###########################################
# cmake module for finding AIDA
#
# requires one of the following vars set:
#   AIDAJNI_HOME
#   RAIDA_HOME
#
# returns:
#   AIDA_FOUND
#   AIDA_INCLUDE_DIRS
#   AIDA_LIBRARIES
#   AIDA_DEFINITIONS
#
# @author Jan Engels, DESY
###########################################

SET( AIDA_FOUND FALSE )
MARK_AS_ADVANCED( AIDA_FOUND )

INCLUDE( "MacroLoadPackage" )

SET( RAIDA_HOME "${AIDA_HOME}" )
LOAD_PACKAGE( RAIDA QUIET )
IF( RAIDA_FOUND )
    SET( RAIDA_HOME ${AIDA_HOME} CACHE PATH "Path to RAIDA" FORCE )
    SET( AIDA_FOUND TRUE )
    SET( AIDA_HOME ${RAIDA_HOME} )
    SET( AIDA_INCLUDE_DIRS ${RAIDA_INCLUDE_DIRS} )
    SET( AIDA_LIBRARIES ${RAIDA_LIBRARIES} )
    SET( AIDA_DEFINITIONS ${RAIDA_DEFINITIONS} )
ENDIF()

IF( NOT AIDA_FOUND )
    SET( AIDAJNI_HOME "${AIDA_HOME}" )
    LOAD_PACKAGE( AIDAJNI QUIET )
    IF( AIDAJNI_FOUND )
        SET( AIDAJNI_HOME ${AIDA_HOME} CACHE PATH "Path to AIDAJNI" FORCE )
        SET( AIDA_FOUND TRUE )
        SET( AIDA_HOME ${AIDAJNI_HOME} )
        SET( AIDA_INCLUDE_DIRS ${AIDAJNI_INCLUDE_DIRS} )
        SET( AIDA_LIBRARIES ${AIDAJNI_LIBRARIES} )
        SET( AIDA_DEFINITIONS ${AIDAJNI_DEFINITIONS} )
    ENDIF()
ENDIF()

IF( AIDA_FOUND )
    MARK_AS_ADVANCED( AIDA_INCLUDE_DIRS AIDA_LIBRARIES AIDA_DEFINITIONS )
    MESSAGE( STATUS "Check for AIDA: ${AIDA_HOME} -- works" )
ELSE()
    IF( AIDA_FIND_REQUIRED )
        MESSAGE( FATAL_ERROR "Check for AIDA: failed to find RAIDA or AIDAJNI in ${AIDA_HOME}" )
    ENDIF()
    IF( NOT AIDA_FIND_QUIETLY )
        MESSAGE( STATUS "Check for AIDA: failed to find RAIDA or AIDAJNI in ${AIDA_HOME}"
            " -- will skip this package..." )
    ENDIF()
ENDIF()
