#####################################
# cmake module for finding CERNLIB
#
# returns:
#   CERNLIB_FOUND
#   CERNLIB_INCLUDE_DIRS
#   CERNLIB_LIBRARIES
#
# @author Jan Engels, DESY
#####################################

SET( CERNLIB_FOUND FALSE )
MARK_AS_ADVANCED( CERNLIB_FOUND )

# include dirs
SET( CERNLIB_INCLUDE_DIR CERNLIB_INCLUDE_DIR-NOTFOUND )
MARK_AS_ADVANCED( CERNLIB_INCLUDE_DIR )

FIND_PATH( CERNLIB_INCLUDE_DIR
    #NAMES stdhep/stdhep.h
    NAMES cfortran/paw.h
    PATHS ${CERNLIB_HOME}
    PATH_SUFFIXES include
    NO_DEFAULT_PATH )

IF( NOT CERNLIB_INCLUDE_DIR AND NOT CERNLIB_FIND_QUIETLY )
    MESSAGE( STATUS "Check for CERNLIB: ${CERNLIB_HOME}"
            " -- failed to find CERNLIB include directory!!" )
ENDIF()

# libraries
SET( CERNLIB_LIB_NAMES mathlib kernlib )
SET( CERNLIB_FINDLIB_FAILED FALSE )
MARK_AS_ADVANCED( CERNLIB_LIB_NAMES CERNLIB_FINDLIB_FAILED )

FOREACH( libname ${CERNLIB_LIB_NAMES} )

    SET( CERNLIB_LIB_${libname} CERNLIB_LIB_${libname}-NOTFOUND )
    MARK_AS_ADVANCED( CERNLIB_LIB_${libname} )

    FIND_LIBRARY( CERNLIB_LIB_${libname}
        NAMES ${libname}
        PATHS ${CERNLIB_HOME}
        PATH_SUFFIXES lib
        NO_DEFAULT_PATH )

    IF( NOT CERNLIB_LIB_${libname} )
        SET( CERNLIB_FINDLIB_FAILED TRUE )
        IF( NOT CERNLIB_FIND_QUIETLY )
            MESSAGE( STATUS "Check for CERNLIB: ${CERNLIB_HOME}"
                    " -- failed to find CERNLIB ${libname} library!!" )
        ENDIF()
    ELSE()
        LIST( APPEND CERNLIB_LIBS ${CERNLIB_LIB_${libname}} )
    ENDIF()
ENDFOREACH()

# set variables and display results
IF( CERNLIB_INCLUDE_DIR AND NOT CERNLIB_FINDLIB_FAILED )
    SET( CERNLIB_FOUND TRUE )
    SET( CERNLIB_INCLUDE_DIRS ${CERNLIB_INCLUDE_DIR} )
    SET( CERNLIB_LIBRARIES ${CERNLIB_LIBS} )
    MARK_AS_ADVANCED( CERNLIB_INCLUDE_DIRS CERNLIB_LIBRARIES )
    MESSAGE( STATUS "Check for CERNLIB: ${CERNLIB_HOME} -- works" )
ELSE()
    IF( CERNLIB_FIND_REQUIRED )
        MESSAGE( FATAL_ERROR "Check for CERNLIB: ${CERNLIB_HOME} -- failed!!" )
    ENDIF()
    IF( NOT CERNLIB_FIND_QUIETLY )
        MESSAGE( STATUS "Check for CERNLIB: ${CERNLIB_HOME}"
                " -- failed!! will skip this package..." )
    ENDIF()
ENDIF()
