########################################
# cmake module for finding CondDBMySQL
#
# returns:
#   CondDBMySQL_FOUND
#   CondDBMySQL_INCLUDE_DIRS
#   CondDBMySQL_LIBRARIES
#   CondDBMySQL_DEFINITIONS
#
# @author Jan Engels, DESY
########################################

SET( CondDBMySQL_FOUND FALSE )
MARK_AS_ADVANCED( CondDBMySQL_FOUND )

# include dirs
SET( CondDBMySQL_INCLUDE_DIR CondDBMySQL_INCLUDE_DIR-NOTFOUND )
MARK_AS_ADVANCED( CondDBMySQL_INCLUDE_DIR )

FIND_PATH( CondDBMySQL_INCLUDE_DIR
    NAMES CondDBInterface.h
    PATHS ${CondDBMySQL_HOME}
    PATH_SUFFIXES include
    NO_DEFAULT_PATH )

IF( NOT CondDBMySQL_INCLUDE_DIR AND NOT CondDBMySQL_FIND_QUIETLY )
    MESSAGE( STATUS "Check for CondDBMySQL: ${CondDBMySQL_HOME}"
            " -- failed to find CondDBMySQL include directory!!" )
ENDIF()

# libraries
SET( CondDBMySQL_LIB CondDBMySQL_LIB-NOTFOUND )
MARK_AS_ADVANCED( CondDBMySQL_LIB )

FIND_LIBRARY( CondDBMySQL_LIB
    NAMES conddb
    PATHS ${CondDBMySQL_HOME}
    PATH_SUFFIXES lib
    NO_DEFAULT_PATH )

IF( NOT CondDBMySQL_LIB AND NOT CondDBMySQL_FIND_QUIETLY )
    MESSAGE( STATUS "Check for CondDBMySQL: ${CondDBMySQL_HOME}"
            " -- failed to find CondDBMySQL library!!" )
ENDIF()

# set variables and display results
IF( CondDBMySQL_INCLUDE_DIR AND CondDBMySQL_LIB )
    SET( CondDBMySQL_FOUND TRUE )
    SET( CondDBMySQL_INCLUDE_DIRS ${CondDBMySQL_INCLUDE_DIR} )
    SET( CondDBMySQL_LIBRARIES ${CondDBMySQL_LIB} )
    SET( CondDBMySQL_DEFINITIONS "-DUSE_CONDDB" )
    MARK_AS_ADVANCED( CondDBMySQL_INCLUDE_DIRS CondDBMySQL_LIBRARIES CondDBMySQL_DEFINITIONS )
    MESSAGE( STATUS "Check for CondDBMySQL: ${CondDBMySQL_HOME} -- works" )
ELSE()
    IF( CondDBMySQL_FIND_REQUIRED )
        MESSAGE( FATAL_ERROR "Check for CondDBMySQL: ${CondDBMySQL_HOME} -- failed!!" )
    ENDIF()
    IF( NOT CondDBMySQL_FIND_QUIETLY )
        MESSAGE( STATUS "Check for CondDBMySQL: ${CondDBMySQL_HOME}"
                " -- failed!! will skip this package..." )
    ENDIF()
ENDIF()
