#####################################
# cmake module for finding GSL
#
# returns:
#   GSL_FOUND
#   GSL_INCLUDE_DIRS
#   GSL_LIBRARIES
#
# @author Jan Engels, DESY
#####################################

SET( GSL_FOUND FALSE )
MARK_AS_ADVANCED( GSL_FOUND )

# include dirs
SET( GSL_INCLUDE_DIR GSL_INCLUDE_DIR-NOTFOUND )
MARK_AS_ADVANCED( GSL_INCLUDE_DIR )

FIND_PATH( GSL_INCLUDE_DIR
    NAMES gsl/gsl_vector.h
    PATHS ${GSL_HOME}
    PATH_SUFFIXES include
    NO_DEFAULT_PATH )

IF( NOT GSL_INCLUDE_DIR AND NOT GSL_FIND_QUIETLY )
    MESSAGE( STATUS "Check for GSL: ${GSL_HOME}"
            " -- failed to find GSL include directory!!" )
ENDIF()


# libraries
SET( GSL_LIB_NAMES gsl gslcblas )
SET( GSL_FINDLIB_FAILED FALSE )
MARK_AS_ADVANCED( GSL_LIB_NAMES GSL_FINDLIB_FAILED )

FOREACH( libname ${GSL_LIB_NAMES} )

    SET( GSL_LIB_${libname} GSL_LIB_${libname}-NOTFOUND )
    MARK_AS_ADVANCED( GSL_LIB_${libname} )

    FIND_LIBRARY( GSL_LIB_${libname}
        NAMES ${libname}
        PATHS ${GSL_HOME}
        PATH_SUFFIXES lib
        NO_DEFAULT_PATH )

    IF( NOT GSL_LIB_${libname} )
        SET( GSL_FINDLIB_FAILED TRUE )
        IF( NOT GSL_FIND_QUIETLY )
            MESSAGE( STATUS "Check for GSL: ${GSL_HOME}"
                    " -- failed to find GSL ${libname} library!!" )
        ENDIF()
    ELSE()
        LIST( APPEND GSL_LIBS ${GSL_LIB_${libname}} )
    ENDIF()
ENDFOREACH()

# set variables and display results
IF( GSL_INCLUDE_DIR AND NOT GSL_FINDLIB_FAILED )
    SET( GSL_FOUND TRUE )
    SET( GSL_INCLUDE_DIRS ${GSL_INCLUDE_DIR} )
    SET( GSL_LIBRARIES ${GSL_LIBS} )
    MARK_AS_ADVANCED( GSL_INCLUDE_DIRS GSL_LIBRARIES )
    MESSAGE( STATUS "Check for GSL: ${GSL_HOME} -- works" )
ELSE()
    IF( GSL_FIND_REQUIRED )
        MESSAGE( FATAL_ERROR "Check for GSL: ${GSL_HOME} -- failed!!" )
    ENDIF()
    IF( NOT GSL_FIND_QUIETLY )
        MESSAGE( STATUS "Check for GSL: ${GSL_HOME}"
                " -- failed!! will skip this package..." )
    ENDIF()
ENDIF()
