#####################################
# cmake module for finding LCIO
#
# requires:
#   ZLIB
#
# returns:
#   LCIO_FOUND
#   LCIO_INCLUDE_DIRS
#   LCIO_LIBRARIES
#   LCIO_DEFINITIONS
#
# @author Jan Engels, DESY
#####################################

SET( LCIO_FOUND FALSE )
MARK_AS_ADVANCED( LCIO_FOUND )

# zlib
FIND_PACKAGE( ZLIB QUIET REQUIRED )

# include dirs
SET( LCIO_INCLUDE_DIR LCIO_INCLUDE_DIR-NOTFOUND )
MARK_AS_ADVANCED( LCIO_INCLUDE_DIR )

FIND_PATH( LCIO_INCLUDE_DIR
    NAMES lcio.h
    PATHS ${LCIO_HOME}
    PATH_SUFFIXES src/cpp/include include
    NO_DEFAULT_PATH )
    
IF( NOT LCIO_INCLUDE_DIR AND NOT LCIO_FIND_QUIETLY )
    MESSAGE( STATUS "Check for LCIO: ${LCIO_HOME}"
            " -- failed to find LCIO include directory!!" )
ENDIF()

# libraries
SET( LCIO_LIB_NAMES lcio sio )
SET( LCIO_FINDLIB_FAILED FALSE )
MARK_AS_ADVANCED( LCIO_LIB_NAMES LCIO_FINDLIB_FAILED )
                                                                                                                                                            
FOREACH( libname ${LCIO_LIB_NAMES} )
                                                                                                                                                            
    SET( LCIO_LIB_${libname} LCIO_LIB_${libname}-NOTFOUND )
    MARK_AS_ADVANCED( LCIO_LIB_${libname} )
                                                                                                                                                            
    FIND_LIBRARY( LCIO_LIB_${libname}
        NAMES ${libname}
        PATHS ${LCIO_HOME}
        PATH_SUFFIXES lib sio/lib
        NO_DEFAULT_PATH )
                                                                                                                                                            
    IF( NOT LCIO_LIB_${libname} )
        SET( LCIO_FINDLIB_FAILED TRUE )
        IF( NOT LCIO_FIND_QUIETLY )
            MESSAGE( STATUS "Check for LCIO: ${LCIO_HOME}"
                    " -- failed to find LCIO ${libname} library!!" )
        ENDIF()
    ELSE()
        LIST( APPEND LCIO_LIBS ${LCIO_LIB_${libname}} )
    ENDIF()
ENDFOREACH()

# set variables
IF( ZLIB_FOUND AND LCIO_INCLUDE_DIR AND NOT LCIO_FINDLIB_FAILED )
    SET( LCIO_FOUND TRUE )
    SET( LCIO_INCLUDE_DIRS ${LCIO_INCLUDE_DIR} )
    SET( LCIO_LIBRARIES ${LCIO_LIBS} ${ZLIB_LIBRARIES} )
    # need long long for int64 for now
    # FIXME: should establish wether we are on a 32bit or 64 bit machine ...
    SET( LCIO_DEFINITIONS "-Wno-long-long" )
    MARK_AS_ADVANCED( LCIO_INCLUDE_DIRS LCIO_LIBRARIES LCIO_DEFINITIONS )
    MESSAGE( STATUS "Check for LCIO: ${LCIO_HOME} -- works" )
ELSE()
    IF( LCIO_FIND_REQUIRED )
        MESSAGE( FATAL_ERROR "Check for LCIO: ${LCIO_HOME} -- failed!!" )
    ENDIF()
    IF( NOT LCIO_FIND_QUIETLY )
        MESSAGE( STATUS "Check for LCIO: ${LCIO_HOME} -- "
                "failed!! will skip this package..." )
    ENDIF()
ENDIF()
