#############################################################
# cmake module for finding mysqlclient
#
# returns:
#   MySQL_INCLUDE_DIRS - where to find mysql.h, etc.
#   MySQL_LIBRARIES    - list of MySQL libraries.
#   MySQL_FOUND        - True if MySQL found.
#
# @author Jan Engels, DESY
#############################################################

SET( MySQL_FOUND FALSE )
MARK_AS_ADVANCED( MySQL_FOUND )

# include dirs
SET( MySQL_INCLUDE_DIR MySQL_INCLUDE_DIR-NOTFOUND )
FIND_PATH( MySQL_INCLUDE_DIR
    NAMES mysql.h
    PATHS ${MySQL_HOME}
    /usr
    /usr/local
    /opt/local
    /opt/products/mysql/5.0.45
    /opt/products/mysql/5.0.26
    PATH_SUFFIXES include include/mysql include/mysql5/mysql
)

IF( NOT MySQL_INCLUDE_DIR AND NOT MySQL_FIND_QUIETLY )
    MESSAGE( STATUS "Check for MySQL: ${MySQL_HOME}"
            " -- failed to find MySQL include directory!!" )
ENDIF()

# libraries
SET( MySQL_LIB MySQL_LIB-NOTFOUND )
FIND_LIBRARY( MySQL_LIB
    NAMES mysqlclient_r mysqlclient
    PATHS ${MySQL_HOME}
    /usr
    /usr/local
    /opt/local
    /opt/products/mysql/5.0.45
    /opt/products/mysql/5.0.26
    PATH_SUFFIXES lib lib/mysql lib/mysql5/mysql
)

IF( NOT MySQL_LIB AND NOT MySQL_FIND_QUIETLY )
    MESSAGE( STATUS "Check for MySQL: ${MySQL_HOME}"
            " -- failed to find MySQL library!!" )
ENDIF()

# set variables and display results
IF( MySQL_INCLUDE_DIR AND MySQL_LIB )
    SET( MySQL_FOUND TRUE )
    SET( MySQL_LIBRARIES ${MySQL_LIB} )
    SET( MySQL_INCLUDE_DIRS ${MySQL_INCLUDE_DIR} )
    MARK_AS_ADVANCED( MySQL_INCLUDE_DIRS MySQL_LIBRARIES )
    #SET( MySQL_DEFINITIONS "-DUSE_MySQL" )
    #MARK_AS_ADVANCED( MySQL_INCLUDE_DIRS MySQL_LIBRARIES MySQL_DEFINITIONS )
    MESSAGE( STATUS "Check for MySQL: ${MySQL_HOME} -- works" )
ELSE()
    IF( MySQL_FIND_REQUIRED )
        MESSAGE( FATAL_ERROR "Check for MySQL: ${MySQL_HOME} -- failed!!" )
    ENDIF()
    IF( NOT MySQL_FIND_QUIETLY )
        MESSAGE( STATUS "Check for MySQL: ${MySQL_HOME}"
                " -- failed!! will skip this package..." )
    ENDIF()
ENDIF()
