#ifndef BIASED_RESIDUALS_PROCESSOR_H
#define BIASED_RESIDUALS_PROCESSOR_H 1

//LCIO
#include <lcio.h>

//MARLIN
#include "marlin/Processor.h"

namespace AIDA{
  class IHistogram1D;
}


namespace marlintpc
{

    class TrackFitterFactory;
//    class TrackFitterBase;

    /** Fill the residuals of the hits into an AIDA histogram.
     *  Remember that LCIO::TRBIT_HITS has to be set, so the hits are stored.
     *
     *  <h4>Input - Prerequisites</h4>
     *  \li A Collection of reconstructed tracks (LCIO::Track)
     *
     *  <h4>Output</h4>
     *  \li Histograms of the residuals in the xy-plane and in z
     *
     *  @param InputTrackes Name of input Tracks collection (default: TPCTracks)
     *  @param HistogramMinValueXY Minimum value in xy histogram (default: -0.3 mm)
     *  @param HistogramMaxValueXY Maximal value in xy histogram (default:  0.3 mm)
     *  @param HistogramMinValueZ  Minimum value in z histogram (default: -1.0 mm)
     *  @param HistogramMaxValueZ  Maximal value in z histogram (default:  1.0 mm)
     *
     *
     *  @author Martin Killenberg, University of Bonn
     */
    class BiasedResidualsProcessor : public marlin::Processor{
			      
    public:
    
	virtual Processor*  newProcessor() { return new BiasedResidualsProcessor ; }
    
	BiasedResidualsProcessor();

	~BiasedResidualsProcessor();

	virtual void init();

	virtual void processRunHeader( EVENT::LCRunHeader* run ) ;

	virtual void processEvent( LCEvent * evt ) ; 
    
    protected:

     std::string _inputColName ;    ///< the name of the input collection
     
//     TrackFitterBase *trackFitter;
     TrackFitterFactory *_fitterFactory; ///< The fitter factory

     AIDA::IHistogram1D * _residualsXYHisto; ///< the residuals in the xy-plane
     AIDA::IHistogram1D * _residualsZHisto; ///< the residuals in z (time)

     float  _histogramMinValueXY; ///< Minimum value in xy histogram
     float  _histogramMaxValueXY; ///< Maximum value in xy histogram

     float  _histogramMinValueZ; ///< Minimum value in z histogram
     float  _histogramMaxValueZ; ///< Maximum value in z histogram
    };
}

#endif // BIASED_RESIDUALS_PROCESSOR_H
