#include "TimePixOccupancyProcessor.h"

//C++
#include <iostream>
using namespace std;

//LCIO
#include <IMPL/LCCollectionVec.h>
#include <EVENT/TrackerData.h>

// Aida
//#ifdef MARLIN_USE_AIDA // the whole processor does not make sense without aida
#include <AIDA/AIDA.h>
#include <marlin/AIDAProcessor.h>
#include <AIDA/IHistogramFactory.h>
#include <AIDA/IHistogram2D.h>
// endif

#include <marlin/Global.h>

using namespace lcio ;
using namespace marlin ;

namespace marlintpc
{

TimePixOccupancyProcessor aTimePixOccupancyProcessor;
    
TimePixOccupancyProcessor::TimePixOccupancyProcessor(): Processor("TimePixOccupancyProcessor")
{
    // register steering parameters: name, description, class-variable, default value
    registerInputCollection(    LCIO::TRACKERDATA,
				"InputColName" , 
				"Name of the input collection  with lcio::TrackerData "
				"(default: TimePixInterpolatedData)"  ,
				_inputColName ,
				std::string("TimePixInterpolData") ) ;
    // modify processor description
    _description = "TimePixOccupancyProcessor: Count how many times a pixel has been hit" ;

  registerProcessorParameter( "NColumns" , 
			      "Number of colmns on the chip (default: 256)"  ,
			      _nColumns ,
			      int(256) ) ;

  registerProcessorParameter( "NRows" , 
			      "Number of rows on the chip (default: 256)"  ,
			      _nRows ,
			      int(256) ) ;
}

TimePixOccupancyProcessor::~TimePixOccupancyProcessor()
{
}

void TimePixOccupancyProcessor::init()
{ 
	// usually a good idea to
	printParameters() ;
}

void TimePixOccupancyProcessor::processRunHeader( EVENT::LCRunHeader* run )
{
}
    
void TimePixOccupancyProcessor::processEvent( LCEvent * evt ) 
{ 
    //read input data
    streamlog_out(MESSAGE3) << "Event: " << evt->getEventNumber() << std::endl;

    // we need a LCCollectionVec to use iterators, as LCCollection is not a std::vector
    LCCollectionVec* InputCollection=0;
    try{
	InputCollection = dynamic_cast<LCCollectionVec*>(evt->getCollection( _inputColName )) ;
    }
    catch(DataNotAvailableException &e)
    {
	// no input data
	// just continue ...
	return;
    }


    // cast did not succeed
    if( InputCollection == 0 )
    {
	streamlog_out(ERROR2) << "InputCollection vector is 0" << std::endl;
	return;
    }

    for(LCCollectionVec::iterator it = InputCollection->begin() ; 
	it < InputCollection->end(); it ++ )
    {
	
	TrackerData *timePixData =  dynamic_cast<TrackerData *>(*it);
	
	// search for histogram with chip ID CellID1 in the map
	std::map<int, AIDA::IHistogram2D *>::iterator histoIter =
	    _timePixOccupancyHistos.find(timePixData->getCellID1());
	
	// if histo was not found create it
	if (histoIter == _timePixOccupancyHistos.end() )
	{
	    // create a name for the histogram
	    std::stringstream histotitle;
	    histotitle << "TimePix Occupancy Chip"<< timePixData->getCellID1();

	    std::stringstream histoname;
	    histoname << "TimePix_occupancy_"<< _inputColName << "_chip" <<timePixData->getCellID1();

	    _timePixOccupancyHistos[timePixData->getCellID1()] = AIDAProcessor::histogramFactory(this)->
		createHistogram2D( histoname.str(),  histotitle.str(), _nColumns, 0, _nColumns , _nRows, 0, _nRows);

	    histoIter = _timePixOccupancyHistos.find(timePixData->getCellID1());
	}

//	    cout << "nr of hits " << hitVector.size()<< endl;
	
//	    int hitNr=0;
	
//	    cout << "\t\t hit\t x \t y \t z \t residual_x/r*phi" << endl;
	for (unsigned int pixelID =  timePixData->getCellID0() ;
	     pixelID <  timePixData->getCellID0() +  timePixData->getChargeValues().size();
	     pixelID++)
	{
	    // histo iter is an iterator of a pair, the histo pointer is pair.second
	    (*histoIter).second->fill(pixelID%_nColumns, pixelID/_nColumns);
	}
    }
    
}


}// namespace 
