#ifndef DBEntryMaker_h
#define DBEntryMaker_h 1

//C++
#include <iostream>

//LCIO
#include <lcio.h>

//MARLIN
#include <marlin/Processor.h>

#include "DBWriter.h"

namespace marlintpc {

    /** abstract processor to extract conditions data and wirie them to
     *  - mySQL database
     *  - data base file (based on LCIO)
     *  - a simple LCIO file
     *  - the marlin data stream
     *
     * @author M. E. Janssen, DESY
     * @version $Id: DBEntryMaker.h,v 1.1 2006/07/07 08:58:52 mjanssen Exp $ 
     */
    class DBEntryMaker : public marlin::Processor, protected DBWriter {
	
    public:
	
	// newProcessor() method must be implented by derived processors

	/** Contructor which also register in- and output collection
	 *  and some other parameters
	 *  (this constructor should be used)
	 * @param typeName name of the Processor (@see Processor)
	 * @param inputCol default name for the input colletion (can be changed via processor parameter)
	 * @param outputCol default name for the output collection (can be changed via processor parameter)
	 */
	DBEntryMaker( const std::string &typeName, std::string inputCol, std::string outputCol );

	/** simple constructor, which calls the super constructor only
	 * @param typeName name of the Processor (@see Processor)
	 */
	DBEntryMaker( const std::string &typeName) : marlin::Processor( typeName ){};
    
	virtual void init() = 0;
    
	virtual void processRunHeader( lcio::LCRunHeader* run ) = 0;
    
	virtual void processEvent( lcio::LCEvent * evt ) = 0; 
    
	virtual void check( lcio::LCEvent * evt ) = 0; 
    
	virtual void end() = 0;

    protected:

	/// common init routine for all DBEntryMaker (must be called in init() );
	virtual void makerInit();
	

	/// common processEvent routine for all DBEntryMaker (must be called in processEvent() );
	virtual void makerProcessEvent( lcio::LCEvent* evt );

	/// common end routine for all DBEntryMaker (must be called in end() );
	virtual void makerEnd();


	virtual lcio::LCCollection* makeLCCDData() = 0;

	int _NRun ;
	int _NEvt ;

	/// name of the collection, where are input data are stored
	std::string   _InputColName;

	int           _WriteEveryNEvents;
	std::string   _FirstSince;
	std::string   _LastTill;

	std::string   _OutputColName;
	std::string   _OutputFileName;
	std::string   _DBOutputFileName;
	std::string   _LCCDDatabaseName;
	std::string   _LCCDDatabaseFolder;
	std::string   _EventStreamColName;

	long long int _lastSinceTime;
	long long int _lastEventTime;

	LCCDEntryVec* _LCCDCollection;

    };

}
#endif
