#ifndef DBWriter_h
#define DBWriter_h 1

//C++
#include <string.h>

//LCIO
#include <lcio.h>
#include <EVENT/LCCollection.h>
#include <EVENT/LCEvent.h>

namespace marlintpc {
    /// structure to store conditions data together with the valid time periode
    struct LCCDEntry{
	long long int        since;
	long long int        till;
	EVENT::LCCollection* Data;
    };
    typedef std::vector<LCCDEntry*> LCCDEntryVec;
  

    /** general class which provide the methods to write LCCD data to
     *  - mySQL database (if compiled with CondDB)
     *  - data base file (based on LCIO)
     *  - a simple LCIO file
     *
     * @author M. E. Janssen, DESY
     * @version $Id: DBWriter.h,v 1.1 2006/07/07 08:58:52 mjanssen Exp $ 
     */  
    class DBWriter {
	
    public:
	
	DBWriter(){};
	
	virtual ~DBWriter(){};
	
// one can only write to a DB when Marlin is compiled with CondDB (USE_LCCD is included in Use_CondDB)
#ifdef USE_CONDDB
	/** write the LCCD collection to a new LCIO DB File with the given name
	 * @param DBName name of the mySQL database
	 * @param DBFolder subfolder of the database, where the data should be stored
	 * @param ColName name of the collection in the folder
	 * @param thisCol collections to be stored
	 */
	virtual void WriteToDB(std::string DBName, std::string DBFolder,
			       std::string ColName, LCCDEntryVec* thisCol);
#endif
	
	/** write the LCCD collection to a new LCIO DB File with the given name
	 * @param FileName name of the file where the data should be stored
	 * @param ColName name of the collection in the file
	 * @param thisCol collections to be stored
	 */
	virtual void WriteToDBFile(std::string FileName, std::string ColName,
				   LCCDEntryVec* thisCol);
	
	/** write the output collection to a new LCIO File with the given name
	 * @param FileName (first part of the) name of the file(s) where the data should be stored
	 * @param ColName name of the collection in the file
	 * @param thisCol collections to be stored
	 * @param addTimeToFileName if true the valid time will be used as suffix for the files
	 */
	virtual void WriteToSimpleFile(std::string FileName, std::string ColName,
				       LCCDEntryVec* thisCol,
				       bool addTimeToFileName = true);


    private:
	/** convert a longlongint to a string
	 */
	std::string longIntToString( long long int i );
	
    };
    
}
#endif
    
