#ifndef PedestalCalculator_h
#define PedestalCalculator_h 1
/* PedestalCalculator.h
 * $Id: PedestalCalculator.h,v 1.1 2006/07/07 08:58:52 mjanssen Exp $
 */

//LCIO
#include <lcio.h>

//MARLIN
#include <marlin/Processor.h>

//MalinTPC
#include "DBEntryMaker.h"

//TPCCondData
#include "Pedestal.h"

namespace marlintpc {

    struct StcPedestal{
	float Mean;
	float Width;
    };
    typedef std::vector<StcPedestal*> PedestalVector_t;

    struct ChannelPedestal{
	int                 ChannelID;
	FloatVec*           Data;
	PedestalVector_t*   PedestalCol;
	StcPedestal*        Average;
	StcPedestal*        AverageError;
    };
    typedef std::vector<ChannelPedestal*> ChannelPedestalCollection_t;
    
    /** a processor for Marlin
     *  calculates the pedestals out of the TPC raw data
     */
    class PedestalCalculator : public DBEntryMaker {
			      
    public:
    
	virtual Processor*  newProcessor() { return new PedestalCalculator ; }
    
	PedestalCalculator();

	PedestalCalculator( const std::string &typeName ) : DBEntryMaker( typeName ){};
    
	/** Called at the begin of the job before anything is read.
	 * Use to initialize the processor, e.g. book histograms.
	 */
	virtual void init();
    
	/** Called for every run.
	 */
	virtual void processRunHeader( lcio::LCRunHeader* run ) ;
    
	/** Called for every event - the working horse.
	 */
	virtual void processEvent( lcio::LCEvent * evt ) ; 
    
    
	virtual void check( lcio::LCEvent * evt ) ; 
    
    
	/** Called after data processing for clean up.
	 */
	virtual void end() ;
    
    protected:

	ChannelPedestalCollection_t* _ChannelPedestalCollection;

	/** calculate the mean and the width of the pedestal out of the given data
	 *  @return a structure Pedestal containing these values 
	 */
	StcPedestal* calcPedestal(FloatVec* Data);

	/** calculate the average of the values for mean and width of the pedestal
	 *  of the given structure and save it in this structure
	 */
	virtual void                   calcAverage(ChannelPedestal* thisChannelPedestal);

	/** create a new collection of Pedestals and fill it with the actual pedestals
	 */
	virtual lcio::LCCollectionVec*       CreatePedestalCollection( ChannelPedestalCollection_t* 
								 thisChannelPedestalCollection);

	/** write the actual pedestal to the std output
	 */
	virtual void                   printPedestal(ChannelPedestalCollection_t*
						     thisChannelPedestalCollection);

	/** write a summery of the actual pedestal to the std output
	 */
	virtual void                   printPedestalShort(ChannelPedestalCollection_t*
							  thisChannelPedestalCollection);

	virtual void                   printPedestal(LCCollection* thisCollection);

	virtual lcio::LCCollection* makeLCCDData();
    
    };
}
#endif
