#include "DBEntryMaker.h"

using namespace lcio ;
using namespace marlin ;

namespace marlintpc {

    DBEntryMaker::DBEntryMaker( const std::string &typeName,
				std::string inputCol, std::string outputCol) : Processor( typeName ){

	// modify processor description
	_description = "base Processor for Processors creating collection of condition data (DO NOT USE)" ;

	registerProcessorParameter( "InputCollectionName", 
				    "Name of the input collection"  ,
				    _InputColName ,
				    inputCol ) ;
    
	registerProcessorParameter( "writeEveryNEvents" , 
				    "write the the collection every N events ( -1: write only at the end)"  ,
				    _WriteEveryNEvents ,
				    -1 );

	registerProcessorParameter( "validSince" , 
				    std::string("since time of first entry in ")+
				    std::string("ns since 1.1.1970 00:00:00 UTC")+
				    std::string("if -1 the time of the first event will be taken"),
				    _FirstSince,
				    std::string("-1") );

	registerProcessorParameter( "validTill" , 
				    std::string("since time of last entry in ns since 1.1.1970 00:00:00 UTC")+
				    std::string("if -1 the time of the last event will be taken"),
				    _LastTill,
				    std::string("-1") );

	registerProcessorParameter( "OutputCollectionName" , 
				    "name of the output collection",
				    _OutputColName,
				    outputCol );

	registerProcessorParameter( "OutputFileName" , 
				    std::string("name of the simple output file (existing file will")+
				    std::string(" be overwritten) if empty no file is written")+
				    std::string(" in one file only one Collection is saved,")+
				    std::string(" valid time will added in the file name before the dot"),
				    _OutputFileName ,
				    std::string("") );

	registerProcessorParameter( "DBOutputFileName" , 
				    std::string("name of the data base output file (existing file will")+
				    std::string(" be overwritten) if empty no file is written"),
				    _DBOutputFileName ,
				    std::string("") );

#ifdef USE_CONDDB
	registerProcessorParameter( "LCCDDatabaseName" , 
				    std::string("name of LCCD data base")+
				    std::string(" if empty no information will be writen to data base"),
				    _LCCDDatabaseName ,
				    std::string("") );

	registerProcessorParameter( "LCCDDatabaseFolder" , 
				    "name of folder in the given LCCD data base",
				    _LCCDDatabaseFolder ,
				    std::string("") );
#endif

	registerProcessorParameter( "EventStreamCollectionName" , 
				    std::string("name of the collection writen to the current data stream")+
				    std::string(" make use of writeEveryNEvents ")+
				    std::string(" if empty no collection is written")+
				    std::string(" PLEASE NOTE: the conditions data will not be valied")+
				    std::string(" in right time periode"),
				    _EventStreamColName ,
				    std::string("") );
    }


    void DBEntryMaker::makerInit(){

	_NRun = 0 ;
	_NEvt = 0 ;
 
	_LCCDCollection = new LCCDEntryVec();

    }

    void DBEntryMaker::makerProcessEvent( LCEvent* evt ){

	if (_NEvt==0){
	    //first event

	    // set the first valid since time to processor parameter
	    _lastSinceTime = atoll( _FirstSince.c_str() );
	    if ( _lastSinceTime < 0 ){
		//or the time of the first event
		_lastSinceTime = evt->getTimeStamp();
	    }
	}
    
	_lastEventTime = evt->getTimeStamp();
    
	_NEvt ++ ;
	if (_NEvt%100 == 0) {
	    std::cout<<_NEvt<<"\r";
	    std::cout.flush();
	}

	if ( ( _WriteEveryNEvents > 0) && ( (_NEvt%_WriteEveryNEvents) == 0 ) ){
	    LCCollection* tempCol = makeLCCDData();
	    if (tempCol){
		LCCDEntry*    newLCCDEntry = new LCCDEntry();
		newLCCDEntry->since = _lastSinceTime;
		newLCCDEntry->till  = _lastEventTime;
		newLCCDEntry->Data  = tempCol;
	    
		_LCCDCollection->push_back( newLCCDEntry );
	    
		_lastSinceTime       = _lastEventTime; //set for next entry

		/* write the output collection to a given LC event (in the marlin data steam)
		 *  PLEASE NOTE: the conditions data will not be valied in right time periode
		 *  (exept written every event)
		 *  so here only one LCCDEntry is written
		 */
		if ( _EventStreamColName.length()>0 ){
		    evt->addCollection( tempCol, _EventStreamColName );
		    std::cout<<"DBEntryMaker: wrote "<<_EventStreamColName<<" to event"<<std::endl;
		}
	    }
	}
	
    }

    void DBEntryMaker::makerEnd(){ 

	long long int lastTillTime = atoll( _LastTill.c_str() );
	if (lastTillTime<=0) {
	    lastTillTime = _lastEventTime;
	}
	LCCollection* tempCol = makeLCCDData();
	if ((tempCol)&&(_lastSinceTime<lastTillTime)){
	    LCCDEntry*    newLCCDEntry = new LCCDEntry();
	    newLCCDEntry->since = _lastSinceTime;
	    newLCCDEntry->till  = lastTillTime+1;
	    newLCCDEntry->Data  = tempCol;
	
	    _LCCDCollection->push_back( newLCCDEntry );
	}
	else{
	    if (_LCCDCollection->size()>0){
		_LCCDCollection->back()->till  = lastTillTime+1;
	    }
	}
    
    
	if ( _OutputFileName.length()>0 ){
	    std::cout<<"Write SimpleFile "<<_OutputFileName<<std::endl;
	    WriteToSimpleFile( _OutputFileName, _OutputColName, _LCCDCollection );
	}

	if ( _DBOutputFileName.length()>0 ){
	    std::cout<<"Write DataBaseFile "<<_DBOutputFileName<<std::endl;
	    WriteToDBFile( _DBOutputFileName, _OutputColName, _LCCDCollection );
	}

#ifdef USE_CONDDB
	if ( _LCCDDatabaseName.length()>0 ){
	    std::cout<<"Write to Data Base "<<_LCCDDatabaseName<<" :: "<<_LCCDDatabaseFolder<<std::endl;
	    WriteToDB( _LCCDDatabaseName, _LCCDDatabaseFolder, _OutputColName, _LCCDCollection );
	}
#endif

    }

}
