#include <DBWriter.h>

//C++
#include<iostream>

//LCIO
#include <IMPL/LCEventImpl.h>
#include <IMPL/LCRunHeaderImpl.h>
#include <IO/LCWriter.h>


#ifdef USE_CONDDB
//LCCD
#include "lccd.h"
#include "lccd/DBInterface.hh"
#endif

namespace marlintpc {

#ifdef USE_CONDDB
    void DBWriter::WriteToDB(std::string DBName, std::string DBFolder, 
			     std::string ColName, LCCDEntryVec* thisCol){

	lccd::DBInterface* Interface = new lccd::DBInterface( DBName, DBFolder, true);

	for (LCCDEntryVec::iterator iEntry=thisCol->begin(); iEntry!=thisCol->end();
	     iEntry++){
	    Interface->storeCollection( (*iEntry)->since, (*iEntry)->till,
					(*iEntry)->Data, ColName);
	}

	delete Interface;

    }
#endif

    void DBWriter::WriteToDBFile(std::string FileName, std::string ColName,
				 LCCDEntryVec* thisCol){
	// create a LCWriter
	IO::LCWriter*    LcWrt    = lcio::LCFactory::getInstance()->createLCWriter();

	// create (overwrite) and  open the file 
	LcWrt->open( FileName, lcio::LCIO::WRITE_NEW ) ;

	// store the valid time periodes

	// create a new run header
	IMPL::LCRunHeaderImpl* newRunHeader = new IMPL::LCRunHeaderImpl();

	/* the differend vaild time periods will be given by the a since and a till time
	 * which are stored as strings in vector in the run haeder
	 */
	EVENT::StringVec SinceTimes;
	EVENT::StringVec TillTimes;
	for (LCCDEntryVec::iterator iEntry=thisCol->begin(); iEntry!=thisCol->end();
	     iEntry++){
	    SinceTimes.push_back( longIntToString( (*iEntry)->since ) );
	    TillTimes.push_back( longIntToString( (*iEntry)->till ) );
	} 
	newRunHeader->parameters().setValues( "DBSince", SinceTimes );
	newRunHeader->parameters().setValues( "DBTill", TillTimes );

	LcWrt->writeRunHeader( newRunHeader );

	// store the conditions data

	// create an event for each valid time period
	IMPL::LCEventImpl* newEvent = 0;
	int          iEvent   = 0;
	for (LCCDEntryVec::iterator iEntry=thisCol->begin(); iEntry!=thisCol->end();
	     iEntry++){

	    // create a new Event
	    newEvent = new IMPL::LCEventImpl();
	    newEvent->setEventNumber( iEvent++ );

	    //add the collection of conditions data to the event, write the event the file
	    newEvent->addCollection(  (*iEntry)->Data, ColName );
	    LcWrt->writeEvent( newEvent ) ;
	}

	LcWrt->close() ;
	delete LcWrt;
    
    }


    void DBWriter::WriteToSimpleFile(std::string FileName, std::string ColName,
				     LCCDEntryVec* thisCol, bool addTimeToFileName){
	
	// create a LCWriter
	IO::LCWriter*    LcWrt    = lcio::LCFactory::getInstance()->createLCWriter() ;
    
	std::string::size_type DotPosition = FileName.find('.', (std::string::size_type)0);
    
	if (DotPosition == std::string::npos) DotPosition = FileName.length();

	/* the conditions data are stored in a simple LCIO file
	 * one file for each valid time period
	 */
	for (LCCDEntryVec::iterator iEntry=thisCol->begin(); iEntry!=thisCol->end();
	     iEntry++){    

	    // create (overwrite) and  open the file 
	    LcWrt->open( (addTimeToFileName?
			  FileName.insert(DotPosition,
					  longIntToString( (*iEntry)->since )+"_"+
					  longIntToString( (*iEntry)->till) ) 
			  :FileName), lcio::LCIO::WRITE_NEW );

	    // create a new Event
	    IMPL::LCEventImpl* newEvent = new IMPL::LCEventImpl();
	    //add the Collection to the event, write the event and close the file
	    newEvent->addCollection( (*iEntry)->Data , ColName );
	
	    LcWrt->writeEvent( newEvent ) ;
	    LcWrt->close() ;

	}
	delete LcWrt;
    }

    std::string DBWriter::longIntToString( long long int i ){

	char* temp = new char[0];
	std::string returnValue("");
	long long int divider   = 10;
	long int      output    = 0;
	for (long long int remainder = i; remainder != 0; remainder /= divider){
	    output = remainder%divider;
	    sprintf( temp, "%li", output );
	    returnValue = std::string(temp)+returnValue;
	    //std::cout<<remainder<<" "<<divider<<" "<<temp<<" "<<returnValue<<std::endl;
	}
	return returnValue;

    }
}
