#ifndef ChargeDistributionProcessor_h
#define ChargeDistributionProcessor_h 1

#include "VoxelTPC.h"

//LCIO
#include "lcio.h"

//MARLIN
#include "marlin/Processor.h"

//C++
#include <string>


/** This processor takes the coordinates of the drifted electrons 
 *  and the charge from amplification and smears the charge 
 *  on the padplane. Broadening of the charge cloud due to 
 *  diffussion between the GEMs is included as well as charge 
 *  transfer parameters in dependence on the GEM settings.
 */

class ChargeDistributionProcessor : public marlin::Processor {
  
 public:
  
  virtual Processor*  newProcessor() { return new ChargeDistributionProcessor ; }
  
  
  ChargeDistributionProcessor() ;
  virtual ~ChargeDistributionProcessor(){}
  
  /** Called at the begin of the job before anything is read.
   *  Use to initialize the processor, e.g. book histograms.
   */
  virtual void init() ;
  
  /** Called for every run.
   */
  virtual void processRunHeader( EVENT::LCRunHeader* run ) ;
  
  /** Called for every event - the working horse.
   */
  virtual void processEvent( EVENT::LCEvent * evt ) ; 
  
  
  virtual void check( EVENT::LCEvent * evt ) ; 
  
  
  /** Called after data processing for clean up.
   */
  virtual void end() ;

  double CalculateTransverseDiffusion(double  EField);

  double CalculateGasParameter(double a, double b ,double c, double d, double e, 
                               double f, double g, double h, double i, double j, 
                               double k, double EField);
  
  double CalculateDiffTransBParam(double a, double b, double c, double d, double e);

protected:

  /** Input/output collection names.
   */
  std::string _inputCollectionName;
  std::string _outputCollectionName;
  std::string _gas;

  float _ETrans1, _ETrans2, _EInd;
  int _transient;
  double _sigma;
  float _BField;
  double _bunchIntervall;
  double _zBinLength;

  gear::PadRowLayout2D const * _padLayout;

  marlintpc::VoxelTPC* _trackerData;


};

#endif 
