#ifndef DriftProcessor_h
#define DriftProcessor_h 1


#include <EVENT/SimTrackerHit.h>
#include "marlin/Processor.h"
#include "lcio.h"

#include <string>
#include <gsl/gsl_rng.h>
#include <gsl/gsl_randist.h>

/** Processor to drift primary electrons according to the characteristics of the choosen gas to pad plane.*/

class DriftProcessor : public marlin::Processor {
  
 public:
  
  virtual Processor*  newProcessor() { return new DriftProcessor ; }
  
  
  DriftProcessor() ;
  virtual  ~DriftProcessor(){}  
  /** Called at the begin of the job before anything is read.
   * Use to initialize the processor, e.g. book histograms.
   */
  virtual void init() ;
  
  /** Called for every run.
   */
  virtual void processRunHeader( lcio::LCRunHeader* run ) ;
  
  /** Called for every event - the working horse.
   */
  virtual void processEvent( lcio::LCEvent * evt ) ; 
  
  
  virtual void check( lcio::LCEvent * evt ) ; 
  
  
  /** Called after data processing for clean up.
   */
  virtual void end() ;

  void InitializeDriftValues();
  void CalculateTransverseDiffusion();
  void MakeElectronDriftCoordinates();
  void GetElectronCoordinates(SimTrackerHit *);
  double CalculateGasParameter(double a, double b ,double c, double d,
			       double e, double f, double g, double h,
			       double i, double j,double k);
  double CalculateDiffTransBParam(double a, double b, double c,double d, double e);

protected:

  /** Input/output collection names.
   */
  std::string _inputCollectionName ;
  std::string _outputCollectionName;

  std::string gas;
  
  int _nRun ;
  int _nEvt ;
 
  float E;
  float _BField;
  const char* Gas;
  int is_transient;
  /*! Random generator for landau and gaussian. */
  gsl_rng *r;
  // coordinates and variables for track
  /*! Coordinates of the electron. */
  double x_e,y_e,z_e,t_e; // e- coordinates
  // particle variables
  int numberofelectrons;
  //needed for drifting
  double _TPCLength;
  double _TPCRadius;
  double _TPCInnerRadius;
  double _distance;
/*! Coordinates of the drifted electron. */
  float xdrift,ydrift,zdrift,tdrift; // coordinates of drifted e-
  float vdrift;  // mm/mus  
  float difflong; // mum/sqrt(cm)
  float difftrans;
};

#endif 
