
#ifndef E_FIELDVECTOR_H
#define E_FIELDVECTOR_H 1

//C++
#include "iostream"
#include "string"

//LCIO
#include "lcio.h"
#include "UTIL/LCFixedObject.h"

#define EFieldVectorNINTVals 3 
#define EFieldVectorNFLOATVals 3
#define EFieldVectorNDOUBLEVals 0

namespace marlintpc{

class EFieldVector : public UTIL::LCFixedObject<EFieldVectorNINTVals,
                                                EFieldVectorNFLOATVals,
                                                EFieldVectorNDOUBLEVals> {

public:

  EFieldVector(int xBin=0., int yBin=0., int zBin = 0., float Ex = 0. , float Ey = 0. , float Ez = 0. );

  EFieldVector(EVENT::LCObject* obj): UTIL::LCFixedObject<EFieldVectorNINTVals,
						          EFieldVectorNFLOATVals,
					                  EFieldVectorNDOUBLEVals>(obj) { } 


  EFieldVector(const EFieldVector &original)
    : UTIL::LCFixedObject<EFieldVectorNINTVals,
			  EFieldVectorNFLOATVals,
			  EFieldVectorNDOUBLEVals>(original) 
  { 
    if( _createdObject )  
       _obj = new LCGenericObjectImpl(*(original._obj));
  } 



    
  virtual ~EFieldVector();

  /// Returns the xBin of the vector
  int getXBin() const;

  /// Returns the yBin of the vector
  int getYBin() const;

  /// Returns the zBin of the vector
  int getZBin() const;

  /// Set the EField in the voxel.
  void setEx(float Ex);
  void setEy(float Ey);
  void setEz(float Ez);

  float getEx() const;
  float getEy() const;
  float getEz() const;

  /** Two voxels are equal if their pad index and time index are equal.
   *  The charge is not considered.
   */
  bool operator==(const EFieldVector &right) const;

    /** < operator for e.g.\ sorting in STL containers. 
     *  They are sorted in pad index, and within this in time index.
     *  Charge is not considdered.
     */
    bool operator<(const EFieldVector &right) const; // needed for STL containers

    EFieldVector& operator=(const EFieldVector& that);

    /// returns the svn revision of the code
    static std::string getRevision();

    /// Print the vector information
    void print ( std::ostream& os = std::cout );

    /** Operator to stream the vector information to an outstream:
     *  \code std::cout << myVector << std::endl; \endcode
     */
    std::ostream& operator<<( std::ostream& os );

    // -------- need to implement abstract methods from LCGenericObject
    /// Implementation of the getTypeName method of LCGenericObject
    virtual const std::string getTypeName() const{
	return std::string("EFieldVector");
    }

    /// Implementation of the getDataDescripton method of LCGenericObject
    /// PadIndex, zBinare all ints. Ex,Ey and Ez are doubles.
    virtual const std::string getDataDescription() const{
	return std::string("i:xBin,i:yBin,i:zBin,f:Ex,f:Ey,f:Ez");	
    }

    /// Set the xBin, only called from the constructor
    void setXBin(int xBin);

    /// Set the yBin, only called from the constructor
    void setYBin(int xBin);

    /// Set the zBin, only called from the constructor
    void setZBin(int zBin);
};

}//namespace marlintpc

#endif // TPC_VOXEL2_H
