#ifndef HITDISTRIBUTOR_H
#define HITDISTRIBUTOR_H 1

//MARLINTPC
#include "TPCVoxel.h"

//C++
#include <vector>
#include <stdexcept>

//LCIO
#include <EVENT/SimTrackerHit.h>

//GEAR
#include <gearimpl/RectangularPadRowLayout.h>
#include <gearimpl/FixedPadSizeDiskLayout.h>

namespace marlintpc{

class HitDistributor{

 public:

  /** Constructors for the class. 
   */
  HitDistributor(gear::PadRowLayout2D  const* padLayout, double zBinLength);

  /** Destructor
   */
  ~HitDistributor();

  /** Only function accessible from the outside. Takes a hit and the diffusion parameters 
   *  for input. Calls all the over function necessary, depending on the type of the pad layout.
   */
  std::vector<TPCVoxel> distributeHit(SimTrackerHit* hit, double sigmaTrans, double sigmaLong = 0);

 protected:
  /** Methods to collect the charge of the hit is distributed on. On for each pad layout type.
   */
  std::vector<int> getPads(gear::FixedPadSizeDiskLayout  const* circPL, double sigmaTrans);
  std::vector<int> getPads(gear::RectangularPadRowLayout const* rectPL, double sigmaTrans);

  /** Called by getPads(). Add all pad indices to the vector which are within the 
   *  distribution range and left/right from the input pad index. Again one function 
   *  for each pad layout.
   */
  std::vector<int> getPadsOnTheLeft(gear::FixedPadSizeDiskLayout  const* circPL, 
                                    std::vector<int> padVec, int centerIndex, double sigmaTrans);
  std::vector<int> getPadsOnTheLeft(gear::RectangularPadRowLayout const* rectPL, 
                                    std::vector<int> padVec, int centerIndex, double sigmaTrans);

  std::vector<int> getPadsOnTheRight(gear::FixedPadSizeDiskLayout  const* circPL, 
                                     std::vector<int> padVec, int centerIndex, double sigmaTrans);
  std::vector<int> getPadsOnTheRight(gear::RectangularPadRowLayout const* rectPL, 
                                     std::vector<int> padVec, int centerIndex, double sigmaTrans);

  /** Check if the distance between the input pad and the hit is within the distribution range.
   */
  bool checkDistance(gear::FixedPadSizeDiskLayout  const* circPL, int padIndex, double sigmaTrans);
  bool checkDistance(gear::RectangularPadRowLayout const* rectPL, int padIndex, double sigmaTrans);


  /** Distributes the charge of the hit on the pads in the input vector by integration 
   *  over a 2D gaussian.
   */
  std::vector<TPCVoxel> distributeHitOnPads(gear::FixedPadSizeDiskLayout  const* circPL, std::vector<int> padIndexVec, 
                                        SimTrackerHit* hit, double sigmaTrans, double sigmaLong);

  std::vector<TPCVoxel> distributeHitOnPads(gear::RectangularPadRowLayout const* rectPL, std::vector<int> padIndexVec, 
                                        SimTrackerHit* hit, double sigmaTrans, double sigmaLong);

 protected:

  /** Pointer on the gear pad layou.
   */
  gear::PadRowLayout2D const* _padLayout;

  /** Initilazation parameter. 
   */
  double _zBinLength;

  /** Diffusion parameters. Come with the hit to
   *  smear and may change from hit to hit.
   */
  double _sigmaTrans,_sigmaLong;

  /** Coordinates of the hit. Are newly set for each hit.
   */
  double _xHit, _yHit;

  /** Type of the pad layout. Decides which smearing 
   *  decides which smearing function is called.
   */
  int _padLayoutType;

  //std::vector<int> padIndexVec;

};

} //end of namespace marlintpc



#endif // HITSMEARER_H
