#ifndef IonBackDriftProcessor_h
#define IonBackDriftProcessor_h 1

//MARLIN
#include "marlin/Processor.h"

//LCIO
#include "lcio.h"

//MARLINTPC
#include "VoxelTPC.h"


class IonBackDriftProcessor : public marlin::Processor {
  
 public:
  
  virtual Processor*  newProcessor() { return new IonBackDriftProcessor;}

  IonBackDriftProcessor();
  virtual  ~IonBackDriftProcessor(){}

  /** Called at the begin of the job before anything is read.
   * Use to initialize the processor, e.g. book histograms.
   */
  virtual void init() ;
  
  /** Called for every run.
   */
  virtual void processRunHeader( EVENT::LCRunHeader* run ) ;
  
  /** Called for every event - the working horse.
   */
  virtual void processEvent( EVENT::LCEvent * evt ) ; 
  
  
  virtual void check( EVENT::LCEvent * evt ) ; 

  virtual void end() ;

  void LoadGEMSettings();



protected: 

  // in-/output collection names
 std::string _inputCollectionName;
  std::string _outputCollectionName;

  //steering parameters
  std::string _paramFile;
  int _transient;

  //VoxelTPC for built up of ion disc
  marlintpc::VoxelTPC* _ionDisc;

  //gear pad layout
  gear::PadRowLayout2D const* _padLayout;

  int _nRun ;
  int _nEvt ;

  //ionbackdrif parameter
  double _ionBackDrift;

  //Drift-, 1st, 2nd transfer and induction field
  double _E,_ETrans1,_ETrans2,_EInd;

  //GEM Voltages
  double _UGEM1,_UGEM2,_UGEM3;
};

#endif 
