#ifndef IonsInVoxelsProcessor_h
#define IonsInVoxelsProcessor_h 1

//MARLINTPC
#include "VoxelTPC.h"

//LCIO
#include <lcio.h>

//MARLIN
#include <marlin/Processor.h>

//C++
#include <string>

//GEAR
#include <gear/PadRowLayout2D.h>


/** This processor takes all the ions from the primary ionisation
 *  and stores there charges in voxels. The binning of the voxels 
 *  is given in the x,y-plane by the pads of the readout. The 
 *  length of the voxel in the z-direction can be chosen via the
 *  steering file. For storing and management of the voxels the 
 *  helper class VoxelTPC is used. Output collections contain
 *  the charges of all the ions emerged to the given event.
 */

class marlintpc:: VoxelTPC;

class IonsInVoxelsProcessor : public marlin::Processor {
  
 public:
  
  virtual Processor*  newProcessor() { return new IonsInVoxelsProcessor ; }  
  
  IonsInVoxelsProcessor() ;
  virtual ~IonsInVoxelsProcessor(){}
  
  /** Called at the begin of the job before anything is read.
   *  Use to initialize the processor, e.g. book histograms.
   */
  virtual void init() ;
  
  /** Called for every run.
   */
  virtual void processRunHeader( EVENT::LCRunHeader* run ) ;
  
  /** Called for every event - the working horse.
   */
  virtual void processEvent( EVENT::LCEvent * evt ) ; 
    
  virtual void check( EVENT::LCEvent * evt ) ; 

  /** Called after data processing for clean up.
   */
  virtual void end() ;

  /** Function to get the pad corresponding to a hit for 
   *  the different pad layouts
   */
  int getPad(gear::PadRowLayout2D const* padLayout, SimTrackerHit* hit);
  
  
protected:

  //input and output collections
  std::string _inputIonsInVoxels;
  std::string _outputIonsInVoxels;

  //padLayout pointer
  gear::PadRowLayout2D const * _padLayout;

  //the pad layout type (cartesian or polar)
  int _padLayoutType;

  //pointer to the VoxelTPC which stores the ions
  marlintpc::VoxelTPC* _ionVoxel;

  //input parameters from steering file
  int _transient;
  int _switch;
  double _binLength;

  //TPC parameters
  double _maxDriftLength;
};

#endif 
