#include "TPCVoxel.h"
#include <iostream>

namespace marlintpc{

TPCVoxel::TPCVoxel(int padIndex, int timeIndex, int charge )
{
    setPadIndex(padIndex);
    setTimeIndex(timeIndex);
    setCharge(charge);
}

TPCVoxel::~TPCVoxel(){ }

int TPCVoxel::getPadIndex() const
{
    return getIntVal( 0 ) ;
}

int TPCVoxel::getTimeIndex() const
{
    return getIntVal( 1 ) ;
}

int TPCVoxel::getCharge() const
{
    return getIntVal( 2 ) ;
}

bool TPCVoxel::operator==(const TPCVoxel &right) const
{
  return (  (this->getPadIndex() == right.getPadIndex()) 
	    && (this->getTimeIndex() == right.getTimeIndex() ) );
}
  
bool TPCVoxel::operator<(const TPCVoxel &right) const
{
    // first test the pad indices
    if (this->getPadIndex() < right.getPadIndex()) return true;
    if (this->getPadIndex() > right.getPadIndex()) return false;

    // pad indices are equal, test time indices
    if (this->getTimeIndex() < right.getTimeIndex()) return true;
    // we can skip this as the return value would be false, which is returned anyhow
//    if (this->getTimeIndex() > right.getTimeIndex()) return false;
    return false;
}


TPCVoxel& TPCVoxel::operator=(const TPCVoxel& that)
{
  if (_createdObject)
    delete _obj;

  _createdObject=that._createdObject;

  if (_createdObject)
    _obj = new  LCGenericObjectImpl(*(that._obj));
  else
    _obj = that._obj;

  return *this;
}



void TPCVoxel::print ( std::ostream& os ){
    os << this << std::endl;
} 

std::ostream& TPCVoxel::operator<<( std::ostream& os ){
    
    os<<"TPCVoxel: PadIndeX  "<<getPadIndex()
      <<" ; TimeIndex "<<getTimeIndex()
      <<" ; Charge : "<<getCharge();
    return os;
}

void TPCVoxel::setPadIndex(int padIndex){
    obj()->setIntVal( 0, padIndex) ;
}

void TPCVoxel::setTimeIndex(int timeIndex){
    obj()->setIntVal( 1, timeIndex) ;
}

void TPCVoxel::setCharge(int charge){
    obj()->setIntVal( 2, charge) ;
}

std::string TPCVoxel::getRevision(){
    return std::string("$Rev:$");
}

}
