#ifndef MAPPING_HANDLER_H
#define MAPPING_HANDLER_H

#include <iostream>
#include <ADCChannelMapping.h>
#include <IMPL/LCCollectionVec.h>
#include <IMPL/LCGenericObjectImpl.h>


namespace marlintpc{

  /** A helper class for convenient usage of the channel mapping information.
   *  Allows to get the channelID (hardware channel number/ cellID)
   *  if the padID (logical cellID / padIndex from GEAR) is known;
   *  and vice versa.
   *
   *  @author Martin Killenberg, University of Bonn
   */
class MappingHandler
{
 public:
  MappingHandler();

  /** Set a new channel map.
   *  Internally the vectors to access channelID from padID
   *  and vice versa are recalculated.
   */
  void setChannelMap(LCCollection *chanMapCol);

  /// Get the channelID if the padID is known.
  int getChannelID(int padID);

  /// Get the padID if the channelID is known.
  int getPadID(int channelID);


 private:

  /** A nested helper class for sorting 
   *  ADCChannelMapping objects according to channelID
   */
  class  sortChannelID
  {
    public:
      bool operator()(EVENT::LCObject* adccm1, EVENT::LCObject* adccm2)
      {
	return ( tpcconddata::ADCChannelMapping(adccm1).getChannelID() <
		 tpcconddata::ADCChannelMapping(adccm2).getChannelID() );
      }
  };

  /** A nested helper class for sorting 
   *  ADCChannelMapping objects according to padID
   */
  class  sortPadID
  {
    public:
      bool operator()(EVENT::LCObject* adccm1, EVENT::LCObject* adccm2)
      {
	return ( tpcconddata::ADCChannelMapping(adccm1).getPadID() <
		 tpcconddata::ADCChannelMapping(adccm2).getPadID() );
      }
  };

  /** A nested helper class for comparing an int to an
   *  ADCChannelMapping object's channelID
   */
  class  cmpChannelID
  {
    public:
      bool operator()(EVENT::LCObject* adccm1, int chanID)
      {
	return ( tpcconddata::ADCChannelMapping(adccm1).getChannelID() 
		 < chanID );
      }
  };

  /** A nested helper class for comparing an int to an
   *  ADCChannelMapping object's padID
   */
  class  cmpPadID
  {
    public:
      bool operator()(EVENT::LCObject* adccm1, int padID)
      {
	return ( tpcconddata::ADCChannelMapping(adccm1).getChannelID() 
		 < padID );
      }
  };

 private:
  LCCollectionVec chanMapSortedChanID; ///< The mapping information sorted by channel ID
  LCCollectionVec chanMapSortedPadID; ///< The mapping information sorted by pad ID

};


}// namespace marlintpc

#endif // MAPPING_HANDLER_H
