#ifndef PedestalHandler_h
#define PedestalHandler_h 1

//C++
#include "iostream"

//LCIO
#include "lcio.h"
#include "EVENT/LCCollection.h"

#ifdef USE_LCCD
//LCCD
#include "lccd/IConditionsChangeListener.hh"
#endif

//Marlin
#include "marlin/ConditionsProcessor.h"

//TPCCondData
#include <Pedestal.h>


// the package schould be capsulated in a name space
/// the MarlinTPC Package: reconstruction of TPC data
namespace marlintpc{

    /// map channel to pedestal object (see: TPCCondData)
    typedef std::map< int, tpcconddata::Pedestal* > PedestalMap;
    typedef std::pair< int, tpcconddata::Pedestal* > PedestalMapPair;
    
//#ifdef USE_LCCD
    /** Helper Class which provides the acutal pedestals for every event.
     *  It is derived from IConditionsChangeListener,
     *  so the pedestals are updated every time the conditions data change.
     *  (Obsolete? One could use LCCDConditionsMap instead)
     */
    class PedestalHandler : public lccd::IConditionsChangeListener {
//#else
//    class PedestalHandler {
//#endif
	
    public:

//#ifdef USE_LCCD
	/** constructor
	 * @param ColName name of the collection in which the pedestals are stored
	 */
	PedestalHandler( std::string ColName = tpcconddata::Pedestal::getDefaultColName() );
//#else
//	/** constructor (without use of LCCD)
//	 * @param ColName name of the collection in which the pedestals are stored
//	 * @param FileName name of the file in wwhich the pedestals are stored
//	 */
//	PedestalHandler( std::string ColName, std::string FileName );
//#endif	
	/// destructor
	virtual ~PedestalHandler( );

	static std::string getDefaultColName(){
 	    return  tpcconddata::Pedestal::getDefaultColName();
 	};
	
	/** called by the Marlin ConditionsProcessor, if the condition change
	 *  inplemetentation of the IConditionsChangeListener interface
	 */
	void conditionsChanged( EVENT::LCCollection* col );
	
	/** return the stored pedestal object for the given channel
	 *  @param Channel channel ID (cell ID) of the channel
	 *  @return Pedestal object (see: TPCCondData)
	 */
	tpcconddata::Pedestal* getPedestal( int Channel );
	
	/** return the pedestal value for the given channel
	 *  @param Channel channel ID (cell ID) of the channel
	 *  @return pedestal mean value (0 if no val;ue is stored for the ID)
	 */
	float getPedestalValue( int Channel );
	
	/** return the pedestal value for the given channel
	 *  @param Channel channel ID (cell ID) of the channel
	 *  @return pedestal width (-1 if no val;ue is stored for the ID)
	 */
	float getPedestalWidth( int Channel );
	
	
    private:
	
	PedestalMap _PedestalMap;

    };
    
}
#endif
    
