#ifndef TIMESHIFTCORRECTORPROCESSOR_H
#define TIMESHIFTCORRECTORPROCESSOR_H

#include <marlin/Processor.h>
#include <ChannelCorrectionHandler.h>

namespace marlintpc {

/**
 * Processor to apply time offset correction to individual channels.
 * The offset for each channel is read from LCCD.
 *
 * @author Peter Wienemann, University of Bonn
 */
class TimeShiftCorrectorProcessor : public marlin::Processor {
 public:

  /**
   * Constructor
   */
  TimeShiftCorrectorProcessor();

  /**
   * Overload newProcessor() method of marlin::Processor base class
   * to get a new instance of this processor
   */
  virtual marlin::Processor* newProcessor() { return new TimeShiftCorrectorProcessor; }

  /**
   * Write information into the run header
   *
   * @param run pointer to LCRunHeader to be processed
   */
  virtual void processRunHeader( EVENT::LCRunHeader* run );

  /**
   * Called for every event - the working horse.
   *
   * @param run pointer to LCEvent to be processed
   */
  virtual void processEvent( EVENT::LCEvent* evt );

  /**
   * Optional checks
   *
   * @param run pointer to LCEvent to be checked
   */
  virtual void check( lcio::LCEvent* evt ) { ; }

  /**
   * Called after data processing for clean up.
   */
  virtual void end() { ; }

 private:

  ChannelCorrectionHandler  _channelCorrectionHandler;

  std::string  _inputColName;         ///< the name of the input collection
  std::string  _channelCorrColName;   ///< the name of the collection with the channel correction data
  std::string  _outputColName;        ///< the name of the output collection

  /**
   * Status which is set to the transient flag of the output collection.
   * It is an int instead of a boolean, because the processor parameter cannot be a boolean
   */
  int          _outputIsTransient;

};

} // namespace marlintpc

#endif // TIMESHIFTCORRECTOR_H
