#include <ChannelCorrectionHandler.h>

namespace marlintpc {


ChannelCorrectionHandler::ChannelCorrectionHandler( EVENT::LCCollection* corrCol )
  : _sortedChanCorrColl("ChannelCorrection") {

  if ( corrCol ) {
    setChannelCorrectionCollection( corrCol );
  }

}


void ChannelCorrectionHandler::setChannelCorrectionCollection( EVENT::LCCollection* corrCol ) {

  // get a local copy of the (still unsorted) channel correction collection
  _sortedChanCorrColl = *( dynamic_cast<IMPL::LCCollectionVec*>( corrCol ) );

  // sort local copy
  std::sort( _sortedChanCorrColl.begin(), _sortedChanCorrColl.end(), sortChannelID() );
  
}


IMPL::LCCollectionVec::iterator ChannelCorrectionHandler::getChannelCorrection( int channelID ) {

  // look for channelID in _sortedChanCorrColl
  IMPL::LCCollectionVec::iterator it = lower_bound( _sortedChanCorrColl.begin(),
						    _sortedChanCorrColl.end(),
						    channelID,
						    cmpChannelID() );

  // check whether the wanted data could be retrieved
  if ( it == _sortedChanCorrColl.end() ||
       tpcconddata::ChannelCorrection( *it ).getChannel() != channelID ) {
    throw( EVENT::DataNotAvailableException( std::string("ChannelCorrectionHandler: Requested channelID not available in channel correction collection") ) );
  }
  else {
    return it;
  }

}


int ChannelCorrectionHandler::getChannelQuality( int channelID ) {
  return tpcconddata::ChannelCorrection( *( getChannelCorrection( channelID ) ) ).getChannelQuality();
}


int ChannelCorrectionHandler::getNChannelCalibrationFactors( int channelID ) {
  return tpcconddata::ChannelCorrection( *( getChannelCorrection( channelID ) ) ).getNChannelCalibrationFactors();
}


float ChannelCorrectionHandler::getChannelCalibrationFactor( int channelID, int index ) {
  return tpcconddata::ChannelCorrection( *( getChannelCorrection( channelID ) ) ).getChannelCalibrationFactor( index );
}


std::vector<float> ChannelCorrectionHandler::getChannelCalibrationFactors( int channelID ) {
  return tpcconddata::ChannelCorrection( *( getChannelCorrection( channelID ) ) ).getChannelCalibrationFactors();
}


float ChannelCorrectionHandler::getChannelTimeOffset( int channelID ) {
  return tpcconddata::ChannelCorrection( *( getChannelCorrection( channelID ) ) ).getChannelTimeOffset();
}


bool ChannelCorrectionHandler::isBroken( int channelID ) {
  return tpcconddata::ChannelCorrection( *( getChannelCorrection( channelID ) ) ).isBroken();
}


bool ChannelCorrectionHandler::isNoisy( int channelID ) {
  return tpcconddata::ChannelCorrection( *( getChannelCorrection( channelID ) ) ).isNoisy();
}


} // namespace marlintpc
