#include "MappingHandler.h"
#include <iostream>

#include <IMPL/LCCollectionVec.h>
using namespace lcio;
//using namespace marlin;
using namespace std;
using namespace tpcconddata;


namespace marlintpc{

MappingHandler::MappingHandler()
  : chanMapSortedChanID("ADCChannelMapping") ,
    chanMapSortedPadID("ADCChannelMapping")
{
  // constructor is empty, but needed for the
  // constructor initializer list
  // i. e. chanMapSortedChanID and chanMapSortedPadID
  // have to be initialised with a string
}

void MappingHandler:: setChannelMap(LCCollection *chanMapCol)
{  // copy the channel map to a local copy which 
  // is then sortet according to channelID
  // cout << "This is  MappingHandler:: setChannelMap" << endl; // debug
  chanMapSortedChanID=*(dynamic_cast<LCCollectionVec *>(chanMapCol));
  // cout << "length of vector:" <<chanMapSortedChanID.size() << endl; // debug

  sort( chanMapSortedChanID.begin(),
	chanMapSortedChanID.end(),
	sortChannelID() );

  // copy the channel map to a local copy which 
  // is then sortet according to padID
  chanMapSortedPadID=*(dynamic_cast<LCCollectionVec *>(chanMapCol));
  sort( chanMapSortedPadID.begin(),
	chanMapSortedPadID.end(),
	sortPadID() );

  /*
  // debug: dump the sorted channel maps
  cout << "sortet by chanID:" << endl;
  for (LCCollectionVec::iterator it=chanMapSortedChanID.begin();
       it < chanMapSortedChanID.end(); it++)
  {
    cout << ADCChannelMapping(*it).getChannelID() << "\t"<<  ADCChannelMapping(*it).getPadID() << endl;
  }

  cout << "sortet by padID:" << endl;
  for (LCCollectionVec::iterator it=chanMapSortedPadID.begin();
       it < chanMapSortedPadID.end(); it++)
  {
    cout << ADCChannelMapping(*it).getChannelID() << "\t"<<  ADCChannelMapping(*it).getPadID() << endl;
  }
  */

  //  cout << "This is  MappingHandler:: setChannelMap" << endl; //debug
}

int MappingHandler::getChannelID(int padID)
{
  // The lower_bound performs a binary search
  // and returns an iterator to the lowest 
  // possible position where this value could be
  // inserted. If there already is an element with 
  // the specific value, it can be accessed by this
  // iterator.
  // As the padID is given, we use the collection
  // sorted in this order.
  LCCollectionVec::iterator it
    =lower_bound(chanMapSortedPadID.begin(),
		 chanMapSortedPadID.end(),
		 padID,
		 cmpPadID() );
  
  // Check whether the iterator realy accesses the correct
  // data. If the requested padID is not in the map
  // this is not the case. Throw an exception in this case.

  if ( (it ==  chanMapSortedPadID.end()) ||
       (ADCChannelMapping(*it).getPadID()
	!= padID) )
    {
      throw(DataNotAvailableException(
	    string("MappingHandler: Requested padID not available in ADCChannelMap!")));
    }
  else
    return ADCChannelMapping(*it).getChannelID();
}

int MappingHandler::getPadID(int channelID)
{
  // As the channelID is given, we use the collection
  // sorted in this order.
  LCCollectionVec::iterator it
    =lower_bound(chanMapSortedChanID.begin(),
		 chanMapSortedChanID.end(),
		 channelID,
		 cmpChannelID() );
  
  // Check whether the iterator realy accesses the correct
  // data. If the requested padID is not in the map
  // this is not the case. Throw an exception in this case.

  if ( (it ==  chanMapSortedChanID.end()) ||
       (ADCChannelMapping(*it).getChannelID() 
	!= channelID) )
    {
      throw(DataNotAvailableException(
	    string("MappingHandler: Requested channelID not available in ADCChannelMap!")));
    }
  else
    return ADCChannelMapping(*it).getPadID();
}

}//namespace marlintpc
