#include "TimePixHitSorterProcessor.h"
//#include "TimePixPixelMode.h"

// C++
#include <iostream>
//#include <queue>
#include <vector>
#include <string>
#include <sstream>
#include "math.h"
//#include "time.h"

// LCIO
#include <EVENT/LCCollection.h>
#include <EVENT/TrackerHit.h>
#include <EVENT/Track.h>
#include <IMPL/LCCollectionVec.h>
#include <IMPL/TrackerHitImpl.h>
#include <IMPL/TrackImpl.h>
#include "UTIL/LCTOOLS.h"

// Aida
#ifdef MARLIN_USE_AIDA
#include <AIDA/AIDA.h>
#include <marlin/AIDAProcessor.h>
#include <AIDA/IPlotter.h>
#include <AIDA/IPlotterFactory.h>
#include <RAIDA/RAIDAUtil.h>
#include <RAIDA/IBrushStyleROOT.h>
#include <RAIDA/IPlotterFactoryROOT.h>
#include <AIDA/IMarkerStyle.h>


//marlin
#include <marlin/ConditionsProcessor.h>


//Root
#include "TROOT.h"
#include "TCanvas.h"
#include "TFile.h"
#include "TH2F.h"
#include "THStack.h"
#include "TLine.h"

#endif

using std::string;
//using namespace tpcconddata;
using namespace lcio ;
using namespace marlin ;

namespace marlintpc{

    TimePixHitSorterProcessor aTimePixHitSorterProcessor ;
    

    TimePixHitSorterProcessor::TimePixHitSorterProcessor() : 
    Processor("TimePixHitSorterProcessor") {
	/* the constructor calls first the super constructor which is the name 
	 * of the processor
	 * this must by the same as the class name.
	 */
	
	/* description of the processor which will be displayed in the
	 * steering file autometicly generated by 
	 * marlin (use MarlinTPC -x (XML-format) or MarlinTPXC -l (text format))
	 */
      _description = "TimePixHitSorterProcessor sorts the TimePixHits in y,x,z (in that order) to be used in TrackFinderRectangularProcessor ";
          	
	
	/* register steering parameters: name, description, class-variable, default value
	 * the type will by definded by the type of the default value
	 * string, double, float and int are possible
	 * plaese note to get a float you must cast in C++: (float) 1.3 
	 */
	registerInputCollection( LCIO::TRACKERHIT,
				 "InputCollectionName" , 
				 "Name of the input collection"  ,
				 _inputColName ,
				 std::string("TimePixHits") ) ;
	
	registerOutputCollection( LCIO::TRACKERHIT,
				  "OutputCollectionName" , 
				  "Name of the output collection"  ,
				  _outputColName ,
				  std::string("TimePixSortedHits") ) ;


	registerProcessorParameter( "SetOutputTransient" , 
				    "if not 0 the output collection is set transient"  ,
				    _outputIsTransient,
				    0);

	registerProcessorParameter( "pitch" , 
				    "pitch of the chip in mm"  ,
				    _pitch,
				    0.055);

	registerProcessorParameter( "nRow" , 
				    "number of rows on the chip"  ,
				    _nRow,
				    256);

	registerProcessorParameter( "nColumn" , 
				    "number of columns on the chip"  ,
				    _nColumn,
				    256);

	registerProcessorParameter( "maxTime" , 
				    "maximum value of time"  ,
				    _maxTime,
				    11810);


    }
    
    
    void TimePixHitSorterProcessor::init() { 
	
	// usually a good idea to
	printParameters() ;



    }
    
    void TimePixHitSorterProcessor::processRunHeader( LCRunHeader* run) { 
	

      run->parameters().setValue( _processorName + "_revision", "$Rev: 547 $");

      for ( ProcParamMap::iterator i = _map.begin(); i != _map.end(); i++ ) {
	if ( ! i->second->isOptional() || i->second->valueSet() ) {
	  run->parameters().setValue( _processorName + "_" + i->second->name(), i->second->value() );
	}
      }

					       

    } 
    
    void TimePixHitSorterProcessor::processEvent( LCEvent * evt ) { 

      streamlog_out(MESSAGE0)<<"TimePixHitSorterProcessor"<<std::endl;

      streamlog_out(DEBUG3)<<"processEvent( "<<evt->getEventNumber()<<")"<<std::endl;

      // get the input collection from the event
      LCCollection* inputCol=0;
      try{      
	inputCol = evt->getCollection( _inputColName ) ;
      }

      catch(DataNotAvailableException){
	// if there is not input collection give a warning to the user
	streamlog_out(ERROR4)
	  <<"Warning of TimePixHitSorterProcessor::processEvent: input collection "
	  <<_inputColName<<" cannot be found in event "<<evt->getEventNumber()<<std::endl;	
      }

	
      // create the  output collections with a string defining the TYPE (see lcio.h)
      LCCollectionVec* outputCol = new LCCollectionVec( LCIO::TRACKERHIT ); 
	
      // set the transient flag as definded by the processor parameter
      outputCol->setTransient( (_outputIsTransient != 0) );

      double oldxmax = _pitch * _nRow;
      double oldymax = _pitch * _nColumn;
      double oldzmax = _maxTime;

      for ( int i = 0; i < inputCol->getNumberOfElements(); i++ ) {

	TrackerHitImpl* NewRawHit = new TrackerHitImpl();


      //STEP1
      //-----
      //switch x and y
      for(int iHit=0; iHit < inputCol->getNumberOfElements() ; iHit++){
	
	TrackerHit* RawHit = 
	  dynamic_cast<TrackerHit*>( inputCol->getElementAt( iHit ) ) ;
	
	
	if( RawHit != 0 ) {

	  double xmax = 0;
	  double ymax = 0;
	  double zmax = 0;

	  const double *pos = RawHit->getPosition(); //hit position x, y, z in mm

	  if ( pos[0] > ymax && pos[0] < oldymax) {
	    //RawHit might be the largest element

	    ymax = pos[0];
	    xmax = pos[1];
	    zmax = pos[2];

	    double newPos[3] = {pos[1], pos[0], pos[2]};
	    NewRawHit->setPosition ( newPos );

	  }

	  if ( pos[0] == ymax ) {
	    
	    if ( pos[1] > xmax && pos[0] < oldxmax) {
	      //RawHit might be the largest element

	      ymax = pos[0];
	      xmax = pos[1];
	      zmax = pos[2];

	      double newPos[3] = {pos[1], pos[0], pos[2]};
	      NewRawHit->setPosition ( newPos );

	    }

	    if ( pos[1] == xmax ) {
	      
	      if ( pos[2] > zmax && pos[2] < oldzmax) {
		//RawHit might be the largest element
	
		ymax = pos[0];
		xmax = pos[1];
		zmax = pos[2];

		double newPos[3] = {pos[1], pos[0], pos[2]};
		NewRawHit->setPosition ( newPos );
	  
	      }

	    }

	  }
	  
	 

	} //end if( RawHit != 0 )


      }//end for(int iHit=0; iHit < inputCol->getNumberOfElements() ; iHit++){
      
      oldxmax = (NewRawHit->getPosition())[0];
      oldymax = (NewRawHit->getPosition())[1];
      oldzmax = (NewRawHit->getPosition())[2];

      outputCol->addElement( NewRawHit );


    }

      // save the processor parameters in the first collection stored
      if (isFirstCollection){
	outputCol->parameters().setValue("InputCollectionName",_inputColName);
	// TO DO: does this make sence? 
	outputCol->parameters().setValue("SetOutputTransient",_outputIsTransient);
	isFirstCollection = false;
      }
	
      evt->addCollection( outputCol, _outputColName );

	
      streamlog_out(DEBUG1)<<"end process event "<<evt->getEventNumber()<<std::endl;

      
    }
    
    
    void TimePixHitSorterProcessor::check( LCEvent * evt ) { 
     streamlog_out(DEBUG1)<<"TimePixHitSorterProcessor::check( evt "
			  <<evt->getEventNumber()<<")"<<std::endl;

     LCTOOLS::dumpEvent( evt );

     streamlog_out(DEBUG1)<<"end check( evt "<<evt->getEventNumber()<<")"<<std::endl;
    }
    
    
    void TimePixHitSorterProcessor::end(){ 

      streamlog_out(DEBUG1)<<"TimePixHitSorterProcessor::end()"<<std::endl;
      /*      	
      streamlog_out(MESSAGE4) << "TimePixHitSorterProcessor::end()  " << name() 
		<< " processed " << evt->getEventNumber() << " events in " << _nRun << " runs "
		<< std::endl ;
      */
    }





} // end of namespace marlintpc{
