#include "TrackFitterFactory.h"
#include "TrackFitterBase.h"
#include "TrackFitterChiSquare.h"
#include "TrackFitterChiSquarePads.h"
#include "TrackFitterBase.h"

#include <gear/GEAR.h>
#include <marlin/Global.h>
#include <gear/PadRowLayout2D.h>
#include <gear/TPCParameters.h>

#include <EVENT/LCRunHeader.h>
#include <EVENT/LCEvent.h>

namespace marlintpc
{

TrackFitterFactory::TrackFitterFactory(int const &type)
    : _theFitter(0)
{
    setType( type );  
}

TrackFitterFactory::TrackFitterFactory(EVENT::LCRunHeader  const *)
    : _theFitter(0)
{
    throw gear::NotImplementedException(std::string("TrackFitterFactory: Initialisation from run header not implemented yet!"));
    
    
    // TO BE IMPLEMENTED: Get the fitter type from the run header
    /* 
    int type = rh->????;

    setType( type );
    */
}

TrackFitterFactory::~TrackFitterFactory()
{
    delete _theFitter;
}

// this function has only been introduced to throw the exception and avoid double coding
void TrackFitterFactory::setType(int const &type)
{
    switch (type)
    {
	case TrackFitterBase::FitterTypes::CHISQUARE : 
	case TrackFitterBase::FitterTypes::CHISQUAREPADS :
	    _fitterType = type;
	    break;
	default:
	    throw gear::NotImplementedException(std::string("TrackFitterFactory: Unknown track fitter type!"));
    }
}

TrackFitterBase * TrackFitterFactory::createTrackFitter(EVENT::LCEvent const * )
{
    // in case there is no instance of the fitter create it in memory
    if (_theFitter == 0)
    {
	switch (_fitterType)
	{
	    case TrackFitterBase::FitterTypes::CHISQUARE :
		_theFitter = new TrackFitterChiSquare();
		break;
	    case TrackFitterBase::FitterTypes::CHISQUAREPADS :
		_theFitter = new TrackFitterChiSquarePads( & marlin::Global::GEAR->getTPCParameters() );
		break;
	}// switch

    }// if (_theFitter == 0)

/* not implemented yet, some example code
    else  // reinitialise the fitter if it depends on per-event information
    {
	switch (_fitterType)
	{
	    case TrackFitterBase::FitterTypes::LIKELIHOOD :
		// get the conndata from the event 
		conddata = ... to be implemented
		// recreate the fitter at the same memory position, no new allocation needed
		new(_theFitter) TrackFitterLikelihood(conddata... to be implemented);
		break;

	    default:
		// no need to reinitialise one of the other fitters
	}// switch
	
    } // else (_theFitter == 0)
*/

    return _theFitter;
}

} // namespace marlintpc
