#ifndef ELECTRONCLOUD_H
#define ELECTRONCLOUD_H 1

// stl
#include <string>

// CLHEP
#include <CLHEP/Vector/ThreeVector.h>

// LCIO
#include <UTIL/LCFixedObject.h>

namespace CLHEP {

	class Hep3Vector;
}

namespace marlintpc {

/** ElectronCloud : A cloud of electrons
 *
 *    @author J. Abernathy (UVic)<br>
 */
class ElectronCloud : public UTIL::LCFixedObject<2, 0, 6> {

public:
	/**
	 * constructor
	 */
	ElectronCloud();

	/** gets the number of electrons
	 */
	int get_electron_count( void ) const;

	/** sets the number of electrons
	 */
	void set_electron_count( int );


	/** gets the mean x position
	 */
	double get_x_position( void ) const;

	/** sets the mean x position
	 */
	void set_x_position( double );


	/** gets the mean y position
	 */
	double get_y_position( void ) const;

	/** sets the mean y position
	 */
	void set_y_position( double );


	/** gets the mean z position
	 */
	double get_z_position( void ) const;

	/** sets the mean z position
	 */
	void set_z_position( double );


	/** gets the mean time of the cloud
	 */
	double get_time( void ) const;

	/** sets the mean time of the cloud
	 */
	void set_time( double );


	/** gets the transverse diffusion of the electrons in the cloud
	 */
	double get_transverse_stdev( void ) const;

	/** sets he transverse diffusion of the electrons in the cloud
	 */
	void set_transverse_stdev( double );


	/** gets the longitudinal diffusion of the electrons in the cloud
	 */
	double get_longitudinal_stdev( void ) const;

	/** sets he longitudinal diffusion of the electrons in the cloud
	 */
	void set_longitudinal_stdev( double );


	/** gets the mean position of the cloud
	 */
	CLHEP::Hep3Vector get_mean_position( void ) const;
	
	/** sets the mean position of the cloud
	 */
	void set_mean_position( CLHEP::Hep3Vector );


	/** sets the unique id of this cloud
	 */
	int get_cloud_id( void ) const;

	/** sets the unique id of this cloud
	 */
	void set_cloud_id( int );


	/** returns the typename of the class
	 */
	const std::string getTypeName( void ) const;


	/** returns the type of data in the generic object
	 */
	const std::string getDataDescription( void ) const;

protected:

private:


};

} // namespace marlintpc

#endif // ELECTRONCLOUD_H
