#ifndef ELECTRONCLOUDCHARGEDEPOSITPROCESSOR_H
#define ELECTRONCLOUDCHARGEDEPOSITPROCESSOR_H 1

// Marlin
#include <marlin/Processor.h>

namespace EVENT {

	class LCRunHeader;
	class LCEvent;
}

namespace marlintpc {

/** ElectronCloudChargeDepositProcessor : This processor takes clouds of electrons and parameterizes the charge they place on the readout electronics
 *
 *    @author J. Abernathy (UVic)<br>
 */
class ElectronCloudChargeDepositProcessor : public marlin::Processor {

public:
	/** constructor
	 */
	ElectronCloudChargeDepositProcessor();

	/** destructor
	 */
	~ElectronCloudChargeDepositProcessor();

	/**
	 * used by Marlin to create a new processor
	 */
	virtual Processor* newProcessor() { return new ElectronCloudChargeDepositProcessor() ; }

	/**
	 * Called at the begin of the job before anything is read.
	 * Use to initialize the processor, e.g. book histograms.
	 */
	virtual void init();

	/**
	 * Called for every run
	 */
	virtual void processRunHeader( EVENT::LCRunHeader* run );

	/**
	 * Called for every event - the working horse.
	 */
	virtual void processEvent( EVENT::LCEvent * evt );

	/**
	 * Called after the event has been processed
	 */
	virtual void check( EVENT::LCEvent * evt );

	/**
	 * Called after data processing for clean up.
	 */
	virtual void end();

		/** used to uniquely identify a charge deposited by a track (important for overlaying tracks)
	 */
	struct ChargeSignalTag {

		bool operator < ( const ChargeSignalTag& tag ) const {

			if(tag.cloud_id != cloud_id) {

				return tag.cloud_id < cloud_id;
			}
			else if( tag.pad_id != pad_id ) {

				return tag.pad_id < pad_id;
			}
			else {

				return tag.module_id < module_id;
			}
		}
		int cloud_id;
		int pad_id;
		int module_id;
	};

protected:

private:

	/**
	 * the name of the input cloud collection
	 */
	std::string _input_collection_name;

	/**
	* the name of the output charge signal collection from this processor
	 */
	std::string _output_collection_name;

	/** the name of the tpc conditions collection containing the drift velocity
	 */
	std::string _tpc_conditions_collection_name;
};

} // namespace marlintpc

#endif // ELECTRONCLOUDCHARGEDEPOSITPROCESSOR_H
