#ifndef ELECTRONCLOUDHITREADOUTPADSPROCESSOR_H
#define ELECTRONCLOUDHITREADOUTPADSPROCESSOR_H 1

// CLHEP
#include <CLHEP/Vector/ThreeVector.h>

// Marlin
#include <marlin/Processor.h>

namespace EVENT {

	class LCRunHeader;
	class LCEvent;
}

namespace marlintpc {

/** ElectronCloudGEMAmplificationProcessor : This processor takes a cloud of electrons and passes it through a GEM simulating an electron avalanche
 *
 *    @author J. Abernathy (UVic)<br>
 */
class ElectronCloudGEMAmplificationProcessor : public marlin::Processor {

public:
	/**
	 * constructor
	 */
	ElectronCloudGEMAmplificationProcessor();

	/**
	 * used by Marlin to create a new processor
	 */
	virtual Processor* newProcessor() { return new ElectronCloudGEMAmplificationProcessor() ; }

	/**
	 * Called at the begin of the job before anything is read.
	 * Use to initialize the processor, e.g. book histograms.
	 */
	virtual void init();

	/**
	 * Called for every run
	 */
	virtual void processRunHeader( EVENT::LCRunHeader* run );

	/**
	 * Called for every event - the working horse.
	 */
	virtual void processEvent( EVENT::LCEvent * evt );

	/**
	 * Called after the event has been processed
	 */
	virtual void check( EVENT::LCEvent * evt );

	/**
	 * Called after data processing for clean up.
	 */
	virtual void end();

protected:

private:

	/**
	 * the name of the SimTrackerHit input collection from the simulation
	 */
	std::string _input_trackerhit_collection_name;

	/**
	* the name of the output TrackerHit collection from this processor
	 */
	std::string _output_trackerhit_collection_name;

	/**
	* the name of the tpcconddata collection with tpc conditions
	 */
	std::string _tpcconditions_collection_name;

	/**
	 * the parameterized collection efficiency of the GEM [0..1]
	 */
	double _collection_efficiency;

	/**
	 * the parameterized extraction efficiency of the GEM [0..1]
	 */
	double _extraction_efficiency;

	/**
	 * the parameterized amplification of the GEM
	 */
	double _amplification;

	/**
	 * the parameterized longitudinal defocussing of this GEM (mm)
	 */
	double _longitudinal_defocussing;

	/**
	 * the parameterized transverse defocussing of this GEM (mm)
	 */
	double _transverse_defocussing;

	/**
	 * the width of the GEM (mm)
	 */
	double _width;

};

} // namespace marlintpc

#endif // ELECTRONCLOUDHITREADOUTPADSPROCESSOR_H
