
// MarlinTPC
#include "ChargeSignal.h"

using namespace std;

namespace marlintpc {

/**
* constructor
*/
ChargeSignal::ChargeSignal(int charge_id, int pad_id, int module_id) {

	set_electron_count(0);
	set_charge_id(charge_id);
	set_pad_id(pad_id);
	set_module_id(module_id);
	set_time_variance(0.);
	set_mean_time(0.);
}


/** adds an electron to the signal
 */
void ChargeSignal::add_electron( const double time ) {

	const double n = get_electron_count() + 1;

	set_electron_count( (int)n );
	
	const double delta = time - get_mean_time();

	set_mean_time( get_mean_time() + (delta/n) );

	const double v = get_time_variance();

	if( n >= 2 ) {

		set_time_variance( ( ((n-2) * v) + ((n-1)/n * delta*delta) ) / ( n - 1 ) );
	}
}


/** gets the number of electrons
*/
int ChargeSignal::get_electron_count( void ) const {

	return _obj->getIntVal(0);
}

/** sets the number of electrons
*/
void ChargeSignal::set_electron_count( const int count ) {

	_obj->setIntVal(0, count);
}


/** gets the unique id of this charge signal
*/
int ChargeSignal::get_charge_id( void ) const {

	return _obj->getIntVal(1);
}

/** sets the unique id of this charge signal
*/
void ChargeSignal::set_charge_id( const int id ) {

	_obj->setIntVal(1, id);
}


/** gets the pad id of this signal
*/
int ChargeSignal::get_pad_id( void ) const {

	return _obj->getIntVal(2);
}

/** sets the pad id of this signal
*/
void ChargeSignal::set_pad_id( const int id ) {

	_obj->setIntVal(2, id);
}


/** gets the module id of this charge signal
*/
int ChargeSignal::get_module_id( void ) const {

	return _obj->getIntVal(3);
}

/** sets the module id of this charge signal
*/
void ChargeSignal::set_module_id( const int id ) {

	_obj->setIntVal(3, id);
}


/** gets the variance in the time
*/
double ChargeSignal::get_time_variance( void ) const {

	return _obj->getDoubleVal(0);
}

/** sets the variance in the time
*/
void ChargeSignal::set_time_variance( const double time_variance ) {

	_obj->setDoubleVal(0, time_variance);
}


/** gets the mean time of the cloud
*/
double ChargeSignal::get_mean_time( void ) const {

	return _obj->getDoubleVal(1);
}

/** gets the variance in the time
*/
void ChargeSignal::set_mean_time( const double mean_time ) {

	_obj->setDoubleVal(1, mean_time);
}



/** returns the typename of the class
*/
const string ChargeSignal::getTypeName( void ) const {

	return string("ChargeSignal");
}


/** returns the type of data in the generic object
*/
const std::string ChargeSignal::getDataDescription( void ) const {

	return string("i:electron_count;i:charge_id;i:pad_id;i:module_id;d:mean_time;d:time_variance");
}

} // namespace marlintpc
