#ifndef SOLENOIDZMAPMAGNETICFIELD_H
#define SOLENOIDZMAPMAGNETICFIELD_H 1

// stl
#include <string>
#include <vector>

// CLHEP
#include <CLHEP/Matrix/Vector.h>

// MarlinTPC
#include "Field.h"

namespace EVENT {

	class LCCollection;
}

namespace marlintpc {

/** SolenoidZMapMagneticField : A magnetic field from a solenoid map (mapped in z)
 *
 *    @author J. Abernathy (UVic)<br>
 */
class SolenoidZMapMagneticField: public Field {

public:
	/** constructor
	 */
	SolenoidZMapMagneticField();

	/** destructor
	 */
	virtual ~SolenoidZMapMagneticField();

	/** returns a new field... similar to how the Marlin processor registration works
	 */
	virtual Field *new_field( void );

	/** constructs the field from an LCCollection of data objects (could be anything... field bin values or just parameters)
	 */
	virtual bool construct( EVENT::LCCollection * );

	/** gets the field value at a specified point
	 */
	CLHEP::HepVector get_field_value( const CLHEP::HepVector & );

	/** returns the typename of the field
	 */
	std::string get_typename();

	struct FieldMapEntry {

		double _z;
		double _Bz;
	};

protected:

private:

	/** the boundaries of the solenoid map
	 */
	double _r_min;
	double _r_max;

	/** the distance between z bins
	 */
	double _dz;

	/** the entries of the field map (the field vector at different positions)
	 */
	std::vector<FieldMapEntry> _field_map;
};

} // namespace marlintpc

#endif // SOLENOIDZMAPMAGNETICFIELD_H
