
// stl
#include <string>
#include <cassert>

// LCIO
#include <EVENT/LCCollection.h>
#include <EVENT/LCGenericObject.h>

// marlintpc
#include "ConstantUnidirectionalMagneticField.h"

using namespace CLHEP;
using namespace EVENT;
using namespace std;

namespace marlintpc {

ConstantUnidirectionalMagneticField aConstantUnidirectionalMagneticField;

/** constructor
 */
ConstantUnidirectionalMagneticField::ConstantUnidirectionalMagneticField():
	Field("ConstantUnidirectionalMagneticField") {
}


/** destructor
 */
ConstantUnidirectionalMagneticField::~ConstantUnidirectionalMagneticField() {}


/** returns a new field... similar to how the Marlin processor registration works
*/
Field *ConstantUnidirectionalMagneticField::new_field( void ) { return new ConstantUnidirectionalMagneticField; }


/** constructs the field from an LCCollection of data objects (could be anything... field bin values or just parameters)
*/
bool ConstantUnidirectionalMagneticField::construct( LCCollection *collection ) {

	for( int map_counter = 0; map_counter < collection->getNumberOfElements(); map_counter++ ) {

		LCGenericObject *field_map_object = dynamic_cast<LCGenericObject*>( collection->getElementAt(map_counter) );
		assert(field_map_object);

		FieldMapEntry map_entry;

		map_entry._z_min = field_map_object->getDoubleVal(0);
		map_entry._z_max = field_map_object->getDoubleVal(1);
		map_entry._r_min = field_map_object->getDoubleVal(2);
		map_entry._r_max = field_map_object->getDoubleVal(3);

		map_entry._field_x = field_map_object->getDoubleVal(4);
		map_entry._field_y = field_map_object->getDoubleVal(5);
		map_entry._field_z = field_map_object->getDoubleVal(6);

		_field_map.push_back(map_entry);
	}

	return true;
}


/** gets the field value at a specified point
*/
CLHEP::HepVector ConstantUnidirectionalMagneticField::get_field_value( const CLHEP::HepVector &position ) {
	
	for( vector<FieldMapEntry>::const_iterator field_entry = _field_map.begin(); field_entry != _field_map.end(); field_entry++ ) {

		const double r = sqrt( (position[0]*position[0]) + (position[1]*position[1]) );

		if( ( r < (*field_entry)._r_max )
			&& ( r > (*field_entry)._r_min )
			&& ( position[2] < (*field_entry)._z_max )
			&& ( position[2] > (*field_entry)._z_min ) ) {

			HepVector field_vector(3);

			field_vector[0] = (*field_entry)._field_x;
			field_vector[1] = (*field_entry)._field_y;
			field_vector[2] = (*field_entry)._field_z;

			return field_vector;
		}
	}

	return HepVector(3);
}

} // namespace marlintpc
