
// stl
#include <string>
#include <cassert>

// LCIO
#include <EVENT/LCCollection.h>
#include <EVENT/LCGenericObject.h>

// marlintpc
#include "DIDZMapMagneticField.h"

using namespace CLHEP;
using namespace EVENT;
using namespace std;

namespace marlintpc {

DIDZMapMagneticField aDIDZMapMagneticField;

/** constructor
 */
DIDZMapMagneticField::DIDZMapMagneticField():
	Field("DIDZMapMagneticField") {
}


/** destructor
 */
DIDZMapMagneticField::~DIDZMapMagneticField() {}


/** returns a new field... similar to how the Marlin processor registration works
*/
Field *DIDZMapMagneticField::new_field( void ) { return new DIDZMapMagneticField; }


/** constructs the field from an LCCollection of data objects (could be anything... field bin values or just parameters)
*/
bool DIDZMapMagneticField::construct( LCCollection *collection ) {

	_field_map.clear();

	// grab the bounds from the collection
	_r_min = collection->getParameters().getFloatVal("r_min");
	_r_max = collection->getParameters().getFloatVal("r_max");
	_dz = collection->getParameters().getFloatVal("dz");

	for( int map_counter = 0; map_counter < collection->getNumberOfElements(); map_counter++ ) {

		LCGenericObject *field_map_object = dynamic_cast<LCGenericObject*>( collection->getElementAt(map_counter) );
		assert(field_map_object);

		FieldMapEntry map_entry;

		map_entry._z = field_map_object->getDoubleVal(0);
		map_entry._Bx = field_map_object->getDoubleVal(1);

		_field_map.push_back(map_entry);
	}

	return true;
}


/** gets the field value at a specified point
*/
CLHEP::HepVector DIDZMapMagneticField::get_field_value( const CLHEP::HepVector &position ) {

	HepVector field_vector(3);

	const int signZ = (position[2] >= 0)? 1: -1; // neglect the case z = 0 here...

	const double 	raw_index 	= fabs(position[2] / _dz);	// the exact position in units of bins (fractional)
	const uint		floor_index = int(raw_index); 			// the lower neighbouring bin (integer)
	const uint		ceil_index 	= int(raw_index + 1.0);		// the higher neighbouring bin (integer)

	if ( ceil_index >= _field_map.size() ) {

		return field_vector;
	}

	const double floor_weight = ceil_index - raw_index;	// how close are we to the lower bin? (one minus distance)
	const double ceil_weight = raw_index - floor_index;	// how close are we to the higher bin? (one minus distance)
	const double Bx = floor_weight * _field_map[floor_index]._Bx + ceil_weight * _field_map[ceil_index]._Bx; // interpolation

	field_vector[2] = Bx * signZ;

	return HepVector(3);
}

} // namespace marlintpc
