
// stl
#include <string>
#include <cassert>

// LCIO
#include <EVENT/LCCollection.h>
#include <EVENT/LCGenericObject.h>

// marlintpc
#include "IdealSolenoidField.h"

using namespace CLHEP;
using namespace EVENT;
using namespace std;

namespace marlintpc {

IdealSolenoidField anIdealSolenoidField;

/** constructor
 */
IdealSolenoidField::IdealSolenoidField():
	Field("IdealSolenoidField") {
}


/** destructor
 */
IdealSolenoidField::~IdealSolenoidField() {}


/** returns a new field... similar to how the Marlin processor registration works
*/
Field *IdealSolenoidField::new_field( void ) { return new IdealSolenoidField; }


/** constructs the field from an LCCollection of data objects (could be anything... field bin values or just parameters)
*/
bool IdealSolenoidField::construct( LCCollection *collection ) {

	// grab the bounds from the collection
	_r_min = collection->getParameters().getFloatVal("r_min");
	_r_max = collection->getParameters().getFloatVal("r_max");
	_z_max = collection->getParameters().getFloatVal("z_max");

	LCGenericObject *field_value = dynamic_cast<LCGenericObject*>( collection->getElementAt(0) );
	assert(field_value);

	_field_value = HepVector(3);

	_field_value[0] = field_value->getDoubleVal(0);
	_field_value[1] = field_value->getDoubleVal(1);
	_field_value[2] = field_value->getDoubleVal(2);

	return true;
}


/** gets the field value at a specified point
*/
CLHEP::HepVector IdealSolenoidField::get_field_value( const CLHEP::HepVector &position ) {

	HepVector no_field(3);

	if( fabs(position[2]) > _z_max ) {

		return no_field;
	}

	const double radius = sqrt( (position[0]*position[0]) + (position[1]*position[1]) );

	if( (radius < _r_min) || (radius > _r_max) ) {

		return no_field;
	}

	return _field_value;
}

} // namespace marlintpc
