#include "LCObjectCopier.h"

#include "vector"

namespace lcio
{

IMPL::TrackerPulseImpl * LCObjectCopier::copy(const EVENT::TrackerPulse * const inPulse)
{
  IMPL::TrackerPulseImpl * newPulse= new  IMPL::TrackerPulseImpl;
  
  newPulse->setCellID0( inPulse->getCellID0());

  newPulse->setCellID1( inPulse->getCellID1());

  newPulse->setTime(  inPulse->getTime());

  newPulse->setCharge(  inPulse->getCharge());
					   
  newPulse->setQuality(  inPulse->getQuality());

  // attention, does only copy the vector
  // this is no clone/ deep copy function
  newPulse->setTrackerData(  inPulse->getTrackerData() );

  return newPulse;
}

IMPL::TrackerDataImpl* LCObjectCopier::copy(const EVENT::TrackerData* const inData) {

  IMPL::TrackerDataImpl* newData = new IMPL::TrackerDataImpl;

  newData->setCellID0( inData->getCellID0() );
  newData->setCellID1( inData->getCellID1() );
  newData->setTime( inData->getTime() );
  newData->setChargeValues( inData->getChargeValues() );

  return newData;
}

IMPL::MCParticleImpl* LCObjectCopier::copy(const EVENT::MCParticle* const inParticle){

  //ATTENTION THE COPIED MCPARTICLE MISSES MOTHER/CHILD DEPENDENCIES

  double mom[3] =  {inParticle->getMomentum()[0], inParticle->getMomentum()[1], inParticle->getMomentum()[2]};
  double vert[3] = {inParticle->getVertex()[0], inParticle->getVertex()[1], inParticle->getVertex()[2]};
  double ept[3] = {inParticle->getEndpoint()[0], inParticle->getEndpoint()[1],inParticle->getEndpoint()[2]};

  double* momentum = mom;
  double* vertex = vert;
  double* endpoint = ept;

  IMPL::MCParticleImpl* newParticle = new IMPL::MCParticleImpl;

  newParticle->setCharge( inParticle->getCharge() );
  newParticle->setMass( inParticle->getMass() );
  newParticle->setMomentum( momentum );
  newParticle->setEndpoint( endpoint );
  newParticle->setTime(inParticle->getTime() );
  newParticle->setVertex( vertex );
  newParticle->setSimulatorStatus(newParticle->getSimulatorStatus() );
  newParticle->setGeneratorStatus( inParticle->getGeneratorStatus() );
  newParticle->setPDG( inParticle->getPDG() );
  newParticle->setCreatedInSimulation( inParticle->isCreatedInSimulation() );
  newParticle->setStopped( inParticle->isStopped() );
  newParticle->setHasLeftDetector( inParticle->hasLeftDetector() );
  newParticle->setBackscatter( inParticle->isBackscatter() );
  newParticle->setVertexIsNotEndpointOfParent( inParticle->vertexIsNotEndpointOfParent() );
  newParticle->setDecayedInTracker( inParticle->isDecayedInTracker() );
  newParticle->setDecayedInCalorimeter( inParticle->isDecayedInCalorimeter() );

  return newParticle;
}


IMPL::SimTrackerHitImpl* LCObjectCopier::copy(const EVENT::SimTrackerHit* const inHit) {

  IMPL::SimTrackerHitImpl* newHit = new IMPL::SimTrackerHitImpl;

  double position[3] = {inHit->getPosition()[0], inHit->getPosition()[1], inHit->getPosition()[2]};
  double* pos = position;

  float momentum[3] = {inHit->getMomentum()[0], inHit->getMomentum()[1], inHit->getMomentum()[2]};
  float* mom = momentum;

  newHit->setCellID( inHit->getCellID() );
  newHit->setPosition( pos  );
  newHit->setdEdx( inHit->getdEdx() );
  newHit->setTime( inHit->getTime() );

  //ATTEENTION THE COPIED MCPARTICLE MISSES MOTHER CHILD DEPENDENCIES
  newHit->setMCParticle( LCObjectCopier::copy(inHit->getMCParticle()) );
  newHit->setMomentum( mom );
  newHit->setPathLength( inHit->getPathLength() );

  return newHit;

}


}// namespace lcio
