#ifndef ChannelCorrection_h
#define ChannelCorrection_h 1

//C++
#include "iostream"
#include "string"

//LCIO
#include "lcio.h"
#include "EVENT/LCGenericObject.h"
#include "IMPL/LCGenericObjectImpl.h"

namespace tpcconddata{

class ChannelCorrection;

/** class to store information for channel corrections
 *  e.g. the quality of a channel
 *  or a calibration factor
 */
class ChannelCorrection : public EVENT::LCGenericObject {
  
public: 
  
    /** constructor wich only sets channel
     */
    ChannelCorrection(int channel);

    /** 'Copy constructor' needed to interpret LCCollection read from file/database.
     */
    ChannelCorrection(EVENT::LCObject* obj); 
    
    /** Important for memory handling*/
    virtual ~ChannelCorrection();
  
    // the class interface:

    /// return the Channel ID
    int   getChannel();

    /** return all status bits as an int
     *  bit 0: channel is not working
     *  bit 1: channel is noisy
     */
    int   getChannelQuality();

    /// return calibration factor (more than one can be stored num)
    float getChannelCalibrationFactor( int Num = 0 );

    ///return the number of calibration factors
    int   getNChannelCalibrationFactors();

    ///return a vector containing all calibration factors
    std::vector<float> getChannelCalibrationFactors();

    /// return time off set of the channel in ns
    float getChannelTimeOffset();

    /// return true if the channel is not working properly
    bool  isBroken();

    /// return true if the cahnnel is noisy
    bool  isNoisy();

    void setChannelQuality( int Value );
    void setChannelCalibrationFactor( int Num, float Value );
    void setNChannelCalibrationFactors( int Value );
    void setChannelCalibrationFactors( std::vector<float> ValueVec );
    void setChannelTimeOffset( float Value );
    void setBroken( bool setIt = true );
    void setNoisy( bool setIt = true );

    IMPL::LCGenericObjectImpl* obj(){ return myObj; };
    
    virtual int id(){ return myObj->id(); }

    static std::string getRevision();
    static std::string getDefaultColName();

    void print( std::ostream& os = std::cout );

    std::ostream& operator<<( std::ostream& os );

    // ---- need to implement LCGenericObject interface:
    
    int getNInt() const;   
    int getNFloat() const; 
    int getNDouble() const;
    
    int getIntVal(int index) const;
    float getFloatVal(int index) const;
    double getDoubleVal(int index) const;

    bool isFixedSize() const { return false; };

    const std::string getTypeName() const { 
	return std::string("ChannelCorrection");
    } 
    const std::string getDataDescription() const {
	return std::string("i:Channel,Quality,NCaliFactors,f:TimeOffset,CalibrationFactors"); 
    }
    

protected:

    bool createObj;
    IMPL::LCGenericObjectImpl* myObj;

    void setChannel(int channel);

    bool getQualityBit( int bit );
    void setQualityBit( int bit, bool setIt = true);

}; //class

}

#endif
