#ifndef Pedestal_h
#define Pedestal_h 1

//C++
#include "iostream"
#include "string"

//LCIO
#include "lcio.h"
#include "UTIL/LCFixedObject.h"


#define PedestalNINTVals 1  // electric Channel or ID here called channel
#define PedestalNFLOATVals 2 //Pedestal Value,PedestalWidth
#define PedestalNDOUBLEVals 0

namespace tpcconddata{

class Pedestal ;

typedef Pedestal LCPedestal;

/** Class that gathers information on pedestals for storage.
 *  The electric channel or channel ID is compound with
 *  Pedestal value and width into a single object of type LCGenericObject.
 *  Based on the LCFixedObject template.
 *  This can be used to be stored in a LCCollectionVec.
 */
class Pedestal : public UTIL::LCFixedObject<PedestalNINTVals,PedestalNFLOATVals,PedestalNDOUBLEVals> {
  
public: 
  
    /** Convenient constructor.
     */
    Pedestal(int channel, float PedestalValue, float PedestalWidth);

    /** constructor wich only sets channel
     */
    Pedestal(int channel);

    /** 'Copy constructor' needed to interpret LCCollection read from file/database.
     */
    Pedestal(EVENT::LCObject* obj) : UTIL::LCFixedObject<PedestalNINTVals,
							 PedestalNFLOATVals,
							 PedestalNDOUBLEVals>(obj) { } 
    
    /** Important for memory handling*/
    virtual ~Pedestal();
  
    // the class interface:
    int   getChannel();
    float getPedestalValue();
    float getPedestalWidth();
    
    void setPedestalValue(float PedestalValue);
    void setPedestalWidth(float PedestalWidth);

    static std::string getRevision();
    static std::string getDefaultColName();

    void print(  std::ostream& os = std::cout );

    std::ostream& operator<<( std::ostream& os );

    // -------- need to implement abstract methods from LCGenericObject
    const std::string getTypeName() const { 
	return std::string("Pedestal");
    } 
    const std::string getDataDescription() const {
	return std::string("i:Channel,f:PedestalValue,PedestalWidth"); 
    }
    

protected:

    void setChannel(int channel);
}; // class

}

#endif
