#ifndef TPCConditions_h
#define TPCConditions_h 1

#include "iostream"
#include "string"

#include "lcio.h"
#include "UTIL/LCFixedObject.h"


#define TPCConditionsNINTVals 0
#define TPCConditionsNFLOATVals 6
#define TPCConditionsNDOUBLEVals 0


namespace tpcconddata{

class TPCConditions ;

class TPCConditions : public UTIL::LCFixedObject<TPCConditionsNINTVals,
		      TPCConditionsNFLOATVals,TPCConditionsNDOUBLEVals> {
  
public: 
  
    /** Convenient constructor.
     */
    TPCConditions();
    
    /** 'Copy constructor' needed to interpret LCCollection read from file/database.
     */
    TPCConditions(EVENT::LCObject* obj) : UTIL::LCFixedObject<TPCConditionsNINTVals,
							      TPCConditionsNFLOATVals,
							      TPCConditionsNDOUBLEVals>(obj) { } 
    
    /** Important for memory handling*/
    virtual ~TPCConditions();
    
    // the class interface:
    
    /// return the drift velocity (in mm/mus)
    float getDriftVelocity();

    /// return the diffusion coefficent for the longitudinal diffusion (in sqrt(mm))
    float getLongDiffusionCoef();
 
    /// return the defocussion constant for the longitudinal diffusion (in mm)
    float getLongDefocussing();
 
    /// return the diffusion coefficent for the transversal diffusion (in sqrt(mm)) 
    float getTransDiffusionCoef();
    
    /// return the defocussion constant for the transversal diffusion (in mm)
    float getTransDefocussing();
    
    /// return the amplification of the gas amplification device (e.g. GEMs)
    float getAmplification();
    
    /// set the drift velocity in mm/mus
    void setDriftVelocity(float Value);

    /// set the diffusion coefficient for the longitudinal diffusion (in sqrt(mm))
    void setLongDiffusionCoef(float Value);

    /// set the defocussion constant for the longitudinal diffusion (in mm)
    void setLongDefocussing(float Value);
    
    /// set the diffusion coefficient for the transversal diffusion (in sqrt(mm)) 
    void setTransDiffusionCoef(float Value);
    
    /// set the defocussion constant for the transversal diffusion (in mm)
    void setTransDefocussing(float Value) ;
    
    /// set the amplification of the gas amplification device (e.g. GEMs)
    void setAmplification(float Value);
    
    static std::string getRevision();
    static std::string getDefaultColName();
    
    void print( std::ostream& os = std::cout );
    
    std::ostream& operator<<( std::ostream& os );
    
    // -------- need to implement abstract methods from LCGenericObject
    const std::string getTypeName() const {
	return "TPCConditions";
    } 
    const std::string getDataDescription() const {
	return std::string("f:Driftvelocity,longDiffusionCoef,longDefocussing,")+
	    std::string("transDiffusionCoef,transDefocussing,Amplification");
    }
    
}; // class

}
#endif
