#ifndef WeatherConditions_h
#define WeatherConditions_h 1

//C++
#include "iostream"
#include "string"

//LCIO
#include "lcio.h"
#include "UTIL/LCFixedObject.h"


#define WeatherConditionsNINTVals 0
#define WeatherConditionsNFLOATVals 3
#define WeatherConditionsNDOUBLEVals 0

namespace tpcconddata{

class WeatherConditions ;

/** class to store the environment of a gaseous detector (e.g. TPC)
 *  the weather
 */
class WeatherConditions : public UTIL::LCFixedObject<WeatherConditionsNINTVals,
			  WeatherConditionsNFLOATVals,WeatherConditionsNDOUBLEVals> {
  
public: 
  
    /** constructor wich only sets channel
     */
    WeatherConditions();

    /** 'Copy constructor' needed to interpret LCCollection read from file/database.
     */
    WeatherConditions(EVENT::LCObject* obj) :UTIL::LCFixedObject<WeatherConditionsNINTVals,
								 WeatherConditionsNFLOATVals,
								 WeatherConditionsNDOUBLEVals>(obj) { } 
    
    /** Important for memory handling*/
    virtual ~WeatherConditions();

    /// return the temperature in C
    float getTemperatur();

    /// return the athmospheric pressure in hPa
    float getPressure();

    /// return relative humidity in %
    float getRelHumidity();

    void setTemperatur( float Value );
    void setPressure( float Value );
    void setRelHumidity( float Value );

    static std::string getRevision();
    static std::string getDefaultColName();

    void print( std::ostream& os = std::cout );
    std::ostream& operator<<( std::ostream& os );

    // -------- need to implement abstract methods from LCGenericObject
    const std::string getTypeName() const {
	return "WeatherConditions";
    } 
    const std::string getDataDescription() const {
	return std::string("f:Temperature,athmosPreasure,relHumidity");
    }

}; // class

}

#endif
