/* ChannelCorrection.cc
 * $Id$ 
 */

#include "ChannelCorrection.h"

namespace tpcconddata{

std::string ChannelCorrection::getRevision(){
    return std::string("$Rev: 1026 $");
}
std::string ChannelCorrection::getDefaultColName(){
    return std::string("TPCChannelCorrection");
}

ChannelCorrection::ChannelCorrection(int channel ){
    createObj = true;
    myObj     = new IMPL::LCGenericObjectImpl();

	myObj->setIntVal(2, 0);

    setChannel( channel );
    setChannelQuality( 0 );
    setChannelTimeOffset( 0 );
    setNChannelCalibrationFactors( 0 );

}

ChannelCorrection::ChannelCorrection(EVENT::LCObject* Obj){
    createObj=false;
    myObj = dynamic_cast<IMPL::LCGenericObjectImpl*>( Obj );
    
    if (myObj ==0){
	//could be an ChannelCorrection
	ChannelCorrection* dummyObj = dynamic_cast<ChannelCorrection*>( Obj );
	if (dummyObj != 0){
	    myObj=dummyObj->obj();
	}
    }
}

ChannelCorrection::~ChannelCorrection(){
    if (createObj) delete myObj;
}
  
int ChannelCorrection::getChannel(){
    return myObj->getIntVal( 0 );
}

int ChannelCorrection::getChannelQuality(){
    return myObj->getIntVal( 1 );
}

float ChannelCorrection::getChannelCalibrationFactor( int Num ){
    if ( Num<getNChannelCalibrationFactors() ){
	return myObj->getFloatVal( 1 + Num);
    }
    else{
	std::cerr<<"ERROR:ChannelCorrection::getChannelCalibrationFactor:"
		 <<" can not get CalibrationFactor "<<Num
		 <<" only "<<getNChannelCalibrationFactors()<<" available"<<std::endl;
    }
    return 0.0;
}

int ChannelCorrection::getNChannelCalibrationFactors(){

	if( myObj->getNInt() >= 2 ) {

		return myObj->getIntVal( 2 );
	}

	return 0;
}

std::vector<float> ChannelCorrection::getChannelCalibrationFactors(){
    std::vector<float> returnValue;
    returnValue.clear();
    for (int iFactor = 0; iFactor<getNChannelCalibrationFactors(); iFactor++){
	returnValue.push_back( getChannelCalibrationFactor( iFactor ) );
    }
    return returnValue;
}

float ChannelCorrection::getChannelTimeOffset(){
    return myObj->getFloatVal( 0 );
}

bool ChannelCorrection::isBroken(){
    return getQualityBit( 0 );
}

bool ChannelCorrection::isNoisy(){
    return getQualityBit( 1 );
}

void ChannelCorrection::setChannelQuality( int Value ){
    myObj->setIntVal( 1, Value );
}

void ChannelCorrection::setChannelCalibrationFactor( int Num , float Value ){
    if ( Num<getNChannelCalibrationFactors() ){
	myObj->setFloatVal( 1+Num, Value );
    }
    else{
	std::cerr<<"ERROR:ChannelCorrection::setChannelCalibrationFactor:"
		 <<" can not set CalibrationFactor "<<Num
		 <<" only "<<getNChannelCalibrationFactors()<<" available"<<std::endl;
    }
}

void ChannelCorrection::setNChannelCalibrationFactors( int Value ){

    int oldValue = getNChannelCalibrationFactors();

    myObj->setIntVal( 2, Value ); 

    // initialise the new factors
    for (int iVal = oldValue; iVal < Value; iVal ++){
	setChannelCalibrationFactor( iVal, 0.0 );
    }
}

void ChannelCorrection::setChannelCalibrationFactors( std::vector<float> ValueVec ){
    setNChannelCalibrationFactors(ValueVec.size());
    int Pos = 1;
    for (std::vector<float>::iterator iValue = ValueVec.begin(); iValue != ValueVec.end(); iValue++){
	myObj->setFloatVal( Pos++, *iValue );
    }
}

void ChannelCorrection::setChannelTimeOffset( float Value ){
    return myObj->setFloatVal( 0, Value );
}

void ChannelCorrection::setBroken( bool setIt ){
    setQualityBit( 0, setIt ) ;
}

void ChannelCorrection::setNoisy( bool setIt ){
    setQualityBit( 1, setIt ) ;
}

void ChannelCorrection::print(  std::ostream& os ){

    os<<"ChannelCorrection: Channel "<<getChannel();
    if ( getChannelQuality() == 0 ){
	os<<" is working properly, ";
    }
    if ( isBroken() ){
	os<<" is broken, ";
    }
    if ( isNoisy() ){
	os<<" is noisy, ";
    }
    os<<" has a time offset of "<< getChannelTimeOffset()<<" ns";
    if ( getNChannelCalibrationFactors() > 0 ){
	os<<" and following calibration constant";
	os<<(getNChannelCalibrationFactors()==1?": ":"s:");
	for (int iFactor = 0; iFactor < getNChannelCalibrationFactors()-1; iFactor++){
	    os<<getChannelCalibrationFactor( iFactor )<<"; ";
	}
	os<<getChannelCalibrationFactor( getNChannelCalibrationFactors()-1 );
    }
    else{
	os<<" and no calibration is set";
    }
    os<<std::endl;

}

std::ostream& ChannelCorrection::operator<<( std::ostream& os ){
    print( os );
    return os;
}

void ChannelCorrection::setChannel(int channel){
    myObj->setIntVal( 0, channel );
}

bool ChannelCorrection::getQualityBit( int bit ){
    return ( (getChannelQuality() & ( 1 << bit ) ) > 0);
}

void ChannelCorrection::setQualityBit( int bit, bool setIt ){
    if( setIt )
	setChannelQuality( getChannelQuality() | ( 1 << bit ) );
    else
	setChannelQuality( getChannelQuality() & ~( 1 << bit ) ) ;
}

int ChannelCorrection::getNInt()    const { return 3; } 
int ChannelCorrection::getNFloat()  const { return myObj->getNFloat(); }
int ChannelCorrection::getNDouble() const { return 0; } 
 
int ChannelCorrection::getIntVal(int index) const {
    return myObj->getIntVal( index ) ;
}
float ChannelCorrection::getFloatVal(int index) const { 
    return myObj->getFloatVal( index ) ;
}
double ChannelCorrection::getDoubleVal(int index) const {
    return 0.0;//myObj->getDoubleVal( index ) ;
}

}
