/* GasConditions.cc
 * $Id$ 
 */

#include "GasConditions.h"

namespace tpcconddata{

std::string GasConditions::getRevision(){
    return std::string("$Rev: 237 $");
}

std::string GasConditions::getDefaultColName(){
    return std::string("TPCGasConditions");
}

GasConditions::GasConditions(){
    createObj=true;
    myObj = new IMPL::LCGenericObjectImpl();

    setTemperature( 0 );
    setPressure( 0 );
    setOverPressure( 0 );
    setFlow( 0 );
    setWaterContent( 0 );
    setOxigenContent( 0 );
    
}

GasConditions::GasConditions(EVENT::LCObject* Obj){
    createObj=false;
    myObj = dynamic_cast<IMPL::LCGenericObjectImpl*>( Obj );
    
    if (myObj ==0){
	//could be an GasConditions
	  GasConditions* dummyObj = dynamic_cast<GasConditions*>( Obj );
	  if (dummyObj != 0){
	      myObj=dummyObj->obj();
	  }
    }
}

GasConditions::~GasConditions(){
    if (createObj) delete myObj;
}

float GasConditions::getMixture( int Content ){
    for (int iContent = 0; iContent < getNMixtureContents(); iContent++){
	if ( getContent( iContent ) == Content ){
	    return getFraction( iContent );
	}
    }
    return 0.0;
}

float GasConditions::getTemperature(){
    return myObj->getFloatVal( 0 ) ; 
}

float GasConditions::getPressure(){
    return myObj->getFloatVal( 1 ) ; 
}

float GasConditions::getOverPressure(){
    return myObj->getFloatVal( 2 ) ; 
}

float GasConditions::getFlow(){
    return myObj->getFloatVal( 3 ) ; 
}

float GasConditions::getWaterContent(){
    return myObj->getFloatVal( 4 ) ; 
}

float GasConditions::getOxigenContent(){
    return myObj->getFloatVal( 5 ) ; 
}

void GasConditions::setMixture( int Content, float Value ){
    for (int iContent = 0; iContent < getNMixtureContents(); iContent++){
	if ( getContent( iContent ) == Content ){
	    setFraction( iContent, Value );
	    return;
	}
    }
    addMixtureContents( Content, Value  );    
}

void GasConditions::setTemperature( float Value ) {
    myObj->setFloatVal( 0 , Value ) ;
}

void GasConditions::setPressure( float Value ) {
    myObj->setFloatVal( 1 , Value ) ;
}

void GasConditions::setOverPressure( float Value ) {
    myObj->setFloatVal( 2 , Value ) ;
}

void GasConditions::setFlow( float Value ) {
    myObj->setFloatVal( 3 , Value ) ;
}

void GasConditions::setWaterContent( float Value ) {
    myObj->setFloatVal( 4 , Value ) ;
}

void GasConditions::setOxigenContent( float Value ) {
    myObj->setFloatVal( 5 , Value ) ;
}

void GasConditions::print( std::ostream& os ){

    if (getNMixtureContents() > 0 ){
	std::streamsize WidthBefore = os.width();
	os.width( 7 );
	os<<"mixtrue: ";
	for (int iMix = 0; iMix < getNMixtureContents(); iMix++){
	    os<<getContent( iMix )<<":";
	}
	os<<std::endl;
	os<<"        ";
	for (int iMix = 0; iMix < getNMixtureContents(); iMix++){
	    os<<getFraction( iMix )<<":";
	}
	os.width( WidthBefore );
    }
    else{
	os<<"no mixture defined";
    }
    os<<std::endl;
    os<<"temprature:     "<<getTemperature()<<" C"<<std::endl;
    os<<"presure:        "<<getPressure()<<" hPa"<<std::endl;
    os<<"overpresure:    "<<getOverPressure()<<" mbar"<<std::endl;
    os<<"flow:           "<<getFlow()<<" l/h"<<std::endl;
    os<<"water content:  "<<getWaterContent()<<" ppmV"<<std::endl;
    os<<"oxigen content: "<<getOxigenContent()<<" ppmV"<<std::endl;
}

std::ostream& GasConditions::operator<<( std::ostream& os ){
    print( os );
    return os;
}

int  GasConditions::getNMixtureContents(){
    return myObj->getNInt();
}
int  GasConditions::getContent( int Index ){
    if (Index <  getNMixtureContents() ){
	return myObj->getIntVal( Index );
    }
    return -1;
}
float GasConditions::getFraction( int Index ){
    if (Index <  getNMixtureContents() ){
	return myObj->getFloatVal( 6+Index );
    }
    return 0.0;
}
void GasConditions::setFraction( int Index, float Fraction ){
    if (Index <  getNMixtureContents() ){
	myObj->setFloatVal( 6+Index, Fraction );
    }
}
void GasConditions::addMixtureContents( int Content, float Fraction ){
    myObj->setIntVal( getNMixtureContents(), Content );
    myObj->setFloatVal( 6+getNMixtureContents(), Fraction );
}

int GasConditions::getNInt() const    { return myObj->getNInt();}//myObj->getNInt(); } 
int GasConditions::getNFloat() const  { return myObj->getNFloat(); }  
int GasConditions::getNDouble() const { return 0; } 

int GasConditions::getIntVal(int index) const {
    return myObj->getIntVal( index ) ;
}
float GasConditions::getFloatVal(int index) const { 
    return myObj->getFloatVal( index ) ;
}
double GasConditions::getDoubleVal(int index) const {
    return 0.0;//myObj->getDoubleVal( index ) ;
}

}
