/* TPCConditions.cc
 * $Id$ 
 */

#include "TPCConditions.h"

namespace tpcconddata{

std::string TPCConditions::getRevision(){
    return std::string("$Rev: 237 $");
}

std::string TPCConditions::getDefaultColName(){
    return std::string("TPCConditions");
}

TPCConditions::TPCConditions(){
    setDriftVelocity(0);
    setLongDiffusionCoef(0);
    setLongDefocussing(0);
    setTransDiffusionCoef(0);
    setTransDefocussing(0);
    setAmplification(1);
}

TPCConditions::~TPCConditions() { /* no op*/  }

float TPCConditions::getDriftVelocity(){
    return obj()->getFloatVal( 0 ) ; 
} 

float TPCConditions::getLongDiffusionCoef() {
    return obj()->getFloatVal( 1 ) ;
}
 
float TPCConditions::getLongDefocussing() {
    return obj()->getFloatVal( 2 ) ;
}
 
float TPCConditions::getTransDiffusionCoef() {
    return obj()->getFloatVal( 3 ) ;
}

float TPCConditions::getTransDefocussing() {
    return obj()->getFloatVal( 4 ) ;
}
    
float TPCConditions::getAmplification() { 
    return obj()->getFloatVal( 5 ) ;
} 
    
void TPCConditions::setDriftVelocity(float Value) {
    obj()->setFloatVal( 0 , Value ) ;
}

void TPCConditions::setLongDiffusionCoef(float Value) {
    obj()->setFloatVal( 1 , Value ) ;
}

void TPCConditions::setLongDefocussing(float Value) {
    obj()->setFloatVal( 2 , Value ) ;
}
    
void TPCConditions::setTransDiffusionCoef(float Value) {
    obj()->setFloatVal( 3 , Value ) ;
}

void TPCConditions::setTransDefocussing(float Value) {
    obj()->setFloatVal( 4 , Value ) ;
}

void TPCConditions::setAmplification(float Value) {
    obj()->setFloatVal( 5 , Value ) ;
}

    
void TPCConditions::print( std::ostream& os ){
    
    os<<"amplification:          "<<getAmplification()<<std::endl;
    os<<"drift velocity:         ";
    if (getDriftVelocity()>0){
	os<<getDriftVelocity()<<"mm/mus";
    }
    else{
	os<<" not set";
    }
    os<<std::endl;
    os<<"transversal diffusion:  ";
    if (getLongDiffusionCoef()>0){
	os<<"sigma^2 = "<<getLongDiffusionCoef()<<"^2* z (mm) + "<<getLongDefocussing()<<"^2 (mm^2)";
    }
    else{
	os<<"not set";
    }
    os<<std::endl;
    os<<"longitudinal diffusion: ";
    if (getTransDiffusionCoef()>0){
	os<<"sigma^2 = "<<getTransDiffusionCoef()<<"^2* z (mm) + "<<getTransDefocussing()<<"^2 (mm^2)";
    }
    else{
	os<<"not set";
    }
    os<<std::endl;
}

std::ostream& TPCConditions::operator<<( std::ostream& os ){
    
    print( os );
    return os;
    
}

}
