//
//*******************************************************
//*                                                     *
//*                      Mokka                          * 
//*   - the detailed geant4 simulation for ILC          *
//*                                                     *
//* For more information about Mokka, please, go to the *
//*                                                     *
//*  polype.in2p3.fr/geant4/tesla/www/mokka/mokka.html  *
//*                                                     *
//* Mokka home page.                                    *
//*                                                     *
//*******************************************************
//
// $Id: SField01.cc,v 1.3 2005/12/13 13:54:34 adrian Exp $
// $Name: mokka-07-00 $
//
// SField01.cc
//
// History:  
//

#include "MySQLWrapper.hh"
#include "Control.hh"
#include "SField01.hh"

#include "globals.hh"
#include "CGADefs.h"

INSTANTIATE(SField01)

G4bool 
SField01::PreLoadScriptAction(Database* dbtmp,
			     CGAGeometryEnvironment& theGeometryEnvironment)
{
  G4String query;

  // upload the setup parameters
  query =  "CREATE TABLE field_map (\n";
  query += "  zmin float default NULL,\n";
  query += "  zmax float default NULL,\n";
  query += "  rmin float default NULL,\n";
  query += "  rmax float default NULL,\n";
  query += "  mag_field_x float default NULL,\n";
  query += "  mag_field_y float default NULL,\n";
  query += "  mag_field_z float default NULL\n";
  query += ") TYPE=ISAM PACK_KEYS=1;";
  dbtmp->exec(query.data());

  // insert field regions
  G4String FieldZMax = 
    theGeometryEnvironment.GetParameterAsString("TPC_Ecal_Hcal_barrel_halfZ") + " + " + 
    theGeometryEnvironment.GetParameterAsString("Coil_extra_size");
  G4String FieldRMiddle = 
    theGeometryEnvironment.GetParameterAsString("Hcal_R_max") + " + " +
    theGeometryEnvironment.GetParameterAsString("Coil_thickness") + " / 2.";
  G4String FieldRMax = 
    theGeometryEnvironment.GetParameterAsString("Yoke_barrel_inner_radius") + " + " +
    theGeometryEnvironment.GetParameterAsString("Yoke_thickness");
  
  // inside the coil :
  query = "INSERT INTO field_map VALUES (\n";
  query += "0.00, \n"; // zmin
  query +=  FieldZMax + ", \n"; // zmax
  query += "0.00, "; // rmin
  query += FieldRMiddle + ", \n"; // rmax
  query += "0., 0.,";
  query += theGeometryEnvironment.GetParameterAsString("Field_nominal_value") + ");"; // central field mag
  dbtmp->exec(query.data());
  
  // outside the coil :
  query = "INSERT INTO field_map VALUES (\n";
  query += "0.00, \n"; // zmin
  query += FieldZMax + ", \n"; // zmax
  query += FieldRMiddle + ", \n"; // rmin
  query += FieldRMax + ",\n"; // rmax
  query += "0., 0.,";
  query += " - (4.0 / (" + FieldRMiddle + 
    ") * " + theGeometryEnvironment.GetParameterAsString("Yoke_thickness") +
    "));"; // yoke return field
  dbtmp->exec(query.data());

  return true;
}

