#ifndef MyCaloHitExtended_H
#define MyCaloHitExtended_H 1

#include "lcio.h"
#include "EVENT/LCIO.h"
#include "EVENT/CalorimeterHit.h"

typedef enum {
  CALHITTYPE_UNKNOWN =0,
  CALHITTYPE_ECAL,
  CALHITTYPE_HCAL,
  CALHITTYPE_LCAL,
  CALHITTYPE_LHCAL,
  CALHITTYPE_BCAL,
  CALHITTYPE_MUON
} CalorimeterHitType;

typedef enum {
  CALHITZONE_UNKNOWN =0,
  CALHITZONE_BARREL,
  CALHITZONE_ENDCAP,
  CALHITZONE_OVERLAP
} CalorimeterHitZone;

class MyCaloHitExtended;
typedef std::vector<MyCaloHitExtended*> MyCaloHitExtendedVec;

using namespace lcio ;

/**
 * Class extending native LCIO class CalorimeterHit. <br>
 * Class MyCaloHitExtended is used in MyProcessor <br>
 * Based on CaloHitExtended <br>
 * @author M. Thomson (DESY)<br>
 * @version $Id: MyCaloHitExtended.h,v 1.6 2008-12-09 13:35:50 thomson Exp $<br>
 */
class MyCaloHitExtended {

    public:

    MyCaloHitExtended(CalorimeterHit* calhit, CalorimeterHitType type);
    
    ~MyCaloHitExtended();

    void setCalorimeterHitType(CalorimeterHitType type);
    void setCalorimeterHit(CalorimeterHit* calhit);
    void setPseudoLayer(int pL);
    void setPhysicalLayer(int pL);
    void setPseudoEnergy(float pE);
    void setEnergyInMips(float pE);
    void setEnergyEM(float pE);
    void setEnergyHAD(float pE);
    void setMCPFO(int m);
    void setGenericDistance(float d);
    void setDensityWeight(float W);
    void setAbsorberThickness(float t);
    void setLayerThickness(float t);
    void setLocalDensityWeight(float W);
    void setHitSizeZ(float dz);
    void setModule(int m);
    void setHitSizePhi(float dLphi);
    void setMipCandidateFlag(bool flag=true);
    void setIsolatedHitFlag(bool tf=true);
    void setPhoton(bool tf=true);
    void addSurroundingEnergy(float E);

    MyCaloHitExtended* clone();

    const CalorimeterHit * getCalorimeterHit() const;
    const CalorimeterHitType getCalorimeterHitType() const;
    const int getPseudoLayer() const;
    const int getPhysicalLayer() const;
    const int getSector() const;
    const int getMCPFO() const;
    const int getModule() const;
    const int getSectorOfInfluence(int i) const;
    const float getPseudoEnergy() const;
    const float getEnergyInMips() const;
    const float getEnergyEM() const;
    const float getEnergyHAD() const;
    const float getAbsorberThickness() const;
    const float getLayerThickness() const;
    const float getDensityWeight() const;
    const float getLocalDensityWeight() const;
    const float getHitSizeZ() const;
    const float getHitSizePhi() const;
    const float getTime() const;
    const float getGenericDistance() const;
    const float getEnergy() const;
    const float getSurroundingEnergy() const;
    const float * getPosition() const;
    const float * getRadialUnitVec() const;
    const bool isCandidateMip() const;
    const bool isIsolatedHit() const;
    const bool getPhoton() const;

    private:
    
    CalorimeterHit * _calohit;
    CalorimeterHitType _type;
    bool   _mipCandidate;
    bool   _isolatedHit;
    int    _pseudoLayer;
    int    _pseudoRadius;
    int    _physicalLayer;
    int    _isector;
    int    _mcPFO;
    int    _isectorOfInfluence[9];
    float  _absorberThickness;
    float  _layerThickness;
    float  _genericDistance;
    float  _pseudoEnergy;
    float  _energyInMips;
    float  _energyEM;
    float  _energyHAD;
    float  _densityWeight;
    float  _localDensityWeight;
    float  _hitSizeZ;
    float  _hitSizePhi;
    float  _radius;
    float  _surroundingEnergy;
    int    _module;
    float  _rhat[3];
    bool   _truePhotonHit;
};

#endif
