#ifndef MyMCTree_h
#define MyMCTree_h

#include <stdio.h>
#include <iostream>
#include <string>
#include <vector>
#include <map>
#include <set>
#include <stack>
#include <sstream>
#include <queue>
#include <EVENT/LCCollection.h>
#include <EVENT/MCParticle.h>
#include <IMPL/MCParticleImpl.h>
#include <IMPL/LCCollectionVec.h>
#include <iomanip>
#include <lcio.h>


typedef enum {
  MC_PFA_TYPE_UNKNOWN=0,
  MC_PFA_TYPE_TRACK,
  MC_PFA_TYPE_PHOTON,
  MC_PFA_TYPE_NEUTRAL_HADRON,
  MC_PFA_TYPE_ELECTRON,
  MC_PFA_TYPE_MUON,
  MC_PFA_TYPE_NEUTRINO
} McTreePFAType;

using std::map;
using namespace lcio;

/**
 *    Utility class for MC particles.
 *
 */
class MyMCPFO;
class MyTrackExtended;
class ProtoCluster;

class MyMCTree {

 public:

  /**
   * An object of this class is constructed with a pointer to a LCCollection of MC particles
   */
  MyMCTree(LCCollection* col);

  ~MyMCTree();

  void PerfectPFO(int opt);
  void oldThrust();
  float getThrust(){return _cosThrust;};
  int getPrimary(){return _ipdg;};
  unsigned int GetMCPFOIndex(MCParticle* part);
  std::vector<MCParticle*>* getMCTracks(){return &_mcTracks;};
  std::vector<MyMCPFO*>* getMCPFOs(){return &_mcPFOs;};
  std::vector<MCParticle*>* getQuarks(){return &_mcQuarks;};
  
  private:
  
  LCCollection* _col;
  float _cosThrust;
  int   _ipdg;
  std::vector<MCParticle*> _mcTracks;
  std::vector<MCParticle*> _mcQuarks;
  std::vector<MyMCPFO*> _mcPFOs;
  std::map<MCParticle*,unsigned int>_pmcToiMcPFO;
  
};



class MyMCPFO {

 public:

  /**
   * An object of this class is constructed with a pointer to a LCCollection of MC particles
   */
  MyMCPFO(MCParticle*);
  ~MyMCPFO();
  const float* getMomentum(){return _p;};
  bool isTrack(){return _isTrack;};
  bool isPhoton(){return _isPhoton;};
  bool isNeutralHadron(){return _isNeutralHadron;};
  bool isNeutrino(){return _isNeutrino;};
  void setTrack(MyTrackExtended* p){_associatedTrack=p;};
  MyTrackExtended* getTrack(){return _associatedTrack;};
  std::string getName(){return _name;};
  void setCluster(ProtoCluster* p){_associatedCluster=p;};
  ProtoCluster* getCluster(){return _associatedCluster;};
  void print();
  MCParticle* getMCParticle(){return _pMcPart;};
  int  getPDG(){return _ipdg;};
  bool isAssociatedWith(MCParticle*);
  McTreePFAType getPFAType(){return _pfaType;};

 private:
  MCParticle* _pMcPart;
  bool _isTrack;
  bool _isPhoton;
  bool _isNeutralHadron;
  bool _isNeutrino;
  float _p[4];
  int   _ipdg;
  std::vector<MCParticle*> _parentage;
  
  MyTrackExtended* _associatedTrack;
  ProtoCluster*    _associatedCluster;
  McTreePFAType _pfaType;
  std::string _name;
  int _nquarks;
};


#endif
