#ifndef MyPFOAnalysis_h
#define MyPFOAnalysis_h 

#include "marlin/Processor.h"
#include "lcio.h"
#include "Pandora.h"
#include <string>
#include <vector>
#include "EVENT/LCIO.h"
#include "EVENT/ReconstructedParticle.h"
#include "ProtoCluster.h"
#include <UTIL/LCRelationNavigator.h>
#include <EVENT/Track.h>
#include "TFile.h"
#include "MyCaloHitExtended.h"

using namespace lcio;
using namespace marlin;
class MyRecoDisplay;
class MyMCTree;
class MyMCPFO;

class TH1F;
class TH2F;

/** Example processor for marlin. If compiled with MARLIN_USE_AIDA 
 *  it creates a histogram (cloud) of the MCParticle energies.
 */
class MyPFOAnalysis : public Processor {
  
 public:
  
  virtual Processor*  newProcessor() { return new MyPFOAnalysis ; }
  
  
  MyPFOAnalysis() ;
  
  /** Called at the begin of the job before anything is read.
   * Use to initialize the processor, e.g. book histograms.
   */
  virtual void init() ;
  
  /** Called for every run.
   */
  virtual void processRunHeader( LCRunHeader* run ) ;

  
  /** Called for every event - the working horse.
   */
  virtual void processEvent( LCEvent * evt ) ; 
  
  
  virtual void check( LCEvent * evt ) ; 
  
  
  /** Called after data processing for clean up.
   */
  virtual void end() ;
  
  
  
 protected:
  
  // Protected member functions
  /** String specifying output Particle 
   *  collection 
   **/
  std::string _particleCollectionName;
  std::string _clusterCollectionName;
  

 private:
  
  void Clear();
   
  void AnalysePFAPerformance();
  void ConfusionAnalysis();
 
  MyRecoDisplay* _recoDisplay;
  MyMCTree* _myMcTree;
  int _printing;
  std::vector<ReconstructedParticle*>_pfovec;
  std::vector<MyMCPFO*>_pMcPFOs;
  float _ez;
  float _mz;
  float _costz;
  float _costq1;
  float _costq2;


  int _otherAnalysis;
  int _confusionAnalysis;

  LCCollection* _relationCaloCollection;
  LCCollection* _relationTrackCollection;
  LCRelationNavigator* _clusterNavigator;
  LCRelationNavigator* _trackNavigator;

  std::string _relationCaloString;
  std::string _relationTrackString;
  std::string _rootFile;

  int _display;
  int _nRun ;
  int _nEvt ;
  int _postRecoFix ;
  float _eCutLow;
  float _eCutHigh;
  float _cosQQCut;
  float _zOfEndcap;

  std::string _detectorName;

  TH1F* fxproton;
  TH1F* fNPFO;
  TH1F* fNTrackPFO;
  TH1F* fNClusterPFO;

  TH1F* fMcPfoPdgCode;
  TH1F* fPfoPdgCode;
  TH1F* fPFA;
  TH1F* fPFAhcal;
  TH1F* fPFAnu;
  TH1F* fPFAnufwd;
  TH1F* fPFAuds;
  TH1F* fPFAudsHP20;
  TH1F* fPFAudsHP10;
  TH1F* fPFAudsHM10;
  TH1F* fPFAudsHM20;
  TH1F* fPFAudscb;
  TH1F* fPFAcb;
  TH1F* fConfusedGamma;
  TH1F* fConfusedNeutral;
  TH1F* fPFA1;
  TH1F* fPFA2;
  TH1F* fPFA3;
  TH1F* fPFA4;
  TH1F* fPFA5;
  TH1F* fPFA6;
  TH1F* fPFA7;
  TH1F* fPFAL7A;
  TH1F* fPFAL7Aud;
  TH1F* fPFAL7As;
  TH1F* fPFAL7Ac;
  TH1F* fPFAL7Ab;
  TH1F* fPFAL7B;
  TH1F* fPFA8;
  TH1F* fPFA9;
  TH1F* fPFA10;
  TH1F* fPFA11;
  TH1F* fPFA12;
  TH1F* fPFA13;
  TH1F* fPFA14;

  TH1F* fPFAQQ;
  TH1F* fPFAQQ8;
  TH1F* fPFADMZ;
  TH1F* fPFADMZOMZ;
  TH1F* fPFADMZOMZQQ8;
  TH1F* fPFADMZ8;
  TH1F* fPFADMZQQ8;
  TH1F* fPFADMZP8;



  TH1F* fEcalMIP;
  TH1F* fEcal1MIP;
  TH1F* fHcalMIP;
  TH1F* fEcalMIPcorr;
  TH1F* fEcal1MIPcorr;
  TH1F* fHcalMIPcorr;


  TH1F* fETrackud;
  TH1F* fETracks;
  TH1F* fENeutralud;
  TH1F* fENeutrals;
  TH1F* fEGammaud;
  TH1F* fEGammas;


  TH1F* fFragmentEnergy;
  TH1F* fConfusedHadron;
  TH1F* fConfusedPhoton;
  TH1F* fConfusion;
  TH2F* fConfused;

} ;

#endif
