#ifndef MY_RECO_DISPLAY_H
#define MY_RECO_DISPLAY_H

class MyCaloHitExtended;

#include "EVENT/Track.h"
#include "EVENT/TrackerHit.h"
#include "EVENT/CalorimeterHit.h"
#include "EVENT/LCCollection.h"
#include "EVENT/ReconstructedParticle.h"
#include "TTree.h"
#include "TApplication.h"
#include "TNamed.h"
#include "TPad.h"
#include "TArc.h"
#include "TBox.h"
#include "TPolyLine.h"
#include "TPolyLine3D.h"
#include "TCanvas.h" // ROOT include file for drawing canvases
#include "TSystem.h" //
#include "TH1.h"     // ROOT include file for 2d histograms
#include "TH2.h"     // ROOT include file for 2d histograms
#include "TH3.h"     // ROOT include file for 2d histograms
#include "lcio.h"
#include "EVENT/LCIO.h"
#include <iostream>
#include <vector>
#include "MyTrackExtended.h"


using namespace std;
using namespace lcio;
class MyTrackExtended;
class ProtoCluster;

typedef enum {
  RECO_VIEW_XY =0,
  RECO_VIEW_XZ,
  RECO_VIEW_YZ,
  RECO_VIEW_RZ,
  RECO_VIEW_XYZ
} RecoDisplayView;

class MyRecoDisplay
{

 public:

  MyRecoDisplay(int nViews);
  void Clear();
  void CleanUp();
  void SetPlotRange(float r){_plotRange=r;};
  void SetTrackCollection(LCCollection*);
  void SetUnusedTPCHits(std::vector<TrackerHit*>);
  void SetCurlKilledTPCHits(std::vector<TrackerHit*>);
  void SetTrackAR(std::vector<MyTrackExtended*>);
  void SetMuon(std::vector<CalorimeterHit*>);
  void DrawTracks(RecoDisplayView view);
  void DrawMuonHits(RecoDisplayView view);
  void DrawByLayer(std::vector<MyCaloHitExtended*>[200]);
  void DrawByProtoCluster(std::vector<ProtoCluster*> &, int minHits=-1);
  void SetPseudoThrust(std::vector<ProtoCluster*> &);
  void SetPseudoThrust(LCCollection * col);
  void DrawByProtoClusterEnergy(std::vector<ProtoCluster*> &pClusters, int minHits);

  void DrawByJet(LCCollection * col, RecoDisplayView view);

  void DrawByProtoClusterEnergy(std::vector<ProtoCluster*> &pClusters, int minHits, RecoDisplayView view);
  void DrawByProtoClusterBadness(std::vector<ProtoCluster*> &pClusters, int minHits, RecoDisplayView view);
  void DrawSingleProtoCluster(ProtoCluster* pCluster, RecoDisplayView view);
  void DrawByMIP(std::vector<MyCaloHitExtended*>[200]);
  void DrawByEnergy(std::vector<MyCaloHitExtended*>[200]);
  void DrawByDensity(std::vector<MyCaloHitExtended*>[200]);
  void Draw();
  void MakeDetectorOutline();
  void DrawOutline(RecoDisplayView view);
  void DrawByParticle(LCCollection * col, RecoDisplayView view);
  void DrawUnusedHits(RecoDisplayView view);
  void DrawCurlKilledHits(RecoDisplayView view);
  void DrawParticle(ReconstructedParticle* part, RecoDisplayView view, int colour);
  void DrawLongitudinalProfile(ProtoCluster* pCluster, bool truncate=false);
  void DrawTransverseProfile(ProtoCluster* pCluster, int nlayers);
  void DrawTransverseProfileFine(ProtoCluster* pCluster, int nlayers);
  void DrawTransverseProfile(std::vector<ProtoCluster*> &pClusters, int nlayers);

 private:
                               
  int _nViews;
  int _nCount;
  float _fSum[10];
  float _eC[10];
  float _thrustX;
  float _thrustY;
  bool  _setThrust;
  float  _plotRange;

  LCCollection* _trackCollection;
  vector<TrackerHit*>      _unusedTPCHits;
  vector<TrackerHit*>      _curlKilledTPCHits;
  vector<MyTrackExtended*> _extendedTracks;
  vector<CalorimeterHit*> _muonHits;
  TApplication* theApp;
  TCanvas* fCanvas;      // Canvas for displaying histograms
  TPad                        *fxypad;
  TPad                        *frzpad;
  TH2F                        *fxyview;
  TH2F                        *fxzview;
  TH2F                        *frzview;
  TH1F                        *flongprofile;
  TH1F                        *flongshowerprofile;
  TH2F                        *ftransprofile;
  TH2F                        *ftransprofileFine;
  TList* fList; 
  vector<TPolyLine3D*> _3Dlines;
  vector<TPolyLine*> _2DlinesXY;
  vector<TPolyLine*> _2DlinesXZ;
  vector<TBox*> _2DboxesXZ;
  vector<TPolyLine*> _2DlinesRZ;
  vector<TArc*> _2DcircsXY;
  vector<TArc*> _2DcircsXZ;
  vector<TArc*> _2DcircsRZ;
    //  TArc* pCircle;
}; 
#endif 
