#ifndef TRACKMT_H 
#define TRACKMT_H 1

#include "lcio.h"
#include "EVENT/LCIO.h"
#include "EVENT/Track.h"
#include <vector>
#include "ProtoCluster.h"
#include "TrackerHitExtended.h"
#include "GroupTracks.h"
#include "HelixClass.h"
#include <EVENT/MCParticle.h>

using namespace lcio;

typedef enum {
  TRACK_STATUS_UNKNOWN=0,
  TRACK_STATUS_GOOD,
  TRACK_STATUS_LOWPT,
  TRACK_STATUS_UNASSOC,
  TRACK_STATUS_BAD,
  TRACK_STATUS_NON_VERTEX,
  TRACK_STATUS_KILLED,
  TRACK_STATUS_USED_IN_V0,
  TRACK_STATUS_CHARGED_KAON_DECAY
} MyTrackStatus;

class ProtoCluster;
class GroupTracks;
class TrackerHitExtended;
class MyTrackExtended;
class MyMCPFO;

typedef std::vector<MyTrackExtended*> MyTrackExtendedVec;
typedef std::vector<TrackerHitExtended*> TrackerHitExtendedVec;
typedef std::vector<ProtoCluster*>ProtoClusterVec;

/**
 * Class extending native LCIO class Track. <br>
 * Class TrackExtended is used in TrackwiseClustering <br>
 * and Wolf processors. <br>
 * @author A. Raspereza (DESY)<br>
 * @version $Id: MyTrackExtended.h,v 1.6 2008-07-08 10:05:01 aplin Exp $<br>
 */

class MyTrackExtended {

 public:


    MyTrackExtended( );
    MyTrackExtended( TrackerHitExtended * trackerhit );
    MyTrackExtended( Track * track );
    ~MyTrackExtended();
    
    Track * getTrack();
    const float * getSeedDirection();
    const float * getSeedPosition();
    const float * getTpcIntersection();
    const float * getEndcapSeedDirection();
    const float * getTrackEndPosition();
    const float * getTrackEndDirection();
    const float * getTrackStartPosition();
    const float * getTrackStartDirection();
    const float * getEndcapSeedPosition();
    const float * getAltSeedDirection();
    const float * getAltSeedPosition();
    ProtoClusterVec & getClusterVec();
    ProtoCluster * getSuperCluster();
    TrackerHitExtendedVec & getTrackerHitExtendedVec();
    void AddMCPFO(MyMCPFO* p){_associatedMCPFOs.push_back(p);};
    std::vector<MyMCPFO*>& GetMCPFOVec(){return _associatedMCPFOs;};
    int hits();
    int nonSiHits();
    void setTrackStatus(MyTrackStatus status);
    MyTrackStatus trackStatus(){return _status;};
    void addCluster(ProtoCluster * cluster);
    void setSuperCluster(ProtoCluster * superCluster);
    void setSeedDirection( float * seedDirection );
    void setSeedPosition( float * seedPosition);
    void setTpcIntersection( float * position);
    void setAltSeedDirection( float * seedDirection );
    void setAltSeedPosition( float * seedPosition);
    void setEndcapSeedDirection( float * seedDirection );
    void setTrackEndPosition( float * seedPosition);
    void setTrackEndDirection( float * seedDirection );
    void setTrackStartPosition( float * seedPosition);
    void setTrackStartDirection( float * seedDirection );
    void setEndcapSeedPosition( float * seedPosition);
    void setMomentum( float * p);
    void setHelix( HelixClass * h);
    HelixClass* getHelix();
    void setAltHelix( HelixClass * h);
    HelixClass* getAltHelix();
    void setEndMomentum( float * p);
    void setStartMomentum( float * p);
    void addTrackerHitExtended( TrackerHitExtended * trackerhit);
    void ClearTrackerHitExtendedVec();
    void ClearClusters();
    bool isV0();
    bool reachedEcal();
    void reachedEcal(bool tf){_reachedECAL=tf;};
    void isV0(bool tf);
    bool isKinkE();
    void isKinkE(bool tf);
    void addBackScatterTrack(MyTrackExtended* pt){_backScatterDaughters.push_back(pt);};
    unsigned int getNBackScatterDaughters(){return _backScatterDaughters.size();};
    void setBackScatterParent(MyTrackExtended* pt){_backScatterParent=pt;};
    MyTrackExtended* getBackScatterParent(){return _backScatterParent;};
    MyTrackExtended* getBackScatterDaughter(int i){return _backScatterDaughters[i];};



    void addProngDaughter(MyTrackExtended* pt){_prongDaughters.push_back(pt);};
    unsigned int getNProngDaughters(){return _prongDaughters.size();};
    MyTrackExtended* getProngDaughter(int i){return _prongDaughters[i];};
    void setProngParent(MyTrackExtended* pt){_prongParent=pt;};
    MyTrackExtended* getProngParent(){return _prongParent;};


    void setKinkDaughter(MyTrackExtended* pt){_kinkDaughter=pt;};
    MyTrackExtended* getKinkDaughter(){return _kinkDaughter;};
    void setKinkParent(MyTrackExtended* pt){_kinkParent=pt;};
    MyTrackExtended* getKinkParent(){return _kinkParent;};
    void setSplitDaughter(MyTrackExtended* pt){_splitDaughter=pt;};
    MyTrackExtended* getSplitDaughter(){return _splitDaughter;};
    void setSplitParent(MyTrackExtended* pt){_splitParent=pt;};
    MyTrackExtended* getSplitParent(){return _splitParent;};


    void setMCParticle(MCParticle* mcpart);
    MCParticle* getMCParticle(){return _mcPart;};
    bool isKinkS();
    void isKinkS(bool tf);
    bool isProngS(){return _isProngS;};
    void isProngS(bool tf){_isProngS=tf;};
    bool isSplitS(){return _isSplitS;};
    void isSplitS(bool tf){_isSplitS=tf;};
    bool isSplitE(){return _isSplitE;};
    void isSplitE(bool tf){_isSplitE=tf;};

    bool isBackScatter(){return _isBackScatter;};
    void isBackScatter(bool tf){_isBackScatter=tf;};

    bool isProngE(){return _isProngE;};
    void isProngE(bool tf){_isProngE=tf;};
    void setX0(float x0);
    void setY0(float y0);
    void setR0(float r0);
    void setD0(float d0);
    void setZ0(float z0);
    void setBz(float bz);
    void setPhi0(float phi0);
    void setPhi(float phi);
    void setOmega(float omega);
    void setTanLambda(float tanLambda);

    void setStart(float * xStart);
    void setEnd(float * xEnd);


    float getX0();
    float getY0();
    float getD0();
    float getZ0();
    float getOmega();
    float getTanLambda();
    float getPhi();
    float getR0();
    float getBz();
    float getPhi0();
    const float* getMomentum();
    const float* getStartMomentum();
    const float* getEndMomentum();
    float getEnergy();
    float getInnerR();
    float getOuterR();
    float getZMin();
    float getZMax();
    void  setEnergy(float E);
    void  setInnerR(float r);
    void  setOuterR(float r);
    void  setZMin(float r);
    void  setZMax(float r);
    void SetMatchedV0Track(MyTrackExtended* track){_matchedV0Track=track;};
    MyTrackExtended* GetMatchedV0Track(){return _matchedV0Track;};

    
    float * getStart();
    float * getEnd();

    void setGroupTracks( GroupTracks * group );
    GroupTracks * getGroupTracks();



 private:

    ProtoCluster *_superCluster;
    ProtoClusterVec _clusterVec;
    GroupTracks * _group;
    Track * _track;
    bool _isV0;
    bool _isKinkS;
    bool _isKinkE;
    bool _isProngS;
    bool _isSplitE;
    bool _isSplitS;
    bool _isProngE;
    bool _isBackScatter;
    float _seedDirection[3];
    float _seedPosition[3];
    float _tpcIntersection[3];
    float _endcapSeedDirection[3];
    float _endcapSeedPosition[3];
    float _trackEndDirection[3];
    float _trackEndPosition[3];
    float _trackStartDirection[3];
    float _trackStartPosition[3];
    float _altSeedDirection[3];
    float _altSeedPosition[3];
    TrackerHitExtendedVec _trackerHitVector;    

    float _x0;
    float _y0;
    float _r0;
    float _bz;
    float _phi0;

    float _xStart[3];
    float _xEnd[3];

    float _d0; // d0 in canonical parameterisation
    float _z0; // z0 in canonical parameterisation
    float _omega; // omega in canonical parameterisation
    float _tanLambda; // tanlambda in canonical parameterisation
    float _phi; // phi in canonical parameterisation 
    float _energy; // enery in GeV
    float _momentum[4];
    float _endMomentum[4];
    float _startMomentum[4];
    float _innerR;
    float _outerR;
    float _zmin;
    float _zmax;
    bool  _reachedECAL;
    MyTrackStatus _status;    
    std::vector<MyMCPFO*> _associatedMCPFOs;
    MyTrackExtended* _splitParent;
    MyTrackExtended* _splitDaughter;
    MyTrackExtended* _kinkParent;
    MyTrackExtended* _prongParent;
    MyTrackExtended* _backScatterParent;
    MyTrackExtendedVec _prongDaughters;
    MyTrackExtendedVec _backScatterDaughters;
    MyTrackExtended* _kinkDaughter;
    MyTrackExtended* _prongDaughter;
    MyTrackExtended* _matchedV0Track;
    MCParticle* _mcPart;
    HelixClass* _helix;
    HelixClass* _altHelix;
    
};

#endif
