// Singleton object which manufactures ProtoClusters
//   it creates the appropriate type of cluster and deals
//   with configuration of cluster

#ifndef PROTO_CLUSTER_FACTORY
#define PROTO_CLUSTER_FACTORY


#include <vector>
#include "ProtoCluster.h"

class ProtoClusterFactory;
class MyTrackExtended;
class MyCaloHitExtended;

class ProtoClusterFactory{

 public:
  static ProtoClusterFactory* Instance();
  ProtoCluster* Manufacture(MyTrackExtended*);
  ProtoCluster* Manufacture(MyCaloHitExtended*);
  ProtoCluster* CloneAndDelete(ProtoCluster*);
  ProtoCluster* Clone(ProtoCluster*);
  ProtoCluster* Merge(ProtoCluster*, ProtoCluster*);
  void SetDesign(const protoClusterDesign_t design);
  void SetDefaultDesign(const protoClusterDesign_t design);
  void SetDesignToDefault();
  protoClusterDesign_t GetDesign();
  void ReclusteringOn(){_reclustering = true;}
  void ReclusteringOff(){_reclustering = false;}
  void Delete();
  void ManageMemory();
  void ManageMemoryDuringReclustering();
  void ZOfEndcap(float z);
  void ZOfBarrel(float z);
  void ResetCounter(){_uniqueID=0;};
  void SetPhotonIDCuts(float c){_photonID_dcoscut=c;};

 protected:
  ProtoClusterFactory();
 private:
  static ProtoClusterFactory* _instance;
  int _uniqueID;
  float _zOfEndcap;
  float _zOfBarrel;
  std::vector<ProtoCluster*>_clusters;
  protoClusterDesign_t _design;
  protoClusterDesign_t _defaultDesign;
  float _photonID_dcoscut;
  bool _reclustering;

};
#endif
