#include "MyCaloHitExtended.h"
#include "Pandora.h"
#include <math.h>
#include <iostream>

const float pi = acos(-1.0);
const float twopi = 2.0*pi;

MyCaloHitExtended::MyCaloHitExtended(CalorimeterHit* calhit, CalorimeterHitType type) {
    _calohit = calhit;
    _type    = type;
    _mipCandidate = false;
    _isolatedHit  = false;
    _truePhotonHit  = false;
    _mcPFO = -999;
    _surroundingEnergy = 0.;
    float r = 0;
    
    for(int i=0;i<3;++i){
      _rhat[i]    = calhit->getPosition()[i];
      r += _rhat[i]*_rhat[i];
    }

    r = sqrt(r);
    for(int i=0;i<3;++i)_rhat[i]=_rhat[i]/r;
    float phi = atan2(_rhat[1],_rhat[0]);
    if(phi<0)phi += twopi;
    float theta = acos(static_cast<double>(_rhat[2]));
    float thetasector = theta/pi*(NUMBER_OF_THETA_SECTORS-2);
    float phisector   = phi/twopi*NUMBER_OF_PHI_SECTORS;
    int itheta = static_cast<int>(thetasector)+1;
    int iphi   = static_cast<int>(phisector);
    _isector   = itheta*NUMBER_OF_PHI_SECTORS+iphi;

    int icount = 0;
    for(int i = iphi-1;i<=iphi+1;++i){
      for(int j = itheta-1;j<=itheta+1;++j){
	int ii = i;
	if(i>=NUMBER_OF_PHI_SECTORS)ii=0;
	if(i<0)ii=NUMBER_OF_PHI_SECTORS-1;
	int isectorOfInfluence   = j*NUMBER_OF_PHI_SECTORS+ii;
	_isectorOfInfluence[icount] = isectorOfInfluence;
	if(isectorOfInfluence<0){
	  std::cout << " sector error " << itheta << "," << iphi << " -> " << j << "," << ii << std::endl;
	}
	icount++;
      }
    }
    
}

MyCaloHitExtended::~MyCaloHitExtended(){    
}

void MyCaloHitExtended::setCalorimeterHit(CalorimeterHit* calhit) {
    _calohit = calhit;
}

void MyCaloHitExtended::setCalorimeterHitType(CalorimeterHitType type) {
    _type = type;
}

void MyCaloHitExtended::setGenericDistance(float d) {
    _genericDistance = d;
}

void MyCaloHitExtended::setPseudoLayer(int pl) {
    _pseudoLayer = pl;
}

void MyCaloHitExtended::setPhysicalLayer(int l) {
    _physicalLayer = l;
}


void MyCaloHitExtended::setHitSizeZ(float dz) {
    _hitSizeZ = dz;
}

void MyCaloHitExtended::setHitSizePhi(float dphi) {
    _hitSizePhi = dphi;
}

void MyCaloHitExtended::setPseudoEnergy(float pE) {
    _pseudoEnergy = pE;
    this->setGenericDistance(pE);

}


void MyCaloHitExtended::setAbsorberThickness(float t){    
  _absorberThickness=t;
}

void MyCaloHitExtended::setLayerThickness(float t){    
  _layerThickness=t;
}


void MyCaloHitExtended::setEnergyInMips(float eMip) {
    _energyInMips = eMip;

}

void MyCaloHitExtended::setModule(int m) {
    _module = m;

}


void MyCaloHitExtended::setMCPFO(int m) {
    _mcPFO = m;

}

void MyCaloHitExtended::setEnergyEM(float e) {
    _energyEM = e;
}
void MyCaloHitExtended::addSurroundingEnergy(float e) {
    _surroundingEnergy += e;
}

void MyCaloHitExtended::setEnergyHAD(float e) {
    _energyHAD = e;
}



void MyCaloHitExtended::setDensityWeight(float w) {
    _densityWeight = w;
}

void MyCaloHitExtended::setLocalDensityWeight(float w) {
    _localDensityWeight = w;
}

void MyCaloHitExtended::setMipCandidateFlag(bool flag) {
    _mipCandidate = flag;
}


void MyCaloHitExtended::setIsolatedHitFlag(bool flag) {
    _isolatedHit = flag;
}

void MyCaloHitExtended::setPhoton(bool flag) {
    _truePhotonHit = flag;
}

const bool MyCaloHitExtended::isCandidateMip() const {
    return _mipCandidate;
}

const bool MyCaloHitExtended::isIsolatedHit() const {
    return _isolatedHit;
}

MyCaloHitExtended* MyCaloHitExtended::clone(){

  MyCaloHitExtended* newhit = new MyCaloHitExtended(_calohit,this->getCalorimeterHitType());
  newhit->setCalorimeterHitType(_type);
  newhit->setGenericDistance(_genericDistance);
  newhit->setPseudoLayer(_pseudoLayer);
  newhit->setPhysicalLayer(_physicalLayer);
  newhit->setHitSizeZ(_hitSizeZ);
  newhit->setHitSizePhi(_hitSizePhi);
  newhit->setPseudoEnergy(_pseudoEnergy);
  newhit->setEnergyInMips(_energyInMips);
  newhit->setEnergyEM(_energyEM);
  newhit->setEnergyHAD(_energyHAD);
  newhit->setDensityWeight(_densityWeight);
  newhit->setLocalDensityWeight(_localDensityWeight);
  newhit->setMipCandidateFlag(_mipCandidate);
  newhit->addSurroundingEnergy(_surroundingEnergy);
  return newhit;

}

const CalorimeterHit* MyCaloHitExtended::getCalorimeterHit() const{    
    return _calohit;
}

const CalorimeterHitType MyCaloHitExtended::getCalorimeterHitType() const{    
    return _type;
}

const int MyCaloHitExtended::getPseudoLayer() const {    
    return _pseudoLayer;
}


const int MyCaloHitExtended::getSector() const {    
    return _isector;
}

const int MyCaloHitExtended::getMCPFO() const {    
    return _mcPFO;
}

const int MyCaloHitExtended::getModule() const {    
    return _module;
}


const int MyCaloHitExtended::getSectorOfInfluence(int i) const {    
  return _isectorOfInfluence[i];
}

const int MyCaloHitExtended::getPhysicalLayer() const {    
    return _physicalLayer;
}


const float MyCaloHitExtended::getAbsorberThickness() const {    
    return _absorberThickness;
}

const float MyCaloHitExtended::getLayerThickness() const {    
    return _layerThickness;
}


const float MyCaloHitExtended::getDensityWeight() const {    
    return _densityWeight;
}

const float MyCaloHitExtended::getSurroundingEnergy() const {    
    return _surroundingEnergy;
}

const float MyCaloHitExtended::getLocalDensityWeight() const {    
    return _localDensityWeight;
}

const float MyCaloHitExtended::getPseudoEnergy() const {    
    return _pseudoEnergy;
}

const float MyCaloHitExtended::getEnergyEM() const {    
    return _energyEM;
}

const float MyCaloHitExtended::getEnergyHAD() const {    
    return _energyHAD;
}


const float MyCaloHitExtended::getEnergyInMips() const {    
    return _energyInMips;
}

const float MyCaloHitExtended::getHitSizeZ() const {    
    return _hitSizeZ;
}

const float MyCaloHitExtended::getHitSizePhi() const {    
    return _hitSizePhi;
}

const float * MyCaloHitExtended::getPosition() const {    
    return _calohit->getPosition();
}

const float MyCaloHitExtended::getTime() const {    
    return _calohit->getTime();
}

const float * MyCaloHitExtended::getRadialUnitVec() const {    
    return _rhat;
}

const float MyCaloHitExtended::getGenericDistance() const {    
    return _genericDistance;
}

const float MyCaloHitExtended::getEnergy() const {    
    return _calohit->getEnergy();
}


const bool MyCaloHitExtended::getPhoton() const {    
    return   _truePhotonHit;
}
