#include "MyMCTree.h"
#include "ProtoCluster.h"
#include "MyTrackExtended.h"
#include <cstdlib>
#include <cmath>

using namespace lcio ;
using namespace std;
using namespace IMPL;
using namespace EVENT;




// ##########################################
// #####                                #####
// #####   Constructor and Destructor   #####
// #####                                #####
// ##########################################

//=============================================================================

MyMCTree::MyMCTree(LCCollection* col){

  _col = col;
  if (_col->getTypeName() != LCIO::MCPARTICLE )return;
    
  int nParticles =  _col->getNumberOfElements() ;

  float sx=0.;
  float sy=0.;
  float sz=0.;
  float sxz=0.;
  float syz=0.;
  float szz=0.;
  float n=0.;
  float scost =0.;
  
  _mcTracks.clear();
  _mcPFOs.clear();
  _mcQuarks.clear();

  float eobs   = 0.;
  for(int i=0;i<nParticles;++i){
    MCParticle* part =  dynamic_cast<MCParticle*>( _col->getElementAt( i ) ) ;
    float energy = part->getEnergy();
    EVENT::MCParticleVec _daughters = part->getDaughters();
    EVENT::MCParticleVec _parents = part->getParents();
    // find the primary particles



    int ipdg = abs(part->getPDG());
    if(ipdg>=1  && ipdg<=6){
      //float x =  part->getMomentum()[0];
      //float y =  part->getMomentum()[1];
      //float z =  part->getMomentum()[2];
      //float e =  part->getEnergy(); 
      //std::cout << i << " quarks : " << ipdg << " : " << x << "," << y << "," << z << " : " << e << " cost = " <<  _parents.size() << " " << _daughters.size() << std::endl;
      if(_parents.size()==0){
	_mcQuarks.push_back(part);
      }
    }
    if(_parents.size()==0){
      //std::cout << part->getPDG() << " : " << part->getEnergy() << " : " << part->getMomentum()[2]/part->getEnergy() << std::endl;
      int ipdg = abs(part->getPDG());
      float cost = abs(part->getMomentum()[2])/part->getEnergy();

      if( (ipdg>=1  && ipdg<=6) ||
          (ipdg>=12 && ipdg<=16) ||
          (ipdg==11 && cost<0.999) ){
	float x =  part->getMomentum()[0];
	float y =  part->getMomentum()[1];
	float z =  part->getMomentum()[2];
	eobs+=energy;
	sx+= x;
	sy+= y;
	sz+= z;
	szz+=z*z;
	sxz+=x*z;
	syz+=y*z;
	n+=1.0;
	_ipdg = ipdg;
	scost+=cost;
      }
    }    
  }

  if(n>1){
    _cosThrust = scost/n;
  }else{
    this->oldThrust();
  }

}





//=============================================================================

MyMCTree::~MyMCTree() {

  // delete the mcPFOs
  for(unsigned int imcpfo=0;imcpfo<_mcPFOs.size();++imcpfo){
    delete _mcPFOs[imcpfo];
  }

}

//=============================================================================

void MyMCTree::oldThrust() {

  if (_col->getTypeName() != LCIO::MCPARTICLE )return;
  float sx=0.;
  float sy=0.;
  float sz=0.;
  float sxz=0.;
  float syz=0.;
  float szz=0.;
  float n=0.;
    
  int nParticles =  _col->getNumberOfElements() ;

  //std::cout << " N particles : " << nParticles << std::endl;

  float eobs   = 0.;
  for(int i=0;i<nParticles;++i){
    MCParticle* part =  dynamic_cast<MCParticle*>( _col->getElementAt( i ) ) ;
    float energy = part->getEnergy();
    EVENT::MCParticleVec _daughters = part->getDaughters();
    EVENT::MCParticleVec _parents = part->getParents();
    if(_parents.size()==0){
      if(abs(part->getPDG())!=11 || part->getEnergy()<40.){
	eobs+=energy;
	float x =  part->getMomentum()[0];
	float y =  part->getMomentum()[1];
	float z =  part->getMomentum()[2];
	sx+= x;
	sy+= y;
	sz+= z;
	szz+=z*z;
	sxz+=x*z;
	syz+=y*z;
	n+=1.0;
      }
    }    
  }

  float ax = (sz*sx-n*sxz)/(sz*sz-n*szz);
  float ay = (sz*sy-n*syz)/(sz*sz-n*szz);

  _ipdg = 1;
  _cosThrust = 1./sqrt(ax*ax+ay*ay+1);

}



//=============================================================================

void MyMCTree::PerfectPFO(int opt) {

  if (_col->getTypeName() != LCIO::MCPARTICLE )return;
  float etot=0;
  float etrack=0;
  float eem=0;
  float ehad=0;
  float eunk=0;
  float ee=0;
  float emu=0;
  float enu=0;
  bool printing = false;
  if(opt>0)printing =true;

  std::vector<MCParticle*> pUsedMCVec;

    
  int nParticles =  _col->getNumberOfElements() ;
  if(printing)std::cout << " N particles : " << nParticles << std::endl;
  for(int i=0;i<nParticles;++i){
    MCParticle* part =  dynamic_cast<MCParticle*>( _col->getElementAt( i ) ) ;
    int ipdg = abs(part->getPDG()); 
    //    float energy = part->getEnergy();
    EVENT::MCParticleVec _daughters = part->getDaughters();
    EVENT::MCParticleVec _parents = part->getParents();
    float penergy = 0;
    if(_parents.size()>0)penergy = _parents[0]->getEnergy();
    int parent = 0;
    float xs = part->getVertex()[0];
    float ys = part->getVertex()[1];
    float zs = part->getVertex()[2];
    float rs = sqrt(xs*xs+ys*ys+zs*zs);
    float xe = part->getEndpoint()[0];
    float ye = part->getEndpoint()[1];
    float ze = part->getEndpoint()[2];
    float re = sqrt(xe*xe+ye*ye+ze*ze);
    float ce = abs(ze)/re;
    float p  = sqrt(part->getEnergy()*part->getEnergy()-part->getMass()*part->getMass()); 

    if(_parents.size()==0&&printing){
      float ct = 0.;
      if(p>0)ct = fabs(part->getMomentum()[2])/p;
       std::cout << " Primary MC : " << ipdg << " E = " <<  part->getEnergy() << " cosTheta = " << ct << std::endl;

    }
    
    //std::cout << part->getPDG() << " : " << part->getEnergy() << " : " << part->getMomentum()[2]/part->getEnergy() << " : " << _parents.size() << "," << _daughters.size() << " rs re = " << rs << "-" << re << std::endl;

    if(p>0.01 && rs < 500 && re > 500){
      pUsedMCVec.push_back(part);
      
      bool found = false;
      std::vector<MCParticle*> parentage;
      MCParticle* apart = part;
      EVENT::MCParticleVec parents = apart->getParents();
      while(parents.size()>0){
	apart = parents[0];
	parentage.push_back(apart);
	parents = apart->getParents();
	for(unsigned int j=0;j<pUsedMCVec.size();++j){
	  if(apart==pUsedMCVec[j])found = true;
	}
      } 

      McTreePFAType pfaType = MC_PFA_TYPE_UNKNOWN;
      string name = "unknown";
      switch(ipdg){
      case 211:
	name = "   pi+-";
	pfaType = MC_PFA_TYPE_TRACK;
	break;
      case 22:
	name = " photon";
	pfaType = MC_PFA_TYPE_PHOTON;
	break;
      case 11:
	name = " electron";
	pfaType = MC_PFA_TYPE_ELECTRON;
	break;
      case 13:
	name = " muon";
	pfaType = MC_PFA_TYPE_MUON;
	break;
      case 2212:
	name = " proton";
	pfaType = MC_PFA_TYPE_TRACK;
	break;
      case 321:
	name = "    K+-";
	pfaType = MC_PFA_TYPE_TRACK;
	break;
      case 310:
	name = "     KS";
	pfaType = MC_PFA_TYPE_TRACK;
	break;
      case 130:
	name = "     KL";
	pfaType = MC_PFA_TYPE_NEUTRAL_HADRON;
        break;
      case 2112:
	name = "neutron";
	pfaType = MC_PFA_TYPE_NEUTRAL_HADRON;
	break;
      case 3222:
      case 3112:
	name = "sigma+-";
	pfaType = MC_PFA_TYPE_TRACK;
	break;
      case 12:
      case 14:
      case 16:
	name = "neutrino";
	pfaType = MC_PFA_TYPE_NEUTRINO;
	break;
      case 3122:
	name = "  lambda";
	pfaType = MC_PFA_TYPE_TRACK;
        cout << "*****" << i << " " << part->getPDG() << "  " << name << " " << " " << parent << " p = " << part->getEnergy() << " r = " << rs << "-" << re << " cos " << ce << endl;
	break;
      default:
	pfaType = MC_PFA_TYPE_UNKNOWN;
	name = "unknown";
	break;
      }

      bool lownp =false;
      if(part->getPDG()==2112||part->getPDG()==2212){
	if(rs>10 && part->getEnergy()<1.2)lownp = true;
      }


      //      if(printing && !found && !lownp)cout << i << " " << part->getPDG() << "  " << name << " " << " " << parent << " p = " << part->getEnergy() << " r = " << rs << "-" << re << " cos " << ce << endl;
      if(pfaType==MC_PFA_TYPE_TRACK)_mcTracks.push_back(part);
      
      if(!found && !lownp){
	MyMCPFO* pfo = new MyMCPFO(part);
	_mcPFOs.push_back(pfo);
	etot += part->getEnergy();
	if(pfaType==MC_PFA_TYPE_TRACK)etrack += part->getEnergy();
	if(pfaType==MC_PFA_TYPE_UNKNOWN)eunk += part->getEnergy();
	if(pfaType==MC_PFA_TYPE_PHOTON)eem += part->getEnergy();
	if(pfaType==MC_PFA_TYPE_NEUTRAL_HADRON)ehad += part->getEnergy();
	if(pfaType==MC_PFA_TYPE_ELECTRON)ee += part->getEnergy();
	if(pfaType==MC_PFA_TYPE_MUON)emu += part->getEnergy();
	if(pfaType==MC_PFA_TYPE_NEUTRINO)enu += part->getEnergy();
      }
    }
  }
  if(printing){
    cout << " MC ETOT : " << etot << endl;
    cout << " MC Etrack : " << etrack << endl;
    cout << " MC Eem    : " << eem << endl;
    cout << " MC Ehad   : " << ehad << endl;
    cout << " MC Eelect : " << ee << endl;
    cout << " MC Emuon  : " << emu << endl;
    cout << " MC Enu    : " << enu << endl;
    cout << " MC Eunk   : " << eunk << endl;
  }

  // populate the map
  for(unsigned int imcpfo=0;imcpfo<_mcPFOs.size();++imcpfo){
    MCParticle* part = _mcPFOs[imcpfo]->getMCParticle();
    vector<MCParticle*>particles;
    vector<MCParticle*>newParticles;
    particles.push_back(part);
    while(particles.size()>0){
      newParticles.clear();
      for(unsigned int imc=0;imc<particles.size();imc++){
	_pmcToiMcPFO.insert(
	   map<MCParticle*,unsigned int>::value_type(particles[imc],imcpfo)
	   );
	EVENT::MCParticleVec daughters = particles[imc]->getDaughters();
	for(unsigned int id=0;id<daughters.size();id++){
	  newParticles.push_back(daughters[id]);
	}
      }
      particles.clear();
      for(unsigned int imc=0;imc<newParticles.size();imc++)particles.push_back(newParticles[imc]);
    }
  }

  return;

}

//=============================================================================

unsigned int MyMCTree::GetMCPFOIndex(MCParticle* part){
  
  unsigned int retval = 999999;
  map<MCParticle*,unsigned int>::iterator it = _pmcToiMcPFO.find(part);
  if(it!=_pmcToiMcPFO.end())retval = (*it).second;
 
  return retval;
}



MyMCPFO::MyMCPFO(MCParticle* part){


  _pMcPart = part;
  _p[0] = part->getMomentum()[0];
  _p[1] = part->getMomentum()[1];
  _p[2] = part->getMomentum()[2];
  _p[3] = part->getEnergy();
  _ipdg = part->getPDG();
  EVENT::MCParticleVec parents = part->getParents();
  while(parents.size()>0){
    MCParticle* apart = parents[0];
    _parentage.push_back(apart);
    parents = apart->getParents();
  } 


  _associatedTrack = NULL;
  _associatedCluster = NULL;
  _isTrack = false;
  _isPhoton= false;
  _isNeutralHadron=false;
  _isNeutrino=false;

  switch(abs(_ipdg)){
  case 211:
    _name = "   pi+-";
    _pfaType = MC_PFA_TYPE_TRACK;
    break;
  case 22:
    _name = " photon";
    _pfaType = MC_PFA_TYPE_PHOTON;
    break;
  case 11:
    _name = "electro";
    _pfaType = MC_PFA_TYPE_ELECTRON;
    break;
  case 13:
    _name = "   muon";
    _pfaType = MC_PFA_TYPE_MUON;
    break;
  case 2212:
    _name = " proton";
    _pfaType = MC_PFA_TYPE_TRACK;
    break;
  case 321:
    _name = "    K+-";
    _pfaType = MC_PFA_TYPE_TRACK;
    break;
  case 310:
    _name = "     KS";
    _pfaType = MC_PFA_TYPE_TRACK;
    break;
  case 130:
    _name = "     KL";
    _pfaType = MC_PFA_TYPE_NEUTRAL_HADRON;
    break;
  case 2112:
    _name = "neutron";
    _pfaType = MC_PFA_TYPE_NEUTRAL_HADRON;
    break;
  case 3222:
  case 3112:
    _name = "sigma+-";
    _pfaType = MC_PFA_TYPE_TRACK;
    break;
  case 12:
  case 14:
  case 16:
    _name = "neutrino";
    _pfaType = MC_PFA_TYPE_NEUTRINO;
    break;
  case 3122:
    _name = "  lambda";
    _pfaType = MC_PFA_TYPE_TRACK;
    break;
  default:
    _pfaType = MC_PFA_TYPE_UNKNOWN;
    _name = "unknown";
    break;
  }
  
  if(_pfaType==MC_PFA_TYPE_TRACK)_isTrack=true;
  if(_pfaType==MC_PFA_TYPE_MUON)_isTrack=true;
  if(_pfaType==MC_PFA_TYPE_ELECTRON)_isTrack=true;
  if(_pfaType==MC_PFA_TYPE_NEUTRAL_HADRON)_isNeutralHadron=true;
  if(_pfaType==MC_PFA_TYPE_NEUTRINO)_isNeutrino=true;
  if(_pfaType==MC_PFA_TYPE_PHOTON)_isPhoton=true;

  
}


bool MyMCPFO::isAssociatedWith(MCParticle* mcParticle){
  //
  vector<MCParticle*>particles;
  vector<MCParticle*>newParticles;
  particles.push_back(_pMcPart);
 
  bool found = false;
  while(particles.size()>0&&!found){
    newParticles.clear();
    for(unsigned int imc=0;imc<particles.size();imc++){
      if(mcParticle==particles[imc])found=true;
      EVENT::MCParticleVec daughters = particles[imc]->getDaughters();
      for(unsigned int id=0;id<daughters.size();id++){
	newParticles.push_back(daughters[id]);
      }
    }
    particles.clear();
    for(unsigned int imc=0;imc<newParticles.size();imc++)particles.push_back(newParticles[imc]);
  }

  return found;
}

MyMCPFO::~MyMCPFO(){

}
