#include "MyRecoDisplay.h"
#include "TApplication.h"
#include "MyCaloHitExtended.h"
#include <MyTrackExtended.h>
#include "ProtoCluster.h"
#include "EVENT/CalorimeterHit.h"
#include "EVENT/Track.h"
#include <EVENT/LCCollection.h>
#include <cstdio>
#include <vector>
#include <sstream>

#include <math.h>
#include <unistd.h>
#include <fcntl.h>

// GEAR include files
#include <gear/GEAR.h>
#include <gear/CalorimeterParameters.h>
#include <gear/GearParameters.h>
#include <gear/TPCParameters.h>
#include <gear/PadRowLayout2D.h>
#include <gear/LayerLayout.h>

// Marlin stuff
#include <marlin/Global.h>

// root stuff
#include "TFile.h"
#include "TPolyMarker.h"
#include "TPolyMarker3D.h"
#include "TPolyLine.h"
#include "TArrow.h"
#include "TPolyLine3D.h"
#include "TArc.h"


const float pi = acos(-1.0);
const float twopi = 2.0*pi;
const int nlevels =7;

float level[nlevels+1];
int  colours[nlevels];

using namespace marlin ;

MyRecoDisplay::MyRecoDisplay(int nViews) 
{ 



  _plotRange = 4250.;
  _setThrust = false;
  _nCount =0;
  for(unsigned int i=0;i<10;i++)_fSum[i]   =0.;
  _eC[0]=0.20;
  _eC[1]=0.15;
  _eC[2]=0.10;
  _eC[3]=0.09;
  _eC[4]=0.08;
  _eC[5]=0.07;
  _eC[6]=0.05;
  _eC[7]=0.04;
  _eC[8]=0.03;
  _eC[9]=0.02;
  
  level[0] = 0;
  level[1] = 0.5;
  level[2] = 1.0;
  level[3] = 2.5;
  level[4] = 5.0;
  level[5] = 10.0;
  level[6] = 20.0;
  level[7] = 50.;
  
  colours[0] = 5;
  colours[1] = 7;
  colours[2] = 3;
  colours[3] = 2;
  colours[4] = 6;
  colours[5] = 4;
  colours[6] = 1;
  
  _nViews = nViews;
  if(nViews<1)_nViews=1;
  if(nViews>2)_nViews=2;

  fList = new TList();
  
  int argc = 0;
  char* argv;
  argv = "";

  theApp = new TApplication("bob",&argc,&argv);
  theApp->SetReturnFromRun(kTRUE);
  
  
  if(_nViews==1)fCanvas = new TCanvas("MyRecoDisplay","MyRecoDisplay",750,750);


  fxyview = 0;
  frzview = 0;
  fxzview = 0;
  flongprofile = NULL;
  flongshowerprofile = NULL;
  ftransprofile = NULL;
  ftransprofileFine=NULL;
  // Create the pads;  Pad takes xlo, ylo, xhi, yhi in NC
  
  if(_nViews==1)fxypad  = new TPad("xypad","xy pad  ",0.020,0.020,0.980,0.980);
  if(_nViews==2){
    frzpad  = new TPad("yzpad","rz pad  ",0.020,0.020,0.480,0.480);
  }
  fxypad->SetFillColor(10);
  if(_nViews==2)frzpad->SetFillColor(10);


   // tweak the margins to maximize histogram use of pads
  fxypad->SetLeftMargin(0.05);
  fxypad->SetRightMargin(0.05);
  fxypad->SetTopMargin(0.05);
  fxypad->SetBottomMargin(0.05);
  fxypad->Draw();
    if(_nViews==2){
    frzpad->SetLeftMargin(0.05);
    frzpad->SetRightMargin(0.05);
    frzpad->SetTopMargin(0.05);
    frzpad->SetBottomMargin(0.05);
    frzpad->Draw();
  }


  fCanvas->Draw();
  gSystem->ProcessEvents();


  this->Clear();

}


void MyRecoDisplay::MakeDetectorOutline() {

  float xEin[100];
  float yEin[100];
  float xEout[100];
  float yEout[100];
  float xHin[100];
  float yHin[100];
  float xHout[100];
  float yHout[100];
  float xYin[100];
  float yYin[100];
  float xYout[100];
  float yYout[100];

  float xsq[5] = {-1.0,-1.0,+1.0,+1.0,-1.0};
  float ysq[5] = {-1.0,+1.0,+1.0,-1.0,-1.0};

  float xBox[5];
  float yBox[5];
   

  _2DlinesXY.erase(_2DlinesXY.begin(),_2DlinesXY.end());
  _2DlinesXZ.erase(_2DlinesXZ.begin(),_2DlinesXZ.end());
  _2DcircsXY.erase(_2DcircsXY.begin(),_2DcircsXY.end());
  _2DcircsXZ.erase(_2DcircsXZ.begin(),_2DcircsXZ.end());
  _2DboxesXZ.erase(_2DboxesXZ.begin(),_2DboxesXZ.end());
  _3Dlines.erase(_3Dlines.begin(),_3Dlines.end());

  // unpack calorimeter geometry

  const gear::CalorimeterParameters& pEcalBarrel = Global::GEAR->getEcalBarrelParameters();
  const gear::CalorimeterParameters& pYokeBarrel = Global::GEAR->getYokeBarrelParameters();
  const gear::CalorimeterParameters& pYokeEndcap = Global::GEAR->getYokeEndcapParameters();
  const gear::CalorimeterParameters& pEcalEndcap = Global::GEAR->getEcalEndcapParameters();
  const gear::CalorimeterParameters& pHcalBarrel = Global::GEAR->getHcalBarrelParameters();
  const gear::CalorimeterParameters& pHcalEndcap = Global::GEAR->getHcalEndcapParameters();


  double extentEcalBarrel[4];
  double extentEcalEndcap[4];
  double extentYokeBarrel[4];
  double extentYokePlug[4];
  double extentYokeEndcap[4];
  double extentEcalPlug[4];
  double extentHcalBarrel[4];
  double extentHcalRing[4];
  double extentHcalEndcap[4];
  double extentLcal[4];
  double extentBcal[4];
  double extentLHcal[4];

  for(int i=0;i<4;i++){
    extentEcalBarrel[i] = pEcalBarrel.getExtent()[i];
    extentEcalEndcap[i] = pEcalEndcap.getExtent()[i];
    extentHcalBarrel[i] = pHcalBarrel.getExtent()[i];
    extentHcalEndcap[i] = pHcalEndcap.getExtent()[i];
    extentYokeBarrel[i] = pYokeBarrel.getExtent()[i];
    extentYokeEndcap[i] = pYokeEndcap.getExtent()[i];
  }


  int symmetryYoke = pYokeBarrel.getSymmetryOrder();
  if(symmetryYoke>1){
    float nFoldSymmetry = static_cast<float>(symmetryYoke);
    float phi0 = pYokeBarrel.getPhi0();
    float c = cos(pi/nFoldSymmetry);
    for(int i=0;i<=symmetryYoke;++i){
      float phi  = phi0 + i*twopi/nFoldSymmetry;
      float phiL = phi - pi/nFoldSymmetry;
      xYin[i]  = extentYokeBarrel[0]*cos(phiL)/c;
      yYin[i]  = extentYokeBarrel[0]*sin(phiL)/c;
      xYout[i] = extentYokeBarrel[1]*cos(phiL)/c;
      yYout[i] = extentYokeBarrel[1]*sin(phiL)/c;
    }
    TPolyLine* polyYout = new TPolyLine(symmetryYoke+1,xYout,yYout);
    _2DlinesXY.push_back(polyYout);
    TPolyLine* polyYin = new TPolyLine(symmetryYoke+1,xYin,yYin);
    _2DlinesXY.push_back(polyYin);
  }  

  TBox* bYE1 = new TBox( extentYokeEndcap[2],extentYokeEndcap[0],
			 extentYokeEndcap[3],extentYokeEndcap[1]);
  _2DboxesXZ.push_back(bYE1);
  TBox* bYE2 = new TBox( extentYokeEndcap[2],-extentYokeEndcap[0],
			 extentYokeEndcap[3],-extentYokeEndcap[1]);
  _2DboxesXZ.push_back(bYE2);
  TBox* bYE3 = new TBox( -extentYokeEndcap[2],extentYokeEndcap[0],
			 -extentYokeEndcap[3],extentYokeEndcap[1]);
  _2DboxesXZ.push_back(bYE3);
  TBox* bYE4 = new TBox( -extentYokeEndcap[2],-extentYokeEndcap[0],
			 -extentYokeEndcap[3],-extentYokeEndcap[1]);
  _2DboxesXZ.push_back(bYE4);
   




  try {
    const gear::CalorimeterParameters& pEcalPlug   = Global::GEAR->getEcalPlugParameters();
    for(int i=0;i<4;i++)extentEcalPlug[i]   = pEcalPlug.getExtent()[i];
  } catch(gear::UnknownParameterException &e){
    for(int i=0;i<4;i++)extentEcalPlug[i]   = 0.;
  }

  try {
    const gear::CalorimeterParameters& pYokePlug   = Global::GEAR->getYokePlugParameters();
    double pt  = pYokePlug.getDoubleVal("YokePlugThickness");

    for(int i=0;i<4;i++)extentYokePlug[i]   = pYokePlug.getExtent()[i];
    extentYokePlug[3]+= pt;
    
  } catch(gear::UnknownParameterException &e){
    for(int i=0;i<4;i++)extentYokePlug[i]   = 0.;
  }



  try {
    const gear::CalorimeterParameters& pLcal = Global::GEAR->getLcalParameters();
    for(int i=0;i<4;i++)extentLcal[i]       = pLcal.getExtent()[i];
  } catch(gear::UnknownParameterException &e){
    for(int i=0;i<4;i++)extentLcal[i]       = 0.;
  }



  try {
    const gear::CalorimeterParameters& pBcal = Global::GEAR->getBeamCalParameters();
    for(int i=0;i<4;i++)extentBcal[i]       = pBcal.getExtent()[i];
  } catch(gear::UnknownParameterException &e){
    for(int i=0;i<4;i++)extentBcal[i]       = 0.;
  }

  try {
    const gear::CalorimeterParameters& pLHcal = Global::GEAR->getLHcalParameters();
    for(int i=0;i<4;i++)extentLHcal[i]       = pLHcal.getExtent()[i];
  } catch(gear::UnknownParameterException &e){
    for(int i=0;i<4;i++)extentLHcal[i]       = 0.;
  }

  try {
    const gear::CalorimeterParameters& pHcalRing    = Global::GEAR->getHcalRingParameters();
    for(int i=0;i<4;i++)extentHcalRing[i]   = pHcalRing.getExtent()[i];
  } catch(gear::UnknownParameterException &e){
    for(int i=0;i<4;i++)extentHcalRing[i]       = 0.;
  }


  int symmetryEcal = pEcalBarrel.getSymmetryOrder();
  float rHcal = 0;
  int symmetryHcal=0;
  // set symmetry of HCALBarrel to be the polygon symmetry of outer edge
  try {
    symmetryHcal = pHcalBarrel.getIntVal("Hcal_outer_polygon_order");
  }
  catch(gear::UnknownParameterException &e){
     symmetryHcal = pHcalBarrel.getSymmetryOrder();
    if(symmetryHcal*2>symmetryEcal){
      symmetryHcal = symmetryHcal*2;
    }
  }
  try {
    rHcal = pHcalBarrel.getDoubleVal("Hcal_outer_radius");
    symmetryHcal = 0;
  }
  catch(gear::UnknownParameterException &e){
  }


 
  // determine polygon angles

  if(symmetryEcal>1){
    float nFoldSymmetry = static_cast<float>(symmetryEcal);
    float phi0 = pEcalBarrel.getPhi0();
    float c = cos(pi/nFoldSymmetry);
    for(int i=0;i<=symmetryEcal;++i){
      float phi  = phi0 + i*twopi/nFoldSymmetry;
      float phiL = phi - pi/nFoldSymmetry;
      //      float phiH = phi + pi/nFoldSymmetry;
      xEin[i]  = extentEcalBarrel[0]*cos(phiL)/c;
      yEin[i]  = extentEcalBarrel[0]*sin(phiL)/c;
      xEout[i] = extentEcalBarrel[1]*cos(phiL)/c;
      yEout[i] = extentEcalBarrel[1]*sin(phiL)/c;
      xHin[i]  = extentHcalBarrel[0]*cos(phiL)/c;
      yHin[i]  = extentHcalBarrel[0]*sin(phiL)/c;
    }

    // ecal barrel
    TBox* bB1 = new TBox(-extentEcalBarrel[3],extentEcalBarrel[0],
			  extentEcalBarrel[3],extentEcalBarrel[1]);
    _2DboxesXZ.push_back(bB1);

    TBox* bB2 = new TBox(-extentEcalBarrel[3],-extentEcalBarrel[0],
			  extentEcalBarrel[3],-extentEcalBarrel[1]);
    _2DboxesXZ.push_back(bB2);

    TBox* bHB1 = new TBox(-extentHcalBarrel[3],extentHcalBarrel[0],
			  extentHcalBarrel[3],extentHcalBarrel[1]);
    _2DboxesXZ.push_back(bHB1);

    TBox* bHB2 = new TBox(-extentHcalBarrel[3],-extentHcalBarrel[0],
			  extentHcalBarrel[3],-extentHcalBarrel[1]);
    _2DboxesXZ.push_back(bHB2);



    // yoke barrel
    TBox* bYB1 = new TBox(-extentYokeBarrel[3],extentYokeBarrel[0],
			  extentYokeBarrel[3],extentYokeBarrel[1]);
    _2DboxesXZ.push_back(bYB1);

    TBox* bYB2 = new TBox(-extentYokeBarrel[3],-extentYokeBarrel[0],
			  extentYokeBarrel[3],-extentYokeBarrel[1]);
    _2DboxesXZ.push_back(bYB2);


     TBox* bE1 = new TBox( extentEcalEndcap[2],extentEcalEndcap[0],
			  extentEcalEndcap[3],extentEcalEndcap[1]);
    _2DboxesXZ.push_back(bE1);
    TBox* bE2 = new TBox( extentEcalEndcap[2],-extentEcalEndcap[0],
			  extentEcalEndcap[3],-extentEcalEndcap[1]);
    _2DboxesXZ.push_back(bE2);
    TBox* bE3 = new TBox( -extentEcalEndcap[2],extentEcalEndcap[0],
			  -extentEcalEndcap[3],extentEcalEndcap[1]);
    _2DboxesXZ.push_back(bE3);
    TBox* bE4 = new TBox( -extentEcalEndcap[2],-extentEcalEndcap[0],
			  -extentEcalEndcap[3],-extentEcalEndcap[1]);
    _2DboxesXZ.push_back(bE4);


     TBox* bP1 = new TBox( extentEcalPlug[2],extentEcalPlug[0],
			   extentEcalPlug[3],extentEcalPlug[1]);
    _2DboxesXZ.push_back(bP1);
     TBox* bP2 = new TBox( extentEcalPlug[2],-extentEcalPlug[0],
			  extentEcalPlug[3],-extentEcalPlug[1]);
    _2DboxesXZ.push_back(bP2);
     TBox* bP3 = new TBox( -extentEcalPlug[2],extentEcalPlug[0],
			  -extentEcalPlug[3],extentEcalPlug[1]);
    _2DboxesXZ.push_back(bP3);
     TBox* bP4 = new TBox( -extentEcalPlug[2],-extentEcalPlug[0],
			  -extentEcalPlug[3],-extentEcalPlug[1]);
    _2DboxesXZ.push_back(bP4);



     TBox* bYP1 = new TBox( extentYokePlug[2],extentYokePlug[0],
			   extentYokePlug[3],extentYokePlug[1]);
    _2DboxesXZ.push_back(bYP1);
     TBox* bYP2 = new TBox( extentYokePlug[2],-extentYokePlug[0],
			  extentYokePlug[3],-extentYokePlug[1]);
    _2DboxesXZ.push_back(bYP2);
     TBox* bYP3 = new TBox( -extentYokePlug[2],extentYokePlug[0],
			  -extentYokePlug[3],extentYokePlug[1]);
    _2DboxesXZ.push_back(bYP3);
     TBox* bYP4 = new TBox( -extentYokePlug[2],-extentYokePlug[0],
			  -extentYokePlug[3],-extentYokePlug[1]);
    _2DboxesXZ.push_back(bYP4);




     TBox* bLH1 = new TBox( extentLHcal[2],extentLHcal[0],
			   extentLHcal[3],extentLHcal[1]);
    _2DboxesXZ.push_back(bLH1);
     TBox* bLH2 = new TBox( extentLHcal[2],-extentLHcal[0],
			  extentLHcal[3],-extentLHcal[1]);
    _2DboxesXZ.push_back(bLH2);
     TBox* bLH3 = new TBox( -extentLHcal[2],extentLHcal[0],
			  -extentLHcal[3],extentLHcal[1]);
    _2DboxesXZ.push_back(bLH3);
     TBox* bLH4 = new TBox( -extentLHcal[2],-extentLHcal[0],
			  -extentLHcal[3],-extentLHcal[1]);
    _2DboxesXZ.push_back(bLH4);



     TBox* bHE1 = new TBox( extentHcalEndcap[2],extentHcalEndcap[0],
			  extentHcalEndcap[3],extentHcalEndcap[1]);
    _2DboxesXZ.push_back(bHE1);
    TBox* bHE2 = new TBox( extentHcalEndcap[2],-extentHcalEndcap[0],
			  extentHcalEndcap[3],-extentHcalEndcap[1]);
    _2DboxesXZ.push_back(bHE2);
    TBox* bHE3 = new TBox( -extentHcalEndcap[2],extentHcalEndcap[0],
			  -extentHcalEndcap[3],extentHcalEndcap[1]);
    _2DboxesXZ.push_back(bHE3);
    TBox* bHE4 = new TBox( -extentHcalEndcap[2],-extentHcalEndcap[0],
			   -extentHcalEndcap[3],-extentHcalEndcap[1]);
    _2DboxesXZ.push_back(bHE4);
   



     TBox* bHR1 = new TBox( extentHcalRing[2],extentHcalRing[0],
			  extentHcalRing[3],extentHcalRing[1]);
    _2DboxesXZ.push_back(bHR1);
    TBox* bHR2 = new TBox( extentHcalRing[2],-extentHcalRing[0],
			  extentHcalRing[3],-extentHcalRing[1]);
    _2DboxesXZ.push_back(bHR2);
    TBox* bHR3 = new TBox( -extentHcalRing[2],extentHcalRing[0],
			  -extentHcalRing[3],extentHcalRing[1]);
    _2DboxesXZ.push_back(bHR3);
    TBox* bHR4 = new TBox( -extentHcalRing[2],-extentHcalRing[0],
			   -extentHcalRing[3],-extentHcalRing[1]);
    _2DboxesXZ.push_back(bHR4);
   

    // ecal endcap inner box
    for(int i=0;i<5;++i)xBox[i] = xsq[i]*extentEcalEndcap[0];
    for(int i=0;i<5;++i)yBox[i] = ysq[i]*extentEcalEndcap[0];
    TPolyLine* polySq = new TPolyLine(5,xBox,yBox);
    _2DlinesXY.push_back(polySq);

    TPolyLine* polyEin = new TPolyLine(symmetryEcal+1,xEin,yEin);
    _2DlinesXY.push_back(polyEin);
    TPolyLine* polyEout = new TPolyLine(symmetryEcal+1,xEout,yEout);
    _2DlinesXY.push_back(polyEout);

    TPolyLine* polyHin = new TPolyLine(symmetryEcal+1,xHin,yHin);
    _2DlinesXY.push_back(polyHin);
    //    TPolyLine* polyHout = new TPolyLine(symmetryEcal,xHout,yHout);
    //_2DlinesXY.push_back(polyHout);
  }  


  if(symmetryHcal>1){
    float nFoldSymmetry = static_cast<float>(symmetryHcal);
    float phi0 = pHcalBarrel.getPhi0();
    float c = cos(pi/nFoldSymmetry);
    for(int i=0;i<=symmetryHcal;++i){
      float phi  = phi0 + i*twopi/nFoldSymmetry;
      float phiL = phi - pi/nFoldSymmetry;
      xHout[i] = extentHcalBarrel[1]*cos(phiL)/c;
      yHout[i] = extentHcalBarrel[1]*sin(phiL)/c;
    }
    TPolyLine* polyHout = new TPolyLine(symmetryHcal+1,xHout,yHout);
    _2DlinesXY.push_back(polyHout);
  }else{
    TArc* arcRH = new TArc(0.,0.,rHcal);
    _2DcircsXY.push_back(arcRH);
  }  

  if(symmetryEcal==0){
    TArc* arcEin = new TArc(0.,0.,extentEcalBarrel[0]);
    _2DcircsXY.push_back(arcEin);
    TArc* arcEout = new TArc(0.,0.,extentEcalBarrel[1]);
    _2DcircsXY.push_back(arcEout);
    TArc* arcHin = new TArc(0.,0.,extentHcalBarrel[0]);
    _2DcircsXY.push_back(arcHin);
    TArc* arcHout = new TArc(0.,0.,extentHcalBarrel[1]);
    _2DcircsXY.push_back(arcHout);
  }  

  const gear::TPCParameters& pTPC = Global::GEAR->getTPCParameters();
  const gear::PadRowLayout2D& padLayout = pTPC.getPadLayout() ;
  float gearRMin = padLayout.getPlaneExtent()[0];
  float gearRMax = padLayout.getPlaneExtent()[1];
  TArc* arcTin = new TArc(0.,0.,gearRMin);
  _2DcircsXY.push_back(arcTin);
  TArc* arcTout = new TArc(0.,0.,gearRMax);
  _2DcircsXY.push_back(arcTout);

  TBox* bT1 = new TBox( pTPC.getMaxDriftLength(),padLayout.getPlaneExtent()[0],
		       -pTPC.getMaxDriftLength(),padLayout.getPlaneExtent()[1]);
  _2DboxesXZ.push_back(bT1);
  TBox* bT2 = new TBox( pTPC.getMaxDriftLength(),-padLayout.getPlaneExtent()[0],
		       -pTPC.getMaxDriftLength(),-padLayout.getPlaneExtent()[1]);
  _2DboxesXZ.push_back(bT2);


  TArc* arcLin = new TArc(0.,0.,extentLcal[0]);
  _2DcircsXY.push_back(arcLin);
  TArc* arcLout = new TArc(0.,0.,extentLcal[1]);
  _2DcircsXY.push_back(arcLout);


  TBox* bL1 = new TBox(  extentLcal[2],extentLcal[0],
			  extentLcal[3],extentLcal[1]);
  _2DboxesXZ.push_back(bL1);
  TBox* bL2 = new TBox( extentLcal[2],-extentLcal[0],
			extentLcal[3],-extentLcal[1]);
  _2DboxesXZ.push_back(bL2);
  TBox* bL3 = new TBox( -extentLcal[2],extentLcal[0],
			-extentLcal[3],extentLcal[1]);
  _2DboxesXZ.push_back(bL3);
  TBox* bL4 = new TBox( -extentLcal[2],-extentLcal[0],
			-extentLcal[3],-extentLcal[1]);
  _2DboxesXZ.push_back(bL4);


  TBox* bB1 = new TBox(  extentBcal[2],extentBcal[0],
			 extentBcal[3],extentBcal[1]);
  _2DboxesXZ.push_back(bB1);
  TBox* bB2 = new TBox( extentBcal[2],-extentBcal[0],
			extentBcal[3],-extentBcal[1]);
  _2DboxesXZ.push_back(bB2);
  TBox* bB3 = new TBox( -extentBcal[2],extentBcal[0],
			-extentBcal[3],extentBcal[1]);
  _2DboxesXZ.push_back(bB3);
  TBox* bB4 = new TBox( -extentBcal[2],-extentBcal[0],
			-extentBcal[3],-extentBcal[1]);
  _2DboxesXZ.push_back(bB4);


  const gear::GearParameters& cp = Global::GEAR->getGearParameters("CoilParameters") ;
  //  GearParameters* cp =  grMgr->getGearParameters("CoilParameters") ;
  double cz   = cp.getDoubleVal("Coil_cryostat_half_z");
  double cin  = cp.getDoubleVal("Coil_cryostat_inner_radius");
  double cout = cp.getDoubleVal("Coil_cryostat_outer_radius");
  TArc* arccin  = new TArc(0.,0.,cin);
  TArc* arccout = new TArc(0.,0.,cout);
  _2DcircsXY.push_back(arccin);
  _2DcircsXY.push_back(arccout);
  TBox* coil1 = new TBox(  -cz,+cin,+cz,+cout);
  _2DboxesXZ.push_back(coil1);
  TBox* coil2 = new TBox(  -cz,-cin,+cz,-cout);
  _2DboxesXZ.push_back(coil2);


    
}


void MyRecoDisplay::SetTrackCollection(LCCollection* trackCollection) {
  _trackCollection = trackCollection;  
}


void MyRecoDisplay::SetTrackAR(std::vector<MyTrackExtended*> tracksAR){

  for(unsigned int i=0;i<tracksAR.size();i++){
    _extendedTracks.push_back(tracksAR[i]);
  }

}


void MyRecoDisplay::SetMuon(std::vector<CalorimeterHit*> muonHits){

  for(unsigned int i=0;i<muonHits.size();i++){
    _muonHits.push_back(muonHits[i]);
  }

}


void MyRecoDisplay::SetUnusedTPCHits(std::vector<TrackerHit*> hits){
  for(unsigned int i=0;i<hits.size();i++){
    _unusedTPCHits.push_back(hits[i]);
  }
}

void MyRecoDisplay::SetCurlKilledTPCHits(std::vector<TrackerHit*> hits){
  for(unsigned int i=0;i<hits.size();i++){
    _curlKilledTPCHits.push_back(hits[i]);
  }
}



void MyRecoDisplay::DrawTracks(RecoDisplayView view) {

  float x[50000];
  float y[50000];
  float z[50000];
  float r[50000];
  
  unsigned int nelem = _extendedTracks.size();

  for (unsigned int j=0; j < nelem; ++j) {
    float ex = _extendedTracks[j]->getSeedPosition()[0];
    float ey = _extendedTracks[j]->getSeedPosition()[1];
    float ez = _extendedTracks[j]->getSeedPosition()[2];
    float er = sqrt(ex*ex+ey*ey);

    float aex = _extendedTracks[j]->getAltSeedPosition()[0];
    float aey = _extendedTracks[j]->getAltSeedPosition()[1];
    float aez = _extendedTracks[j]->getAltSeedPosition()[2];
    float aer = sqrt(ex*ex+ey*ey);

    float tex = _extendedTracks[j]->getTpcIntersection()[0];
    float tey = _extendedTracks[j]->getTpcIntersection()[1];
    float tez = _extendedTracks[j]->getTpcIntersection()[2];
    float ter = sqrt(ex*ex+ey*ey);

    Track* track = _extendedTracks[j]->getTrack();
    float d0 = track->getD0();
    float z0 = track->getZ0();
    TrackerHitVec hitvec = track->getTrackerHits();
    int nhits = (int)hitvec.size();
    for(int i=0;i<nhits;++i){
      TrackerHit* trkHit = hitvec[i];
      x[i] = (float)trkHit->getPosition()[0];
      y[i] = (float)trkHit->getPosition()[1];
      z[i] = (float)trkHit->getPosition()[2];
      r[i] = x[i]*_thrustX + y[i]*_thrustY;
    }
    TPolyMarker* pPm=NULL;
    TPolyLine* pPl=NULL;
    if(view==RECO_VIEW_XY)pPm = new TPolyMarker(nhits,x,y);
    if(view==RECO_VIEW_XZ)pPm = new TPolyMarker(nhits,z,x);
    if(view==RECO_VIEW_YZ)pPm = new TPolyMarker(nhits,z,y);
    if(view==RECO_VIEW_RZ)pPm = new TPolyMarker(nhits,z,r);
    if(view==RECO_VIEW_XY)pPl = new TPolyLine(nhits,x,y);
    if(view==RECO_VIEW_XY)pPl = new TPolyLine(nhits,x,y);
    if(view==RECO_VIEW_XZ)pPl = new TPolyLine(nhits,z,x);
    if(view==RECO_VIEW_XY)pPl = new TPolyLine(nhits,x,y);
    if(view==RECO_VIEW_RZ)pPl = new TPolyLine(nhits,z,r);

    float E = _extendedTracks[j]->getEnergy();
    int ilevel =0;
    for(int i=0;i<nlevels;++i){
      if(E>level[i])ilevel=i;
    }
    int colour = colours[ilevel];

    colour = 1;
    if(_extendedTracks[j]->trackStatus()==TRACK_STATUS_GOOD || _extendedTracks[j]->trackStatus()==TRACK_STATUS_LOWPT){
      colour =11;
    }
    if(_extendedTracks[j]->trackStatus()==TRACK_STATUS_NON_VERTEX ){
      ProtoClusterVec clusters = _extendedTracks[j]->getClusterVec();
      if(clusters.size()>0){
	colour =3;
      }
    }
    if(_extendedTracks[j]->isProngE() || _extendedTracks[j]->isProngS()){
      colour =2;
    }
    if(_extendedTracks[j]->isV0() ){
      colour =6;
    }
    if(_extendedTracks[j]->isKinkE() ||_extendedTracks[j]->isKinkS() ){
      colour =2;
    }   
    if(_extendedTracks[j]->isBackScatter()){
      colour =4;
    }   

    //        colour = j+1;


    pPm->SetMarkerStyle(20);
    pPm->SetMarkerColor(colour);
    pPm->SetMarkerSize(0.3);
    pPm->Draw();

    // pPl->SetLineWidth(2);
    // pPl->SetLineColor(colour);
    //pPl->Draw();


    
    TPolyMarker* extrap=NULL;

    if(view==RECO_VIEW_XY)extrap = new TPolyMarker(1,&ex,&ey);
    if(view==RECO_VIEW_XZ)extrap = new TPolyMarker(1,&ez,&ex);
    if(view==RECO_VIEW_YZ)extrap = new TPolyMarker(1,&ez,&ey);
    if(view==RECO_VIEW_RZ)extrap = new TPolyMarker(1,&ez,&er);
    if(extrap!=NULL){
      extrap->SetMarkerStyle(24);
      extrap->SetMarkerColor(colour);
      extrap->SetMarkerSize(0.8);
      extrap->Draw();
    }


    TPolyMarker* aextrap=NULL;

    if(view==RECO_VIEW_XY)aextrap = new TPolyMarker(1,&aex,&aey);
    if(view==RECO_VIEW_XZ)aextrap = new TPolyMarker(1,&aez,&aex);
    if(view==RECO_VIEW_YZ)aextrap = new TPolyMarker(1,&aez,&aey);
    if(view==RECO_VIEW_RZ)aextrap = new TPolyMarker(1,&aez,&aer);
    if(aextrap!=NULL){
      aextrap->SetMarkerStyle(28);
      aextrap->SetMarkerColor(colour);
      aextrap->SetMarkerSize(1.0);
      if(_extendedTracks[j]->getEnergy()>100)aextrap->SetMarkerColor(4);
      aextrap->Draw();
    }

    TPolyMarker* textrap=NULL;

    if(view==RECO_VIEW_XY)textrap = new TPolyMarker(1,&tex,&tey);
    if(view==RECO_VIEW_XZ)textrap= new TPolyMarker(1,&tez,&tex);
    if(view==RECO_VIEW_YZ)textrap = new TPolyMarker(1,&tez,&tey);
    if(view==RECO_VIEW_RZ)textrap = new TPolyMarker(1,&tez,&ter);
    if(textrap!=NULL){
      textrap->SetMarkerStyle(24);
      textrap->SetMarkerColor(colour);
      textrap->SetMarkerSize(0.8);
      if(_extendedTracks[j]->getEnergy()>100)textrap->SetMarkerColor(4);
      textrap->Draw();
    }


    float x2 = ex + 100.*_extendedTracks[j]->getSeedDirection()[0];
    float y2 = ey + 100.*_extendedTracks[j]->getSeedDirection()[1];
    float z2 = ez + 100.*_extendedTracks[j]->getSeedDirection()[2];
    float r2 = sqrt(x2*x2+y2*y2);
    TArrow* arrow=NULL;
    if((fabs(d0)<50.0&&fabs(z0)<50.0)||_extendedTracks[j]->isV0()||_extendedTracks[j]->isKinkE()){
      if(view==RECO_VIEW_XY)arrow= new TArrow(ex,ey,x2,y2,0.03);
      if(view==RECO_VIEW_XZ)arrow= new TArrow(ez,ex,z2,x2,0.03);
      if(view==RECO_VIEW_YZ)arrow= new TArrow(ez,ey,z2,y2,0.03);
      if(view==RECO_VIEW_RZ)arrow= new TArrow(ez,er,z2,r2,0.03);
    }
    if(arrow!=NULL){
      arrow->SetLineColor(colour);
      if(_extendedTracks[j]->getEnergy()>100)arrow->SetLineColor(4);
      arrow->Draw();
    }



    float ax2 = aex + 100.*_extendedTracks[j]->getAltSeedDirection()[0];
    float ay2 = aey + 100.*_extendedTracks[j]->getAltSeedDirection()[1];
    float az2 = aez + 100.*_extendedTracks[j]->getAltSeedDirection()[2];
    float ar2 = sqrt(ax2*ax2+ay2*ay2);
    TArrow* aarrow=NULL;
    if((fabs(d0)<50.0&&fabs(z0)<50.0)||_extendedTracks[j]->isV0()||_extendedTracks[j]->isKinkE()){
      if(view==RECO_VIEW_XY)aarrow= new TArrow(aex,aey,ax2,ay2,0.03);
      if(view==RECO_VIEW_XZ)aarrow= new TArrow(aez,aex,az2,ax2,0.03);
      if(view==RECO_VIEW_YZ)aarrow= new TArrow(aez,aey,az2,ay2,0.03);
      if(view==RECO_VIEW_RZ)aarrow= new TArrow(aez,aer,az2,ar2,0.03);
    }
    if(aarrow!=NULL){
      aarrow->SetLineColor(colour);
      if(_extendedTracks[j]->getEnergy()>100)aarrow->SetLineColor(4);
      aarrow->SetLineStyle(2);
      aarrow->Draw();
    }
  }
  this->DrawCurlKilledHits(view);
  this->DrawUnusedHits(view);
  
}












void MyRecoDisplay::DrawMuonHits(RecoDisplayView view) {

  float x[50000];
  float y[50000];
  float z[50000];
  float r[50000];
  
  unsigned int nhits = _muonHits.size();

  for(unsigned int i=0;i<nhits;++i){
    x[i] = _muonHits[i]->getPosition()[0];
    y[i] = _muonHits[i]->getPosition()[1];
    z[i] = _muonHits[i]->getPosition()[2];
    r[i] = sqrt(x[i]*x[i]+y[i]*y[i]);
  }
  TPolyMarker* pPm=NULL;
  TPolyLine* pPl=NULL;
  if(view==RECO_VIEW_XY)pPm = new TPolyMarker(nhits,x,y);
  if(view==RECO_VIEW_XZ)pPm = new TPolyMarker(nhits,z,x);
  if(view==RECO_VIEW_YZ)pPm = new TPolyMarker(nhits,z,y);
  if(view==RECO_VIEW_RZ)pPm = new TPolyMarker(nhits,z,r);
  if(view==RECO_VIEW_XY)pPl = new TPolyLine(nhits,x,y);
  if(view==RECO_VIEW_XY)pPl = new TPolyLine(nhits,x,y);
  if(view==RECO_VIEW_XZ)pPl = new TPolyLine(nhits,z,x);
  if(view==RECO_VIEW_XY)pPl = new TPolyLine(nhits,x,y);
  if(view==RECO_VIEW_RZ)pPl = new TPolyLine(nhits,z,r);
  int colour = 1;
  pPm->SetMarkerStyle(23);
  pPm->SetMarkerColor(colour);
  pPm->SetMarkerSize(0.6);
  pPm->Draw();
  
}

void MyRecoDisplay::DrawUnusedHits(RecoDisplayView view) {

  float x[50000];
  float y[50000];
  float z[50000];
  float r[50000];
  
  unsigned int nhits = _unusedTPCHits.size();
  for (unsigned int j=0; j <nhits ; ++j) {
    TrackerHit* trkHit = _unusedTPCHits[j];
    x[j] = (float)trkHit->getPosition()[0];
    y[j] = (float)trkHit->getPosition()[1];
    z[j] = (float)trkHit->getPosition()[2];
    r[j] = sqrt(x[j]*x[j]+y[j]*y[j]);
  }
  TPolyMarker* pPm=NULL;
  if(view==RECO_VIEW_XY)pPm = new TPolyMarker(nhits,x,y);
  if(view==RECO_VIEW_XZ)pPm = new TPolyMarker(nhits,z,x);
  if(view==RECO_VIEW_YZ)pPm = new TPolyMarker(nhits,z,y);
  if(view==RECO_VIEW_RZ)pPm = new TPolyMarker(nhits,z,r);
  
  int colour = 9;

  pPm->SetMarkerStyle(20);
  pPm->SetMarkerColor(colour);
  pPm->SetMarkerSize(0.3);
  pPm->Draw();

}


void MyRecoDisplay::DrawCurlKilledHits(RecoDisplayView view) {

  float x[50000];
  float y[50000];
  float z[50000];
  float r[50000];
  
  unsigned int nhits = _curlKilledTPCHits.size();
  for (unsigned int j=0; j <nhits ; ++j) {
    TrackerHit* trkHit = _curlKilledTPCHits[j];
    x[j] = (float)trkHit->getPosition()[0];
    y[j] = (float)trkHit->getPosition()[1];
    z[j] = (float)trkHit->getPosition()[2];
    r[j] = sqrt(x[j]*x[j]+y[j]*y[j]);
  }
  TPolyMarker* pPm=NULL;
  if(view==RECO_VIEW_XY)pPm = new TPolyMarker(nhits,x,y);
  if(view==RECO_VIEW_XZ)pPm = new TPolyMarker(nhits,z,x);
  if(view==RECO_VIEW_YZ)pPm = new TPolyMarker(nhits,z,y);
  if(view==RECO_VIEW_RZ)pPm = new TPolyMarker(nhits,z,r);
  
  int colour = 8;

  pPm->SetMarkerStyle(20);
  pPm->SetMarkerColor(colour);
  pPm->SetMarkerSize(0.3);
  pPm->Draw();

}



void MyRecoDisplay::CleanUp() {

  _extendedTracks.erase(_extendedTracks.begin(),_extendedTracks.end());
  _muonHits.erase(_muonHits.begin(),_muonHits.end());
  _unusedTPCHits.clear();
  _curlKilledTPCHits.clear();
}

void MyRecoDisplay::Clear() {

  _setThrust = false;
  // clear the primitives from the pads
  fxypad->Clear("");
  if(_nViews==2)frzpad->Clear("");

   // remove old histograms if they exists
  if ( fxyview ) { delete fxyview;  fxyview=0; }
  if(_nViews==2){ delete frzview;  frzview=0; }
  
  // create histograms that that will use World Coordinates
  
  fxyview  = new TH2F("xyview","xy view",500,-_plotRange,_plotRange,500,-_plotRange,_plotRange);
  fxzview  = new TH2F("xzview","xz view",500,-_plotRange,_plotRange,500,-_plotRange,_plotRange);
  if(_nViews==2)frzview  = new TH2F("yzview","yz view",500,-_plotRange,_plotRange,500,-_plotRange,_plotRange);

  if(flongprofile!=NULL){delete flongprofile; flongprofile=0;}
  flongprofile  = new TH1F("longprofile","longitudinal profile",100,0.,100.);
  flongprofile->SetDirectory(0);

  if(flongshowerprofile!=NULL){delete flongshowerprofile; flongshowerprofile=0;}
  flongshowerprofile  = new TH1F("longshowerprofile","longitudinal shower profile",100,0.,100.);
  flongshowerprofile->SetDirectory(0);


  if(ftransprofile!=NULL){delete ftransprofile; ftransprofile=0;}
  ftransprofile  = new TH2F("transprofile","transverse profile",41,-20.5,20.5,41,-20.5,20.5);
  ftransprofile->SetDirectory(0);

  if(ftransprofileFine!=NULL){delete ftransprofileFine; ftransprofileFine=0;}
  ftransprofileFine  = new TH2F("transprofileFine","fine transverse profile",1001,-500.5,500.5,1001,-500.5,500.5);
  ftransprofileFine->SetDirectory(0);

  // don't draw the Stats box
  fxyview->SetStats(kFALSE);
  fxzview->SetStats(kFALSE);
  if(_nViews==2)frzview->SetStats(kFALSE);

  // take away any association with a "directory" for these histograms
  fxyview->SetDirectory(0);
  fxzview->SetDirectory(0);
  if(_nViews==2)frzview->SetDirectory(0);

}


void MyRecoDisplay::Draw() {

  fCanvas->Update();
  gSystem->ProcessEvents();

  return;

}


void MyRecoDisplay::DrawByLayer(std::vector<MyCaloHitExtended*>hitsByLayer[]){

  vector <MyCaloHitExtended*>::iterator literA;
  float x[10000];
  float y[10000];
  float z[10000];
  float r[10000];
  int   n;

  fxypad->cd();
  fxyview->Draw();


  for(int i=0; i<100; i++){
    n = 0;
    if(hitsByLayer[i].size()>0){
      literA = (hitsByLayer[i]).begin();
      while (literA != hitsByLayer[i].end()){
	x[n] = (*literA)->getPosition()[0];
	y[n] = (*literA)->getPosition()[1];
	z[n] = (*literA)->getPosition()[2];
	n++;
	literA++;
      }
      
    }
 
    TPolyMarker* pPm = new TPolyMarker(n,x,y);
    int colour = i-(i/7)*7 + 1;
    pPm->SetMarkerStyle(20);
    pPm->SetMarkerColor(colour);
    pPm->SetMarkerSize(0.5);
    //   fList->Add(pPm);
    pPm->Draw();
  }
  DrawOutline(RECO_VIEW_XY);
  

  if(_nViews==2){
    frzpad->cd();
    frzview->Draw();
    for(int i=0; i<100; i++){
      n = 0;
      if(hitsByLayer[i].size()>0){
	literA = (hitsByLayer[i]).begin();
	while (literA != hitsByLayer[i].end()){
	  x[n] = (*literA)->getPosition()[0];
	  y[n] = (*literA)->getPosition()[1];
	  z[n] = (*literA)->getPosition()[2];
	  r[n] = sqrt(x[n]*x[n] + y[n]*y[n]);
	  if(y[n]<0)r[n]=-r[n];
	  n++;
	  literA++;
	}
      }
      TPolyMarker* pPm = new TPolyMarker(n,z,r);
      int colour = i-(i/8)*8;
      pPm->SetMarkerStyle(20);
      pPm->SetMarkerColor(colour);
      pPm->SetMarkerSize(0.5);
      // fList->Add(pPm);
      pPm->Draw();
    }
  }

  fCanvas->Update();
  gSystem->ProcessEvents();

 
  theApp->SetReturnFromRun(kTRUE);

  fcntl(STDIN_FILENO, F_SETFL, O_NONBLOCK);
  int key = 0;
  while(1){
    cout <<std::flush;
    //    theApp->StartIdleing();
    gSystem->ProcessEvents();
    //theApp->StopIdleing();
    key = getchar();
    if(key=='\n') break;
     usleep(1000);
  }

  return;

}

void MyRecoDisplay::DrawOutline(RecoDisplayView view){
  MakeDetectorOutline();
  switch(view){
  case RECO_VIEW_XY:
    for(unsigned int i=0;i<_2DlinesXY.size();++i){
      _2DlinesXY[i]->SetLineWidth(2);
      _2DlinesXY[i]->Draw();
    }
    for(unsigned int i=0;i<_2DcircsXY.size();++i){
      _2DcircsXY[i]->SetLineWidth(2);
      _2DcircsXY[i]->Draw();
    }
    break;
  case RECO_VIEW_XZ:
  case RECO_VIEW_RZ:
    for(unsigned int i=0;i<_2DlinesXZ.size();++i){
      _2DlinesXZ[i]->SetLineWidth(2);
      _2DlinesXZ[i]->Draw();
    }
    for(unsigned int i=0;i<_2DboxesXZ.size();++i){
      _2DboxesXZ[i]->SetLineWidth(2);
      _2DboxesXZ[i]->SetLineColor(1);
      _2DboxesXZ[i]->SetFillStyle(0);
      _2DboxesXZ[i]->Draw();
    }
    break;
  default:
    break;
  }

  return;

}



void MyRecoDisplay::DrawByDensity(std::vector<MyCaloHitExtended*>hitsByLayer[]){

  vector <MyCaloHitExtended*>::iterator literA;
  float x[10000];
  float y[10000];
  float z[10000];
  //  float r[10000];
  int   n;

  fxypad->cd();
  fxyview->Draw();


  for(int ilevel=0;ilevel<nlevels;ilevel++){
    for(int i=0; i<100; i++){
      n = 0;
      if(hitsByLayer[i].size()>0){
	literA = (hitsByLayer[i]).begin();
	while (literA != hitsByLayer[i].end()){
	  float density = (*literA)->getDensityWeight();
	  if( density>=level[ilevel]&&density<level[ilevel+1]){
	    x[n] = (*literA)->getPosition()[0];
	    y[n] = (*literA)->getPosition()[1];
	    z[n] = (*literA)->getPosition()[2];
	    n++;
	  }
	  literA++;
	}
      }
      TPolyMarker* pPm = new TPolyMarker(n,x,y);
      int colour = colours[ilevel];
      pPm->SetMarkerStyle(20);
      pPm->SetMarkerColor(colour);
      pPm->SetMarkerSize(0.5);
      //   fList->Add(pPm);
      pPm->Draw();
    }
  }

  this->DrawTracks(RECO_VIEW_XY);
  this->DrawMuonHits(RECO_VIEW_XY);
  this->DrawOutline(RECO_VIEW_XY);

  fCanvas->Update();
  gSystem->ProcessEvents();

 
  theApp->SetReturnFromRun(kTRUE);

  fcntl(STDIN_FILENO, F_SETFL, O_NONBLOCK);
  int key = 0;
  while(1){
    cout <<std::flush;
    //    theApp->StartIdleing();
    gSystem->ProcessEvents();
    //theApp->StopIdleing();
    key = getchar();
    if(key=='\n') break;
     usleep(1000);
  }

  return;

}

void MyRecoDisplay::DrawByProtoCluster(std::vector<ProtoCluster*> &pClusters, int minHits){

  float x[10000];
  float y[10000];
  float z[10000];
  int   n;

  fxypad->cd();
  fxyview->Draw();

  int imax = 0;
  for(unsigned int icluster=0;icluster<pClusters.size();++icluster){
    if(pClusters[icluster]->Hits()>=minHits){
      n = 0;
      for(int i=0; i<100; i++){	
	if(pClusters[icluster]->hitsInLayer(i)>0&&i>imax)imax=i;
	for(int ihit =0; ihit < pClusters[icluster]->hitsInLayer(i);++ihit){
	  MyCaloHitExtended* hiti = pClusters[icluster]->hitInLayer(i,ihit);
	  x[n] = hiti->getPosition()[0];
	  y[n] = hiti->getPosition()[1];
	  z[n] = hiti->getPosition()[2];
	  n++;
	}
      }
      

      TPolyMarker* pPm = new TPolyMarker(n,x,y);
      int jcluster = pClusters[icluster]->getEarliestRelationID();
      if(pClusters[icluster]->getParents()>0){
	//      jcluster = (pClusters[icluster]->getParent(0))->getID();
      }
      int colour = jcluster-(jcluster/10)*10+1;
      if(pClusters[icluster]->StillGrowing()){
	pPm->SetMarkerStyle(24);
	pPm->SetMarkerSize(0.6);
      }else{
	pPm->SetMarkerStyle(20);
	pPm->SetMarkerSize(0.6);
      }

      pPm->SetMarkerColor(colour);
      pPm->Draw();
      
      
      int firstLayer = pClusters[icluster]->FirstLayer();
      n =1;
      x[0] = pClusters[icluster]->GetCentroid(firstLayer)[0];
      y[0] = pClusters[icluster]->GetCentroid(firstLayer)[1];
      z[0] = pClusters[icluster]->GetCentroid(firstLayer)[2];
      pPm = new TPolyMarker(n,x,y);
      pPm->SetMarkerStyle(25);
      pPm->SetMarkerSize(1.);
      pPm->SetMarkerColor(colour);
      pPm->Draw();
      
      
      
      if(pClusters[icluster]->StillGrowing()){
	int lastLayer = pClusters[icluster]->LastLayer();
	float x1 = pClusters[icluster]->GetCentroid(lastLayer)[0];
	float y1 = pClusters[icluster]->GetCentroid(lastLayer)[1];
	float x2 = x1 + 100.*pClusters[icluster]->GetInitialDir()[0];
	float y2 = y1 + 100.*pClusters[icluster]->GetInitialDir()[1];
	TArrow* arrow = new TArrow(x1,y1,x2,y2,0.03);
	arrow->SetLineColor(colour);
	arrow->Draw();
	x2 = x1 + 100.*pClusters[icluster]->GetCurrentDir()[0];
	y2 = y1 + 100.*pClusters[icluster]->GetCurrentDir()[1];
	arrow = new TArrow(x1,y1,x2,y2,0.03);
	arrow->SetLineColor(colour);
	arrow->Draw();
      }
      
      fitResult fitS = pClusters[icluster]->FitStart(10);
      fitResult fitE = pClusters[icluster]->FitEnd(8);
      //if(fitS.ok){
      ///float x1 = fitS.intercept[0];
      //float y1 = fitS.intercept[1];
      //float z1 = fitS.intercept[2];
      //float x2 = x1 - fitS.dir[0]*1000.;
      //float y2 = y1 - fitS.dir[1]*1000.;
      //TArrow* arrow = new TArrow(x1,y1,x2,y2,0.03);
      //arrow->SetLineColor(colour);
      //arrow->Draw();
      //}
    //if(fitE.ok){
    //float x1 = fitE.intercept[0];
    //float y1 = fitE.intercept[1];
    //float x2 = x1 + fitE.dir[0]*100000.;
    //float y2 = y1 + fitE.dir[1]*100000.;
    //TArrow* arrow = new TArrow(x1,y1,x2,y2,0.03);
    //arrow->SetLineColor(colour);
    //arrow->Draw();
    //}
 

    }
  }

  this->DrawTracks(RECO_VIEW_XY);
  this->DrawMuonHits(RECO_VIEW_XY);
  this->DrawOutline(RECO_VIEW_XY);

  fCanvas->Update();

  gSystem->ProcessEvents();

 
  theApp->SetReturnFromRun(kTRUE);

  fcntl(STDIN_FILENO, F_SETFL, O_NONBLOCK);
  int key = 0;
  while(1){
    cout <<std::flush;
    //    theApp->StartIdleing();
    gSystem->ProcessEvents();
    //theApp->StopIdleing();
    key = getchar();
    if(key=='\n') break;
     usleep(1000);
  }


  ostringstream buffer;
  buffer << "Event_" << imax << ".gif"; 
  string file = buffer.str();

  fCanvas->SaveAs(file.c_str());

  return;

}


void MyRecoDisplay::DrawByProtoClusterEnergy(std::vector<ProtoCluster*> &pClusters, int minHits){

  float x[10000];
  float y[10000];
  float z[10000];
  int   n;

  fxypad->cd();
  fxyview->Draw();

  for(unsigned int icluster=0;icluster<pClusters.size();++icluster){
    if(pClusters[icluster]->Hits()>=minHits){
      n = 0;
      for(int i=0; i<100; i++){
	for(int ihit =0; ihit < pClusters[icluster]->hitsInLayer(i);++ihit){
	  MyCaloHitExtended* hiti = pClusters[icluster]->hitInLayer(i,ihit);
	  x[n] = hiti->getPosition()[0];
	  y[n] = hiti->getPosition()[1];
	  z[n] = hiti->getPosition()[2];
	  n++;
	}
      }
      for(int i=0; i<pClusters[icluster]->IsolatedHits();++i){
	MyCaloHitExtended* hiti = pClusters[icluster]->IsolatedHit(i);
	x[n] = hiti->getPosition()[0];
	y[n] = hiti->getPosition()[1];
	z[n] = hiti->getPosition()[2];
	n++;
      }

      
      TPolyMarker* pPm = new TPolyMarker(n,x,y);
      //      int jcluster = pClusters[icluster]->getEarliestRelationID();

      float E;
      if(pClusters[icluster]->IsPhoton()){
	E = pClusters[icluster]->EnergyEM();
      }else{	
	E = pClusters[icluster]->EnergyHAD();
      }
      int ilevel =0;
      for(int i=0;i<nlevels;++i){
	if(E>level[i])ilevel=i;
      }

      int colour = colours[ilevel];
      pPm->SetMarkerStyle(20);
      pPm->SetMarkerSize(0.3);      
      if(pClusters[icluster]->AssociatedTrack()==false){
	pPm->SetMarkerStyle(24);
	pPm->SetMarkerSize(0.8);
      }      
      pPm->SetMarkerColor(colour);
      pPm->Draw();
      
      
      int firstLayer = pClusters[icluster]->FirstLayer();
      n =1;
      x[0] = pClusters[icluster]->GetCentroid(firstLayer)[0];
      y[0] = pClusters[icluster]->GetCentroid(firstLayer)[1];
      z[0] = pClusters[icluster]->GetCentroid(firstLayer)[2];
      pPm = new TPolyMarker(n,x,y);
      pPm->SetMarkerStyle(25);
      pPm->SetMarkerSize(1.);
      pPm->SetMarkerColor(colour);
      pPm->Draw();
      
      
      
      if(pClusters[icluster]->StillGrowing()){
	int lastLayer = pClusters[icluster]->LastLayer();
	float x1 = pClusters[icluster]->GetCentroid(lastLayer)[0];
	float y1 = pClusters[icluster]->GetCentroid(lastLayer)[1];
	float x2 = x1 + 100.*pClusters[icluster]->GetInitialDir()[0];
	float y2 = y1 + 100.*pClusters[icluster]->GetInitialDir()[1];
	TArrow* arrow = new TArrow(x1,y1,x2,y2,0.03);
	arrow->SetLineColor(colour);
	arrow->Draw();
	x2 = x1 + 100.*pClusters[icluster]->GetCurrentDir()[0];
	y2 = y1 + 100.*pClusters[icluster]->GetCurrentDir()[1];
	arrow = new TArrow(x1,y1,x2,y2,0.03);
	arrow->SetLineColor(colour);
	arrow->Draw();
      }
      
      
      
      fitResult fitS = pClusters[icluster]->FitStart(8);
      fitResult fitE = pClusters[icluster]->FitEnd(8);
      
    
    //    if(fitS.ok){
    //float x1 = fitS.intercept[0];
    //float y1 = fitS.intercept[1];
    //float x2 = x1 - fitS.dir[0]*100000.;
    //float y2 = y1 - fitS.dir[1]*100000.;
    //TArrow* arrow = new TArrow(x1,y1,x2,y2,0.03);
    //arrow->SetLineColor(colour);
    //arrow->Draw();
    //}
    //if(fitE.ok){
    //float x1 = fitE.intercept[0];
    //float y1 = fitE.intercept[1];
    //float x2 = x1 + fitE.dir[0]*100000.;
    //float y2 = y1 + fitE.dir[1]*100000.;
    //TArrow* arrow = new TArrow(x1,y1,x2,y2,0.03);
    //arrow->SetLineColor(colour);
    //arrow->Draw();
    //}
 

    }
  }

  this->DrawTracks(RECO_VIEW_XY);
  this->DrawMuonHits(RECO_VIEW_XY);
  this->DrawOutline(RECO_VIEW_XY);

  fCanvas->Update();

  gSystem->ProcessEvents();

 
  theApp->SetReturnFromRun(kTRUE);

  fcntl(STDIN_FILENO, F_SETFL, O_NONBLOCK);
  int key = 0;
  while(1){
    cout <<std::flush;
    //    theApp->StartIdleing();
    gSystem->ProcessEvents();
    //theApp->StopIdleing();
    key = getchar();
    if(key=='\n') break;
     usleep(1000);
  }

  return;

}









void MyRecoDisplay::DrawByProtoClusterEnergy(std::vector<ProtoCluster*> &pClusters, int minHits, RecoDisplayView view){

  float x[50000];
  float y[50000];
  float z[50000];
  float r[50000];
  int   n;
  int   np;
  float xp[10000];
  float yp[10000];
  float zp[10000];
  float rp[10000];


  fxypad->cd();
  fxzview->Draw();

  for(unsigned int icluster=0;icluster<pClusters.size();++icluster){
    if(pClusters[icluster]->Hits()>=minHits){
      n  = 0;
      np = 0;

      if(pClusters[icluster]->AssociatedTrack()){
	float e  = pClusters[icluster]->EnergyHAD();
	float ep = pClusters[icluster]->TruePhotonContribution();
	if(ep>1)std::cout << " PHOTON E : " << ep << " lost in cluster " << e << std::endl; 
      }



      for(int i=0; i<100; i++){
	for(int ihit =0; ihit < pClusters[icluster]->hitsInLayer(i);++ihit){
	  MyCaloHitExtended* hiti = pClusters[icluster]->hitInLayer(i,ihit);
	  x[n] = hiti->getPosition()[0];
	  y[n] = hiti->getPosition()[1];
	  z[n] = hiti->getPosition()[2];
	  r[n] = sqrt(x[n]*x[n]+y[n]+y[n]);
	  n++;
	  if(pClusters[icluster]->AssociatedTrack()){
	    if(hiti->getPhoton()){
	      xp[np] = hiti->getPosition()[0];
	      yp[np] = hiti->getPosition()[1];
	      zp[np] = hiti->getPosition()[2];
	      rp[np] = hiti->getPosition()[2];
	      np++;
	    }
	  }
	}
      }
      
      TPolyMarker* pPm = NULL;
      TPolyMarker* pPh = NULL;    
      if(view==RECO_VIEW_XY)pPm = new TPolyMarker(n,x,y);
      if(view==RECO_VIEW_XZ)pPm = new TPolyMarker(n,z,x);
      if(view==RECO_VIEW_RZ)pPm = new TPolyMarker(n,z,r);
      if(view==RECO_VIEW_XY)pPh = new TPolyMarker(np,xp,yp);
      if(view==RECO_VIEW_XZ)pPh = new TPolyMarker(np,zp,xp);
      if(view==RECO_VIEW_RZ)pPh = new TPolyMarker(np,zp,rp);
      //      int jcluster = pClusters[icluster]->getEarliestRelationID();

      float E;
      if(pClusters[icluster]->IsPhoton()){
	E = pClusters[icluster]->EnergyEM();
      }else{	
	E = pClusters[icluster]->EnergyHAD();
      }
      int ilevel =0;
      for(int i=0;i<nlevels;++i){
	if(E>level[i])ilevel=i;
      }

      int colour = colours[ilevel];
      pPm->SetMarkerStyle(20);
      pPm->SetMarkerSize(0.6);      
      if(pClusters[icluster]->AssociatedTrack()==false){
	pPm->SetMarkerStyle(24);
	pPm->SetMarkerSize(0.6);
      }      
      pPm->SetMarkerColor(colour);
      pPm->Draw();


      pPh->SetMarkerStyle(20);
      pPh->SetMarkerSize(0.8);      
      if(pClusters[icluster]->AssociatedTrack()==false){
	pPh->SetMarkerStyle(24);
	pPh->SetMarkerSize(0.8);
      }      
      pPh->SetMarkerColor(42);
      pPh->Draw();
            
      int firstLayer = pClusters[icluster]->FirstLayer();
      n =1;
      x[0] = pClusters[icluster]->GetCentroid(firstLayer)[0];
      y[0] = pClusters[icluster]->GetCentroid(firstLayer)[1];
      z[0] = pClusters[icluster]->GetCentroid(firstLayer)[2];
      r[0] = sqrt(x[0]*x[0]+y[0]*y[0]);
      if(view==RECO_VIEW_XY)pPm = new TPolyMarker(n,x,y);
      if(view==RECO_VIEW_XZ)pPm = new TPolyMarker(n,z,x);
      if(view==RECO_VIEW_RZ)pPm = new TPolyMarker(n,z,r);

      pPm->SetMarkerStyle(25);
      pPm->SetMarkerSize(1.);
      pPm->SetMarkerColor(colour);
      pPm->Draw();
            
      if(pClusters[icluster]->StillGrowing()){
	int lastLayer = pClusters[icluster]->LastLayer();
	float x1 = pClusters[icluster]->GetCentroid(lastLayer)[0];
	float y1 = pClusters[icluster]->GetCentroid(lastLayer)[1];
	float z1 = pClusters[icluster]->GetCentroid(lastLayer)[2];
	float r1 = sqrt(x1*x1+y1*y1);
	float x2 = x1 + 100.*pClusters[icluster]->GetInitialDir()[0];
	float y2 = y1 + 100.*pClusters[icluster]->GetInitialDir()[1];
	float z2 = z1 + 100.*pClusters[icluster]->GetInitialDir()[2];
	float r2 = sqrt(x2*x2+y2*y2);
	TArrow* arrow=NULL;
	if(view==RECO_VIEW_XY)arrow = new TArrow(x1,y1,x2,y2,0.03);
	if(view==RECO_VIEW_XZ)arrow = new TArrow(z1,x1,z2,x2,0.03);
	if(view==RECO_VIEW_RZ)arrow = new TArrow(z1,r1,z2,r2,0.03);
	arrow->SetLineColor(colour);
	arrow->Draw();
	x2 = x1 + 100.*pClusters[icluster]->GetCurrentDir()[0];
	y2 = y1 + 100.*pClusters[icluster]->GetCurrentDir()[1];
	z2 = z1 + 100.*pClusters[icluster]->GetCurrentDir()[2];
	r2 = sqrt(x2*x2+y2*y2);
	if(view==RECO_VIEW_XY)arrow = new TArrow(x1,y1,x2,y2,0.03);
	if(view==RECO_VIEW_XZ)arrow = new TArrow(z1,x1,z2,x2,0.03);
	if(view==RECO_VIEW_RZ)arrow = new TArrow(z1,r1,z2,r2,0.03);
	arrow->SetLineColor(colour);
	arrow->Draw();
      }
      
           
      fitResult fitS = pClusters[icluster]->FitStart(8);
      fitResult fitE = pClusters[icluster]->FitEnd(8);
    }
  }

  this->DrawTracks(view);
  this->DrawMuonHits(view);
  this->DrawOutline(view);

  fCanvas->Update();

  gSystem->ProcessEvents();

 
  theApp->SetReturnFromRun(kTRUE);

  fcntl(STDIN_FILENO, F_SETFL, O_NONBLOCK);
  int key = 0;
  while(1){
    cout <<std::flush;
    //    theApp->StartIdleing();
    gSystem->ProcessEvents();
    //theApp->StopIdleing();
    key = getchar();
    if(key=='\n') break;
     usleep(1000);
  }

  return;

}




void MyRecoDisplay::DrawByJet(LCCollection * col, RecoDisplayView view){


  fxypad->cd();
  fxzview->Draw();

  for(int i=0; i<col->getNumberOfElements(); i++) {
    ReconstructedParticle* part =
      dynamic_cast<ReconstructedParticle*>( col->getElementAt( i ) );
    ReconstructedParticleVec partVec = part->getParticles();
    int colour = i+1;
    this->DrawParticle(part,view,colour);
  }    

  this->DrawOutline(view);
  fCanvas->Update();
  gSystem->ProcessEvents();
  theApp->SetReturnFromRun(kTRUE);

  fcntl(STDIN_FILENO, F_SETFL, O_NONBLOCK);
  int key = 0;
  while(1){
    cout <<std::flush;
    //    theApp->StartIdleing();
    gSystem->ProcessEvents();
    //theApp->StopIdleing();
    key = getchar();
    if(key=='\n') break;
     usleep(1000);
  }
  return;
}





void MyRecoDisplay::DrawByParticle(LCCollection * col, RecoDisplayView view){

  fxypad->cd();
  fxzview->Draw();
  for(int i=0; i<col->getNumberOfElements(); i++) {
    ReconstructedParticle* part =
      dynamic_cast<ReconstructedParticle*>( col->getElementAt( i ) );
    float E = part->getEnergy();
    int ilevel =0;
    for(int il=0;il<nlevels;++il){
      if(E>level[il])ilevel=il;
    }
    int colour = colours[ilevel];
    this->DrawParticle(part,view,colour);
  }    

  this->DrawOutline(view);
  fCanvas->Update();
  gSystem->ProcessEvents();
  theApp->SetReturnFromRun(kTRUE);

  fcntl(STDIN_FILENO, F_SETFL, O_NONBLOCK);
  int key = 0;
  while(1){
    cout <<std::flush;
    //    theApp->StartIdleing();
    gSystem->ProcessEvents();
    //theApp->StopIdleing();
    key = getchar();
    if(key=='\n') break;
     usleep(1000);
  }
  return;
}

void MyRecoDisplay::DrawParticle(ReconstructedParticle* part, RecoDisplayView view, int colour){
  
  float x[5000];
  float y[5000];
  float z[5000];
  float r[5000];
  int   n=0;
  float xx[5000];
  float yy[5000];
  float zz[5000];
  float rr[5000];
  int   nn=0;

  // recursive call to draw any particles which are part of this PFO
  // e.g. particles in jet
  ReconstructedParticleVec partVec = part->getParticles();
  for(unsigned int ipart=0;ipart<partVec.size();++ipart){
    DrawParticle(partVec[ipart],view,colour);
  }  

  ClusterVec clusters = part->getClusters();
  TrackVec tracks = part->getTracks();

  for(unsigned int icluster=0;icluster<clusters.size();++icluster){
    CalorimeterHitVec hits = clusters[icluster]->getCalorimeterHits();
    for(unsigned int i=0; i<hits.size(); i++){
      CalorimeterHit * hiti = hits[i];
      x[n] = hiti->getPosition()[0];
      y[n] = hiti->getPosition()[1];
      z[n] = hiti->getPosition()[2];
      r[n] = x[n]*_thrustX+y[n]*_thrustY;
      n++;
    }
  }

  for(unsigned int itrack=0;itrack<tracks.size();++itrack){
    Track* track = tracks[itrack];
    TrackerHitVec hitvec = track->getTrackerHits();
    int nhits = (int)hitvec.size(); 
    for(int i=0;i<nhits;++i){
      TrackerHit* trkHit = hitvec[i];
      xx[nn] = (float)trkHit->getPosition()[0];
      yy[nn] = (float)trkHit->getPosition()[1];
      zz[nn] = (float)trkHit->getPosition()[2];
      rr[nn] = xx[nn]*_thrustX+yy[nn]*_thrustY;
      nn++;
    }
  }
  
  TPolyMarker* pPm = NULL;
  if(view==RECO_VIEW_XY)pPm = new TPolyMarker(n,x,y);
  if(view==RECO_VIEW_XZ)pPm = new TPolyMarker(n,z,x);
  if(view==RECO_VIEW_RZ)pPm = new TPolyMarker(n,z,r);
  pPm->SetMarkerStyle(20);
  pPm->SetMarkerSize(0.6);      
  if(tracks.size()==0){
    pPm->SetMarkerStyle(20);
    pPm->SetMarkerSize(0.6);      
  }
  pPm->SetMarkerColor(colour);
  pPm->Draw();

  TPolyMarker* pPt = NULL;
  if(view==RECO_VIEW_XY)pPt = new TPolyMarker(nn,xx,yy);
  if(view==RECO_VIEW_XZ)pPt = new TPolyMarker(nn,zz,xx);
  if(view==RECO_VIEW_RZ)pPt = new TPolyMarker(nn,zz,rr);
  pPt->SetMarkerStyle(20);
  pPt->SetMarkerSize(0.3);      
  pPt->SetMarkerColor(colour);
  pPt->Draw();

}




void MyRecoDisplay::DrawByProtoClusterBadness(std::vector<ProtoCluster*> &pClusters, int minHits, RecoDisplayView view){

  float x[50000];
  float y[50000];
  float z[50000];
  float r[50000];
  int   n;

  fxypad->cd();
  fxzview->Draw();

  for(unsigned int icluster=0;icluster<pClusters.size();++icluster){
    if(pClusters[icluster]->Hits()>=minHits){
      n = 0;
      for(int i=0; i<100; i++){
	for(int ihit =0; ihit < pClusters[icluster]->hitsInLayer(i);++ihit){
	  MyCaloHitExtended* hiti = pClusters[icluster]->hitInLayer(i,ihit);
	  x[n] = hiti->getPosition()[0];
	  y[n] = hiti->getPosition()[1];
	  z[n] = hiti->getPosition()[2];
	  r[n] = sqrt(x[n]*x[n]+y[n]+y[n]);
	  n++;
	}
      }
      for(int i=0; i<pClusters[icluster]->IsolatedHits();++i){
	MyCaloHitExtended* hiti = pClusters[icluster]->IsolatedHit(i);
	x[n] = hiti->getPosition()[0];
	y[n] = hiti->getPosition()[1];
	z[n] = hiti->getPosition()[2];
	r[n] = sqrt(x[n]*x[n]+y[n]+y[n]);
	n++;
      }

      
      TPolyMarker* pPm = NULL;
      if(view==RECO_VIEW_XY)pPm = new TPolyMarker(n,x,y);
      if(view==RECO_VIEW_XZ)pPm = new TPolyMarker(n,z,x);
      if(view==RECO_VIEW_RZ)pPm = new TPolyMarker(n,z,r);
      //      int jcluster = pClusters[icluster]->getEarliestRelationID();


      int colour = 1;
      if(pClusters[icluster]->GetBadness()==0)colour=3;
      if(pClusters[icluster]->GetBadness()==1)colour=4;
      if(pClusters[icluster]->GetBadness()==2)colour=2;

      
      pPm->SetMarkerStyle(24);
      pPm->SetMarkerSize(0.3);      
      if(pClusters[icluster]->AssociatedTrack()==false){
	pPm->SetMarkerStyle(20);
	pPm->SetMarkerSize(0.8);
      }      
      pPm->SetMarkerColor(colour);
      pPm->Draw();
      
      
      int firstLayer = pClusters[icluster]->FirstLayer();
      n =1;
      x[0] = pClusters[icluster]->GetCentroid(firstLayer)[0];
      y[0] = pClusters[icluster]->GetCentroid(firstLayer)[1];
      z[0] = pClusters[icluster]->GetCentroid(firstLayer)[2];
      r[0] = sqrt(x[0]*x[0]+y[0]*y[0]);
      if(view==RECO_VIEW_XY)pPm = new TPolyMarker(n,x,y);
      if(view==RECO_VIEW_XZ)pPm = new TPolyMarker(n,z,x);
      if(view==RECO_VIEW_RZ)pPm = new TPolyMarker(n,z,r);

      pPm->SetMarkerStyle(25);
      pPm->SetMarkerSize(1.);
      pPm->SetMarkerColor(colour);
      pPm->Draw();
      
      
      
      if(pClusters[icluster]->StillGrowing()){
	int lastLayer = pClusters[icluster]->LastLayer();
	float x1 = pClusters[icluster]->GetCentroid(lastLayer)[0];
	float y1 = pClusters[icluster]->GetCentroid(lastLayer)[1];
	float z1 = pClusters[icluster]->GetCentroid(lastLayer)[2];
	float r1 = sqrt(x1*x1+y1*y1);
	float x2 = x1 + 100.*pClusters[icluster]->GetInitialDir()[0];
	float y2 = y1 + 100.*pClusters[icluster]->GetInitialDir()[1];
	float z2 = z1 + 100.*pClusters[icluster]->GetInitialDir()[2];
	float r2 = sqrt(x2*x2+y2*y2);
	TArrow* arrow=NULL;
	if(view==RECO_VIEW_XY)arrow = new TArrow(x1,y1,x2,y2,0.03);
	if(view==RECO_VIEW_XZ)arrow = new TArrow(z1,x1,z2,x2,0.03);
	if(view==RECO_VIEW_RZ)arrow = new TArrow(z1,r1,z2,r2,0.03);
	arrow->SetLineColor(colour);
	arrow->Draw();
	x2 = x1 + 100.*pClusters[icluster]->GetCurrentDir()[0];
	y2 = y1 + 100.*pClusters[icluster]->GetCurrentDir()[1];
	z2 = z1 + 100.*pClusters[icluster]->GetCurrentDir()[2];
	r2 = sqrt(x2*x2+y2*y2);
	if(view==RECO_VIEW_XY)arrow = new TArrow(x1,y1,x2,y2,0.03);
	if(view==RECO_VIEW_XZ)arrow = new TArrow(z1,x1,z2,x2,0.03);
	if(view==RECO_VIEW_RZ)arrow = new TArrow(z1,r1,z2,r2,0.03);
	arrow->SetLineColor(colour);
	arrow->Draw();
      }
      
           
      fitResult fitS = pClusters[icluster]->FitStart(8);
      fitResult fitE = pClusters[icluster]->FitEnd(8);
    }
  }

  this->DrawTracks(view);
  this->DrawMuonHits(view);
  this->DrawOutline(view);

  fCanvas->Update();

  gSystem->ProcessEvents();

 
  theApp->SetReturnFromRun(kTRUE);

  fcntl(STDIN_FILENO, F_SETFL, O_NONBLOCK);
  int key = 0;
  while(1){
    cout <<std::flush;
    //    theApp->StartIdleing();
    gSystem->ProcessEvents();
    //theApp->StopIdleing();
    key = getchar();
    if(key=='\n') break;
     usleep(1000);
  }

  return;

}








void MyRecoDisplay::SetPseudoThrust(std::vector<ProtoCluster*> &pClusters){

  // Calculate a simple thrust like variable... NOT TRUE THRUST

  float ex = 0;
  float ey = 0;
  float s  = 0;

  for(unsigned int icluster=0;icluster<pClusters.size();++icluster){
    int firstLayer = pClusters[icluster]->FirstLayer();
    float x = pClusters[icluster]->GetCentroid(firstLayer)[0];
    float y = pClusters[icluster]->GetCentroid(firstLayer)[1];
    float z = pClusters[icluster]->GetCentroid(firstLayer)[2];
    float r = sqrt(x*x+y*y+z*z);
    ex += pClusters[icluster]->EnergyHAD()*fabs(x)/r;
    ey += pClusters[icluster]->EnergyHAD()*fabs(y)/r;
    if( x*y>0 )s += pClusters[icluster]->EnergyHAD()*(ex*ex+ey*ey)/r;
    if( x*y<0 )s -= pClusters[icluster]->EnergyHAD()*(ex*ex+ey*ey)/r;
  }
  float et = sqrt(ex*ex+ey*ey);
  ex = ex/et;
  ey = ey/et;
  if(s<0)ey = -ey;
  std::cout << " Pseudo-Thrust : " << ex << "," << ey << std::endl;

  return;

}



void MyRecoDisplay::SetPseudoThrust(LCCollection * pfocol){

  // Calculate a simple thrust like variable... NOT TRUE THRUST
  fxypad->cd();
  fxzview->Draw();
  float ex = 0;
  float ey = 0;
  float s  = 0;

  for(int i=0; i<pfocol->getNumberOfElements(); i++) {
    ReconstructedParticle* part =
      dynamic_cast<ReconstructedParticle*>( pfocol->getElementAt( i ) );
    float px = part->getMomentum()[0];
    float py = part->getMomentum()[1];
    ex += fabs(px);
    ey += fabs(py);
    if( px*py>0 )s += (px*px+py*py);
    if( px*py<0 )s -= (px*px+py*py);
  }
  float et = sqrt(ex*ex+ey*ey);
  ex = ex/et;
  ey = ey/et;
  if(s<0)ey = -ey;
  std::cout << " Pseudo-Thrust : " << ex << "," << ey << std::endl;
  _setThrust = true;
  _thrustX = ex;
  _thrustY = ey;
  
  
  return;

}








void MyRecoDisplay::DrawSingleProtoCluster(ProtoCluster* pCluster, RecoDisplayView view){

  float x[50000];
  float y[50000];
  float z[50000];
  float r[50000];
  float xm[50000];
  float ym[50000];
  float zm[50000];
  float rm[50000];
  int   n;
  int   nm;

  //  this->DrawLongitudinalProfile(pCluster);

  fxypad->cd();
  fxzview->Draw();

  n = 0;
  nm = 0;
  for(int i=0; i<100; i++){
    for(int ihit =0; ihit < pCluster->hitsInLayer(i);++ihit){
      MyCaloHitExtended* hiti = pCluster->hitInLayer(i,ihit);

      if(hiti->isCandidateMip()){
	xm[nm] = hiti->getPosition()[0];
	ym[nm] = hiti->getPosition()[1];
	zm[nm] = hiti->getPosition()[2];
	rm[nm] = sqrt(x[nm]*x[nm]+y[nm]+y[nm]);
	nm++;
      }else{
	x[n] = hiti->getPosition()[0];
	y[n] = hiti->getPosition()[1];
	z[n] = hiti->getPosition()[2];
	r[n] = sqrt(x[n]*x[n]+y[n]+y[n]);
	n++;
      }
    }
  }
  //  for(int i=0; i<pCluster->IsolatedHits();++i){
  //  MyCaloHitExtended* hiti = pCluster->IsolatedHit(i);
  // x[n] = hiti->getPosition()[0];
  // y[n] = hiti->getPosition()[1];
  // z[n] = hiti->getPosition()[2];
  // r[n] = sqrt(x[n]*x[n]+y[n]+y[n]);
  //  n++;
  // }
  
  
  TPolyMarker* pP=NULL;
  if(view==RECO_VIEW_XY)pP = new TPolyMarker(n,x,y);
  if(view==RECO_VIEW_XZ)pP = new TPolyMarker(n,z,x);
  if(view==RECO_VIEW_RZ)pP = new TPolyMarker(n,z,r);
  TPolyMarker* pPm=NULL;
  if(view==RECO_VIEW_XY)pPm = new TPolyMarker(nm,xm,ym);
  if(view==RECO_VIEW_XZ)pPm = new TPolyMarker(nm,zm,xm);
  if(view==RECO_VIEW_RZ)pPm = new TPolyMarker(nm,zm,rm);
  //      int jcluster = pClusters[icluster]->getEarliestRelationID();
  
  float E;
  if(pCluster->IsPhoton()){
    E = pCluster->EnergyEM();
  }else{	
    E = pCluster->EnergyHAD();
  }
  int ilevel =0;
  for(int i=0;i<nlevels;++i){
    if(E>level[i])ilevel=i;
  }
  
  int colour = colours[ilevel];
  pP->SetMarkerStyle(20);
  pP->SetMarkerSize(0.3);      
  if(pCluster->AssociatedTrack()==false){
    pP->SetMarkerStyle(20);
    pP->SetMarkerSize(0.8);
  }      
  pP->SetMarkerColor(4);
  pP->Draw();

  pPm->SetMarkerStyle(24);
  pPm->SetMarkerSize(0.5);      
  if(pCluster->AssociatedTrack()==false){
    pPm->SetMarkerStyle(20);
    pPm->SetMarkerSize(0.8);
  }      
  pPm->SetMarkerColor(2);
  pPm->Draw();
  
  
  int firstLayer = pCluster->FirstLayer();
  n =1;
  x[0] = pCluster->GetCentroid(firstLayer)[0];
  y[0] = pCluster->GetCentroid(firstLayer)[1];
  z[0] = pCluster->GetCentroid(firstLayer)[2];
  r[0] = sqrt(x[0]*x[0]+y[0]*y[0]);
  if(view==RECO_VIEW_XY)pPm = new TPolyMarker(n,x,y);
  if(view==RECO_VIEW_XZ)pPm = new TPolyMarker(n,z,x);
  if(view==RECO_VIEW_RZ)pPm = new TPolyMarker(n,z,r);
  
  pPm->SetMarkerStyle(25);
  pPm->SetMarkerSize(1.);
  pPm->SetMarkerColor(colour);
  pPm->Draw();
  
  this->DrawTracks(view);
  this->DrawMuonHits(view);
  this->DrawOutline(view);

  fCanvas->Update();

  gSystem->ProcessEvents();

 
  theApp->SetReturnFromRun(kTRUE);

  fcntl(STDIN_FILENO, F_SETFL, O_NONBLOCK);
  int key = 0;
  while(1){
    cout <<std::flush;
    //    theApp->StartIdleing();
    gSystem->ProcessEvents();
    //theApp->StopIdleing();
    key = getchar();
    if(key=='\n') break;
     usleep(1000);
  }

  return;

}




void MyRecoDisplay::DrawLongitudinalProfile(ProtoCluster* pCluster, bool truncate){

  //  float x[100];

  fxypad->cd();
  fxzview->Draw();
  flongprofile->Reset();
  flongshowerprofile->Reset();

  float X0BinSize = 0.5;
  const unsigned int X0Bins = 100;
  //  float X0Max     = X0Bins*X0BinSize;
  float X0Profile[X0Bins];
  float X0MipProfile[X0Bins];
  float expectedX0Profile[X0Bins];

  float eProfile[MAX_NUMBER_OF_LAYERS];
  float mipProfile[MAX_NUMBER_OF_LAYERS];

  for(unsigned int i=0;i<MAX_NUMBER_OF_LAYERS;i++)eProfile[i]=0.;
  for(unsigned int i=0;i<MAX_NUMBER_OF_LAYERS;i++)mipProfile[i]=0.;
  for(unsigned int i=0;i<X0Bins;i++)X0Profile[i]=0.;
  for(unsigned int i=0;i<X0Bins;i++)X0MipProfile[i]=0.;
  for(unsigned int i=0;i<X0Bins;i++)expectedX0Profile[i]=0.;

  for(int i=0; i<100; i++){
    for(int ihit =0; ihit < pCluster->hitsInLayer(i);++ihit){
      MyCaloHitExtended* hiti = pCluster->hitInLayer(i,ihit);
      eProfile[i]+= hiti->getEnergy();
      mipProfile[i]+= hiti->getEnergyInMips();
      //      flongprofile->Fill(static_cast<float>(i)+0.5,hiti->getEnergy());
    }
  }
  
  float x=0.;
  float y=0.;
  float z=0.;

  x = pCluster->GetCentroid(pCluster->FirstLayer())[0];
  y = pCluster->GetCentroid(pCluster->FirstLayer())[1];
  z = pCluster->GetCentroid(pCluster->FirstLayer())[2];
  float r= sqrt(x*x+y*y+z*z);
  float urx = x/r;
  float ury = y/r;
  float urz = z/r;

  float cost[MAX_NUMBER_OF_LAYERS];
  float sep[MAX_NUMBER_OF_LAYERS];
  for(unsigned int i=1;i<MAX_NUMBER_OF_LAYERS;++i)cost[i] = -999.;
  for(unsigned int i=1;i<MAX_NUMBER_OF_LAYERS;++i)sep[i] = -999.;

  for(int i=pCluster->FirstLayer(); i<=pCluster->LastLayer(); i++){
    if(pCluster->hitsInLayer(i)>0){
      MyCaloHitExtended* hit0 = pCluster->hitInLayer(i,0);
      int physLayer = hit0->getPhysicalLayer();
      if(physLayer<30)sep[i]=1.4;
      if(physLayer>=30)sep[i]=4.2;
    }
    if(pCluster->hitsInLayer(i)>2){
      MyCaloHitExtended* hit0 = pCluster->hitInLayer(i,0);
      MyCaloHitExtended* hit1 = pCluster->hitInLayer(i,1);
      MyCaloHitExtended* hit2 = pCluster->hitInLayer(i,2);
      float x0 = hit0->getPosition()[0];
      float y0 = hit0->getPosition()[1];
      float z0 = hit0->getPosition()[2];
      float dx1 = (hit1->getPosition()[0] -x0);
      float dy1 = (hit1->getPosition()[1] -y0);
      float dz1 = (hit1->getPosition()[2] -z0);
      float dx2 = (hit2->getPosition()[0] -x0);
      float dy2 = (hit2->getPosition()[1] -y0);
      float dz2 = (hit2->getPosition()[2] -z0);
      float xn = dy1*dz2-dz1*dy2;
      float yn = dz1*dx2-dx1*dz2;
      float zn = dx1*dy2-dy1*dx2;
      float rn = sqrt(xn*xn+yn*yn+zn*zn);
      if(rn>10.){
	xn = xn/rn;
	yn = yn/rn;
	zn = zn/rn;
	cost[i] = fabs(xn*urx+yn*ury+zn*urz);
	if(cost[i]<0.3)cost[i]=-999.;
      }
    }
  }

  int ilast  = 0;
  for(int i=pCluster->FirstLayer(); i<=40; i++){
    if(sep[i]>0){
      if(ilast==0){
	for(int j=1;j<i;j++)sep[j]=sep[i];
	ilast = i;
      }
      if(i>ilast+1){
	for(int j=ilast+1;j<i;j++)sep[j]=sep[ilast];
      }
      ilast = i;
    }
  }
  if(ilast==0)sep[0]=1.4;
  for(int i=ilast+1; i<=40; i++)sep[i]=sep[ilast];

  ilast  = 0;
  for(int i=pCluster->FirstLayer(); i<=40; i++){
    if(cost[i]>0){
      if(ilast==0){
	for(int j=1;j<i;j++)cost[j]=cost[i];
	ilast = i;
      }
      if(i>ilast+1){
	for(int j=ilast+1;j<i;j++)cost[j]=cost[ilast];
      }
      ilast = i;
    }
  }
  if(ilast==0)cost[0]=1.;
  for(int i=ilast+1; i<=40; i++)cost[i]=cost[ilast];

  float x0    = 3.5;
  float E0 = pCluster->EnergyEM();
  // Effective critical energy
  float EC = 0.08;

  std::cout << E0 << std::endl;
  double a = 1.25+0.5*log(E0/EC);
  double lngammaa = lgamma(a);
  double gammaa = exp(lngammaa);
  //  float diffE = 0.;

  float dist=0.;
  int   X0ProfileEnd=0;
  // step through ECAL making profile in radiation lengths
  float ecalE = 0.;
  int maxLayer = 40;
  if(truncate)maxLayer = 35;
  for(int i=1; i<=maxLayer; i++){
    ecalE += eProfile[i];
    float dr = sep[i]/cost[i];
    float dt = dr/x0;
    float distStart = dist;
    dist+= dt;
    float deltaX0Bins = dt/X0BinSize;
    float bStart = distStart/X0BinSize;
    float bEnd   = dist/X0BinSize;
    unsigned int iStart = static_cast<int>(bStart);
    unsigned int iEnd = static_cast<int>(bEnd);
    float deltaStart = bStart-iStart;
    float deltaEnd   = 1.0-bEnd+iEnd;
    for(unsigned int ibin=iStart;ibin<=iEnd;++ibin){
      float delta = 1.;
      if(ibin==iStart)delta = delta - deltaStart;
      if(ibin==iEnd)delta   = delta - deltaEnd;
      float frac = delta/deltaX0Bins;
      X0Profile[ibin] += eProfile[i]*frac; 
      X0MipProfile[ibin] += mipProfile[i]*frac; 
    }
    X0ProfileEnd = iEnd; 
  }
  if(truncate)E0=ecalE;

  for(int i=0; i<100; i++){

    flongprofile->Fill(static_cast<float>(i)+0.5,X0Profile[i]);
    //    flongprofile->Fill(static_cast<float>(i)+0.5,mipProfile[i]/cost[i]);

  }

 
  float aa = static_cast<float>(a);
  float t =0.;
  for(int i=0; i<100; i++){
    t+= X0BinSize;
    float de = E0/2.0*pow(t/2,aa-1)*exp(-t/2)*X0BinSize/gammaa;
    expectedX0Profile[i] = de;
  }

  float diffEmin = 9999.;
  int   ioffmin  = 0;
  bool  stillgoing = true;
  for(int ioffset=0; stillgoing && ioffset<40; ioffset++){
    float diffE = 0;
    for(int i=0; i<X0ProfileEnd; i++){
      if(i-ioffset<0){
	diffE += X0Profile[i];
      }else{
	diffE += fabs(expectedX0Profile[i-ioffset]-X0Profile[i]);
      }
    }
    if(diffE<diffEmin){
      diffEmin = diffE;
      ioffmin = ioffset;
    }
    if(diffE-diffEmin>0.1)stillgoing = false;
  }

  for(int i=ioffmin; i<X0ProfileEnd; i++){
    flongshowerprofile->Fill(static_cast<float>(i)+0.5,expectedX0Profile[i-ioffmin]);
  }
  
  if(ecalE>0){
    std::cout << " MyRecoD Fraction profile difference : " << diffEmin/ecalE << std::endl;
    std::cout << " MyRecoD Shower Start                : " << ioffmin*X0BinSize << std::endl;
  }

  flongprofile->Draw();
  flongshowerprofile->Draw("same p");
  fCanvas->Update();

  gSystem->ProcessEvents();

 
  theApp->SetReturnFromRun(kTRUE);

  fcntl(STDIN_FILENO, F_SETFL, O_NONBLOCK);
  int key = 0;
  while(1){
    cout <<std::flush;
    //    theApp->StartIdleing();
    gSystem->ProcessEvents();
    //theApp->StopIdleing();
    key = getchar();
    if(key=='\n') break;
     usleep(1000);
  }

  return;

}


void MyRecoDisplay::DrawTransverseProfile(ProtoCluster* pCluster, int nlayers){

  //  float x[100];

  fxypad->cd();
  fxzview->Draw();
  ftransprofile->Reset();

  int nbins   = 41;
  //int ioffset = nbins/2; 
  int ifirst = pCluster->FirstLayer();
  float x0   = pCluster->GetCentroid(ifirst)[0];
  float y0   = pCluster->GetCentroid(ifirst)[1];
  float z0   = pCluster->GetCentroid(ifirst)[2];
  float r0   = sqrt(x0*x0+y0*y0+z0*z0);
  float ux   = x0/r0;
  float uy   = y0/r0;
  float uz   = z0/r0;
  float utx  = uy/sqrt(ux*ux+uy*uy);
  float uty  = -ux/sqrt(ux*ux+uy*uy);
  float utz  = 0;
  float vtx=0;
  float vty=0;
  float vtz=0;
  if(utx!=0&&uz!=0){
    vtx  = -uty/utx;
    vtz  = (uty*ux-uy*utx)/utx/uz;
    vty  = 1;
    float v    = sqrt(vtx*vtx+vty*vty+vtz*vtz);
    vtx = vtx/v;
    vty = vty/v;
    vtz = vtz/v;
  }
  if(utx==0&&uz!=0){
    vtx  = 1;
    vtz  = ux/uz;
    vty  = 0;
    float v    = sqrt(vtx*vtx+vty*vty+vtz*vtz);
    vtx = vtx/v;
    vty = vty/v;
    vtz = vtz/v;
  }
  if(utx==0&&uz==0){
    vtx  = 0;
    vtz  = 1;
    vty  = 0;
  }

  int  done[nbins][nbins];
  bool assigned[nbins][nbins];
  float tprofile[nbins][nbins];
  for(int i=0; i<nbins; i++){
    for(int j=0; j<nbins; j++){
      tprofile[i][j]=0;
      done[i][j]=assigned[i][j]=false;
    }
  }

  bool first = true;
  float pixelsize = 10.; 
  for(int i=0; i<nlayers; i++){
    for(int ihit =0; ihit < pCluster->hitsInLayer(i);++ihit){
      MyCaloHitExtended* hiti = pCluster->hitInLayer(i,ihit);
      if(first){
	pixelsize  = hiti->getHitSizeZ();
      }
      float dx = (hiti->getPosition()[0] -x0)/pixelsize;
      float dy = (hiti->getPosition()[1] -y0)/pixelsize;
      float dz = (hiti->getPosition()[2] -z0)/pixelsize;
      float dut = dx*utx+dy*uty+dz*utz;
      float dvt = dx*vtx+dy*vty+dz*vtz;
      //      int idut  = static_cast<int>(dut+0.5);
      //int idvt  = static_cast<int>(dvt+0.5);
      ftransprofile->Fill(dut,dvt,hiti->getEnergyEM());  
    }
  }


  ftransprofile->Draw("box");
  


  fCanvas->Update();

  gSystem->ProcessEvents();

 
  theApp->SetReturnFromRun(kTRUE);

  fcntl(STDIN_FILENO, F_SETFL, O_NONBLOCK);
  int key = 0;
  while(1){
    cout <<std::flush;
    //    theApp->StartIdleing();
    gSystem->ProcessEvents();
    //theApp->StopIdleing();
    key = getchar();
    if(key=='\n') break;
     usleep(1000);
  }

  this->DrawLongitudinalProfile(pCluster);

  return;

}







void MyRecoDisplay::DrawTransverseProfileFine(ProtoCluster* pCluster, int nlayers){

  //  float x[100];
  return;
  fxypad->cd();
  fxzview->Draw();
  ftransprofileFine->Reset();

  int nbins   = 1001;
  //  int ioffset = nbins/2; 
  int ifirst = pCluster->FirstLayer();
  float x0   = pCluster->GetCentroid(ifirst)[0];
  float y0   = pCluster->GetCentroid(ifirst)[1];
  float z0   = pCluster->GetCentroid(ifirst)[2];
  float r0   = sqrt(x0*x0+y0*y0+z0*z0);
  float ux   = x0/r0;
  float uy   = y0/r0;
  float uz   = z0/r0;
  float utx  = uy/sqrt(ux*ux+uy*uy);
  float uty  = -ux/sqrt(ux*ux+uy*uy);
  float utz  = 0;
  float vtx=0;
  float vty=0;
  float vtz=0;
  if(utx!=0&&uz!=0){
    vtx  = -uty/utx;
    vtz  = (uty*ux-uy*utx)/utx/uz;
    vty  = 1;
    float v    = sqrt(vtx*vtx+vty*vty+vtz*vtz);
    vtx = vtx/v;
    vty = vty/v;
    vtz = vtz/v;
  }
  if(utx==0&&uz!=0){
    vtx  = 1;
    vtz  = ux/uz;
    vty  = 0;
    float v    = sqrt(vtx*vtx+vty*vty+vtz*vtz);
    vtx = vtx/v;
    vty = vty/v;
    vtz = vtz/v;
  }
  if(utx==0&&uz==0){
    vtx  = 0;
    vtz  = 1;
    vty  = 0;
  }

  int  done[nbins][nbins];
  bool assigned[nbins][nbins];
  float tprofile[nbins][nbins];
  for(int i=0; i<nbins; i++){
    for(int j=0; j<nbins; j++){
      tprofile[i][j]=0;
      done[i][j]=assigned[i][j]=false;
    }
  }

  bool first = true;
  float pixelsize = 10.; 
  float sdut=0;
  float sdvt=0;
  float s2dut=0;
  float s2dvt=0;
  float nhits=0;
  int radialDistributionInMM[1000];
  for(int i=0;i<1000;i++)radialDistributionInMM[i]=0;

  for(int i=0; i<nlayers; i++){
    for(int ihit =0; ihit < pCluster->hitsInLayer(i);++ihit){
      MyCaloHitExtended* hiti = pCluster->hitInLayer(i,ihit);
      if(first){
	pixelsize  = hiti->getHitSizeZ();
      }
      float dx = (hiti->getPosition()[0] -x0);
      float dy = (hiti->getPosition()[1] -y0);
      float dz = (hiti->getPosition()[2] -z0);
      float dut = (dx*utx+dy*uty+dz*utz)/pixelsize;
      float dvt = (dx*vtx+dy*vty+dz*vtz)/pixelsize;
      sdut+= dut*pixelsize;
      sdvt+= dvt*pixelsize;
      s2dut+= dut*dut*pixelsize*pixelsize;
      s2dvt+= dvt*dvt*pixelsize*pixelsize;
      //      int idut  = static_cast<int>(dut);
      //int idvt  = static_cast<int>(dvt);
      float r = sqrt(dut*dut+dvt*dvt);
      int ir = static_cast<int>(r);
      if(ir<1000)radialDistributionInMM[ir]++;      
      nhits+=1.;
      ftransprofileFine->Fill(dut,dvt,hiti->getEnergyEM());  
    }
  }
  std::cout << sdut << ","  << nhits << std::endl;

  float umean = sdut/nhits;
  float vmean = sdvt/nhits;

  for(int i=0; i<nlayers; i++){
    for(int ihit =0; ihit < pCluster->hitsInLayer(i);++ihit){
      MyCaloHitExtended* hiti = pCluster->hitInLayer(i,ihit);
      float dx = (hiti->getPosition()[0] -x0);
      float dy = (hiti->getPosition()[1] -y0);
      float dz = (hiti->getPosition()[2] -z0);
      float dut = (dx*utx+dy*uty+dz*utz)-umean;
      float dvt = (dx*vtx+dy*vty+dz*vtz)-vmean;
      float r = sqrt(dut*dut+dvt*dvt);
      int ir = static_cast<int>(r);
      std::cout << " r ir " << r << "," << ir << std::endl;
      if(ir<1000)radialDistributionInMM[ir]++;      
    }
  }


  float count=0.;
  float r90 = 1000.;
  float r50 = 1000.;
  bool notDone=true;
  for(int i=0;i<1000&&notDone;i++){
    count += radialDistributionInMM[i];
    std::cout << " r[" << i<< "] " << radialDistributionInMM[i]<< std::endl; 
    if(count>0.9*nhits){
      notDone =false;
      r90 = i;
    }
    if(count>0.5*nhits&&r50>999){
      r50 = i;
    }
  }

  float urms  = s2dut/nhits-umean*umean;
  float vrms  = s2dvt/nhits-vmean*vmean;

  std::cout << " mean : " << umean << " , " << vmean << std::endl;
  std::cout << " rms  : " <<  urms << " , " <<  vrms << std::endl;
  std::cout << " r90  : " << r90 << std::endl;
  std::cout << " r50  : " << r50 << std::endl;

 


  ftransprofileFine->Draw("col");
  


  fCanvas->Update();

  gSystem->ProcessEvents();

 
  theApp->SetReturnFromRun(kTRUE);

  fcntl(STDIN_FILENO, F_SETFL, O_NONBLOCK);
  int key = 0;
  while(1){
    cout <<std::flush;
    //    theApp->StartIdleing();
    gSystem->ProcessEvents();
    //theApp->StopIdleing();
    key = getchar();
    if(key=='\n') break;
     usleep(1000);
  }

  return;

}






void MyRecoDisplay::DrawTransverseProfile(std::vector<ProtoCluster*> &pClusters, int nlayers){

  //  float x[100];

  fxypad->cd();
  fxzview->Draw();
  ftransprofile->Reset();

  int nbins   = 41;
  //  int ioffset = nbins/2; 

  int ifirst = pClusters[0]->FirstLayer();
  float x0   = pClusters[0]->GetCentroid(ifirst)[0];
  float y0   = pClusters[0]->GetCentroid(ifirst)[1];
  float z0   = pClusters[0]->GetCentroid(ifirst)[2];
  float r0   = sqrt(x0*x0+y0*y0+z0*z0);
  float ux   = x0/r0;
  float uy   = y0/r0;
  float uz   = z0/r0;
  float utx  = uy/sqrt(ux*ux+uy*uy);
  float uty  = -ux/sqrt(ux*ux+uy*uy);
  float utz  = 0;
  float vtx=0;
  float vty=0;
  float vtz=0;
  if(utx!=0&&uz!=0){
    vtx  = -uty/utx;
    vtz  = (uty*ux-uy*utx)/utx/uz;
    vty  = 1;
    float v    = sqrt(vtx*vtx+vty*vty+vtz*vtz);
    vtx = vtx/v;
    vty = vty/v;
    vtz = vtz/v;
  }
  if(utx==0&&uz!=0){
    vtx  = 1;
    vtz  = ux/uz;
    vty  = 0;
    float v    = sqrt(vtx*vtx+vty*vty+vtz*vtz);
    vtx = vtx/v;
    vty = vty/v;
    vtz = vtz/v;
  }
  if(utx==0&&uz==0){
    vtx  = 0;
    vtz  = 1;
    vty  = 0;
  }

  int  done[nbins][nbins];
  bool assigned[nbins][nbins];
  float tprofile[nbins][nbins];
  for(int i=0; i<nbins; i++){
    for(int j=0; j<nbins; j++){
      tprofile[i][j]=0;
      done[i][j]=assigned[i][j]=false;
    }
  }


  bool first = true;
  float pixelsize = 10.; 
  for(unsigned int ic = 0;ic<pClusters.size();ic++){
    for(int i=0; i<nlayers; i++){
      for(int ihit =0; ihit < pClusters[ic]->hitsInLayer(i);++ihit){
	MyCaloHitExtended* hiti = pClusters[ic]->hitInLayer(i,ihit);
	if(first){
	  pixelsize  = hiti->getHitSizeZ();
	}
	float dx = (hiti->getPosition()[0] -x0)/pixelsize;
	float dy = (hiti->getPosition()[1] -y0)/pixelsize;
	float dz = (hiti->getPosition()[2] -z0)/pixelsize;
	float dut = dx*utx+dy*uty+dz*utz;
	float dvt = dx*vtx+dy*vty+dz*vtz;
	//	int idut  = static_cast<int>(dut);
	//int idvt  = static_cast<int>(dvt);
	ftransprofile->Fill(dut,dvt,hiti->getEnergyEM());  
      }
    }
  }

  ftransprofile->Draw("box");
  


  fCanvas->Update();

  gSystem->ProcessEvents();

 
  theApp->SetReturnFromRun(kTRUE);

  fcntl(STDIN_FILENO, F_SETFL, O_NONBLOCK);
  int key = 0;
  while(1){
    cout <<std::flush;
    //    theApp->StartIdleing();
    gSystem->ProcessEvents();
    //theApp->StopIdleing();
    key = getchar();
    if(key=='\n') break;
     usleep(1000);
  }


  return;

}





void MyRecoDisplay::DrawByMIP(std::vector<MyCaloHitExtended*>hitsByLayer[]){

  vector <MyCaloHitExtended*>::iterator literA;
  float x[10000];
  float y[10000];
  float z[10000];
  //  float r[10000];

  fxypad->cd();
  fxyview->Draw();

  int n;



  for(int i=0; i<100; i++){
    n = 0;
    if(hitsByLayer[i].size()>0){
      literA = (hitsByLayer[i]).begin();
      while (literA != hitsByLayer[i].end()){
	if((*literA)->isCandidateMip()){
	  x[n] = (*literA)->getPosition()[0];
	  y[n] = (*literA)->getPosition()[1];
	  z[n] = (*literA)->getPosition()[2];
	  n++;
	}
	literA++;
      }
    }
    TPolyMarker* pPm = new TPolyMarker(n,x,y);
    pPm->SetMarkerStyle(20);
    pPm->SetMarkerColor(1);
    pPm->SetMarkerSize(0.5);
    //   fList->Add(pPm);
    pPm->Draw();
  }


  for(int i=0; i<100; i++){
    n = 0;
    if(hitsByLayer[i].size()>0){
      literA = (hitsByLayer[i]).begin();
      while (literA != hitsByLayer[i].end()){
	if(!(*literA)->isCandidateMip()){
	  x[n] = (*literA)->getPosition()[0];
	  y[n] = (*literA)->getPosition()[1];
	  z[n] = (*literA)->getPosition()[2];
	  n++;
	}
	literA++;
      }
    }
    TPolyMarker* pPm = new TPolyMarker(n,x,y);
    pPm->SetMarkerStyle(20);
    pPm->SetMarkerColor(5);
    pPm->SetMarkerSize(0.5);
    //   fList->Add(pPm);
    pPm->Draw();
  }





  this->DrawTracks(RECO_VIEW_XY);
  this->DrawMuonHits(RECO_VIEW_XY);
  this->DrawOutline(RECO_VIEW_XY);

  fCanvas->Update();
  gSystem->ProcessEvents();

 
  theApp->SetReturnFromRun(kTRUE);

  fcntl(STDIN_FILENO, F_SETFL, O_NONBLOCK);
  int key = 0;
  while(1){
    cout <<std::flush;
    //    theApp->StartIdleing();
    gSystem->ProcessEvents();
    //theApp->StopIdleing();
    key = getchar();
    if(key=='\n') break;
     usleep(1000);
  }

  return;

}



void MyRecoDisplay::DrawByEnergy(std::vector<MyCaloHitExtended*>hitsByLayer[]){

  vector <MyCaloHitExtended*>::iterator literA;
  float x[10000];
  float y[10000];
  float z[10000];
  //  float r[10000];
  int   n;

  fxypad->cd();
  fxyview->Draw();


  for(int ilevel=0;ilevel<nlevels;ilevel++){
    for(int i=0; i<100; i++){
      n = 0;
      if(hitsByLayer[i].size()>0){
	literA = (hitsByLayer[i]).begin();
	while (literA != hitsByLayer[i].end()){
	  float energy = (*literA)->getPseudoEnergy();
	  if( energy>=level[ilevel]&&energy<level[ilevel+1]){
	    x[n] = (*literA)->getPosition()[0];
	    y[n] = (*literA)->getPosition()[1];
	    z[n] = (*literA)->getPosition()[2];
	    n++;
	  }
	  literA++;
	}
      }
      TPolyMarker* pPm = new TPolyMarker(n,z,x);
      int colour = colours[ilevel];
      pPm->SetMarkerStyle(20);
      pPm->SetMarkerColor(colour);
      pPm->SetMarkerSize(0.5);
      //   fList->Add(pPm);
      pPm->Draw();
    }
  }

  this->DrawTracks(RECO_VIEW_XZ);
  this->DrawMuonHits(RECO_VIEW_XZ);
  this->DrawOutline(RECO_VIEW_XZ);

  fCanvas->Update();
  gSystem->ProcessEvents();

 
  theApp->SetReturnFromRun(kTRUE);

  fcntl(STDIN_FILENO, F_SETFL, O_NONBLOCK);
  int key = 0;
  while(1){
    cout <<std::flush;
    //    theApp->StartIdleing();
    gSystem->ProcessEvents();
    //theApp->StopIdleing();
    key = getchar();
    if(key=='\n') break;
     usleep(1000);
  }

  return;

}


