#include "MyTrackExtended.h"
#include "ClusterShapes.h"
#include <math.h>
#include <iostream>

MyTrackExtended::MyTrackExtended( ) {
    _track = NULL;
    _reachedECAL = true;
    _isV0 = false;
    _isKinkE = false;
    _isKinkS = false;
    _isProngE = false;
    _isProngS = false;
    _isBackScatter = false;
    _backScatterDaughters.clear();
    _backScatterParent = NULL;
    _prongDaughters.clear();
    _isSplitE = false;
    _isSplitS = false;
    _splitDaughter = NULL;
    _splitParent = NULL;
    _superCluster = NULL;
    _trackerHitVector.clear();
    _clusterVec.clear();
    _status = TRACK_STATUS_UNKNOWN;
    _kinkParent = NULL;
    _kinkDaughter = NULL;
    _mcPart = NULL;
    _helix = NULL;
    _altHelix = NULL;

}

MyTrackExtended::MyTrackExtended( Track * track) {
    _track = track;
    _reachedECAL = true;
    _isV0 = false;
    _isKinkE = false;
    _isKinkS = false;
    _isProngE = false;
    _isProngS = false;
    _prongDaughters.clear();
    _superCluster = NULL;
    _trackerHitVector.clear();
    _clusterVec.clear();
    _status = TRACK_STATUS_UNKNOWN;
    TrackerHitVec hitvec = track->getTrackerHits();   
    _kinkParent = NULL;
    _kinkDaughter = NULL;
    _mcPart = NULL;
    _helix = NULL;
    _altHelix = NULL;
    _isBackScatter = false;
    _backScatterDaughters.clear();
    _backScatterParent = NULL;
    _isSplitE = false;
    _isSplitS = false;
    _splitDaughter = NULL;
    _splitParent = NULL;

}

MyTrackExtended::MyTrackExtended( TrackerHitExtended * trackerhit) {
    _trackerHitVector.clear();
    _trackerHitVector.push_back(trackerhit);
    _reachedECAL = true;
    _track = NULL;
    _isV0 = false;
    _isKinkS = false;
    _isKinkE = false;
    _isProngE = false;
    _isProngS = false;
    _prongDaughters.clear();
    _superCluster = NULL;
    _clusterVec.clear();
    _status = TRACK_STATUS_UNKNOWN;
    _kinkParent = NULL;
    _kinkDaughter = NULL;
    _mcPart = NULL;
    _helix = NULL;
    _altHelix = NULL;
    _isBackScatter = false;
    _backScatterDaughters.clear();
    _backScatterParent = NULL;
    _isSplitE = false;
    _isSplitS = false;
    _splitDaughter = NULL;
    _splitParent = NULL;
}


MyTrackExtended::~MyTrackExtended() 
{
   if(_helix!=NULL)delete _helix;
   if(_altHelix!=NULL)delete _altHelix;
}

Track * MyTrackExtended::getTrack() {
    return _track;
}

void MyTrackExtended::setHelix(HelixClass* h) {
  if(_helix!=NULL)delete _helix;
  _helix = h;
}

HelixClass* MyTrackExtended::getHelix() {
  return _helix;
}


void MyTrackExtended::setAltHelix(HelixClass* h) {
  if(_altHelix!=NULL)delete _altHelix;
  _altHelix = h;
}

HelixClass* MyTrackExtended::getAltHelix() {
  return _altHelix;
}


bool MyTrackExtended::reachedEcal() {
    return _reachedECAL;
}

int MyTrackExtended::hits(){
  TrackerHitVec hitvec = _track->getTrackerHits();
  int nhits = (int)hitvec.size();
  return nhits;
}


int MyTrackExtended::nonSiHits(){
  TrackerHitVec hitvec = _track->getTrackerHits();
  int nhits = (int)hitvec.size();
  int count = 0;
  for(int ih =0;ih<nhits;++ih){
    float x = (float)hitvec[ih]->getPosition()[0];
    float y = (float)hitvec[ih]->getPosition()[1];
    float z = (float)hitvec[ih]->getPosition()[2];
    float r2 = x*x+y*y+z*z;
    if(r2>4225)count++;
  }

  return count;
}


const float * MyTrackExtended::getSeedPosition() {
    return _seedPosition;
}


const float * MyTrackExtended::getTpcIntersection() {
    return _tpcIntersection;
}

const float * MyTrackExtended::getTrackEndPosition() {
    return _trackEndPosition;
}

const float * MyTrackExtended::getTrackStartPosition() {
    return _trackStartPosition;
}



const float * MyTrackExtended::getSeedDirection() {
    return _seedDirection;
}


const float * MyTrackExtended::getTrackEndDirection() {
    return _trackEndDirection;
}

const float * MyTrackExtended::getTrackStartDirection() {
    return _trackStartDirection;
}

const float * MyTrackExtended::getEndcapSeedPosition() {
    return _endcapSeedPosition;
}

const float * MyTrackExtended::getEndcapSeedDirection() {
    return _endcapSeedDirection;
}

const float * MyTrackExtended::getAltSeedPosition() {
    return _altSeedPosition;
}

const float * MyTrackExtended::getAltSeedDirection() {
    return _altSeedDirection;
}


const float * MyTrackExtended::getMomentum() {
    return _momentum;
}


const float * MyTrackExtended::getEndMomentum() {
    return _endMomentum;
}

const float * MyTrackExtended::getStartMomentum() {
    return _startMomentum;
}


ProtoClusterVec & MyTrackExtended::getClusterVec() {
    return _clusterVec;
}

ProtoCluster * MyTrackExtended::getSuperCluster() {
    return _superCluster;
}

TrackerHitExtendedVec & MyTrackExtended::getTrackerHitExtendedVec() {

    return _trackerHitVector;
}

void MyTrackExtended::isV0(bool tf){
  _isV0=tf;
}

bool MyTrackExtended::isV0(){
  return _isV0;
}


void MyTrackExtended::isKinkE(bool tf){
  _isKinkS=tf;
}

bool MyTrackExtended::isKinkE(){
  return _isKinkS;
}


void MyTrackExtended::isKinkS(bool tf){
  _isKinkE=tf;
}

bool MyTrackExtended::isKinkS(){
  return _isKinkE;
}


void MyTrackExtended::addCluster(ProtoCluster * cluster) {
    _clusterVec.push_back(cluster);
}

void MyTrackExtended::setSuperCluster(ProtoCluster * superCluster) {
    _superCluster = superCluster;
}


void MyTrackExtended::setMCParticle(MCParticle* mcpart) {
    _mcPart = mcpart;
    if(_mcPart!=NULL){
      float p[4];
      p[0] = mcpart->getMomentum()[0];
      p[1] = mcpart->getMomentum()[1];
      p[2] = mcpart->getMomentum()[2];
      p[3] = sqrt(p[0]*p[0]+p[1]*p[1]+p[2]*p[2]);
      for(int i=0;i<4;i++)_momentum[i]=p[i];
      _energy = sqrt(p[3]*p[3]+0.139*0.139);
      //    std::cout << "   MC TRACK " << p[3] << " -> " << _momentum[3] << std::endl;
    }else{
      _energy = 0;
    }

}

void MyTrackExtended::setSeedDirection( float * seedDirection ) {
    _seedDirection[0] = seedDirection[0];
    _seedDirection[1] = seedDirection[1];
    _seedDirection[2] = seedDirection[2];
}

void MyTrackExtended::setSeedPosition( float * seedPosition) {
    _seedPosition[0] = seedPosition[0];
    _seedPosition[1] = seedPosition[1];
    _seedPosition[2] = seedPosition[2];
}


void MyTrackExtended::setTpcIntersection( float * pos) {
    _tpcIntersection[0] = pos[0];
    _tpcIntersection[1] = pos[1];
    _tpcIntersection[2] = pos[2];
}



void MyTrackExtended::setEndcapSeedDirection( float * seedDirection ) {
    _endcapSeedDirection[0] = seedDirection[0];
    _endcapSeedDirection[1] = seedDirection[1];
    _endcapSeedDirection[2] = seedDirection[2];
}

void MyTrackExtended::setEndcapSeedPosition( float * seedPosition) {
    _endcapSeedPosition[0] = seedPosition[0];
    _endcapSeedPosition[1] = seedPosition[1];
    _endcapSeedPosition[2] = seedPosition[2];
}



void MyTrackExtended::setTrackEndDirection( float * seedDirection ) {
    _trackEndDirection[0] = seedDirection[0];
    _trackEndDirection[1] = seedDirection[1];
    _trackEndDirection[2] = seedDirection[2];
}


void MyTrackExtended::setTrackStartDirection( float * seedDirection ) {
    _trackStartDirection[0] = seedDirection[0];
    _trackStartDirection[1] = seedDirection[1];
    _trackStartDirection[2] = seedDirection[2];
}

void MyTrackExtended::setTrackEndPosition( float * seedPosition) {
    _trackEndPosition[0] = seedPosition[0];
    _trackEndPosition[1] = seedPosition[1];
    _trackEndPosition[2] = seedPosition[2];
}

void MyTrackExtended::setTrackStartPosition( float * seedPosition) {
    _trackStartPosition[0] = seedPosition[0];
    _trackStartPosition[1] = seedPosition[1];
    _trackStartPosition[2] = seedPosition[2];
}


void MyTrackExtended::setAltSeedDirection( float * seedDirection ) {
    _altSeedDirection[0] = seedDirection[0];
    _altSeedDirection[1] = seedDirection[1];
    _altSeedDirection[2] = seedDirection[2];
}

void MyTrackExtended::setAltSeedPosition( float * seedPosition) {
    _altSeedPosition[0] = seedPosition[0];
    _altSeedPosition[1] = seedPosition[1];
    _altSeedPosition[2] = seedPosition[2];
}


void MyTrackExtended::setMomentum( float * p) {
    _momentum[0] = p[0];
    _momentum[1] = p[1];
    _momentum[2] = p[2];
    _momentum[3] = sqrt(p[0]*p[0]+p[1]*p[1]+p[2]*p[2]);
}


void MyTrackExtended::setStartMomentum( float * p) {
    _startMomentum[0] = p[0];
    _startMomentum[1] = p[1];
    _startMomentum[2] = p[2];
    _startMomentum[3] = sqrt(p[0]*p[0]+p[1]*p[1]+p[2]*p[2]);
}

void MyTrackExtended::setEndMomentum( float * p) {
    _endMomentum[0] = p[0];
    _endMomentum[1] = p[1];
    _endMomentum[2] = p[2];
    _endMomentum[3] = sqrt(p[0]*p[0]+p[1]*p[1]+p[2]*p[2]);
}

void MyTrackExtended::setTrackStatus(MyTrackStatus status){

  if(_status==TRACK_STATUS_KILLED)std::cout << " Setting KILLED track status to " << status << std::endl;
  _status = status;
}


void MyTrackExtended::addTrackerHitExtended( TrackerHitExtended * trackerhit) {
  _trackerHitVector.push_back(trackerhit);

}

void MyTrackExtended::ClearTrackerHitExtendedVec() {
  _trackerHitVector.clear();
}


void MyTrackExtended::ClearClusters() {
  _clusterVec.clear();
}


void MyTrackExtended::setX0( float x0 ) {
  _x0 = x0;
}

void MyTrackExtended::setD0( float d0 ) {
  _d0 = d0;
}

void MyTrackExtended::setZ0( float z0 ) {
  _z0 = z0;
}

void MyTrackExtended::setY0( float y0 ) {
  _y0 = y0;
}

void MyTrackExtended::setR0( float r0 ) {
  _r0 = r0;
}

void MyTrackExtended::setBz( float bz ) {
  _bz = bz;
}

void MyTrackExtended::setPhi0( float phi0 ) {
  _phi0 = phi0;
}

void MyTrackExtended::setPhi( float phi ) {
  _phi = phi;
}

void MyTrackExtended::setOmega( float omega ) {
  _omega = omega;
}

void MyTrackExtended::setTanLambda( float tanLambda ) {
  _tanLambda = tanLambda;
}

void MyTrackExtended::setEnergy( float E) {
  _energy = E;
}

void MyTrackExtended::setZMin( float z) {
  _zmin = z;
}

void MyTrackExtended::setZMax( float z) {
  _zmax = z;
}


float MyTrackExtended::getZMin() {
  return _zmin;
}

float MyTrackExtended::getZMax() {
  return _zmax;
}



void MyTrackExtended::setInnerR( float rinner) {
  _innerR = rinner;
}


void MyTrackExtended::setOuterR( float router) {
  _outerR = router;
}



float MyTrackExtended::getX0() {
  return _x0;
}

float MyTrackExtended::getY0() {
  return _y0;
}

float MyTrackExtended::getR0() {
  return _r0;
}

float MyTrackExtended::getBz() {
  return _bz;
}


float MyTrackExtended::getInnerR() {
  return _innerR;
}

float MyTrackExtended::getOuterR() {
  return _outerR;
}

float MyTrackExtended::getD0() {
  return _d0;
}

float MyTrackExtended::getZ0() {
  return _z0;
}

float MyTrackExtended::getPhi0() {
  return _phi0;
}

float MyTrackExtended::getPhi() {
  return _phi;
}

float MyTrackExtended::getOmega() {
  return _omega;
}

float MyTrackExtended::getTanLambda() {
  return _tanLambda;
}

float MyTrackExtended::getEnergy() {
  return _energy;
}

void MyTrackExtended::setStart(float * xStart) {
  _xStart[0] = xStart[0];
  _xStart[1] = xStart[1];
  _xStart[2] = xStart[2];
}

void MyTrackExtended::setEnd(float * xEnd) {
  _xEnd[0] = xEnd[0];
  _xEnd[1] = xEnd[1];
  _xEnd[2] = xEnd[2];
}

float * MyTrackExtended::getStart() {
  return _xStart;
}

float * MyTrackExtended::getEnd() {
  return _xEnd;
}

void MyTrackExtended::setGroupTracks( GroupTracks * group ) {
  _group = group;
}

GroupTracks * MyTrackExtended::getGroupTracks() {
  return _group;
}

