#include "PandoraPFAEventDisplay.h"
#include "EVENT/LCIO.h"
#include "EVENT/LCRunHeader.h"
//#include "EVENT/LCParameters.h"
#include "EVENT/LCCollection.h"
#include "EVENT/SimCalorimeterHit.h"
#include "EVENT/CalorimeterHit.h"
#include "IMPL/ClusterImpl.h"
#include "EVENT/MCParticle.h"
#include "EVENT/Cluster.h"
#include "EVENT/Track.h"
#include "EVENT/ReconstructedParticle.h"
#include "IMPL/LCFlagImpl.h"
// Marlin stuff
#include <marlin/Global.h>
// the event display
#include "MyRecoDisplay.h"

using namespace lcio;

PandoraPFAEventDisplay aPandoraPFAEventDisplay;

PandoraPFAEventDisplay::PandoraPFAEventDisplay() : marlin::Processor("PandoraPFAEventDisplay") {

  std::string a = "JetAtDetector";
  registerProcessorParameter( "JetCollection" ,
                              "Jet Collection to display (data only)",
			      _jetCollectionName,
			       a);


  std::string b = "PandoraPFOs";
  registerProcessorParameter( "PFOCollection" ,
                              "PFO Collection to display (data only)",
			      _pfoCollectionName,
			       b);


  registerProcessorParameter( "DrawXY" ,
                              "Show XY view ?",
			      _drawXY,
			      (int)1 );


  registerProcessorParameter( "DrawUnused" ,
                              "Draw Unused TPC hits",
			      _drawUnused,
			      (int)0 );


  registerProcessorParameter( "DrawXZ" ,
                              "Show XZ view ?",
			      _drawXZ,
			      (int)1 );

  registerProcessorParameter( "DrawRZ" ,
                              "Show RZ view ?",
			      _drawRZ,
			      (int)1 );

  registerProcessorParameter( "DrawJets" ,
                              "Show Jets",
			      _drawJets,
			      (int)1 );


  registerProcessorParameter( "DrawPFOs" ,
                              "Show PFOs",
			      _drawPFOs,
			      (int)0 );

  registerProcessorParameter( "RecoDisplayPlotRange" ,
                              "zyz range to be displayed ",
			      _plotRange,
			      (float)4250. );


}


void PandoraPFAEventDisplay::init() { 

  _recoDisplay = new MyRecoDisplay(1);
  _recoDisplay->SetPlotRange(_plotRange);


}

void PandoraPFAEventDisplay::processRunHeader( LCRunHeader* run) { 
  _nRun++ ;
}

void PandoraPFAEventDisplay::processEvent( LCEvent * evt ) { 


  _recoDisplay->Clear();
  if(_nEvt%100==0)std::cout << _nEvt << " events" << std::endl; 

  if(_drawPFOs>0)DrawPFOs( evt );
  if(_drawJets>0)JetsAtDetector( evt );

  _nEvt++;
}

void PandoraPFAEventDisplay::end(){ 

}

void PandoraPFAEventDisplay::JetsAtDetector( LCEvent* evt ) {

  //Jets at detector level: PFlow as input
  try {
    LCCollection* col = evt->getCollection(_jetCollectionName);
    if(_drawXY>0)_recoDisplay->DrawByJet(col,RECO_VIEW_XY);
    if(_drawXZ>0)_recoDisplay->DrawByJet(col,RECO_VIEW_XZ);
  }
  catch(DataNotAvailableException) {}
  
}

void PandoraPFAEventDisplay::DrawPFOs( LCEvent* evt ) {

  //Jets at detector level: PFlow as input
  try {
    LCCollection* col = evt->getCollection(_pfoCollectionName);
    if(_drawXY>0){
      _recoDisplay->DrawByParticle(col,RECO_VIEW_XY);
      if(_drawUnused>0)_recoDisplay->DrawUnusedHits(RECO_VIEW_XY);
    }
    if(_drawXZ>0){
      _recoDisplay->DrawByParticle(col,RECO_VIEW_XZ);
      if(_drawUnused>0)_recoDisplay->DrawUnusedHits(RECO_VIEW_XZ);
    }
    if(_drawRZ>0){
      _recoDisplay->SetPseudoThrust(col);
      _recoDisplay->DrawByParticle(col,RECO_VIEW_RZ);
      if(_drawUnused>0)_recoDisplay->DrawUnusedHits(RECO_VIEW_RZ);
    }
  }
  catch(DataNotAvailableException) {}
  
}
