#include "ProtoClusterFactory.h"
#include "ProtoCluster.h"
#include "MyTrackExtended.h"
#include "MyCaloHitExtended.h"
#include <iostream>
using std::cout;
using std::endl;

ProtoClusterFactory* ProtoClusterFactory::_instance =0;

ProtoClusterFactory* ProtoClusterFactory::Instance(){
  if(_instance==0){
    _instance = new ProtoClusterFactory;
  }
  return _instance;
}

ProtoClusterFactory::ProtoClusterFactory(){
  
  _reclustering = false;
  _uniqueID = 0;
  _design.tanConeAngleECAL    = 0.4;
  _design.tanConeAngleHCAL    = 1.0;
  _design.additionalPadsECAL  = 1.5;
  _design.additionalPadsHCAL  = 2.5;
  _design.sameLayerPadCutECAL = 1.8;
  _design.sameLayerPadCutHCAL = 1.8;
  _design.maximumDistanceForConeAssociationECAL = 250.;
  _design.maximumDistanceForConeAssociationHCAL = 500.;
  _design.trackingWeight = 1;
  _design.trackingCutOff = 0;
  _design.trackingTube   = 0.;

  return;
}

void ProtoClusterFactory::Delete(){
  
  for(unsigned int i=0;i<_clusters.size();++i)delete _clusters[i];
  _clusters.erase(_clusters.begin(),_clusters.end());
  _uniqueID = 0;

  return;
}

void ProtoClusterFactory::ManageMemory(){
  
  ProtoClusterVec temp;
  temp.clear();

  int count = 0;
  for(unsigned int i=0;i<_clusters.size();++i){
    if(_clusters[i]->IsDefunct()){
      delete _clusters[i];
      count++;
    }else{
      temp.push_back(_clusters[i]);
    }
  }  

  if(count>0){
    _clusters.clear();
    for(unsigned int i=0;i<temp.size();++i){
      _clusters.push_back(temp[i]);
    }
  }
  //  std::cout << " Deleted : " << count << " leaving    Alive " << _clusters.size() << " from " << _uniqueID << std::endl; 
  
  return;
}


void ProtoClusterFactory::ManageMemoryDuringReclustering(){
  
  ProtoClusterVec temp;
  temp.clear();

  int count = 0;
  for(unsigned int i=0;i<_clusters.size();++i){
    if(_clusters[i]->GetTemporaryReclusteringFlag()){
      count++;
      delete _clusters[i];
    }else{
      temp.push_back(_clusters[i]);
    }
  }  

  if(count>0){
    _clusters.clear();
    for(unsigned int i=0;i<temp.size();++i){
      _clusters.push_back(temp[i]);
    }
  }
  
  //  std::cout << " Deleted : " << count << " leaving  R Alive " << _clusters.size() << " from " << _uniqueID << std::endl; 
  
  return;
}


ProtoCluster* ProtoClusterFactory::Manufacture(MyTrackExtended* track){

  ProtoCluster* p = new ProtoCluster(track);
  p->SetID(_uniqueID);
  p->SetZOfEndcap(_zOfEndcap);
  p->SetZOfEndcap(_zOfBarrel);
  p->ConfigureGrowth(_design);
  p->SetPhotonIDCuts(_photonID_dcoscut);
  _uniqueID++;
  _clusters.push_back(p);

  if(_reclustering)p->SetTemporaryReclusteringFlag(true);

  return p;

}

void ProtoClusterFactory::ZOfEndcap(float z){
  _zOfEndcap = z;

}
void ProtoClusterFactory::ZOfBarrel(float z){
  _zOfBarrel = z;
}

ProtoCluster* ProtoClusterFactory::Manufacture(MyCaloHitExtended* hit){

  ProtoCluster* p = new ProtoCluster(hit);
  p->SetID(_uniqueID);
  p->SetZOfEndcap(_zOfEndcap);
  p->SetZOfBarrel(_zOfBarrel);
  p->ConfigureGrowth(_design);
  p->SetPhotonIDCuts(_photonID_dcoscut);
  _uniqueID++;
  _clusters.push_back(p);
  if(_reclustering)p->SetTemporaryReclusteringFlag(true);

 
  return p;


}

protoClusterDesign_t ProtoClusterFactory::GetDesign(){
  return _design;
}

ProtoCluster* ProtoClusterFactory::CloneAndDelete(ProtoCluster* pC){

  // clone a cluster and the delete original
  ProtoCluster* p = NULL;
  for(int ihit = 0;ihit<pC->Hits();++ihit){
    MyCaloHitExtended* hiti = pC->Hit(ihit);
    if(p==NULL){
      p = this->Manufacture(hiti);
    }else{
      p->AddHit(hiti);
    }
  }
  // Make sure the new cluster properties are uptodate
  if(p!=NULL){
    p->Update(MAX_NUMBER_OF_LAYERS);
    p->ClassifyYourself();
  }
  delete pC;
  return p;

}

ProtoCluster* ProtoClusterFactory::Clone(ProtoCluster* pC){

  // clone a cluster and the delete original
  ProtoCluster* p = NULL;
  for(int ihit = 0;ihit<pC->Hits();++ihit){
    MyCaloHitExtended* hiti = pC->Hit(ihit);
    if(p==NULL){
      p = this->Manufacture(hiti);
    }else{
      p->AddHit(hiti);
    }
  }
  // Make sure the new cluster properties are uptodate
  if(p!=NULL){
    p->Update(MAX_NUMBER_OF_LAYERS);
    p->ClassifyYourself();
  }

  return p;

}


ProtoCluster* ProtoClusterFactory::Merge(ProtoCluster* pCi, ProtoCluster* pCj){

  ProtoCluster* p = NULL;
  for(int ihit = 0;ihit<pCi->Hits();++ihit){
    MyCaloHitExtended* hiti = pCi->Hit(ihit);
    if(p==NULL){
      p = this->Manufacture(hiti);
    }else{
      p->AddHit(hiti);
    }
  }
  for(int ihit = 0;ihit<pCj->Hits();++ihit){
    MyCaloHitExtended* hiti = pCj->Hit(ihit);
    if(p==NULL){
      p = this->Manufacture(hiti);
    }else{
      p->AddHit(hiti);
    }
  }

  // Make sure the new cluster properties are uptodate
  if(p!=NULL){
    p->Update(MAX_NUMBER_OF_LAYERS);
    p->ClassifyYourself();
  }
  return p;

}


void ProtoClusterFactory::SetDesign(const protoClusterDesign_t design){
  _design = design;
}


void ProtoClusterFactory::SetDefaultDesign(const protoClusterDesign_t design){
  _defaultDesign = design;
}

void ProtoClusterFactory::SetDesignToDefault(){
  _design = _defaultDesign;
}

