#include "SiTrkClusteringProcessor.h"
#include "SiUtil.h"
#include "SiClusterizer.h"

namespace SICLUPROC{

bool TrkHit_debug = false ;

SiTrkClusteringProcessor aSiTrkClusteringProcessor ;

 /** 
  *  C O N S T R U C T O R
  */
SiTrkClusteringProcessor::SiTrkClusteringProcessor() : Processor("SiTrkClusteringProcessor") {
  
  // Processor description
  _description = "SiTrkClusteringProcessor: to find hits from digis of Si tracker subdetector." ;
  
 _detector = new SIDET::SiTracker();

 registerInputCollection(  LCIO::TRACKERRAWDATA,
                           "InputCollectionName", 
                           "Name of the TrackerRawData collection",
                           _inputColName, SIUTIL::get_sDefault() ) ;

 registerInputCollection(  LCIO::TRACKERRAWDATA,
                           "InputCollectionName_second" , 
                           "Name of the second TrackerRawData collection",
                           _inputColName_second, SIUTIL::get_sDefault() ) ;

 registerInputCollection(  LCIO::LCRELATION,
                           "InputRelColName", 
                           "Name of the raw to sim relation-collection",
                           _inputRelColName, SIUTIL::get_sDefault() ) ;

 registerOutputCollection( LCIO::TRACKERHIT,
                           "OutputCollections", 
                           "Name of the output TrackerHit collection",
                            _outputColName, SIUTIL::get_sDefault() ) ;

 registerOutputCollection( LCIO::LCRELATION ,
                           "OutputRelColName", 
                           "Name of the output raw trackerhit relation collection" ,
                            _outputRelColName, SIUTIL::get_sDefault() ) ;
 /** Parameters :
  */

 registerProcessorParameter("PrintRunHeaderInform",
                            " equals 1 if print run header information",
			    _printRunHeaderInform, int(0) ) ;

 registerProcessorParameter("SubDet_Technology",
 			    "Subdetector name, type of technology"
			    ,_subDet_Technology, SIUTIL::get_sVecDefault() ) ;

// registerProcessorParameter("Layers",
//                            "Layers to find its hits",
//			    _iLayers, SIUTIL::get_iVecDefault() ) ;

} // constructor SiTrkClusteringProcessor.

 /** 
  */
void SiTrkClusteringProcessor::init() { 

try{ 
  printParameters ( ) ; 

 _detector->setTechnology( _subDet_Technology ) ;

 string CC = "From SiTrkClusteringProcessor::init() for " + name() + ":" ;

 _detector->gearInit(  "From SiTrkClusteringProcessor::init() for " + name() + ":"  ) ;

#ifdef _DEBUG_CLUST_
cout <<"From SiTrkClusteringProcessor::init() for " + name() + ":" << endl
     << " subDet = " << sSubDet( _detector->getSubDet() )
     << " Technology = " << sDetUnitType( _detector->getDetUnitType() ) << endl ;
#endif

  if (   (_detector->getDetUnitType() == SIUTIL::barrelZStrips)
       ||(_detector->getDetUnitType() == SIUTIL::barrelPhiStrips) 
       ||(_detector->getDetUnitType() == SIUTIL::diskPhiStrips) 
       ||(_detector->getDetUnitType() == SIUTIL::diskRStrips) 
      )  _stechnology = "strips" ;
  else if(
         (_detector->getDetUnitType() == SIUTIL::barrelPixels)
       ||(_detector->getDetUnitType() == SIUTIL::barrelLaddersPixels)
       ||(_detector->getDetUnitType() == SIUTIL::diskRectanglePixels)
         ) _stechnology = "pixels" ;
  else 
    throw(string(CC+" no such detUnitType "+sDetUnitType( _detector->getDetUnitType() ) ) );



} catch( string report ) {

   cout << "SiTrkClusteringProcessor::init():" << endl 
        <<" Report:"<< report+". Program terminated." << endl;  exit( 0 );

}

 _nRun = 0 ;
 _nEvt = 0 ;

} // init().


 /**  
  */
void SiTrkClusteringProcessor::processRunHeader( LCRunHeader* run ) { 
 string CC = "SiTrkClusteringProcessor::processRunHeader for " + name() + ":" ;

#ifdef _DEBUG_CLUST_
cout << CC + " start." << endl ;
#endif

 /**  add new header parameters 
  */
  run->parameters().setValue ("[ProcessorName]", name() ) ;
  run->parameters().setValue (name()+"[ProcessorType]"   , "SiTrkClusteringProcessor" ) ;
  run->parameters().setValue (name()+"[InputCollection]" , _inputColName ) ;
  run->parameters().setValue (name()+"[OutputCollection]", _outputColName ) ;

 /**  show all run header parameters
  */
  if( _printRunHeaderInform == 1 ) SIUTIL::ShowRunHeaderParameters( CC + " Before add new run parameters: ", run ) ;

 /**  show all run header parameters
  */
  if( _printRunHeaderInform == 1 ) SIUTIL::ShowRunHeaderParameters( CC + "  After add new run parameters: ", run ) ;

  _nRun++ ;

}  // processRunHeader().


 /** 
  */
void SiTrkClusteringProcessor::processEvent( LCEvent * evt ) { 

 string CC = "SiTrkClusteringProcessor::processEvent for " + name() + ":" ;
 cout << endl <<">>> processEvent(" << evt->getEventNumber() 
      << ") by Processor " << name() 
      << ", Run N " << evt->getRunNumber() << endl ;

 ++_nEvt ;

 LCCollection* inColl = NULL ;
 LCCollection* inColl_second = NULL ;
 LCCollection* inRelColl = NULL ;

try{

  inColl = evt->getCollection( _inputColName );
  
//  SiTrkDetUnitClusterizer::_decoder 
//            = new UTIL::CellIDDecoder<TrackerRawDataImpl>( inColl ) ;

  SIDETUNIT::DetUnit::_decoder = new UTIL::CellIDDecoder<TrackerRawDataImpl>( inColl ) ;

  inRelColl = evt->getCollection( _inputRelColName ) ;

  if ( _stechnology == "strips" ) 
    inColl_second = evt->getCollection( _inputColName_second );
  else if( _stechnology != "pixels" ) 
    throw(string(CC+" no such detUnitType "+sDetUnitType( _detector->getDetUnitType() ) ) );

 if( inColl == NULL ) 
   throw CC + " It impossible after evt->getCollection: input collection does not exists." ;

  vector<LCCollection*> outCollVec ;

  if ( _stechnology == "pixels" ) {

    createOutputCollections_pixels( inColl, inRelColl, outCollVec ) ;

    evt->addCollection( outCollVec[0], _outputColName ) ;

  } else  throw " such technology (" + _stechnology +"), det.unit type="
             + sDetUnitType( _detector->getDetUnitType() )
             + " is not realized yet, for subdet " + sSubDet( _detector->getSubDet() ) ;

} catch(EVENT::DataNotAvailableException& e){
   cout << CC << e.what() << endl ;
   return;
} catch( string report ) {
   cout << CC + report + " Program terminated." << endl;  exit( 0 ) ;
} catch(...) {
   cout << CC + " Unknown exeption. Continue..." << endl;
}


} // processEvent().


/**
 */
void SiTrkClusteringProcessor::check( LCEvent * evt ) { 
  // nothing to check here - could be used to fill checkplots in reconstruction processor
}

/**
 */
void SiTrkClusteringProcessor::end(){ 
//   cout << "SiTrkClusteringProcessor::end()  " << name() 
// 	    << " processed " << _nEvt << " events in " << _nRun << " runs "
// 	    << endl ;

}

} // namespace SITRK.
