#ifndef PixelReco_BinaryClusterParameterEstimator_H
#define PixelReco_BinaryClusterParameterEstimator_H

#include "ClusterParameterEstimator.h"
#include "Cluster.h"
#include "DetGeometry/GeomVector/LocalVector.h"

#include "Pixel/SiRectangularDetUnit.h"

/** CMS software, ORCA.
 *   Estimates the position of pixel hits as the digital (binary) average,
 *   that is the center of the cluster is used.
 *
 *  Adopted in MarlinReco : S Shulga 2007/06/06
 *  @version $Id: BinaryClusterParameterEstimator.h, v 1 2007/06/06 shulga
 */

class Topology;

namespace SICLU {
  
  class BinaryClusterParameterEstimator : public ClusterParameterEstimator {
    
  public:
    
    BinaryClusterParameterEstimator( const SIDETUNIT::DetUnit&);
    
    virtual SIUTIL::LocalError localError( const Cluster&) const;
    
    virtual SIUTIL::LocalPoint localPosition( const Cluster&) const;
    
  private:
    
    const SIDETUNIT::DetUnit* theDet;
    const SIUTIL::Topology& theTopol;
    
    float theXErr2;
    float theYErr2;
    
    float lorentzShift() const;
  };
  
} //namespace SICLU.

#endif
