#ifndef CPEFromDetPosition_H
#define CPEFromDetPosition_H

#define _DEBUG_CPEFROMDETPOSITION_

#include <utility>
#include <vector>


#include "Pixel/SiRectangularDetUnit.h"

#include "Cluster.h"
#include "ClusterParameterEstimator.h"
#include "Enumerators.h"
#include "EtaCorrection.h"
#include "DetGeometry/GeomVector/LocalPoint.h"
#include "DetGeometry/LocalError.h"
#include "DetGeometry/MeasurementError.h"
#include "DetGeometry/MeasurementPoint.h"

/** CMS software, ORCA.
 *  Perform the position and error evaluation of pixel hits using 
 *  the Det angle to estimate the track impact angle 
 *
 *  Adapted in MarlinReco : S Shulga 2007/06/06
 *  @version $Id: CPEFromDetPosition.h, v 1 2007/06/06 shulga
 */

namespace SICLU {
  
  class CPEFromDetPosition : public ClusterParameterEstimator {
    
  public:
    CPEFromDetPosition( const SIDETUNIT::DetUnit& det );
    
    SIUTIL::MeasurementError measurementError(const Cluster&)const;
    
    SIUTIL::LocalError localError( const Cluster&) const;
    
    SIUTIL::MeasurementPoint measurementPosition( const Cluster&) const;
    
    SIUTIL::LocalPoint localPosition(const Cluster&) const;
    
    float chaWidth2X(const float&) const;

  private:
    //members
    const SIDETUNIT::DetUnit* theDet;
    Part thePart;
    EtaCorrection theEtaFunc;
    float theThickness;
    float thePitchX;
    float thePitchY;
    float theOffsetX;
    float theOffsetY;
    float theNumOfRow;
    float theNumOfCol;
    float theDetZ;
    float theDetR;
    float theLShift;
    float theSign;

    //methods
    float err2X(bool&, int&) const;
    float err2Y(bool&, int&) const;
    bool isFlipped() const;
    // parameters for the position assignment
    float chargeWidthX()const;
    float chargeWidthY()const;
    float geomCorrection()const;
    float estimatedAlphaForBarrel(const float&) const;

    // Determine the Lorentz shift correction
    float lorentzShift()const;
    // returns position in x
    float xpos( const Cluster& ) const;
    //returns position in y
    float ypos( const Cluster& ) const;

    // charge on the first, last  and  inner pixels on x and y 
    vector<float> 
      xCharge(const vector<Cluster::Pixel>&, const float&, const float&)const; 
    vector<float> 
      yCharge(const vector<Cluster::Pixel>&, const float&, const float&)const; 
  };
}  // namespace SICLU.
#endif
