#ifndef PixelReco_FixedClusterParameterEstimator_H
#define PixelReco_FixedClusterParameterEstimator_H

#include <iostream>

#include "ClusterParameterEstimator.h"
#include "Cluster.h"

#include "DetGeometry/GeomVector/LocalPoint.h"
#include "Pixel/SiRectangularDetUnit.h"
#include "Topology.h"


/** CMS software, ORCA.
 *  Estimates the position of pixel hits as the berycenter of cluster and 
 *
 *  Adapted in MarlinReco : S Shulga 2007/06/06
 *  @version $Id: EtaCorrection.h, v 1 2007/06/06 shulga
 */

namespace SICLU {
  
  class FixedClusterParameterEstimator : public ClusterParameterEstimator {
    
  public:
    
    FixedClusterParameterEstimator( const SIDETUNIT::DetUnit&);
    
    virtual SIUTIL::LocalError localError( const Cluster&) const;
    
    virtual SIUTIL::LocalPoint localPosition( const Cluster&) const;
    
  private:
    
    const SIUTIL::Topology& theTopol;
    const SIDETUNIT::DetUnit* theDet;
    
    float theXErr2;
    float theYErr2;
    
    float lorentzShift() const;
  };
  
} // namespace SICLU.

#endif
