#ifndef _TRACKER_PIXEL_ARRAY_BUFFER_H_
#define _TRACKER_PIXEL_ARRAY_BUFFER_H_

#include <vector>

#include "PixelPos.h"

/** CMS software, ORCA.
 *  Define the buffer where the pixel adc  are stored.
 *  The size is the number of rows and cols into a
 *  ROC and it is set in the Clusterizer
 *
 *  Adapted in MarlinReco : S Shulga 2007/06/06
 *  @version $Id: PixelArrayBuffer.h , v 1 2007/06/06 shulga
 */
namespace SICLU {

class PixelArrayBuffer {

public:

  inline PixelArrayBuffer( int rows, int cols);
  inline PixelArrayBuffer( ){}

  inline void setSize( int rows, int cols);

  inline int operator()( int row, int col) const;
  inline int operator()( const PixelPos&) const;

  inline int rows() const { return nrows;}
  inline int columns() const { return ncols;}

  inline bool inside(int row, int col) const;
  inline void set_adc( int row, int col, int adc);
  inline void set_adc( const PixelPos&, int adc);
  int size() const { return pixel_vec.size();}

  /// Definition of indexing within the buffer.
  int index( int row, int col) const {return col*nrows+row;}
  int index( const PixelPos& pix) const {return index(pix.row(),pix.col());}

private:

  int nrows ;
  int ncols ;

  vector<int> pixel_vec ;

} ; // class PixelArrayBuffer.

PixelArrayBuffer::PixelArrayBuffer( int rows, int cols) : 
  nrows(rows), ncols(cols) {
  pixel_vec.resize(rows*cols);

  // Some STL implementations have problems with default values 
  // so a initialization loop is used instead
  for (vector<int>::iterator i=pixel_vec.begin(); i!=pixel_vec.end(); i++) {
    *i = 0;
  }
}

void PixelArrayBuffer::setSize( int rows, int cols) {
  nrows = rows;
  ncols = cols;
  pixel_vec.resize(rows*cols);
 // cout << " Resize the clusterize pixel buffer " << (rows*cols) << endl;
  // Some STL implementations have problems with default values 
  // so a initialization loop is used instead
  for (vector<int>::iterator i=pixel_vec.begin(); i!=pixel_vec.end(); i++) {
    *i = 0;
  }
}

bool PixelArrayBuffer::inside(int row, int col) const {
  return ( row >= 0 && row < nrows && col >= 0 && col < ncols);
}

int PixelArrayBuffer::operator()(int row, int col) const {
  if (inside(row,col))  return pixel_vec[index(row,col)];
  else  return 0;
}

int PixelArrayBuffer::operator()(const PixelPos& pix) const {
  if (inside( pix.row(), pix.col())) return pixel_vec[index(pix)];
  else return 0;
}

// unchecked!
void PixelArrayBuffer::set_adc( int row, int col, int adc) {
  pixel_vec[index(row,col)] = adc;
}

void PixelArrayBuffer::set_adc( const PixelPos& pix, int adc) {
  pixel_vec[index(pix)] = adc;
}

} // namespace SICLU.

#endif
