#ifndef _TRACKER_PIXEL_ERROR_MATRIX_H_
#define _TRACKER_PIXEL_ERROR_MATRIX_H_

#include <utility>
#include <vector>
#include <string>
#include <iosfwd>
#include <iostream>
#include <fstream>
#include <iterator>
#include <cmath>


#include "Enumerators.h"
#include "SiUtil.h"

using namespace std ;

/** CMS software, ORCA.
 *  Performs a 3-dimensional parametrization of pixel hits
 *  as a funcion of the cluster size and the two projections
 *  of the impact track angle 
 *  !!! it is valid for 100 X 150 mu pixel size !!!
 *
 *  Adapted in MarlinReco : S Shulga 2007/06/06
 *  @version $Id: PixelClusterizer.h, v 1 2007/06/06 shulga
 */


namespace SICLU{

class PixelErrorParametrization{

public:

  PixelErrorParametrization();
  ~PixelErrorParametrization();


  pair<float,float> 
    getError(Part pixelPart, int sizex, int sizey, 
	      float alpha, float beta);

private:
  float a_min; 
  float a_max;
  float a_bin; 
  string theParametrizationType;
  typedef vector<vector<vector<float> > > P3D; 
  typedef vector<vector<float> > P2D;

  // MATRIX FOR Y BARREL ERRORS
  //
  // vector depending on cluster size:
  P3D ybarrel_3D;

  //vector containing beta range
  vector<pair<float,float> > brange_yb;
  //
  // MATRIX FOR X BARREL ERRORS
  //
  // vector depending cluster size
  P3D xbarrel_3D;

  // 2D vector containing beta bins 
  // for X-BARREL depending on x cluster size
  P2D bbins_xb;
  //
  // MATRIX FOR Y FORWARD ERRORS
  //
  // vector depending cluster size
  P3D yforward_3D;

  // pair containing abs(pi/2-beta) range 
  // for Y-FORWARD
  pair<float,float> brange_yf;

  // MATRIX FOR X FORWARD ERRORS
  //
  // vector depending cluster size
  P3D xforward_3D;

  // 1D vector containing beta bins 
  // for X-FORWARD
  vector<float> bbins_xf;

  float error_XB(int sizex, float alpha, float beta);
  float error_XF(int sizex, float alpha, float beta);
  float error_YB(int sizey, float alpha, float beta);
  float error_YF(int sizey, float alpha, float beta);
  float interpolation(vector<float>&, float&, 
		      pair<float,float>&);
  int betaIndex(int&, vector<float>&, float&);
  float linParametrize(bool&, int&, int&, float&);  

  float quadParametrize(bool&, int&, int&, float&);  
/*
  void readXB( P3D& vec3D, const string& prefix, 
	       const string& postfix1, const string& postfix2);

  void readYB( P3D& vec3D, const string& prefix, 
	       const string& postfix1, const string& postfix2);

  void readF( P3D& vec3D, const string& prefix, 
	      const string& postfix1, const string& postfix2);

  vector<float> readVec( const string& name);

*/


};

} // namespace SICLU.

#endif
