#ifndef SiTrkDetUnitClusterizer_H
#define SiTrkDetUnitClusterizer_H

//#define _DEBUG_CLUSTER_

#include <stack>

#include "PixelClusterizer.h"
#include "PixelArrayBuffer.h"
#include "PixelPos.h"

#include "Cluster.h"
#include "ClusterParameterEstimatorFactory.h"

#include "Pixel/SiRectangularDetUnit.h"

//#include "UTIL/CellIDDecoder.h"
#include "UTIL/CellIDEncoder.h"
#include "UTIL/BitField64.h"

/** ORCA, CMS software, 2006
 *
 *  class SiTrkDetUnitClusterizer
 *  Sets the PixelArrayBuffer dimensions and pixel thresholds.
 *  Makes clusters and stores them in theCache if the option
 *  useCache has been set.
 *
 *  Adapted in MarlinReco, Sergey Shulga, May, 2007
 *  @version $Id: SiTrkDetUnitClusterizer.h, v 1 2007/06/06 shulga
 */

namespace SICLU{

class SiTrkDetUnitClusterizer : public PixelClusterizer {
public:

  SiTrkDetUnitClusterizer();

  ~SiTrkDetUnitClusterizer();

  virtual void clusterize( SIDETUNIT::DetUnit * rd );

  vector<Cluster> clusters( SIDETUNIT::DetUnit *rd );

  vector<TrackerHitImpl *> & getRecHits() { return theRecHits ; };


private:

  void addRawHitsToCollection( const Cluster & cl, 
	                       TrackerHitImpl * trkHit,
			       SIDETUNIT::DetUnit * rd ) ;
  // data
  PixelArrayBuffer theBuffer ;
  vector<PixelPos> theSeeds ;
  vector<Cluster>  theClusters ;

  vector<TrackerHitImpl *> theRecHits ;

  int   thePixelThreshold ;   //Pixel threshold in adc counts
  int   theSeedThreshold ;    //Seed threshold in units of adc counts 
  float theClusterThreshold ; //Cluster threshold in units of adc counts

  int  theNumOfRows ;
  int  theNumOfCols ;

  ClusterParameterEstimatorFactory * theEstimatorFactory ;

  // methods
  bool setup( SIDETUNIT::DetUnit * rd) throw(string) ;

  void copy_to_buffer( SIDETUNIT::DetUnit * readout ) throw(string) ;   

  void clear_buffer( SIDETUNIT::DetUnit * readout );   

  Cluster make_cluster( const PixelPos& pix) ;

  bool  bufferAlreadySet;
  
  CellIDEncoder<TrackerRawDataImpl> *_encoderTrackerRawData ;

  BitField64 *_bf64 ;

// public:
//  static BitField64 *_bbf ;
  
};

} // namespace SICLU.

#endif
