#include "BinaryClusterParameterEstimator.h"

#include <iostream>

namespace SICLU {

BinaryClusterParameterEstimator::
BinaryClusterParameterEstimator( const SIDETUNIT::DetUnit& det) :
  theDet(&det), theTopol( (det.topology()) )
{
  static float xerr = 0.001;
  static float yerr = 0.002;

  theXErr2 = xerr*xerr;
  theYErr2 = yerr*yerr;
}

SIUTIL::LocalError BinaryClusterParameterEstimator::localError( const Cluster&) const
{
  return SIUTIL::LocalError( theXErr2, 0, theYErr2);
}
// Get the cluster position as the geometrical cluster center (digital
// average). The analog amplitueds are ignored.
SIUTIL::LocalPoint 
BinaryClusterParameterEstimator::localPosition( const Cluster& clust) const
{
  // Get the cluster limits.
  int minX = clust.minPixelRow();
  int maxX = clust.maxPixelRow();
  int minY = clust.minPixelCol();
  int maxY = clust.maxPixelCol();
  // Get the center
  float centerX = float(maxX+minX+1)/2.;
  float centerY = float(maxY+minY+1)/2.;

  return theTopol.localPosition( SIUTIL::MeasurementPoint( centerX-lorentzShift(), 
						   centerY ));
} 

float BinaryClusterParameterEstimator::lorentzShift() const
{
  // Implement only the x direction shift for now (OK for barrel)
  SIUTIL::LocalVector dir = SIUTIL::LocalVector( (theDet->driftDirection( )).x(), 
  			         (theDet->driftDirection( )).y(), 
				 (theDet->driftDirection( )).z() ) ;
  float thickness = (float)(theDet->getThickness()) ;
  float pitchX = (float)(theDet->pixelSizeX()) ;
  // max shift in cm 

  float xdrift = dir.x()/dir.z() * thickness;
  // divide by 2 to get the average correction
  float lshift = xdrift / pitchX / 2;  
  return lshift;  
}

}  // namespace SICLU.
