#include "Cluster.h"
#include "PixelDigi.h"

using namespace SIUTIL ;

//using namespace PixelReco ;

namespace SICLU{

Cluster::Cluster( const PixelPos& pix, float adc) :
  // The center of pixel with row # N is at N+0.5 in the meas. frame!
  theSumX( (pix.row()+0.5) * adc ), 
  theSumY( (pix.col()+0.5) * adc ),
  theCharge( adc ),
  theMinPixelRow( pix.row() ),
  theMaxPixelRow( pix.row() ),
  theMinPixelCol( pix.col() ),
  theMaxPixelCol( pix.col() ) {
  // First pixel in this cluster.
  thePixels.push_back( Pixel( pix.row()+0.5, pix.col()+0.5, adc));

} // c-tor Cluster.




void Cluster::add( const PixelPos & pix, float adc) {

  // The center of pixel with row # N is at N+0.5 in the meas. frame!
  theSumX += (pix.row()+0.5) * adc; 
  theSumY += (pix.col()+0.5) * adc; 
  theCharge += adc;

  thePixels.push_back( Pixel( (pix.row()+0.5), (pix.col()+0.5),adc ) );

  if (pix.row() < theMinPixelRow) theMinPixelRow = pix.row();
  if (pix.row() > theMaxPixelRow) theMaxPixelRow = pix.row();
  if (pix.col() < theMinPixelCol) theMinPixelCol = pix.col();
  if (pix.col() > theMaxPixelCol) theMaxPixelCol = pix.col();
}


bool Cluster::edgeHitX() const {
  bool t1 = (theMinPixelRow == 0);
  bool t2 = false; /// !buffer.inside(theMaxPixelRow+1,0);
  return (t1||t2);
}


bool Cluster::edgeHitY() const {
  bool t1 = (theMinPixelCol == 0);
  bool t2 = false; /// !buffer.inside(0,theMaxPixelCol+1);
  return (t1||t2);
}



Cluster::ChannelContainer Cluster::channels() const {
  ChannelContainer result; 
  result.reserve( pixels().size() ) ;
  for (vector<Pixel>::const_iterator i = pixels().begin();
                                     i!= pixels().end(); i++ ) {
    result.push_back( PixelDigi::pixelToChannel( int(i->x), int(i->y)) );
  }
  return result;
}

} // namespace SICLU.
