#include "ClusterParameterEstimatorFactory.h"
#include "FixedClusterParameterEstimator.h"
#include "BinaryClusterParameterEstimator.h"
#include "CPEFromDetPosition.h"

namespace SICLU {

  ClusterParameterEstimatorFactory::~ClusterParameterEstimatorFactory()
  {
    // do not delete theErrorMatrix since it is shared by all instances
    // ot the factory
  }

  ClusterParameterEstimator*
  ClusterParameterEstimatorFactory::get(const SIDETUNIT::DetUnit& det) const 
                     throw(string)  {
    static string name = "precise" ; // SSh

    if      ( name == "fixed" )
              return new FixedClusterParameterEstimator( det ) ;
    else if ( name == "precise" )
              return new CPEFromDetPosition( det ) ;
    else if ( name == "binary" )
              return new BinaryClusterParameterEstimator( det ) ;
    else throw(string("ClusterParameterEstimatorFactory::get: Unknown PixelHitParameters name: ") + name ) ;

  } // get.

  PixelErrorParametrization* ClusterParameterEstimatorFactory::theErrorMatrix = 0;

} // namespace SICLU.
