#include "FixedClusterParameterEstimator.h"
#include "DetGeometry/GeomVector/LocalVector.h"

namespace SICLU {

FixedClusterParameterEstimator::
FixedClusterParameterEstimator( const SIDETUNIT::DetUnit& det) :
 theTopol( det.topology()), theDet(&det)
{
  static float xerr = 0.001  ;
  static float yerr = 0.002 ;

  theXErr2 = xerr*xerr;
  theYErr2 = yerr*yerr;
}

SIUTIL::LocalError FixedClusterParameterEstimator::localError( const Cluster&) const
{
  return SIUTIL::LocalError( theXErr2, 0, theYErr2);
}

SIUTIL::LocalPoint 
FixedClusterParameterEstimator::localPosition( const Cluster& clust) const
{
  return theTopol.localPosition( SIUTIL::MeasurementPoint( clust.x()-lorentzShift(), 
						   clust.y()));
} 

float FixedClusterParameterEstimator::lorentzShift() const
{
  // Implement only the x direction shift for now (OK for barrel)
  SIUTIL::LocalVector dir = SIUTIL::LocalVector( (theDet->driftDirection()).x(), 
  				 (theDet->driftDirection()).y(),
				 (theDet->driftDirection()).z() ) ;

  float thickness = theDet->getThickness() ;

  float pitchX = theDet->topology().pitch().first;

  // max shift in cm 
  float xdrift = dir.x()/dir.z() * thickness;
  // divide by 2 to get the average correction
  float lshift = xdrift / pitchX / 2;  
  return lshift;  
}

} // namespace SICLU.
