#include "PixelErrorParametrization.h"

namespace SICLU {

PixelErrorParametrization::PixelErrorParametrization(){
  
//  static SimpleConfigurable<string> paramType("oscar", "CMSsimulation");
//SSh  theParametrizationType = paramType.value();
  theParametrizationType = "oscar" ;
  ////////////////////////////////////////////////////////
  // define alpha and beta ranges-bins for Y BARREL errors 
  /////////////////////////////////////////////////////////
  
  // MAGIC NUMBERS: alpha bins 
  a_min = 1.37078;
  a_max = 1.77078;
  a_bin = 0.1;

  // MACIG NUMBERS: beta ranges depending on y cluster size
  brange_yb.resize(6);
  brange_yb[0] = pair<float,float>(0., 0.6);   // ysize=1
  brange_yb[1] = pair<float,float>(0.1, 0.9);     // ysize = 2
  brange_yb[2] = pair<float,float>(0.6, 1.05);  // ysize = 3
  brange_yb[3] = pair<float,float>(0.9, 1.15);  // ysize = 4 
  brange_yb[4] = pair<float,float>(1.05, 1.22); // ysize = 5
  brange_yb[5] = pair<float,float>(1.15, 1.41); // ysize >= 6 

  /////////////////////////////////////////////////////
  // fill Y-BARREL matrix with sigma from gaussian fit 
  // of residuals
  /////////////////////////////////////////////////////
  // fill with the resolution points in order 
  // to make an error interpolation 

//SSh  readYB( ybarrel_3D, "yres_npix", "_alpha", "_b.vec");

  ///////////////////////////////////////////////////////
  // define alpha and beta range/bins for X-BARREL errors
  ///////////////////////////////////////////////////////
  
  // MAGIC NUMBERS:
  // abs(pi/2-beta) bins depending on X cluster size
  bbins_xb.resize(3);
  // xsize = 1 all beta range
  (bbins_xb[0]).resize(1); 
  (bbins_xb[0])[0] = 100.; 
  // xsize = 2 4 beta-bins
  (bbins_xb[1]).resize(3);
  (bbins_xb[1])[0] = 0.7; 
  (bbins_xb[1])[1] = 1.; 
  (bbins_xb[1])[2] = 1.2; 
  // xsize >= 3 same 4 beta-bins as for xsize=2
  (bbins_xb[1]).resize(3);
  (bbins_xb[1])[0] = 0.7; 
  (bbins_xb[1])[1] = 1.; 
  (bbins_xb[1])[2] = 1.2; 

  ///////////////////////////////////////////////////////
  // fill X-BARREL matrix with parameters to perform a 
  // linear parametrization of x erros
  ///////////////////////////////////////////////////////
  // for each beta bin: p1 + p2*alpha

//SSh  readXB( xbarrel_3D, "xpar_npix", "_beta", "_b.vec");

  ///////////////////////////////////////////////////////
  // define alpha and beta range/bins for Y-FORWARD errors
  ///////////////////////////////////////////////////////

  // MAGIC NUMBERS:
  // abs(pi/2-beta) range independent on Y cluster size
  brange_yf = pair<float,float>(0.3, 0.4);     

  //////////////////////////////////////////////////////
  // fill Y-FORWARD matrix with parameters to perform  
  // a parametrization of Y erros
  //////////////////////////////////////////////////////
  // for npix=1 and all alpha range:
  // p1 + p2*beta + p3*beta**2 + p4*beta**3 + p5*beta**4
  // for npix>=2 and all alpha range:
  // p1 + p2*beta + p3*beta**2 

//SSh  readF( yforward_3D, "ypar_npix", "_alpha", "_f.vec");

  //////////////////////////////////////////////////////
  // fill X-FORWARD matrix with parameters to perform  
  // a linear parametrization on alpha for all beta range
  //////////////////////////////////////////////////////

//SSh  readF( xforward_3D, "xpar_npix", "_beta", "_f.vec");

}     

PixelErrorParametrization::~PixelErrorParametrization(){}
 
pair<float,float> 
PixelErrorParametrization::getError(Part pixelPart, int sizex, int sizey, 
			   float alpha, float beta){
  pair<float,float> element;
  if(pixelPart == barrel){
    element = pair<float,float>(error_XB(sizex, alpha, beta), 
				error_YB(sizey, alpha, beta));
  }else {
    element =  pair<float,float>(error_XF(sizex, alpha, beta),
				 error_YF(sizey, alpha, beta));
  }
#ifdef DEBUG
  cout << " ErrorMatrix gives error: " << element.first << " , " 
       <<element.second << endl;
#endif
  return element;
}

float PixelErrorParametrization::error_XB(int sizex, float alpha, float beta){
#ifdef DEBUG 
  cout << "I'M AT THE BEGIN IN ERROR XB METHOD" << endl;
#endif
   bool barrelPart = true;
 // find size index
  int i_size = min(sizex-1,2);

  // find beta index
  int i_beta = betaIndex(i_size, bbins_xb[i_size], beta);

  // if ( i_size==0 ) return linParametrize(barrelPart, i_size, i_beta, alpha);
  //else return quadParametrize(barrelPart, i_size, i_beta, alpha);
  return quadParametrize(barrelPart, i_size, i_beta, alpha);
}

float PixelErrorParametrization::error_XF(int sizex, float alpha, float beta){
#ifdef DEBUG 
  cout << "I'M AT THE BEGIN IN ERROR XF METHOD" << endl;
#endif
  bool barrelPart = false;
  //symmetrization w.r.t. orthogonal direction
  float alpha_prime = fabs(3.14159/2.-alpha);
  // find x size index
  int i_size = min(sizex-1,2);
  // no beta parametrization!!!
  int i_beta = 0;
  // find beta index
  // int i_beta = betaIndex(i_size, bbins_xf, beta);
#ifdef DEBUG 
  cout << "size index = " << i_size << endl;
  cout << "no beta index " << endl;
  cout << " betaprime = " << beta_prime << endl;
#endif
  return linParametrize(barrelPart, i_size, i_beta, alpha_prime);
}

float PixelErrorParametrization::error_YB(int sizey, float alpha, float beta){  
#ifdef DEBUG
  cout << "I'M AT THE BEGIN IN ERROR YB METHOD" << endl;
#endif
  int i_alpha;
  int i_size = min(sizey-1,5);
#ifdef DEBUG
  cout << "I found size index = " << i_size << endl;
#endif
  if (sizey < 4) {      // 3 alpha bins
    if (alpha <= a_min + a_bin) { 
      i_alpha = 0;
    } else if (alpha < a_max-a_bin) {
      i_alpha = 1;
    }else {
      i_alpha = 2; 
    }
  }else{                // 1 alpha bin 
    i_alpha = 0;
  }
#ifdef DEBUG  
  cout << "I found alpha index = " << i_alpha << endl;
#endif
  // vector of beta parametrization
  vector<float> ybarrel_1D = (ybarrel_3D[i_size])[i_alpha];

#ifdef DEBUG  
  cout << "beta vec has dimensions = " << ybarrel_1D.size()<< endl;
  cout << "beta = " << beta << " beta max = " << brange_yb[i_size].second 
       << " beta min = " << brange_yb[i_size].first << endl;
#endif

  // beta --> abs(pi/2-beta) to be symmetric w.r.t. pi/2 axis
  float beta_prime = fabs(3.14159/2.-beta);
  if (beta_prime <= brange_yb[i_size].first){ 
    return ybarrel_1D[0];
  }else if (beta_prime >= brange_yb[i_size].second){
    return ybarrel_1D[ybarrel_1D.size()-1];
  } else {
    return interpolation(ybarrel_1D, beta_prime, brange_yb[i_size] );
  }  
}

float PixelErrorParametrization::error_YF(int sizey, float alpha, float beta){
#ifdef DEBUG 
  cout << "I'M AT THE BEGIN IN ERROR YF METHOD" << endl;
#endif
  // find y size index
  int i_size = min(sizey-1,1);
  // no parametrization in alpha
  int i_alpha = 0;
  // beta --> abs(pi/2-beta) to be symmetric w.r.t. pi/2 axis
  float beta_prime = fabs(3.14159/2.-beta);
  if (beta_prime < brange_yf.first) beta_prime = brange_yf.first;
  if (beta_prime > brange_yf.second) beta_prime = brange_yf.second;
  float err_par = 0;
  for(int ii=0; ii < (int)(( (yforward_3D[i_size])[i_alpha] ).size()); ii++){
    err_par += ( (yforward_3D[i_size])[i_alpha] )[ii] * pow(beta_prime,ii);
  }
  return err_par; 
}

float PixelErrorParametrization::interpolation(vector<float>& vector_1D, 
					       float& angle, pair<float,float>& range){

  if (angle < range.first) 
    cout << " IT IS NOT NECESSARY TO DO AN INTERPOLATION" << endl;

  float bin = (range.second-range.first)/float(vector_1D.size());
  float curr = range.first;
  int i_bin = -1;
  for(int i = 0; i< (int)(vector_1D.size()); i++){
    //cout<< "index = "<< i << endl;
    curr += bin;
    if (angle <= curr){
      i_bin = i;
      break;
    }
  }
  // cout << " ibin= " << i_bin << " value= " << curr <<endl;
  float y1;
  float y2;
  float x1;
  float x2;
  // test_1
  if(i_bin == 0){
    x1 = range.first + bin/2;
    x2 = x1 + bin;
    y1 = vector_1D[0];
    y2 = vector_1D[1];
  }else if ( i_bin == (int)((vector_1D.size()-1)) || i_bin == -1){
    x2 = range.second - bin/2;
    x1 = x2 - bin;
    y2 = vector_1D[(vector_1D.size()-1)]; 
    y1 = vector_1D[(vector_1D.size()-2)]; 
  } else if ( (curr-angle) < bin/2){
    x1 = curr - bin/2;
    x2 = x1 + bin;
    y1 = vector_1D[i_bin];
    y2 = vector_1D[i_bin+1];
  } else {
    x2 = curr - bin/2;
    x1 = x2 - bin;
    y2 = vector_1D[i_bin];
    y1 = vector_1D[i_bin-1];    
  }
  float y = ((y2-y1)*angle + (y1*x2-y2*x1))/(x2-x1);
  //end test1
#ifdef DEBUG
  cout << "INTERPOLATION GIVES" << endl;
  cout << "(x1,y1) = " << x1 << " , " << y1 <<endl;
  cout << "(x2,y2) = " << x2 << " , " << y2 <<endl;
  cout << "(angle,y) = " << angle << " , " << y <<endl;
#endif
  
  return y;
}

int PixelErrorParametrization::betaIndex(int& i_size, vector<float>& betabins, 
					 float& beta){
  int i_beta = -1;
  // beta --> abs(pi/2-beta) to be symmetric w.r.t. pi/2 axis
  float beta_prime = fabs(3.14159/2.-beta);
  for(int i = 0; i < (int)(betabins.size()); i++){
    if (beta_prime < betabins[i]) {
      i_beta = i;
      break;
    } 
  }
  if (i_beta < 0) i_beta = betabins.size();
  return i_beta;
}
float PixelErrorParametrization::linParametrize(bool& barrelPart, int& i_size, 
						int& i_angle1, float& angle2){
#ifdef DEBUG
  cout << "we are in linParametrize metod" <<endl;
  cout << "barrel? " << barrelPart << endl;
  cout << "size index = " << i_size << endl;
  cout << "angle index = " << i_angle1 << endl;
  cout << "angle variable = " << angle2 << endl;
#endif
  float par0 = 0;
  float par1 = 0;
  if (barrelPart) {
    par0 = ((xbarrel_3D[i_size])[i_angle1])[0]; 
    par1 = ((xbarrel_3D[i_size])[i_angle1])[1]; 
  } else {
    par0 = ((xforward_3D[i_size])[i_angle1])[0]; 
    par1 = ((xforward_3D[i_size])[i_angle1])[1]; 
  }
#ifdef DEBUG
  cout << "PAR0 = " << par0 << endl;
  cout << "PAR1 = " << par1 << endl;
  cout << "PAR1 = " << angle2 << endl;
  cout << "X error = " << (par0 + par1*angle2) << endl;
#endif
  return par0 + par1*angle2;
}

float PixelErrorParametrization::quadParametrize(bool& barrelPart, int& i_size, 
						 int& i_angle1, float& angle2){
#ifdef DEBUG
  cout << "we are in quadParametrize metod" <<endl;
  cout << "barrel? " << barrelPart << endl;
  cout << "size index = " << i_size << endl;
  cout << "angle index = " << i_angle1 << endl;
  cout << "angle variable = " << angle2 << endl;
#endif
  float par0 = 0;
  float par1 = 0;
  float par2 = 0;
  if (barrelPart) {
    par0 = ((xbarrel_3D[i_size])[i_angle1])[0]; 
    par1 = ((xbarrel_3D[i_size])[i_angle1])[1]; 
    par2 = ((xbarrel_3D[i_size])[i_angle1])[2]; 
  } else {
    par0 = ((xforward_3D[i_size])[i_angle1])[0]; 
    par1 = ((xforward_3D[i_size])[i_angle1])[1]; 
    par2 = ((xforward_3D[i_size])[i_angle1])[2]; 
  }
#ifdef DEBUG
  cout << "PAR0 = " << par0 << endl;
  cout << "PAR1 = " << par1 << endl;
  cout << "PAR2 = " << par1 << endl;
  cout << "ANGLE = " << angle2 << endl;
  cout << "X error = " << (par0 + par1*angle2 + par2*angle2*angle2) << endl;
#endif
  return par0 + par1*angle2 + par2*angle2*angle2;
}

} // namespace SICLU.
