#ifndef _BARRELDETUNIT_H_
#define _BARRELDETUNIT_H_

#include "math.h"
#include <string>
#include <sstream>

#include "CLHEP/Matrix/Matrix.h"
#include "CLHEP/Matrix/Vector.h"
#include "CLHEP/Vector/ThreeVector.h"

#include "Pixel/SiRectangularDetUnit.h"

using namespace std ;
using namespace CLHEP ;

/** Class BarrelDetUnit.
 * 
 * @author S.Shulga
 * @version $Id: BarrelDetUnit.h,v 1 2007/06/06 shulga
 */
namespace SIDETUNIT {

class BarrelDetUnit : public DetUnit{

 public:
  
  virtual ~BarrelDetUnit() ;
  
  BarrelDetUnit() : DetUnit(),  _RdetUnit_refPoint(0.),
				_PhiDetUnit_refPoint(0.),
				_DR_detUnit(SIUTIL::dblDummy), 
				_DPhi_detUnit(SIUTIL::dblDummy){ 
    set_isRefPointDone( false ) ;
  } ;


  BarrelDetUnit( SIUTIL::DetUnitType  detUnitType,
  	   unsigned int layerN, 
	   unsigned int detUnitsInLayer, 
	   unsigned int detUnitN,
	   double Rmin, double Rmax, 
	   double Zmin, double Zmax,
	   double pixelSizeX_duFrame, double pixelSizeY_duFrame,
	   double vxdBField,
	   double tanLorentzAnglePerTesla,
	   double pixThresholdInNoiseUnits,
	   double seedThresholdInNoiseUnits,
	   double clusterThresholdInNoiseUnits ) ;
		
  void getDetUnit_refPoint( double &RdetUnit_refPoint, 
  			    double &PhiDetUnit_refPoint)
                                     const  throw(string) ;
//--
  virtual int getLayerN() const {  return _layerN ; }

  virtual unsigned int getWedgeN() const throw(string){ 
    throw(string("BarrelDetUnit::getWedgeN(): It is impossible to call this function for barrel mode. " ) );
  }

  virtual unsigned int getDetUnitN() const { return _detUnitN ; }

  virtual unsigned int getNumbDetUnitsInLayer() const throw(string) { return _detUnitsInLayer ; }

  virtual unsigned int getNumbDetUnitsInWedge() const throw(string) { 
    throw(string("BarrelDetUnit::getNumbDetUnitsInWedge(): It is impossible to call this function for barrel mode. " ) );
  }
//--

  void setHit_tmp( SimTrackerHit *simHit ) throw(string) ;
  void setHit_tmp( SimTrackerHit *simHit, int dummy ) throw(string) ;

  virtual void make_digis( LCCollectionVec * rawVec, 
			   LCCollectionVec * relVec ) throw(string) ;

  void getPixelPos( unsigned int pixelX, 
  		    unsigned int pixelY, double * recoHitPos ) throw(string) ;

  double getThickness() const { return _thickness ; } ;

  bool setRawHit( TrackerRawDataImpl *rawHit ) throw(string) ;

  double get_RdetUnit_refPoint() const { return _RdetUnit_refPoint ; } ;

  double get_ZdetUnit_refPoint() const { return 0. ; } ; // ladder in [-_Z,_Z]

  void getGlobalPos(  Hep3Vector &local_Pos ) const ;
  void getGlobalError(float locErr[3], float gloErr[TRKHITNCOVMATRIX]) const ;
  void getLocalError( const EVENT::FloatVec gloErr, float * locErr ) const ;

 private:

  bool false_cluster( int & dX, int & dY, 
                      double & distX, double & distY ) ;

  bool false_cluster_accurate( int k, double & distX, double & distY ) ;

  void getLocalPos(  Hep3Vector &global_Pos ) const ;

  void getGlobalVector(  Hep3Vector &local_Vec ) const ;

  void getLocalVector(  Hep3Vector &global_Vec ) const ;

  bool checkHit(Hep3Vector &hitGlobalPos,
   		Hep3Vector &hitGlobalMom ) throw(string) ;

  unsigned int _layerN ; // layer number in sub detector (>=0 for barrel)

  unsigned int _detUnitsInLayer ; 

  unsigned int _detUnitN ;    // det.unit number in layer

 /**  Global detUnit position (ref.point) in cylindrical coord.system 
  *   (z_ref_point = 0, d.u. center)
  */
  double _thickness, _RdetUnit_refPoint, _PhiDetUnit_refPoint ;

  HepVector _refPoint ;
  HepMatrix _g2l, _l2g ;

  double _DR_detUnit, _DPhi_detUnit ;

 public :
 
  static string _cellIDEncoding ;

} ; // class BarrelDetUnit.


} // namespace SIDETUNIT.

#endif
