/** class GaussianTailNoiseGenerator
 *  Generation of random noise
 *  on "numberOfChannels" channels with a given threshold.
 *  The generated noise :
 *   - corresponds to a Gaussian distribution of RMS = "noiseRMS"
 *   - is larger than threshold*noiseRMS. <BR>
 *
 *  Initial author : Veronique Lefebure 08.10.98
 *                  according to the FORTRAN code tgreset.F from Pascal Vanlaer
 *
 *  Fills in a map < channel number, generated noise >
 *
 *  Adapted in MarlinReco, Sergey Shulga, May, 2007
 *  @version $Id: GaussianTailNoiseGenerator.h,v 1 2007/06/06 shulga
 */

#ifndef GaussianTailNoiseGenerator_h
#define GaussianTailNoiseGenerator_h _ 1

//#define _DEBUG_NOISE_

#include <map>
#include "CLHEP/Random/RandPoisson.h"
#include "CLHEP/Random/RandFlat.h"
#include <gsl/gsl_sf_erf.h>
#include <gsl/gsl_sf_result.h>
#include <gsl/gsl_randist.h>
#include <gsl/gsl_rng.h>

using namespace std ;
using namespace CLHEP ;

namespace SIDETUNIT{


class GaussianTailNoiseGenerator {

  public:
     
      GaussianTailNoiseGenerator(){}
     ~GaussianTailNoiseGenerator(){}

      void generate(int NumberOfchannels, double threshold
                                        , double noiseRMS 
                                        , map<int,double, less<int> >& theMap );

};

}

#endif
