#ifndef _TRACKER_RECTANGULAR_PIXEL_TOPOLOGY_H_
#define _TRACKER_RECTANGULAR_PIXEL_TOPOLOGY_H_

#include "Pixel/PixelTopology.h"
#include "PixelDigi.h"
#include "DetGeometry/GeomVector/LocalPoint.h"

/** ORCA
 *  Topology for rectangular pixel detector. 
 *  The only available.
 * 
 *  @version $Id: DeUnit.h,v 1 2007/06/06 shulga
 */
namespace SIDETUNIT{

class RectangularPixelTopology : public PixelTopology {

 public:
 
  RectangularPixelTopology( int   nrows,  int ncols, 
  			    double pitchx, double pitchy)
   : m_nrows(nrows), m_ncols(ncols), 
     m_pitchx(pitchx), m_pitchy(pitchy) {
     m_xoffset = -nrows/2. * pitchx ;
     m_yoffset = -ncols/2. * pitchy ;
  }

  // Topology interface
  // SSh throw
  virtual SIUTIL::LocalPoint localPosition( const SIUTIL::MeasurementPoint& mp) const throw(string){
    if( (int)(mp.x()) >= m_nrows ) 
     throw( string("RectangularPixelTopology::localPosition(): pix_x N = ") 
           +SIUTIL::d2s(mp.x()) + " >= nrows =" + SIUTIL::i2s(m_nrows) + "\n" ) ;
    if( (int)(mp.y()) >= m_ncols ) 
     throw( string("RectangularPixelTopology::localPosition(): pix_y N = ") 
           +SIUTIL::d2s(mp.y()) + " >= ncols =" + SIUTIL::i2s(m_ncols) + "\n" ) ;

    return SIUTIL::LocalPoint( mp.x()*m_pitchx + m_xoffset, 
		       mp.y()*m_pitchy + m_yoffset ) ;
  }
    
  virtual SIUTIL::LocalError localError( const SIUTIL::MeasurementPoint&,
				 const SIUTIL::MeasurementError& ) const;
  
//   virtual LocalPoint localPosition( double channel) const {
//     pair<int,int> p = PixelDigi::channelToPixel( channel);
//     return LocalPoint( p.first*m_pitchx + m_xoffset, 
// 		       p.second*m_pitchy + m_yoffset);
//   }

/*
  virtual SIUTIL::MeasurementPoint measurementPosition( const SIUTIL::LocalPoint& lp ) const throw(string){
    pair<double,double> p = pixel(lp) ;

    if( floor( p.first ) == m_nrows  ) p.first -= 1;
    else if( floor( p.first ) == -1  ) p.first += 1;
    else if( floor( p.first ) == -2  ){ cout<<"SIUTIL::MeasurementPoint measurementPosition WARNING: p.first="<<p.first<<" correction: p.first += 2"<< endl ; p.first += 2; }
    else if( ( floor(p.first) > m_nrows) || ( floor(p.first) < -1) )
     throw( string("RectangularPixelTopology::measurementPosition(): pix_x N = ") 
           +SIUTIL::d2s(p.first) + ", nrows =" + SIUTIL::i2s(m_nrows) + "\n" ) ;

    if( floor(p.second) == m_ncols ) p.second -= 1 ;
    else if( floor(p.second) == -1 ) p.second += 1 ;
    else if( floor( p.second ) == -2  ){ cout<<"SIUTIL::MeasurementPoint measurementPosition WARNING: p.second="<<p.second<<" correction: p.second += 2"<< endl ; p.second += 2; }
    else if( ( floor(p.second) > m_ncols) || ( floor(p.second) < -1) )
     throw( string("RectangularPixelTopology::measurementPosition(): pix_y N = ") 
           +SIUTIL::d2s(p.second) + ", ncols =" + SIUTIL::i2s(m_ncols) + "\n" ) ;
    
    SIUTIL::MeasurementPoint mp = SIUTIL::MeasurementPoint( p.first, p.second) ;

    return mp ;
  }
*/

  virtual SIUTIL::MeasurementPoint measurementPosition( const SIUTIL::LocalPoint& lp ) const {
    pair<double,double> p = pixel(lp) ;

    if( floor( p.first ) >= m_nrows  ) p.first = m_nrows - 1 ;
    else if( floor( p.first ) <  0  ) p.first = 0 ;

    if( floor(p.second) >= m_ncols ) p.second = m_ncols - 1 ;
    else if( floor(p.second) < 0 ) p.second = 0 ;
    
    SIUTIL::MeasurementPoint mp = SIUTIL::MeasurementPoint( p.first, p.second) ;

    return mp ;
  }


  virtual SIUTIL::MeasurementError 
  measurementError( const SIUTIL::LocalPoint&, const SIUTIL::LocalError& ) const;

  virtual int channel( const SIUTIL::LocalPoint& lp) const {
    pair<double,double> p = pixel(lp);
    return SIUTIL::PixelDigi::pixelToChannel( int(p.first), int(p.second));
  }

  // PixelTopology interface

  virtual pair<double,double> pixel( const SIUTIL::LocalPoint& p ) const {
    return pair<double,double>( (p.x() - m_xoffset) / m_pitchx,
                              (p.y() - m_yoffset) / m_pitchy);
  }
      
  virtual pair<double,double> pitch() const {
    return pair<double,double>( m_pitchx, m_pitchy);
  }

  virtual int nrows() const {
    return m_nrows;
  }

  virtual int ncolumns() const {
    return m_ncols;
  }
  

private:
  int m_nrows;
  int m_ncols;
  double m_pitchx;
  double m_pitchy;
  double m_xoffset;
  double m_yoffset;
};

} //namespace SIDETUNIT{

#endif
