#ifndef _DETUNIT_H_
#define _DETUNIT_H_

//#define _DEBUG_DU_

#include "math.h"
#include <string>
#include <sstream>

#include "IMPL/TrackerRawDataImpl.h"
#include "IMPL/TrackerHitImpl.h"

#include "UTIL/BitField64.h"

#include "SiUtil.h"

#include "Pixel/SiTrkSimHitExtended.h"
#include "Pixel/GaussianTailNoiseGenerator.h"

#include "Pixel/RectangularPixelTopology.h"
#include "Pixel/Amplitude.h"


using namespace std ;
using namespace EVENT ;
using namespace CLHEP ;
using namespace IMPL ;

/** Abstract base class DetUnit.
 * 
 * @author S.Shulga
 * @version $Id: DeUnit.h, v 1 2007/06/06 shulga
 */

namespace SIDETUNIT{


/** Map to store pixels (int chan) by charge_fraction (double)
 *  (used in SiPixelDetUnitDigitizer::induce_signal 
 *  and DetUnit::set_signal )
 */
typedef map< int, double, less<int> > hit_map_type ;


class DetUnit{

 public:

  DetUnit() ;

  DetUnit( SIUTIL::DetUnitType  detUnitType,
	   double Rmin, double Rmax,
	   double Zmin, double Zmax,
	   double X, double Y, double Z,// half sizes in local coordinate system (du_Frame)
	   double pixelSizeX_duFrame, double pixelSizeY_duFrame,
	   double BField,
	   double tanLorentzAnglePerTesla,
	   double pixThresholdInNoiseUnits,
	   double seedThresholdInNoiseUnits,
	   double clusterThresholdInNoiseUnits ) ;

  virtual ~DetUnit() ;
  
  unsigned int getHalfNpixelsX() const { return (unsigned int)(_X/_pixelSizeX_duFrame) ; } ;

  unsigned int getHalfNpixelsY() const { return (unsigned int)(_Y/_pixelSizeY_duFrame) ; } ;

  void setHit() ;

  void add_signal_from_simHit( hit_map_type & hit_signal, 
			       SiTrkSimHitExtended *simHit ) ;

  void add_noise() throw(string) ;

  void pixel_inefficiency() throw(string) ;

  void clear() ;

  void setHit_tmp( SimTrackerHit *simHit ) throw(string) ;

  void setHit_tmp( SimTrackerHit *simHit, bool bb ) throw(string) ; // for debug

  void setHit_tmp( SimTrackerHit *simHit, int dummy ) throw(string) ;// for debug

// virtual = 0

  virtual bool checkHit(Hep3Vector &hitGlobalPos,
   			Hep3Vector &hitGlobalMom ) throw(string) = 0 ;// used in setHit_tmp

  virtual void make_digis( LCCollectionVec * rawVec, 
			   LCCollectionVec * relVec ) throw(string) = 0 ;

  virtual bool setRawHit( TrackerRawDataImpl *rawHit ) = 0 ; 

  virtual int getLayerN() const = 0 ;

  virtual unsigned int getWedgeN() const throw(string) = 0 ;

  virtual unsigned int getDetUnitN() const = 0 ;

  virtual unsigned int getNumbDetUnitsInLayer() const throw(string) = 0 ;

  virtual unsigned int getNumbDetUnitsInWedge() const throw(string) = 0 ;

  void addRecHit( TrackerHitImpl* recHit ) ;

//17.10.2007  void addCluster( SICLU::Cluster cluster ) ;

  void clearRecHits() ;
//17.10.2007  void clearClusters() ;

// virtual get = 0

  virtual double getThickness() const = 0 ;

  virtual double get_RdetUnit_refPoint() const = 0 ;

  virtual double get_ZdetUnit_refPoint() const = 0 ;

  virtual void getPixelPos( unsigned int pixelX, 
  		    	    unsigned int pixelY, double * recoHitPos ) throw(string) = 0 ;

  virtual void getLocalPos( Hep3Vector &global_Pos ) const = 0 ;

  virtual void getLocalVector( Hep3Vector &global_Vec ) const = 0 ;

  virtual void getGlobalPos(  Hep3Vector &local_Pos ) const = 0;
  virtual void getGlobalError(float locErr[3], 
                              float gloErr[TRKHITNCOVMATRIX] ) const = 0;
  virtual void getLocalError( const EVENT::FloatVec gloErr, 
  			      float * locErr ) const = 0;

// get methods

  SIUTIL::DetUnitType getDetUnitType() const { return _detUnitType; } ;

  unsigned int getNumberOfSimHits() const throw(string) { return _simHits.size() ; } ;
  unsigned int getNumberOfRawHits() const throw(string) { return _rawHits.size() ; } ;

  void getPixelSizes( double &sizeX, double &sizeY ) const { sizeX = _pixelSizeX_duFrame ; sizeY = _pixelSizeY_duFrame ; } ;

  double pixelSizeX() const { return _pixelSizeX_duFrame ; } ;
  double pixelSizeY() const { return _pixelSizeY_duFrame ; } ;

  double pxsSizeX() const { return  (double)(2 * getHalfNpixelsX()) * pixelSizeX() ; } //.
  double rocsSizeX() const { return ((double)(2 * getHalfNpixelsX()/_rocSizeInX)  )*(double)(_rocSizeInX) *  pixelSizeX() ; } //.

  double pxsSizeY() const { return  (double)(2 * getHalfNpixelsY()) * pixelSizeY() ;} //.
  double rocsSizeY() const { return ((double)(2 * getHalfNpixelsY()/_rocSizeInY)  )*(double)(_rocSizeInY) *  pixelSizeY() ; } //.

  void getRZ( double &Rmin, double &Rmax, double &Zmin, double &Zmax ) const {
    Rmin = _Rmin ; Rmax = _Rmax ; Zmin = _Zmin ; Zmax = _Zmax ;  } ;
  
  SiTrkSimHitExtended * get_simHit_tmp()  { return _simHit_tmp ; } ;
  
  SiTrkSimHitExtendedVec  getSimHits() { return _simHits ;  } ;

  vector<TrackerRawDataImpl *> getRawHits() const { return _rawHits ; } ;

  double get_X() const { return _X ; } ;
  double get_Y() const { return _Y ; } ;
  double get_Z() const { return _Z ; } ;

  Hep3Vector get_BFieldLocal() const throw(string) { 
   if( _isRefPointDone ) return _BFieldLocal ; 
   throw(string("DetUnit::get_BFieldLocal: reference point is not done.")) ;
  } ;

  Hep3Vector driftDirection() const {
   if( _isRefPointDone ) return _theDriftDirection ;
   throw(string("DetUnit::get_BFieldLocal: reference point is not done.")) ;
  }

  bool isSetTmpHit() const { return _setHitTmp_XY ; } ;

  bool isAddNoise() const { return _addNoise ; } ;

  bool isAddNoisyPixels() const { return _addNoisyPixels  ; } ;

  const RectangularPixelTopology &topology() const { return  _topology ; } ;

  double get_thresholdInNoiseUnits() const { return _thresholdInNoiseUnits ; } ;

  double get_noiseInAdcCounts() const { return _noiseInAdcCounts ; } ;

  int get_pixelThresholdInADC() const { return _pixelThresholdInADC ; } ;

  double get_seedThresholdInNoiseUnits() const { return _seedThresholdInNoiseUnits ; } ;

  double get_clusterThresholdInNoiseUnits() const { return _clusterThresholdInNoiseUnits ; } ;

  int get_seedThresholdInADC() const { return _seedThresholdInADC ; } ;

  int get_clusterThresholdInADC() const { return _clusterThresholdInADC ; } ;

  virtual void show(string) ;

  virtual void show( SimTrackerHit *simHit) ;

  virtual void check() throw(string) ;

  vector<TrackerHitImpl*> getRecHits() const { return _recHits ; } ;

  double getElectronPerADC() const { return _electronPerADC ; } ;

// get end.

 protected:

  void setRZ( double Rmin, double Rmax, double Zmin, double Zmax ) ;

  bool hitEntryExit( Hep3Vector &A, Hep3Vector &n, SiTrkSimHitExtended * simHit ) throw(string) ;

  bool hitEntryExit( Hep3Vector &A, Hep3Vector &n,
		     SiTrkSimHitExtended * simHit,
		     bool &bentry_par, bool &bexit_par ) throw(string) ;

  bool hitEntryExit( Hep3Vector &A, Hep3Vector &n, SiTrkSimHitExtended * simHit, int dummy ) throw(string) ; // used for debug

  void set_isRefPointDone( bool ) ;

  bool isRefPointDone() const { return _isRefPointDone ; }  ;

  double get_globalBField() const { return _globalBField ; } ;

  double get_tanLorentzAnglePerTesla() const { return _tanLorentzAnglePerTesla; } ;

  SIUTIL::DetUnitType _detUnitType ;

 private:
  
  bool _isRefPointDone ;

  double _Rmin, _Rmax, _Zmin, _Zmax ; // subdetector sizes

 // half-sizes of detUnit in det.Unit frame, see function detUnitFrame( double * v )
  double _X, _Y, _Z ;

  double _pixelSizeX_duFrame, _pixelSizeY_duFrame ;

  double _globalBField ; // B field in detector frame (4 Tesla) along Z (beam axis)

  double _tanLorentzAnglePerTesla ;
  
 protected:

  Hep3Vector _BFieldLocal ;
  
  SiTrkSimHitExtended *_simHit_tmp ; // sim.hits with extended information
 
  Hep3Vector _theDriftDirection ;  // Drift direction

  bool _setHitTmp_XY ;

  signal_map_type     _signal ;

  // Adc callibration 
  double _electronPerADC ;

  /** ADC calibration 1adc count = 135e.
   *  Corresponds to 2adc/kev, 270[e/kev]/135[e/adc]=2[adc/kev]
   *  This parameter is also used to 
   *  calculate the noise in adc counts from noise in electrons.
   *  Both defaults should be the same.
   *  See also: 
   *  SiPixelDetUnitDigitizer.cc : _GeVperElectron   = 3.7E-09 ;
   *   1 electrons=3.7eV, 1keV=270.3e
   */

  // ADC saturation value, 255=8bit adc.
  // _theAdcFullScale = 255 ;
  int _adcFullScale ;

  // noise
  double _noiseInElectrons ;  // Noise (RMS) in units of electrons.
  double _noiseInAdcCounts, _thresholdInNoiseUnits ;
  double _pixelThresholdInE ; // pixel threshold in electrons
  int    _pixelThresholdInADC ;
  
  bool _addNoise, _addNoisyPixels ;

  //-- pixel efficiency
  bool _applyPixelInefficiency ;      // Switch on pixel ineffciency

  vector<double> _thePixelEfficiency ; // Single pixel efficiency
  vector<double> _thePixelColEfficiency ;  // Column efficiency
  vector<double> _thePixelChipEfficiency ; // ROC efficiency

  int _rocSizeInX ; //in pixels, num of row's pixels per ROC
  int _rocSizeInY ; //in pixels, num of column's pixels per ROC (for pix ineff.)
  unsigned int _nbHitPixelsInROCdeadZone ;

  unsigned int _nLayersInDet ; // used in pixel_inefficiency()

  bool _useRocDeadZone ;

  GaussianTailNoiseGenerator * _theNoiser ; // init in  

  vector<TrackerRawDataImpl *> _rawHits ;

  // clustering
  const RectangularPixelTopology _topology ;

  double _seedThresholdInNoiseUnits ;    //Pixel cluster seed in units of noise
  double _clusterThresholdInNoiseUnits ; //Cluster threshold in units of noise
  int _seedThresholdInADC, _clusterThresholdInADC ;

 public :

  SiTrkSimHitExtendedVec _simHits ; // sim.hits with extended information
  			            // adapted for given det.unit
  vector<TrackerHitImpl*> _recHits ;
//17.10.2007  vector<SICLU::Cluster> _clusters ;

  // for digi (e.g.: init in c-tor BarrelDetUnit, use in BarrelDetUnit::make_digis)
  UTIL::BitField64 *_bf64 ; 
  UTIL::BitField64 *_cellID_bf64 ; // used in BarrelDetUnit::setRawHit

  static UTIL::CellIDDecoder<TrackerRawDataImpl> *_decoder ;

} ; // class DetUnit.

//--------------------------------------

typedef vector<DetUnit *> DetUnitVec ;
typedef vector<DetUnitVec> DetUnitVecVec ;

} // namespace SIDETUNIT.

#endif
