#include "Pixel/SiRectangularDetUnit.h"

namespace SIDETUNIT{

// Static members.

UTIL::CellIDDecoder<TrackerRawDataImpl> *
DetUnit::_decoder = 0 ; // init in SiTrkClusteringProcessor.cc

//BitField64 *DetUnit::_bf64 = new BitField64("") ;

DetUnit::~DetUnit() { 

  cout <<"~DetUnit() destructor: " << endl ;

  _simHits.clear() ;

  _setHitTmp_XY = false ;
 
} // d-tor


//17.10.2007 void DetUnit::addCluster( SIUTIL::Cluster cluster ) { _clusters.push_back( cluster ) ;} //.


void DetUnit::clear() {

  _simHits.clear() ;
  
  _setHitTmp_XY = false ;

  _signal.clear() ;

  _nbHitPixelsInROCdeadZone = 0 ;

  _rawHits.clear()  ;

} // .


void DetUnit::clearRecHits() {

  _recHits.clear()  ;

} // .


//17.10.2007 void DetUnit::clearClusters() {  _clusters.clear()  ;} // .


bool bFirst_DU = true ;

DetUnit::DetUnit( SIUTIL::DetUnitType  detUnitType, 
	   double Rmin, double Rmax, 
	   double Zmin, double Zmax,
	   double X, double Y, double Z,
	   double pixelSizeX_duFrame, double pixelSizeY_duFrame,
	   double BField,
	   double tanLorentzAnglePerTesla,
	   double pixThresholdInNoiseUnits,
	   double seedThresholdInNoiseUnits,
	   double clusterThresholdInNoiseUnits )
	       :_detUnitType( detUnitType ),
		_isRefPointDone( false ),
		_Rmin( Rmin ), _Rmax( Rmax ),
		_Zmin( Zmin ), _Zmax( Zmax ),
		_X(X), _Y(Y), _Z(Z),
		_pixelSizeX_duFrame( pixelSizeX_duFrame ),
		_pixelSizeY_duFrame( pixelSizeY_duFrame ),
		_globalBField( BField ),
		_tanLorentzAnglePerTesla( tanLorentzAnglePerTesla ),
		_addNoise( true ),
		_addNoisyPixels( true ),
		_applyPixelInefficiency( true ),
                // Get the DetUnit size (for pixel inefficiency)
                // The default numbers are for pixels 100*150 microns.
                _rocSizeInX(20), //24(rows) in pixels, for 100*150pix
		_rocSizeInY(52), //(columns) in pixels
		_nLayersInDet(5), // used in pixel_inefficiency() only
		_useRocDeadZone( false ),
                _topology( RectangularPixelTopology
                     ( 2*getHalfNpixelsX(), 2*getHalfNpixelsY(),
		     _pixelSizeX_duFrame, _pixelSizeY_duFrame ) ),
		// clustering
		_seedThresholdInNoiseUnits(seedThresholdInNoiseUnits ), // originaly 6.
		_clusterThresholdInNoiseUnits(clusterThresholdInNoiseUnits  ) {
//show("c-tor DetUnit and exit.") ; exit(0) ;

 // Adc callibration
 _electronPerADC =  50. ; // 135.
 _adcFullScale   = 511 ;  // 255*50 = 12750 electrons

 double xPy = pixelSizeX_duFrame*pixelSizeY_duFrame ;

 if     ( xPy < 0.021*0.021) _noiseInElectrons =  70. ;
 else if( xPy < 0.026*0.026) _noiseInElectrons = 100. ;
 else if( xPy < 0.051*0.051) _noiseInElectrons = 200. ;
 else if( xPy < 0.101*0.101) _noiseInElectrons = 250. ;
 else                        _noiseInElectrons = 500. ;

 _noiseInAdcCounts      = _noiseInElectrons/_electronPerADC ;   // sigma
 _thresholdInNoiseUnits =  pixThresholdInNoiseUnits ;
 _pixelThresholdInE     = _thresholdInNoiseUnits * _noiseInElectrons ;
 _pixelThresholdInADC   = int( _thresholdInNoiseUnits * _noiseInAdcCounts ) ;

  if( _pixelThresholdInADC == 0 ){
     _pixelThresholdInADC = 1 ;
     _pixelThresholdInE   = _electronPerADC ;
     _thresholdInNoiseUnits = _pixelThresholdInE/_noiseInElectrons  ;
     cout<< "c-tor DetUnit: WARNING: _pixelThresholdInADC = 0 [adc units] and will be set = 1 [adc]" << endl ;
  }

  _seedThresholdInADC =  int (_seedThresholdInNoiseUnits * _noiseInAdcCounts ) ;
  _clusterThresholdInADC = int (_clusterThresholdInNoiseUnits * _noiseInAdcCounts ) ;

  if( bFirst_DU ) {
   cout <<" ===================================== "<< endl ;
   cout <<"c-tor DetUnit: noise In Electrons = "<<_noiseInElectrons << endl ;
   cout <<"               noise In Adc Counts = "<< _noiseInAdcCounts<< endl ;
   cout <<"               pixel Threshold in e- = "<< _pixelThresholdInE<< endl ;
   cout <<"               seed Threshold In e- = "<< _seedThresholdInNoiseUnits *_noiseInElectrons << endl ;
   cout <<"               cluster Threshold In e- = "<<_clusterThresholdInNoiseUnits *_noiseInElectrons << endl ;
   cout <<"              _applyPixelInefficiency = " << _applyPixelInefficiency
        << endl <<" ===================================== bFirst_DU = "<< bFirst_DU << endl ;
   bFirst_DU = false ;
  }

  _simHits.clear() ;

  _setHitTmp_XY = false ;

  if( _addNoisyPixels )  // Option to do noise in non-hit pixels
    _theNoiser = new GaussianTailNoiseGenerator() ;

  if( _applyPixelInefficiency ){

   _nbHitPixelsInROCdeadZone = 0 ;

   if( _useRocDeadZone ){
    cout <<"c-tor DetUnit:" <<" pxs(X,Y)= ("
         <<2*getHalfNpixelsX()<<", "<<2*getHalfNpixelsY() 
	 <<")   rocs= ("<<_rocSizeInX<<", "<<_rocSizeInY <<")" << endl ;
    cout <<"   ROCs defect (X,Y) = ("<< 100.*( -rocsSizeX() + pxsSizeX() )/pxsSizeX()<<", "
         << 100.*( -rocsSizeY() + pxsSizeY() )/pxsSizeY()<<") %" << endl ;
   }

   for ( unsigned int i = 0; i < _nLayersInDet; i++ ) {

    // Assume 1% inefficiency for single pixels, 
    _thePixelEfficiency.push_back( 1.-0.01 ) ;  // pixels = 99%
    // For columns make 1% default.
    _thePixelColEfficiency.push_back( 1.-0.01 ) ;  // columns = 99%
    // A flat 0.25% inefficiency due to lost data packets
    _thePixelChipEfficiency.push_back( 1.-0.0025 ) ; // chips = 99.75%


#ifdef _DEBUG_DU_
cout << "       DBG in constructor DetUnit: " << endl ;
cout <<"        DetUnit: "<< endl ;
cout <<" PixelEfficiency["<< i <<"] = "<< _thePixelEfficiency[i] ;
cout <<" PixelColEfficiency["<< i <<"] = "<< _thePixelColEfficiency[i] ;
cout <<" PixelChipEfficiency["<< i <<"] = "<< _thePixelChipEfficiency[i] ;
cout << endl ;
#endif

   } //  for ( int i.

  } // if( _applyPixelInefficiency .

  // delta cutoff in MeV, has to be same as in OSCAR=0.030/cmsim=1.0 MeV
  // _deltaProductionCut = 0.030; // In MeV

} // c-tor




DetUnit::DetUnit():_detUnitType( SIUTIL::dummy ),
  	    _Rmin(0.), _Rmax(0.),
	    _Zmin(0.), _Zmax(0.),
	    _X(0.),_Y(0.), _Z(0.),
            _topology( RectangularPixelTopology(0,0,0.,0.) ) { }



void DetUnit::set_isRefPointDone( bool isRefPointDone ){
  _isRefPointDone = isRefPointDone ;
}




void DetUnit::setRZ( double Rmin, double Rmax,
		     double Zmin, double Zmax ){
 _Rmin = Rmin ; _Rmax = Rmax ; _Zmin = Zmin ; _Zmax = Zmax ;
}


void DetUnit::setHit(){

  _simHits.push_back( _simHit_tmp ) ;

} // setHit.



bool DetUnit::hitEntryExit( Hep3Vector & A,
			    Hep3Vector & n,
			    SiTrkSimHitExtended * simHit ) throw(string) {// in det.unit frame

 const double n_epsil = 2. ;

 if( fabs( n.z() )< SIUTIL::get_epsil() )
  throw(string(" DetUnit::hitEntryExit: momentum in Z to small: x=" 
         + SIUTIL::d2s(n.x())+", y=" + SIUTIL::d2s(n.y())+", z=" + SIUTIL::d2s(n.z()) ) ) ;

 Hep3Vector entry, exit ;

 if( ( fabs(-_Z - A.z() ) < SIUTIL::get_epsil() ) 
  || ( fabs( _Z - A.z() ) < SIUTIL::get_epsil() ) ) A[2] += n_epsil*SIUTIL::get_epsil() ;

 double t1 = (-_Z - A.z())/n.z() ;
 double t2 = ( _Z - A.z())/n.z() ;

 switch( t2 - t1 > 0. ){

  case true:  // t1: entry, t2:exit

    entry[2] =-_Z; exit[2] = _Z;

    entry[0] = A[0] + n[0] * t1 ;  entry[1] = A[1] + n[1] * t1 ;
    exit [0] = A[0] + n[0] * t2 ;  exit [1] = A[1] + n[1] * t2 ;

   break;

  default:   // t2: entry, t1:exit

    entry[2] = _Z; exit[2] = -_Z; 

    exit [0] = A[0] + n[0] * t1 ; exit [1] = A[1] + n[1] * t1 ;
    entry[0] = A[0] + n[0] * t2 ; entry[1] = A[1] + n[1] * t2 ;

 } //  switch.

 bool bEntry = (entry.x() > -_X) && (entry.x() < _X) 
            && (entry.y() > -_Y) && (entry.y() < _Y) ;

 bool bExit  = (exit.x() > -_X) && (exit.x() < _X) 
            && (exit.y() > -_Y) && (exit.y() < _Y) ;

 if( (!bEntry) && (!bExit) ) return false; 

 simHit->setEntryExitPoints( entry, exit ) ;

 return true ; 

} // hitEntryExit.






bool DetUnit::hitEntryExit( Hep3Vector & A,
			    Hep3Vector & n,
			    SiTrkSimHitExtended * simHit,
			    bool &bentry_par, bool &bexit_par ) throw(string) {// in det.unit frame

 const double n_epsil = 2. ;

 if( fabs( n.z() )< SIUTIL::get_epsil() )
  throw(string(" DetUnit::hitEntryExit: momentum in Z to small: x=" 
         + SIUTIL::d2s(n.x())+", y=" + SIUTIL::d2s(n.y())+", z=" + SIUTIL::d2s(n.z()) ) ) ;

 Hep3Vector entry, exit ;

 if( ( fabs(-_Z - A.z() ) < SIUTIL::get_epsil() ) 
  || ( fabs( _Z - A.z() ) < SIUTIL::get_epsil() ) ) A[2] += n_epsil*SIUTIL::get_epsil() ;

 double t1 = (-_Z - A.z())/n.z() ;
 double t2 = ( _Z - A.z())/n.z() ;

 switch( t2 - t1 > 0. ){

  case true:  // t1: entry, t2:exit

    entry[2] =-_Z; exit[2] = _Z;

    entry[0] = A[0] + n[0] * t1 ;  entry[1] = A[1] + n[1] * t1 ;
    exit [0] = A[0] + n[0] * t2 ;  exit [1] = A[1] + n[1] * t2 ;

   break;

  default:   // t2: entry, t1:exit

    entry[2] = _Z; exit[2] = -_Z; 

    exit [0] = A[0] + n[0] * t1 ; exit [1] = A[1] + n[1] * t1 ;
    entry[0] = A[0] + n[0] * t2 ; entry[1] = A[1] + n[1] * t2 ;

 } //  switch.

 bool bEntry = (entry.x() > -_X) && (entry.x() < _X) 
            && (entry.y() > -_Y) && (entry.y() < _Y) ;

 bool bExit  = (exit.x() > -_X) && (exit.x() < _X) 
            && (exit.y() > -_Y) && (exit.y() < _Y) ;

 if( (!bEntry) && (!bExit) ) return false; 

 simHit->setEntryExitPoints( entry, exit ) ;

 bentry_par = bEntry ;
 bexit_par = bExit;
 
 return true ; 

} // hitEntryExit.





bool DetUnit::hitEntryExit( Hep3Vector & A, 
			    Hep3Vector & n, 
			    SiTrkSimHitExtended * simHit, 
			    int dummy )  throw(string) {// in det.unit frame

 const double n_epsil = 2. ;

 if( fabs( n.z() )< SIUTIL::get_epsil() )
  throw(string(" DetUnit::hitEntryExit: momentum in Z to small: x=" 
         + SIUTIL::d2s(n.x())+", y=" + SIUTIL::d2s(n.y())+", z=" + SIUTIL::d2s(n.z()) ) ) ;

 Hep3Vector entry, exit ;

 if( ( fabs(-_Z - A.z() ) < SIUTIL::get_epsil() ) 
  || ( fabs( _Z - A.z() ) < SIUTIL::get_epsil() ) ) A[2] += n_epsil*SIUTIL::get_epsil() ;

 double t1 = (-_Z - A.z())/n.z() ;
 double t2 = ( _Z - A.z())/n.z() ;

 switch( t2 - t1 > 0. ){

  case true:  // 1: entry, 2:exit

    entry[2] =-_Z; exit[2] = _Z;

    entry[0] = A[0] + n[0] * t1 ;  entry[1] = A[1] + n[1] * t1 ;
    exit [0] = A[0] + n[0] * t2 ;  exit [1] = A[1] + n[1] * t2 ;

   break;

  default:   // 2: entry, 1:exit

    entry[2] = _Z; exit[2] = -_Z; 

    exit [0] = A[0] + n[0] * t1 ; exit [1] = A[1] + n[1] * t1 ;
    entry[0] = A[0] + n[0] * t2 ; entry[1] = A[1] + n[1] * t2 ;

 } //  switch.

 bool bEntry = (entry.x() > -_X) && (entry.x() < _X) 
            && (entry.y() > -_Y) && (entry.y() < _Y) ;

 bool bExit  = (exit.x() > -_X) && (exit.x() < _X) 
            && (exit.y() > -_Y) && (exit.y() < _Y) ;

 // Two following lines are changed as compare to previous function:
 simHit->setEntryExitPoints( entry, exit ) ;

 if( (!bEntry) && (!bExit) )   return false; 

 return true ; 

} // hitEntryExit.



void DetUnit::setHit_tmp( SimTrackerHit *simHit ) throw(string) {

  if( !isRefPointDone() ) throw(string("DetUnit::setHit: ref.point is not set."));

  _simHit_tmp = new SiTrkSimHitExtended( 
                *( dynamic_cast< SimTrackerHitImpl* >(simHit) ) ) ;

  const double *hitGlobal_Pos = simHit->getPosition() ;
  const float  *hitGlobal_Mom = simHit->getMomentum() ;

  Hep3Vector hitPos_tmp = Hep3Vector( hitGlobal_Pos[0], hitGlobal_Pos[1], hitGlobal_Pos[2] ) ;
  Hep3Vector hitMom_tmp = Hep3Vector( hitGlobal_Mom[0], hitGlobal_Mom[1], hitGlobal_Mom[2] ) ;

  // set global pos, mom to sim hit extended:
  _simHit_tmp->setHitGlobalPosMom( hitPos_tmp, hitMom_tmp ) ;

  if( !checkHit( hitPos_tmp, hitMom_tmp ) ) throw( string("1 DetUnit::setHit_tmp: it is impossible here.") ) ;

  // transform to local coordinate system
  getLocalPos( hitPos_tmp ) ;
  getLocalVector( hitMom_tmp ) ;

  // set local pos, mom to sim hit extended:
  _simHit_tmp->setHitLocalPosMom( hitPos_tmp, hitMom_tmp ) ;

  if( hitEntryExit( hitPos_tmp, hitMom_tmp, _simHit_tmp ) ){ // _hitEntry_tmp, _hitExit_tmp

   _setHitTmp_XY = true ;

  }else {

    delete _simHit_tmp ;

    _setHitTmp_XY = false ;

  }
  
  return ;

} // MAIN setHit_tmp .



void DetUnit::setHit_tmp( SimTrackerHit *simHit, bool bb ) throw(string) {

  if( !isRefPointDone() ) throw(string("DetUnit::setHit: ref.point is not set."));

  _simHit_tmp = new SiTrkSimHitExtended( 
                *( dynamic_cast< SimTrackerHitImpl* >(simHit) ) ) ;

  const double *hitGlobal_Pos = simHit->getPosition() ;
  const float  *hitGlobal_Mom = simHit->getMomentum() ;

  Hep3Vector hitPos_tmp = Hep3Vector( hitGlobal_Pos[0], hitGlobal_Pos[1], hitGlobal_Pos[2] ) ;
  Hep3Vector hitMom_tmp = Hep3Vector( hitGlobal_Mom[0], hitGlobal_Mom[1], hitGlobal_Mom[2] ) ;

  // set global pos, mom to sim hit extended:
  _simHit_tmp->setHitGlobalPosMom( hitPos_tmp, hitMom_tmp ) ;

  if( !checkHit( hitPos_tmp, hitMom_tmp ) ) throw( string("1 DetUnit::setHit_tmp: it is impossible here.") ) ;

  // transform to local coordinate system
  getLocalPos( hitPos_tmp ) ;
  getLocalVector( hitMom_tmp ) ;

  // set local pos, mom to sim hit extended:
  _simHit_tmp->setHitLocalPosMom( hitPos_tmp, hitMom_tmp ) ;

  bool b_entry, b_exit ;
  if( hitEntryExit( hitPos_tmp, hitMom_tmp, _simHit_tmp, b_entry, b_exit ) ){ // _hitEntry_tmp, _hitExit_tmp

   _setHitTmp_XY = true ;

if( ( (b_entry)&&(!b_exit) )||( (!b_entry)&&(b_exit) ) ){
 cout <<"DetUnit::setHit_tmp: hitEntryExit= true! [layer,wedge,du] = ["
      << getLayerN() << ", "<< getWedgeN()<<", "<<getDetUnitN() 
      <<"] "<< endl ;
 cout <<"DetUnit::setHit_tmp: hitEntryExit true: d.u.=["<<get_X()<<","<<get_Y()<<","<<get_Z()<<"]"<< endl ;
 cout <<" entryPoint="<< _simHit_tmp->entryPoint() << endl ;
 cout <<" exitPoint="<< _simHit_tmp->exitPoint() << endl ;
 cout <<" getHitLocalPos() = "<<  _simHit_tmp->getHitLocalPos() << endl ;
 cout <<" getHitGlobalPos() = "<<  _simHit_tmp->getHitGlobalPos() << endl  << endl ;
}

  }else {

    delete _simHit_tmp ;

    _setHitTmp_XY = false ;

  }
  
  return ;

} // setHit_tmp .





// used in show for debug only:
void DetUnit::setHit_tmp( SimTrackerHit *simHit, int dummy ) throw(string) {

  if( !isRefPointDone() ) throw(string("DetUnit::setHit: ref.point is not set."));

  _simHit_tmp = new SiTrkSimHitExtended( 
                *( dynamic_cast< SimTrackerHitImpl* >(simHit) ) ) ;

  const double *hitGlobal_Pos = simHit->getPosition() ;
  const float  *hitGlobal_Mom = simHit->getMomentum() ;

  Hep3Vector hitPos_tmp = Hep3Vector( hitGlobal_Pos[0], hitGlobal_Pos[1], hitGlobal_Pos[2] ) ;
  Hep3Vector hitMom_tmp = Hep3Vector( hitGlobal_Mom[0], hitGlobal_Mom[1], hitGlobal_Mom[2] ) ;

  // set global pos, mom to sim hit extended:
  _simHit_tmp->setHitGlobalPosMom( hitPos_tmp, hitMom_tmp ) ;

  if( !checkHit( hitPos_tmp, hitMom_tmp ) ) throw( string("1 DetUnit::setHit_tmp: it is impossible here.") ) ;

  // transform to local coordinate system
  getLocalPos( hitPos_tmp ) ;
  getLocalVector( hitMom_tmp ) ;

  // set local pos, mom to sim hit extended:
  _simHit_tmp->setHitLocalPosMom( hitPos_tmp, hitMom_tmp ) ;

  if( hitEntryExit( hitPos_tmp, hitMom_tmp, _simHit_tmp, dummy ) ) // _hitEntry_tmp, _hitExit_tmp

   _setHitTmp_XY = true ;

  else {

    _setHitTmp_XY = false ;

  }
  
  return ;

} // setHit_tmp(... dummy) .













void DetUnit::add_signal_from_simHit( hit_map_type & hit_signal, 
				      SiTrkSimHitExtended * simHit ) {

  // Fill the global map with all hit pixels from this event
  for ( hit_map_type::const_iterator im = hit_signal.begin() ;
	im != hit_signal.end(); im++) {

    _signal[(*im).first] += Amplitude( (*im).second, simHit ) ;

#ifdef _DEBUG_DU_1_
int chan =  (*im).first ;

pair<int,int> ip = PixelDigi::channelToPixel(chan) ;

cout << "DetUnit::add_signal_from_simHit: chan="<< chan <<" pixel (" << ip.first << "," << ip.second <<")"
	     << endl
     << "  signal[(*im).first] =" << _signal[(*im).first] << endl ;
#endif

  } // for.

}  //  DetUnit::set_signal.






void DetUnit::addRecHit( TrackerHitImpl* recHit ) {

 _recHits.push_back( recHit ) ;

} // .







void DetUnit::check() throw(string) {

 string CC = "DetUnit::check():" ;
 
 for(SiTrkSimHitExtendedVec::const_iterator ihit = _simHits.begin(); 
 			      	    ihit != _simHits.end() ; ++ihit ) {
 
  const double * pos = (*ihit)->getPosition() ;
  const float  * mom = (*ihit)->getMomentum() ;

  cout << CC + " hit global pos="<< pos[0] <<", "<< pos[1] <<", "<< pos[2]
       << " global mom="<< mom[0]<<", "<< mom[1]<<", "<< mom[2]
       << endl ;

 } // for(vector<double *>::iterator pos.

} // check.




void DetUnit::show( string SS ){

 cout << SS << endl <<" DetUnit:show(): (X,Y,Z)=("<<_X <<", "<<_Y <<", "<<_Z << endl << endl ;

 cout <<" Rmin = "<< _Rmin << " Rmax = " << _Rmax 
      <<" Zmin = "<< _Zmin << " Zmax = " << _Zmax << endl
      <<" _X = " << _X << " _Y = " << _Y << " _Z = " << _Z << endl 
      <<" _pixelSizeX = " << _pixelSizeX_duFrame
      <<" _pixelSizeY = " << _pixelSizeY_duFrame
      <<" B field = " << _globalBField
      <<"_tanLorentzAnglePerTesla = " << _tanLorentzAnglePerTesla << endl
      <<" _electronPerADC = " << _electronPerADC
      <<" _adcFullScale = " << _adcFullScale << endl
      <<" _noiseInElectrons = " << _noiseInElectrons
      <<" _noiseInAdcCounts = "<< _noiseInAdcCounts << endl
      <<" _thresholdInNoiseUnits = " <<_thresholdInNoiseUnits
      <<" _pixelThresholdInE = " << _pixelThresholdInE<< endl
      <<" _addNoise = " << _addNoise
      <<" _addNoisyPixels = " << _addNoisyPixels
      <<" _applyPixelInefficiency = "<< _applyPixelInefficiency<< endl
      <<" _rocSizeInX = "<<_rocSizeInX
      <<" _rocSizeInY = " << _rocSizeInY
      <<" _nLayersInDet = " << _nLayersInDet << endl ;
} // show.









void DetUnit::show( SimTrackerHit *simHit ){

 setHit_tmp( simHit, 999 ) ;

 cout<< endl <<"DetUnit::show: _setHitTmp_XY = "<< _setHitTmp_XY << endl ;

 cout <<" (X,Y,Z)=("<<_X <<", "<<_Y <<", "<<_Z <<")"<< endl << endl ;

 SIUTIL::out3("Entry:", _simHit_tmp->entryPoint() ) ;
 SIUTIL::out3("Exit:", _simHit_tmp->exitPoint() ) ;
 SIUTIL::out3("hitLocalPos:", _simHit_tmp->getHitLocalPos() ) ;

 Hep3Vector a = _simHit_tmp->getHitLocalPos() - _simHit_tmp->entryPoint() ;
 cout <<" dist(Entry-hitLocal) = "<< a.mag() << endl ;

 cout <<"mass = " << _simHit_tmp->getMCParticle()->getMass() << ", pid="<<_simHit_tmp->getMCParticle()-> getPDG()  << endl << endl ;

} // show.


} // namespace .
