#include "Pixel/SiRectangularDetUnit.h"
#include "CLHEP/Random/RandGauss.h"

namespace SIDETUNIT{


void DetUnit::add_noise() throw(string) {


#ifdef _DEBUG_NOISE_
  cout << "DetUnit::add_noise() " << _noiseInElectrons << endl;
#endif

  if ( !_addNoise ) return ;

  // First add noise to hit pixels
  for ( signal_map_iterator i = _signal.begin(); i != _signal.end(); i++) {
     double noise  = RandGauss::shoot(0.,_noiseInElectrons) ;
     (*i).second += Amplitude( noise,0);
#ifdef _DEBUG_NOISE_
cout <<" chan = "<< (*i).first << " charge = " << (*i).second 
     << " noise = " << noise  << endl;
#endif
  } // for ( signal_map_iterator i.

  if( !_addNoisyPixels )  // Option to skip noise in non-hit pixels
    return;

  // Add noise on non-hit pixels
  int   num_Y = 2 * getHalfNpixelsY(),
  	num_X = 2 * getHalfNpixelsX() ;

  int numberOfPixels = num_X * num_Y ;

  map<int,double, less<int> > otherPixels ;
  map<int,double, less<int> >::iterator mapI ;

  _theNoiser->generate( numberOfPixels, 
                       _thresholdInNoiseUnits, //thr. in un. of nois, =5.
		       _noiseInElectrons, // noise in elec. 
                        otherPixels );

#ifdef _DEBUG_NOISE_ 
  cout <<  " Add noisy pixels det(colX, rowY) = (" << num_X << ", " << num_Y
       << ") noiseInElectrons = " << _noiseInElectrons << endl  
       << " thresholdInNoiseUnits = " << _thresholdInNoiseUnits 
       << " numberOfPixels = " << numberOfPixels  << endl
       << " number of non-hit noise pixels = " << otherPixels.size() << endl;
#endif

  for (mapI = otherPixels.begin(); mapI != otherPixels.end(); mapI++) {
    int iy = ((*mapI).first) / num_X ;
    int ix = ((*mapI).first) - (iy * num_X);

    // Keep for a while for testing.
    if( iy < 0 || iy > (num_Y - 1) )  
       throw(string(" DetUnit::add_noise: error in iy ") + SIUTIL::i2s(iy)+ string(" max Y = ") + SIUTIL::i2s(num_Y - 1) ) ;
    if( ix < 0 || ix > (num_X - 1) )
       throw(string(" DetUnit::add_noise: error in ix ") + SIUTIL::i2s(ix)+ string(" max X = ") + SIUTIL::i2s(num_X - 1) ) ;

    int chan = SIUTIL::PixelDigi::pixelToChannel(ix, iy);

#ifdef _DEBUG_NOISE_ 
    cout<<" Storing noise: mapI.first =" << (*mapI).first << " ampl = " << (*mapI).second 
	<< " pix(x,y)=(" << ix << ", " << iy << ") chan = " << chan << endl ;
#endif

    if(_signal[chan] == 0) {
      double noise = double( (*mapI).second );
      _signal[chan] = Amplitude (noise, 0);
    }

  } //  for (mapI = otherPixels.begin.

} // add_noise.

} //namespace SIDETUNIT{
