#include "Pixel/DiskDetUnit.h"
#include "UTIL/CellIDEncoder.h"
#include "IMPL/LCCollectionVec.h"
#include "IMPL/LCRelationImpl.h"  

namespace SIDETUNIT{


void DiskDetUnit::make_digis( LCCollectionVec * rawVec, 
			        LCCollectionVec * relVec ) throw(string) {

#ifdef _DEBUG_MAKEDIGIS_
cout << endl <<"DiskDetUnit::make_digis: (Layer, Ndu) = ("<< getNLayer() <<","<< getDetUnitN() <<")" << endl
     << "         pixel threshold " << _pixelThresholdInE << endl
     << "         List pixels passing threshold  _signal.size() = "<<_signal.size() << endl ;
#endif

 CellIDEncoder<TrackerRawDataImpl> *encoderTrackerRawData 
      = new CellIDEncoder<TrackerRawDataImpl>( _cellIDEncoding, rawVec ) ;

 for ( signal_map_iterator i  = _signal.begin(); 
 			   i != _signal.end(); i++) {

   double signalInElectrons = (*i).second ;   // signal in electrons

   // Do only for pixels above threshold
   if ( signalInElectrons < _pixelThresholdInE ) continue ;

   int adc = int( signalInElectrons / _electronPerADC ) ; // calibrate gain
   adc = min(adc, _adcFullScale) ; // Check maximum value
      
   int chan =  (*i).first ;  // channel number
   pair<int,int> ip = SIUTIL::PixelDigi::channelToPixel(chan) ;

#ifdef _DEBUG_MAKEDIGIS_
cout <<" chan="<< (*i).first << " charge=" << (*i).second << " " << signalInElectrons 
     << " adc=" << adc <<" pix(ix,iy)="<< ip.first << " " << ip.second << endl ;
#endif 

  (*_bf64)["technology"] = (unsigned int)getDetUnitType() ;
  (*_bf64)["subdet"    ] = 0 ; // VXD
  (*_bf64)["layer"     ] = getLayerN() ;
  (*_bf64)["ladder"    ] = getDetUnitN() ;
  (*_bf64)["pixelX"    ] = (unsigned int)ip.first ;
  (*_bf64)["pixelY"    ] = (unsigned int)ip.second ;

  SIUTIL::TcellID cellid = _bf64->getValue() ; // lcio::long64, def. in SiTrk.h

  // create elements of collections:
  TrackerRawDataImpl* rawhit = new TrackerRawDataImpl() ;

  encoderTrackerRawData->setValue( cellid ) ;
    
  encoderTrackerRawData->setCellID( rawhit ) ;

  rawhit->setTime( 9999 ) ;

  // according to lcio/v01-08/src/cpp/include/IMPL/TrackerRawDataImpl.h: 
  rawhit->adcValues().clear() ; rawhit->adcValues().resize( 1 ) ;
  rawhit->adcValues()[0] = (short int)( adc ) ;
  //  or  const EVENT::ShortVec AdC; AdC.push_back( adc );// rawhit->setADCValues( AdC );

  rawVec->addElement( rawhit ) ; //  rawVec->push_back( rawhit ) ;

  // To make rel collection
  const SiTrkSimHitExtendedVec simHits =  (*i).second.hits() ;
  
  
  if( simHits.size()==0 ) 
     throw(string("BarrelDetUnit::make_digis : simHits.size() = " + SIUTIL::i2s(simHits.size()) )) ;
  else {
     LCRelationImpl * rel = new LCRelationImpl( rawhit, (*simHits.begin()) ) ;
     relVec->addElement( rel ) ;
  }
  
/**  for(SiTrkSimHitExtendedVec::const_iterator is  = simHits.begin();
 *                                     is != simHits.end(); ++is ){
 *    LCRelationImpl * rel = new LCRelationImpl( rawhit, (*is) ) ;
 *    relVec->addElement( rel ) ; } // for(SiTrkSimHitVec::const_iterator is.
 */

 } // for ( signal_map_iterator i.
#ifdef _DEBUG_MAKEDIGIS_
cout << "      End List pixels passing threshold " << endl ;
#endif 

 delete encoderTrackerRawData ;

} // make_digis.

} //namespace SIDETUNIT{
