#include "Pixel/SiRectangularDetUnit.h"
#include "PixelChipIndices.h"

namespace SIDETUNIT{

#ifdef _DEBUG_INEFFICIENCY_
unsigned int iCalc = 0 ;
#endif

void DetUnit::pixel_inefficiency() throw(string) {

  if( !_applyPixelInefficiency ) {
#ifdef _DEBUG_INEFFICIENCY_
cout << endl <<"DetUnit::pixel_inefficiency: do not apply for (Layer, Ndu) = ("<< getNLayer() <<","<< getDetUnitN() <<")" << endl ;
#endif
    return ;
  }
  // Predefined efficiencies
  float pixelEfficiency  = 1.0;
  float columnEfficiency = 1.0;
  float chipEfficiency   = 1.0;

  int layerIndex = getLayerN() ;

  if ( _detUnitType == SIUTIL::barrelLaddersPixels ) {  // barrel layers

   // int layerIndex = getLayerN() ;
    if( (layerIndex < 0 )||(layerIndex > (int)_nLayersInDet ) ) throw(string("DetUnit::pixel_inefficiency: (layerIndex < 0 )||(layerIndex > _nLayersInDet: ")+SIUTIL::i2s(layerIndex)+" layers in det="+SIUTIL::i2s(_nLayersInDet) ) ;
    pixelEfficiency  = _thePixelEfficiency[layerIndex];
    columnEfficiency = _thePixelColEfficiency[layerIndex];
    chipEfficiency   = _thePixelChipEfficiency[layerIndex];

  } else {  // forward disks

    // For endcaps take same for each endcap
    pixelEfficiency  = _thePixelEfficiency[0];
    columnEfficiency = _thePixelColEfficiency[0];
    chipEfficiency   = _thePixelChipEfficiency[0];

  }

  // Initilize the index converter
  int numColumnsY = (int)(getHalfNpixelsY()) * 2, 
      numRowsX = (int)(getHalfNpixelsX()) * 2 ;

#ifdef _DEBUG_INEFFICIENCY_
cout << endl <<"DetUnit::pixel_inefficiency: (Layer, Ndu) = ("<< getNLayer() <<","<< getDetUnitN() <<")" << endl ;
cout << "  numColumnsY = " << numColumnsY << " numRowsX = " << numRowsX 
     << " _rocSizeInY = " << _rocSizeInY << " _rocSizeInX = " << _rocSizeInX << endl ;
#endif

  SIUTIL::PixelChipIndices indexConverter( _rocSizeInY, _rocSizeInX, // pixels in ROC
				   numColumnsY, numRowsX ) ; // pixels in det.unit (ladder)

  int chipX, chipY, rowX, colY ;
  map<int, int, less<int> > chips, columns;
  map<int, int, less<int> >::iterator iter;

#ifdef _DEBUG_INEFFICIENCY_
 cout <<"    Find out the number of columns and chips hits " << endl
      <<"    Loop over hit pixels(_signal), amplitude in electrons, channel = coded row,col." << endl ;
#endif

  for (signal_map_iterator i = _signal.begin();i != _signal.end();i++) {

    int chan = i->first;
    pair<int,int> ip = SIUTIL::PixelDigi::channelToPixel( chan ) ;
    int pixX = ip.first + 1 ;  //indices start from 1 in class PixelChipIndices
    int pixY = ip.second + 1 ;

    if( !indexConverter.chipIndices(  pixX, pixY,  // from 1 to max
    				     chipX,chipY,  // from 0 to max
				      rowX, colY ) ){ // from 1 to max
      // hit pixel near the board and ourside the ROC's (dead zone)
      _nbHitPixelsInROCdeadZone++ ;
#ifdef _DEBUG_INEFFICIENCY_
 cout <<"      Rare(1 loop) hit pixel outside the rocs: _nbHitPixelsInROCdeadZone "<<_nbHitPixelsInROCdeadZone << endl ;
#endif
      continue ;
    }

    int chipIndex = indexConverter.chipIndex( chipX, chipY ) ;

    pair<int,int> dColId = indexConverter.dColumn( rowX, colY ) ;

    //    int pixInChip  = dColId.first;
    int dColInChip = dColId.second ;
    int dColInDet = indexConverter.dColumnIdInDet( dColInChip, chipIndex ) ;

#ifdef _DEBUG_INEFFICIENCY_
iCalc++;
if( iCalc < 0 ) {
 cout<<" iCalc = "<<iCalc <<"    DBG: exit " << endl ; exit(0) ;
} else {
 cout <<" iCalc = "<< iCalc <<"    DBG: check indexConverter: " << endl ;
 cout <<"chan="<< chan << " pix(" << pixX << "," << pixY << ") charge=" 
      << (*i).second  << endl ;
 cout <<" chip("<< chipX << "," << chipY 
      << ") pix in chip(" << rowX << "," << colY << ")  chipIndex=" 
      << chipIndex << " pixInChip=" << dColId.first 
      << " dColInChip=" << dColInChip << " dColInDet=" << dColInDet << endl;
}
#endif

    chips[chipIndex]++ ;
    columns[dColInDet]++ ;

  } // for (signal_map_iterator i = _signal.begin().

#ifdef _DEBUG_INEFFICIENCY_
cout << " Loop start: chips.size " << chips.size() << endl;
#endif

  for ( iter = chips.begin(); iter != chips.end() ; iter++ ) {
    
#ifdef _DEBUG_INEFFICIENCY_
cout << " chips map: chipIndex " << iter->first << " number of hits = " << iter->second << endl;
#endif

   float rand  = RandFlat::shoot();
    
   if( rand > chipEfficiency ) chips[iter->first] = 0 ;

#ifdef _DEBUG_INEFFICIENCY_
cout <<" rand="<< rand << " chipIndex="  << iter->first << " number of hits = " << iter->second << endl;
if( iter->second == 0 ) cout << " chip erased " << endl ;
else  cout << " chip keeped " << endl;
#endif

  } // for ( iter.


#ifdef _DEBUG_INEFFICIENCY_
cout << " Loop start: columns.size() " << columns.size() << endl ;
#endif

  for ( iter = columns.begin(); iter != columns.end() ; iter++ ) {

#ifdef _DEBUG_INEFFICIENCY_
cout <<" columns map: dColInDet "<< iter->first << " number of hits = " << iter->second << endl;
#endif

    float rand  = RandFlat::shoot();

    if( rand > columnEfficiency ) columns[iter->first] = 0 ;

#ifdef _DEBUG_INEFFICIENCY_
cout <<" rand="<< rand << " dColInDet="  << iter->first << " number of hits = " << iter->second << endl;
if( iter->second == 0 ) cout << " column erased " << endl;
else  cout << " column keeped " << endl;
#endif

  } // for ( iter.

#ifdef _DEBUG_INEFFICIENCY_
cout << "  _signal.size() = " << _signal.size() 
     << "      Now loop again over pixel to kill some " << endl ;
#endif

  // Loop over hit pixels, amplitude in electrons, channel = coded row,col

  _nbHitPixelsInROCdeadZone = 0 ;

  for(signal_map_iterator i = _signal.begin();i != _signal.end(); i++) {

#ifdef _DEBUG_INEFFICIENCY_
cout << "       pix " << i->first << " charge=" << float(i->second) << endl;    
#endif
    //    int chan = i->first;

    pair<int,int> ip = SIUTIL::PixelDigi::channelToPixel(i->first);//get pixel pos

    int pixX = ip.first + 1 ;  // my indices start from 1
    int pixY = ip.second + 1 ;

    if( !indexConverter.chipIndices
         (pixX,pixY,chipX,chipY,rowX,colY) ){
      _nbHitPixelsInROCdeadZone++ ;

      if( _useRocDeadZone ){
        i->second.set(0.); // reset amplitude 
#ifdef _DEBUG_INEFFICIENCY_
 cout <<"      Rare(2 loop) hit pixel outside the rocs: _nbHitPixelsInROCdeadZone "<<_nbHitPixelsInROCdeadZone << endl ;
#endif
      } // if( _useRocDeadZone.
      continue ;
    } // if( !.
	 
    int chipIndex = indexConverter.chipIndex(chipX,chipY);

    pair<int,int> dColId = indexConverter.dColumn(rowX,colY);  // get dcol index
    int dColInDet = indexConverter.dColumnIdInDet(dColId.second,chipIndex);

#ifdef _DEBUG_INEFFICIENCY_
cout <<" chan ="<< i->first << " pix(" << pixX << "," << pixY 
     << ")  charge=" << (*i).second  << endl;
cout <<" chip("<< chipX << "," << chipY 
     << ")  pixID in chip(" << rowX << "," << colY 
     << ") chipIndex=" << chipIndex << " dColInDet=" << dColInDet << endl;
cout <<" number of hits in chip ="<<  chips[chipIndex] 
     << " number of hits in dColumn=" <<  columns[dColInDet] << endl; 
#endif

    float rand  = RandFlat::shoot();
    if( chips[chipIndex]==0 ||  columns[dColInDet]==0 
	|| rand > pixelEfficiency ) {
      // make pixel amplitude =0, pixel will be lost at clusterization    
      i->second.set(0.); // reset amplitude, 
#ifdef _DEBUG_INEFFICIENCY_
cout << "     pixel is killed i->second=" << float(i->second) << " rand=" 
     << rand << " number of hits in chip=" << chips[chipIndex] 
     << " number of hits in dColumn=" << columns[dColInDet] << endl ;
#endif

   } // if.

  }  //  for(signal_map_iterator i.

#ifdef _DEBUG_INEFFICIENCY_
  cout << "DetUnit::pixel_inefficiency : _nbHitPixelsInROCdeadZone = " 
       << _nbHitPixelsInROCdeadZone <<" for detUnit (Nlayer, Ndu)= ("<<getNLayer() <<","<<getDetUnitN() <<")"<< endl ;
#endif

} // pixel_inefficiency.

} //namespace SIDETUNIT{
