#ifndef SiTrkDigiProcessor_h
#define SiTrkDigiProcessor_h 1

//#define _DEBUG_PHYS_
//#define _DEBUG_

#include <iostream>
#include <list>

#include "marlin/Processor.h"
#include <string>
#include <vector>

#include "UTIL/BitField64.h"

#include "SiTracker.h"

using namespace std ;
using namespace lcio ;
using namespace marlin ;

/**  Marlin processor for digitization of Silicon barrel and disk layer
 * 
 * @author S.Shulga
 * @version $Id: SiTrkDigiProcessor.h,v 1.1 2007-11-07 10:52:37 shulga Exp $ 
 */

namespace SIDIGPROC{

class SiTrkDigiProcessor : public Processor {
  
 public:
  
  virtual Processor*  newProcessor() { return new SiTrkDigiProcessor ; }
  
  
  // constructor
  SiTrkDigiProcessor() ;
  // Destructor
  ~SiTrkDigiProcessor() {};
  
  /** Called at the begin of the job before anything is read.
   * Use to initialize the processor, e.g. book histograms.
   */
  virtual void init() ;

  /** Called by init():
   */
  void checkInit( string CC ) ;

  /** Called by init():
   */
  void show_geom( string CC ) ;
  
  /** Called for every run.
   */
  virtual void processRunHeader( LCRunHeader* run ) ;
  
  /** Called for every event - the working horse.
   */
  virtual void processEvent( LCEvent * evt ) ; 
  
  /** 
   */
  void createOutputCollections_digi ( LCCollection * simHits, 
                                 vector<LCCollection*> & collVec ) 
				       throw( string ) ;
  /** Called by createOutputCollections_digi, createOutputCollections_digi_phys
   */
  bool recognizeSimHit( SimTrackerHit *, SIUTIL::DetUnitType  &);

  /** Called by createOutputCollections_digi
   */
  void getCellID_bf64 (bool 	       ModeDetUnit, // to identify ReadOut Chip only
		        SIUTIL::DetUnitType    detUnitType,           // input
		        SimTrackerHit* simHit ,             // input
			bool         & Hit_In_DeadZone ,    // output
		        string       & cellID_hitEncoding,  // output
		        BitField64   * &cellID_hit_bf64,    // output
		        string       & cellID_rocEncoding,  // output
		        BitField64   * &cellID_roc_bf64 ) const throw( string ) ;

  /** Called for every event
   */
  virtual void check( LCEvent * evt ) ; 
  
  /** Called after data processing for clean up.
   */
  virtual void end() ;
    
 protected:

  /** Parameters from steering file.
   */
  string _inputColName, _outputColName, _outputRelColName ;

  StringVec  _subDet_Technology ;
  
  int _printRunHeaderInform ; 

//  IntVec _iLayers ;

  SIDET::SiTracker *_detector ;

  double  _timeCut ; // init in init(), used in createOutputCollections_digi.CC

  int _nRun ;
  int _nEvt ;

} ;

} // namespace SITRK{.

#endif
