#include "SiTrkDigiProcessor.h"
#include "SiUtil.h"
#include "EVENT/LCParameters.h"

namespace SIDIGPROC{

SiTrkDigiProcessor aSiTrkDigiProcessor ;

 /** 
  *  C O N S T R U C T O R
  */
SiTrkDigiProcessor::SiTrkDigiProcessor() : Processor("SiTrkDigiProcessor") {
  
 // Processor description
 _description = "SiTrkDigiProcessor: digitization of Si tracker subdetector." ;
  
 _detector = new SIDET::SiTracker() ;

 registerInputCollection(  LCIO::SIMTRACKERHIT,
                           "InputCollectionName" , 
                           "Name of the SimTrackerHit collection"  ,
                           _inputColName, SIUTIL::get_sDefault() ) ;

 registerOutputCollection( LCIO::TRACKERRAWDATA,
                           "OutputCollections" , 
                           "Name of the output raw tracker data collection" ,
                            _outputColName, SIUTIL::get_sDefault() ) ;

 registerOutputCollection( LCIO::LCRELATION ,
                           "OutputRelColName" , 
                           "Name of the output raw to sim. relation collection" ,
                            _outputRelColName , SIUTIL::get_sDefault() ) ;
 /** Parameters :
  */
 registerProcessorParameter("SubDet_Technology",
 			    "Subdetector name, type of technology"
			    ,_subDet_Technology, SIUTIL::get_sVecDefault() ) ;

 registerProcessorParameter("PrintRunHeaderInform",
                            " equals 1 if print run header information",
			    _printRunHeaderInform, int(0) ) ;

// registerProcessorParameter("Layers",
//                            "Layers to be digitize",
//			    _iLayers, SIUTIL::get_iVecDefault() ) ;

} // constructor SiTrkDigiProcessor.


 /** 
  */
void SiTrkDigiProcessor::init() { 
  
  cout<<"DBG in SiTrkDigiProcessor::init() starteded." << endl ;
try{ 
  printParameters ( ) ; 

 _timeCut = 10000000. ; // time cut off

 _detector->setTechnology( _subDet_Technology ) ;

 _detector->gearInit(  "From SiTrkDigiProcessor::init() for " + name() + ":"  ) ;

} catch( string report ) {

   cout << "SiTrkDigiProcessor::init():" << endl 
        <<" Report:"<< report+". Program terminated." << endl;  exit( 0 );

}
 _nRun = 0 ;
 _nEvt = 0 ;

  cout<<"DBG in SiTrkDigiProcessor::init() ended." << endl ;

} // init().

 /**  
  */
void SiTrkDigiProcessor::processRunHeader( LCRunHeader* run ) { 

 /**  add new header parameters 
  */
  run->parameters().setValue ("[ProcessorName]", name() ) ;
  run->parameters().setValue (name()+"[ProcessorType]"   , "SiTrkDigiProcessor" ) ;
  run->parameters().setValue (name()+"[InputCollection]" , _inputColName ) ;
  run->parameters().setValues(name()+"[subDet&Technology]" , _subDet_Technology ) ;
  run->parameters().setValue (name()+"[OutputCollection]", _outputColName ) ;

 /**  show all run header parameters
  */
  if( _printRunHeaderInform == 1 ) SIUTIL::ShowRunHeaderParameters(" SiTrkDigiProcessor::processRunHeader: " +  name() + ":  After add new run parameters: ", run ) ;

  _nRun++ ;

}  // processRunHeader().








 /** 
  */
void SiTrkDigiProcessor::processEvent( LCEvent * evt ) { 
 string CC = "SiTrkDigiProcessor::processEvent(" 
 	    + SIUTIL::i2s( evt->getEventNumber() ) 
	    + ") by processor " + name() ;
	    
 cout << endl << CC + ", Run N " << evt->getRunNumber() << endl ;

 ++_nEvt ;

 LCCollection* simHits = NULL ;

 vector<LCCollection*> collVec ;

 try{ 

   simHits = evt->getCollection( _inputColName ); 

#ifdef _DEBUG_PHYS_
int flag = simHits->getFlag() ; 
LCFlagImpl *Fl = new LCFlagImpl( flag ) ;
cout<< CC+" bitSet( LCIO::THBIT_MOMENTUM ) = " << Fl->bitSet( LCIO::THBIT_MOMENTUM ) << endl ;
#endif
   
   createOutputCollections_digi( simHits, collVec ) ;

 } catch( EVENT::DataNotAvailableException &e ){

   cout << CC << e.what() << endl ;

   return;

 } catch( string report ) {

   cout << CC << endl 
        <<" Report:"<< report+". Program terminated." << endl;  exit( 0 );

 } catch( EVENT::ReadOnlyException &e ){
 
   cout << CC + " ReadOnlyException:" << e.what()
        << " Program terminated." << endl ;  exit( 0 ) ;
 
 }

#ifdef _DEBUG_PHYS_1_
cout<< CC+" oOOo number of output coll's = " << collVec.size() << endl ;
cout<< " oOOo collVec.name 1 = " << _outputColName << " oOOo collVec.name 2 = " << _outputRelColName << endl
    << " oOOo input coll size(sim hits)= " << simHits->getNumberOfElements() << endl
    << " oOOo output coll size(raw)= " << collVec[0]->getNumberOfElements() << endl ;
if(collVec.size() > 1) 
  cout<< " oOOo second output coll size(rel)= " << collVec[1]->getNumberOfElements() << endl ;
#endif
 if(collVec.size() > 0)
   evt->addCollection( collVec[0], _outputColName );
 
 if(collVec.size() > 1) 
   evt->addCollection( collVec[1], _outputRelColName );

} // processEvent().




/**
 */
void SiTrkDigiProcessor::check( LCEvent * evt ) { 
  // nothing to check here - could be used to fill checkplots in reconstruction processor
}

/**
 */
void SiTrkDigiProcessor::end(){ 
//   cout << "SiTrkDigiProcessor::end()  " << name() 
// 	    << " processed " << _nEvt << " events in " << _nRun << " runs "
// 	    << endl ;

}

} // namespace SITRK.
