#include "SiTrkDigiProcessor.h"
#include "EVENT/LCCollection.h"

namespace SIDIGPROC{

/**  Prepares collections to be appended to LCEvent:
 *   Called by SiTrkDigiProcessor::processEvent.
 */

void SiTrkDigiProcessor::createOutputCollections_digi
                      ( LCCollection           * simHits,  
		        vector<LCCollection*>  & result ) 
			   throw( string ) {
 string CC = "SiTrkDigiProcessor::createOutputCollections_digi" ;

 /** Loop to fill _detector.
  */

 int NHits = simHits->getNumberOfElements();

 int NcalcP = 0 ;
 double minMom = 1000. ; // MeV
 _detector->_hitsInDeadZone = 0 ;
 _detector->_hitsInTwoDU = 0 ;

 for( int i = 0; i < NHits; ++i ) {

  SimTrackerHit* simHit = dynamic_cast< SimTrackerHit* > ( simHits->getElementAt( i ) );

  const float  *mom = simHit->getMomentum() ;
  Hep3Vector mH = Hep3Vector(mom[0], mom[1], mom[2]) ;

  if( mH.mag() > minMom ){

    NcalcP++ ;

#ifdef _DEBUG_PHYS_
cout<<"<><><> sim.pos = "<< pH <<" R="<<pH.perp() <<" phi="<<pH.phi()<<" mom=" << mH << endl ;
#endif

  }

  if( simHit->getTime() > _timeCut ) continue ;  // time cut off, not used yet

  _detector->fillDetUnits( simHit ) ;  

 } // for( int i = 0; i < NHits...

 cout << "Final: " + CC
      << " NcalcP = "<< NcalcP <<" with p > "<<minMom<<" MeV "
      << " from total " << NHits << endl 
      << " Total accepted sim hits " << _detector->getTotalAcceptedSimHits()<< endl 
      << " Hits in dead zone " << _detector->_hitsInDeadZone << endl 
      << " Hits in two det.units " << _detector->_hitsInTwoDU << endl ;
 
 _detector->howManySimHits() ;
//----

 /** return collections
   */

 _detector->digitizeDetUnits( result ) ;

 _detector->clearDetUnits() ;

#ifdef _DEBUG_PHYS_
cout << "<<end>> debug in "+CC << endl;
#endif

} //  SiTrkDigiProcessor::createOutputCollections_digi.


} // namespace SITRK.
