#ifndef _SIPIXEL_DETUNIT_DIGITIZER_H_
#define _SIPIXEL_DETUNIT_DIGITIZER_H_

//#define _DEBUG_ 
//#define _DEBUG_1_
//#define _DEBUG_NEXT_HIT_
//#define _DEBUG_NOISE_

#include "CLHEP/Vector/TwoVector.h"
#include "CLHEP/Vector/ThreeVector.h"

#include "SiUtil.h"
#include "Pixel/SiRectangularDetUnit.h"
#include "MyG4UniversalFluctuationForSi.h"
#include "Pixel/SiTrkSimHitExtended.h"

using namespace CLHEP ;

  /** A simple Pixel Digitizer (following to CMS software).
   *
   *  @version $Id: SiPixelDetUnitDigitizer.h, v 1 2007/06/06 shulga
   */
namespace SIDIGIR{


  /** CMS software, ORCA
   *  Define a class for 3D ionization points and energy
   */
class EnergyDepositUnit {
  
  public:
    
    EnergyDepositUnit(): _energy(0),_position(0,0,0){}
    
    EnergyDepositUnit(  double energy,
    			double x, 
			double y, 
			double z)
			: _energy(energy),_position(x,y,z){}

    EnergyDepositUnit(  double energy, 
    			Hep3Vector position ) 
		: _energy(energy),_position(position){}

    double x() const { return _position.x(); }
    double y() const { return _position.y(); }
    double z() const { return _position.z(); }

    double energy() const { return _energy; }

  private:

    double _energy;

    Hep3Vector _position;

};



  /** CMS software, ORCA
   *  Define class to store signals on the collection surface
   */
class SignalPoint {

  public:

    SignalPoint() :_pos(0,0), 
    		   _time(0),
		   _amplitude(0),
		   _sigma_x(1.),
		   _sigma_y(1.),
		   _hitp(0) {}

    SignalPoint( double x, 
    		 double y, 
		 double sigma_x, double sigma_y,
		 double t, double a = 1.0 ) 
	: _pos(x,y), 
	  _time(t),
	  _amplitude(a),
	  _sigma_x(sigma_x),
	  _sigma_y(sigma_y),
	  _hitp(0) {}

    SignalPoint( double x, 
    		 double y,
		 double sigma_x,
		 double sigma_y,
		 double t,
		 const SIDETUNIT::SiTrkSimHitExtended *hit, 
		 double a=1.0 ) 
	 : _pos( x, y ), 
	   _time( t ), 
	   _amplitude( a ),
	   _sigma_x( sigma_x ),
	   _sigma_y( sigma_y ),
	   _hitp( hit ) {}

    const Hep2Vector& position() const { return _pos;}

    double x()         const { return _pos.x();}
    double y()         const { return _pos.y();}

    double sigma_x()   const { return _sigma_x;}
    double sigma_y()   const { return _sigma_y;}

    double time()      const { return _time;}
    double amplitude() const { return _amplitude;}

    const SIDETUNIT::SiTrkSimHitExtended& hit()     { return *_hitp;}

    SignalPoint& set_amplitude( double amp) { _amplitude = amp; return *this;}

  private:

    Hep2Vector _pos ;

    double     _time ;

    double     _amplitude ;

    double     _sigma_x ;   // gaussian sigma in the x direction (cm)

    double     _sigma_y ;   //    "       "          y direction (cm)

    const SIDETUNIT::SiTrkSimHitExtended *_hitp ;

};






class SiPixelDetUnitDigitizer {

public:  

  SiPixelDetUnitDigitizer() ;

  ~SiPixelDetUnitDigitizer() ;

  void digitize( SIDETUNIT::DetUnit *det ) throw(string) ;

  void compute_charges( SIDETUNIT::SiTrkSimHitExtended * hit ) throw(string) ;

  void primary_ionization( SIDETUNIT::SiTrkSimHitExtended * hit ) throw(string) ;

// Fluctuate the average eloss in a segment, use the Geant3 routine glandz
  void fluctuateEloss(  float particleMass,
			float particleMomentum,
			float eloss,
			float length,
			int NumberOfSegs,
			float elossVector[] )  throw(string);

  void drift( SIDETUNIT::SiTrkSimHitExtended * hit ) throw(string) ;

  void induce_signal( SIDETUNIT::SiTrkSimHitExtended * hit ) throw(string) ;

  int get_sumSimHits() const { return _sum_sim_Hits ; } ;
  int get_sumInduceSignals() const { return _sum_induce_signals ; } ;

  void reset_sumSimHits() ;
  void reset_sumInduceSignals() ;

private: 

  //-- primary ionization
  unsigned int    _NumberOfSegments;  // =20 does not work

  //-- induce_signal
  double _ClusterWidth;       // Gaussian charge cutoff width in sigma units

  // go from Geant energy GeV to number of electrons
  double _GeVperElectron;     // 3.7E-09 

  //-- drift
  double _Sigma0;   //=0.0007  // Charge diffusion in microns for 300 micron Si
  double _Dist300;  //=0.0300  // Define 300microns for normalization 

  bool _fluctuateCharge;

  bool   _isItBarrel ; 		// Is it barrel or forward

  unsigned int _numPixelsY ; // number of pixel columns in a detUnit
  unsigned int _numPixelsX ;    // number          rows

  double _pixelSizeX, _pixelSizeY ;

  double  _moduleThickness ;

  SIDETUNIT::DetUnit *_detp; // points to Det currently being digitized

  vector<SignalPoint> _collection_points ;

  vector<EnergyDepositUnit> _ionization_points ;

  //-- charge fluctuation
  double _deltaProductionCut ;  // The delta production cut, should be as in OSCAR = 30keV
				//                                           cmsim = 100keV
  // The eloss fluctuation class from G4 
  MyG4UniversalFluctuationForSi _fluctuate;

  int _sum_sim_Hits, _sum_induce_signals ;

}; // SiPixelDetUnitDigitizer.

} // namespace SIDIGIR.

#endif
