#include "SiDigitizer.h"

namespace SIDIGIR{

/** Digitizer steering routine.
 */
void SiPixelDetUnitDigitizer::digitize( SIDETUNIT::DetUnit *det ) throw(string) {

#ifdef _DEBUG_
cout <<"SiPixelDetUnitDigitizer::digitize: begin:"<< endl ;
#endif

  _detp = det ;    // cache for convenience

  SIDETUNIT::SiTrkSimHitExtendedVec simHits = _detp->getSimHits() ;

  unsigned int NbSimHits = simHits.size() ; 
  
  _sum_sim_Hits += NbSimHits ;

  if( ( NbSimHits < 1 ) && ( !_detp->isAddNoisyPixels() ) ) return ;

//  _signal.clear() ; // clear signal array

  _numPixelsY = 2*_detp->getHalfNpixelsY() ;  //  number of cols&rows of pixels

  _numPixelsX = 2*_detp->getHalfNpixelsX() ; 

  _moduleThickness = _detp->getThickness(); // full detector thickness

  _detp->getPixelSizes( _pixelSizeX, _pixelSizeY ) ;

  // produce SignalPoint's for all SimHit's in detector
  // Loop over hits
  unsigned int i_H = 0 ;
  for(SIDETUNIT::SiTrkSimHitExtendedVec::iterator  ihit  = simHits.begin(); 
  				ihit != simHits.end(); ++ihit, ++i_H ){

#ifdef _DEBUG_NEXT_HIT_
cout <<" SiPixelDetUnitDigitizer::digitize: digitize hit N = "<< i_H << " from given det.unit ("<<_detp->getNLayer() <<","<< _detp->getDetUnitN() <<")"<< endl ;
#endif
    _collection_points.clear() ;  // Clear the container 

    // fill _collection_points for this SimHit, indpendent of topology
    compute_charges( *ihit ) ;

    // compute induced signal on readout elements 
    // and add to _detp->_signal
    induce_signal( *ihit ) ;

#ifdef _DEBUG_
cout <<""<< endl ;
if( i_H > 3 ){ cout <<"SiPixelDetUnitDigitizer::digitize: stop after "<< i_H<<"-th hit digitizing  " << endl ;
               exit(0) ; }
#endif

  } // for.
    
  _detp->add_noise();  // generate noise in det.unit

  _detp->pixel_inefficiency(); // Kill some pixels

//  make_digis(); // add hit pixels to the digis list

#ifdef _DEBUG_NEXT_HIT_
cout <<"SiPixelDetUnitDigitizer::digitize: end for det.unit (Layer,Ndu)=("<<_detp->getNLayer() <<","<< _detp->getDetUnitN() <<")"<< endl ;
#endif

}  // digitize.


} //namespace SIDIGIR.
