#include "SiDigitizer.h"

namespace SIDIGIR{

/**
 * Drift the primary ionizatioon points to the collection plane
 */
void SiPixelDetUnitDigitizer::drift( SIDETUNIT::SiTrkSimHitExtended * hit) throw(string) {

#ifdef _DEBUG_
  cout << endl << "SiPixelDetUnitDigitizer::drift: begin drift " << endl;
#endif
  
  _collection_points.resize( _ionization_points.size() ) ; // set size

  Hep3Vector driftDir = _detp->driftDirection() ; // drift in center

  if( driftDir.z() ==0. )
    throw(string("SiPixelDetUnitDigitizer::drift: drift in z is zero: it is impossible.")) ;
                                              // see also BarrelDetUnit.cc : setDetUnit_refPoint()
  double TanLorenzAngleX = driftDir.x()/driftDir.z() ; // tangen of Lorentz angle
  double TanLorenzAngleY = 0. ; // force to 0, driftDir.y()/driftDir.z();
  // I also need cosines to estimate the path length
  double CosLorenzAngleX = 1./sqrt(1.+TanLorenzAngleX*TanLorenzAngleX) ; //cosine
  double CosLorenzAngleY = 1. ;
  
#ifdef _DEBUG_1_
cout << " Lorentz Tan (X,Y) = (" << TanLorenzAngleX << "," << TanLorenzAngleY 
     <<") cos (X,Y) = ("<< CosLorenzAngleX << "," << CosLorenzAngleY << "); drift Xmax="
     << _moduleThickness*TanLorenzAngleX << " DriftDir=" << driftDir << endl;
#endif

  double Sigma_x = 1. ;  // Charge spread 
  double Sigma_y = 1. ;
  double DriftDistance ; // Distance between charge generation and collection 
  double DriftLength ;   // Actual Drift Lentgh
  double Sigma ;

#ifdef _DEBUG_1_
cout << " drift: _ionization_points.size() = " << _ionization_points.size() << endl;
#endif
  // loop over ionization points
  for (unsigned int i = 0; i != _ionization_points.size(); i++ ) {
        
    double SegX, SegY, SegZ; // position
    SegX = _ionization_points[i].x();
    SegY = _ionization_points[i].y();
    SegZ = _ionization_points[i].z();

    // Distance from the collection plane
    // Change drift direction to compensate for the tilt/cooridinate problem
    // Drift to +z, F_Electron in +z (see c-tor BarrelDetUnit)
    
    DriftDistance = (_moduleThickness/2. - SegZ); 
    
    //SSh: DriftDistance = (_moduleThickness/2. + SegZ); // Drift to -z

    if( (DriftDistance < 0.)||(DriftDistance > _moduleThickness) )
       throw(string("SiPixelDetUnitDigitizer::drift: _moduleThickness, DriftDistance = ")
	       + SIUTIL::d2s(_moduleThickness) + ", " + SIUTIL::d2s(DriftDistance) 
	       + string("\n") + " ion.points Nb= " + SIUTIL::i2s(_ionization_points.size()) ) ;
	
    // Assume full depletion now, partial depletion will come later.
    double XDriftDueToMagField = DriftDistance * TanLorenzAngleX;
    double YDriftDueToMagField = DriftDistance * TanLorenzAngleY;

    // Shift cloud center
    double CloudCenterX = SegX + XDriftDueToMagField;
    double CloudCenterY = SegY + YDriftDueToMagField;

    // Calculate how long is the charge drift path
    DriftLength = sqrt( DriftDistance * DriftDistance + 
                        XDriftDueToMagField * XDriftDueToMagField +
                        YDriftDueToMagField * YDriftDueToMagField );

    // What is the charge diffusion after this path
    Sigma = sqrt( DriftLength/_Dist300 ) * _Sigma0 ; //[mm]

    // Project the diffusion sigma on the collection plane
    Sigma_x = Sigma / CosLorenzAngleX ;
    Sigma_y = Sigma / CosLorenzAngleY ;

    SignalPoint sp( CloudCenterX, CloudCenterY,
     Sigma_x, Sigma_y, hit->getTime(), _ionization_points[i].energy() );

    // Load the Charge distribution parameters
    _collection_points[i] = (sp);

#ifdef _DEBUG_1_
cout <<"  drift: "<< i << "-th IP, drift: " << endl << " pos("
     << _collection_points[i].x() << ", "
     << _collection_points[i].y() << ") sigma("
     << _collection_points[i].sigma_x() << ", "
     << _collection_points[i].sigma_y() << ") ampl = "
     << _collection_points[i].amplitude() << endl ;
cout <<"  drift length = "<< DriftLength << endl ;
#endif

  } // loop over ionization points, i.

#ifdef _DEBUG_
  cout << "SiPixelDetUnitDigitizer::drift: end drift. " << endl;
#endif

}


} // namespace SIDIGIR.
