#include "SiDigitizer.h"

namespace SIDIGIR{


//***********************************************************************
// Fluctuate the average eloss in a segment, use the Geant3 routine glandz
void SiPixelDetUnitDigitizer::fluctuateEloss(  float particleMass,
					float particleMomentum,
					float eloss, 
					float length,
					int NumberOfSegs,
					float elossVector[]) throw(string) {
  // Get dedx for this track
  float dedx;

  if( length > 0.) dedx = eloss/length;
  else dedx = eloss;

  // What is the track segment length.
  float segmentLength = length/NumberOfSegs ;

  // Generate charge fluctuations.
  float de = 0. ;
  float sum = 0. ;

  double segmentEloss = (1000.*eloss)/NumberOfSegs ; //eloss in MeV

  for (int i = 0; i < NumberOfSegs; i++ ) {
    //       material,*,   momentum,energy,*, *,  mass
    //myglandz_(14.,segmentLength,2.,2.,dedx,de,0.14);
    // The G4 routine needs momentum in MeV, mass in Mev, delta-cut in MeV,
    // track segment length in mm, segment eloss in MeV 
    // Returns fluctuated eloss in MeV

    double deltaCutoff = _deltaProductionCut; // the cutoff is sometimes redefined inside, so fix it.

    de = _fluctuate.SampleFluctuations( double(particleMomentum*1000.),
				       particleMass, deltaCutoff, 
				       double(segmentLength*10.),
				       segmentEloss )
				       /1000. ; //convert to GeV

    //cout << i <<" "<<de <<" "<< de2 <<" "<< segmentEloss << endl;
    elossVector[i] = de ;

    sum += de ;

  } // for (int i.

  if( sum > 0. ) {  // If fluctuations give eloss > 0 .

    // Rescale to the same total eloss
    float ratio = eloss/sum ;

    //cout << " fluctuate charge, ratio = " << ratio <<" "<< eloss <<" "
    // << sum <<" "<< length<<" " <<dedx<<" "<<NumberOfSegs<<" " 
    // << _deltaProductionCut << " " << particleMass << endl;
    for (int ii = 0; ii < NumberOfSegs; ii++ ) 
              elossVector[ii]= ratio*elossVector[ii];

  } else {  // If fluctuations gives 0 eloss

    float averageEloss = eloss/NumberOfSegs ;
    for ( int ii = 0; ii < NumberOfSegs; ii++ ) 
             elossVector[ii]= averageEloss ; 

  } // if( sum.

  return ;

} //  fluctuateEloss.


} //   namespace  SIDIGIR.
