#include "SiDigitizer.h"
#include "Pixel/Amplitude.h" /* for hit_map_type */
#include <gsl/gsl_sf_erf.h>

namespace SIDIGIR{

/** Integrate charges distributed on the collection plane 
 *  within pixel borders.
 */
void SiPixelDetUnitDigitizer::induce_signal( SIDETUNIT::SiTrkSimHitExtended *hit) throw(string) {

  SIDETUNIT::hit_map_type hit_signal ;

  // map to store pixel integrals in the x and in the y directions
  map<int, double, less<int> > x, y ; 

  // Assign signals to readout channels and store sorted by channel number

  // Iterate over collection points on the collection plane
  unsigned int i_SP = 0 ;

#ifdef  _DEBUG_
cout << "SiPixelDetUnitDigitizer::induce_signal: begin: "
     << endl <<" _collection_points.size() = " << _collection_points.size() << endl ;
#endif

  for ( vector<SignalPoint>::const_iterator i  = _collection_points.begin();
					    i != _collection_points.end(); 
					    i++, i_SP++) {
    double CloudCenterX = i->position().x(); // Charge position in x
    double CloudCenterY = i->position().y(); //                 in y
    double SigmaX = i->sigma_x();            // Charge spread in x
    double SigmaY = i->sigma_y();            //               in y
    double Charge = i->amplitude();          // Charge amplitude

    // Find the maximum cloud spread in 2D plane , assume 3*sigma
    double CloudRight = CloudCenterX + _ClusterWidth*SigmaX;
    double CloudLeft  = CloudCenterX - _ClusterWidth*SigmaX;
    double CloudUp    = CloudCenterY + _ClusterWidth*SigmaY;
    double CloudDown  = CloudCenterY - _ClusterWidth*SigmaY;

    /** Convert the 2D points to pixel indices
     */
    SIUTIL::MeasurementPoint mpRightUp = (_detp->topology()).measurementPosition( SIUTIL::LocalPoint( CloudRight, CloudUp ) ) ;
    unsigned int IPixRightUpX = int( mpRightUp.x() ) ;
    unsigned int IPixRightUpY = int( mpRightUp.y() ) ;

    SIUTIL::MeasurementPoint mpLeftDown = (_detp->topology()).measurementPosition( SIUTIL::LocalPoint( CloudLeft, CloudDown ) ) ;
    unsigned int IPixLeftDownX = int( mpLeftDown.x() ) ;
    unsigned int IPixLeftDownY = int( mpLeftDown.y() ) ;

#ifdef  _DEBUG_ 
#include "check_inc/check_is1.CC"
#endif

    x.clear(); // clear temporary integration array
    y.clear();

#ifdef _DEBUG_
cout << i_SP <<"=sp; First integarte strips in x pix ["
     << IPixLeftDownX <<", "<< IPixRightUpX<<"] pix sizeX="
     <<_detp->pixelSizeX()<<" Ncl*SigmaX="<< _ClusterWidth*SigmaX << endl ;
#endif
    for (unsigned int ix = IPixLeftDownX; ix <= IPixRightUpX; ix++){// loop over x index

      double UpperBound, LowerBound;

      if (ix == 0){
       LowerBound = 0.;
#ifdef _DEBUG_
cout <<" min ix = "<< ix << " CloudCenterX = " << CloudCenterX<< "  SigmaX = "<<SigmaX << endl ;
#endif
      } else {
        //SSh  Hep2Vector mp ; mp.set( double(ix), 0.0) ;
	//	double oLowerBound = freq_( (xLB-CloudCenterX)/SigmaX);
        /**  Function: int gsl_sf_erf_Q_e (double X, gsl_sf_result * RESULT)
	 *   These routine compute the upper tail of the Gaussian probability
	 *   function Q(x) = (1/\sqrt{2\pi}) \int_x^\infty dt \exp(-t^2/2).
         */
//        double xLB = _detp->coordX( ix ) ;
        double xLB = _detp->topology().localPosition( SIUTIL::MeasurementPoint( (double)ix, 0.0 ) ).x() ;

	gsl_sf_result result;

	int status = gsl_sf_erf_Q_e( (xLB - CloudCenterX)/SigmaX, &result) ;

	if (status != 0) throw(string("SiPixelDetUnitDigitizer::induce_signal : could not compute gaussian tail probability for the threshold chosen"));

	LowerBound = 1. - result.val ;  //cout <<" LOWERB PIXEL "<<oLowerBound<<" " <<LowerBound<<" " <<oLowerBound-LowerBound<<endl;
#ifdef _DEBUG_
cout <<" LB, int_(-inf)^("<< ( xLB - CloudCenterX )/SigmaX<<") = "<< result.val<<"  xLB, CloudCenterX = "<< xLB<<", "<<CloudCenterX << "  SigmaX = " <<SigmaX << endl ;
#endif
      } // if (ix == 0) else.

      if (ix == _numPixelsX - 1){
       UpperBound = 1.;
#ifdef _DEBUG_
cout <<" max ix = "<< ix << " CloudCenterX = " << CloudCenterX << "  SigmaX = "<<SigmaX << endl ;
#endif
      }  else {  //SSh mp = MeasurementPoint( double(ix+1), 0.0);
		 //double oUpperBound = freq_( (xUB-CloudCenterX)/SigmaX);
//        double xUB = _detp->coordX( ix + 1 ) ;
        double xUB = _detp->topology().localPosition( SIUTIL::MeasurementPoint( (double)( ix + 1 ), 0.0 ) ).x() ;

	gsl_sf_result result ;

	int status = gsl_sf_erf_Q_e( ( xUB - CloudCenterX )/SigmaX, &result) ;

	if (status != 0) throw(string("SiPixelDetUnitDigitizer::induce_signal : could not compute gaussian tail probability for the threshold chosen"));

	UpperBound = 1. - result.val ; //cout <<" LOWERB PIXEL "<<oUpperBound<<" " <<UpperBound<<" " <<oUpperBound-UpperBound<<endl;
#ifdef _DEBUG_
cout <<" UB, int_(-inf)^("<< ( xUB - CloudCenterX )/SigmaX<<") = "<< result.val<<"  xUB, CloudCenterX = "<< xUB<<", "<<CloudCenterX << "  SigmaX = " <<SigmaX << endl ;
#endif
      }

      double   TotalIntegrationRange = UpperBound - LowerBound ; // get strip
      x[ix] = TotalIntegrationRange; // save strip integral 

#ifdef _DEBUG_
cout << i_SP <<"=sp; x["<<ix<<"] = "<< x[ix] <<" UB, LB = "<< UpperBound<<","<<LowerBound <<" Npix X All="<<_numPixelsX<< endl ;
#endif
    } // for (ix=.

#ifdef _DEBUG_
cout << i_SP <<"=sp; Now integarte strips in y pix ["
     << IPixLeftDownY <<","<< IPixRightUpY<<"]  pix sizeY = "
     <<_detp->pixelSizeY()<<" Ncl*SigmaY="<< _ClusterWidth*SigmaY << endl ;
#endif
    for (unsigned int iy = IPixLeftDownY ; iy <= IPixRightUpY; iy++) { //loope over y ind  

      double UpperBound, LowerBound ;

      if (iy == 0){
       LowerBound = 0. ;
#ifdef _DEBUG_
cout <<" min iy = "<< iy << " CloudCenterY = " << CloudCenterY<< "  SigmaY = "<<SigmaY << endl ;
#endif
      } else { //SSh mp = MeasurementPoint( 0.0, double(iy) );

//        double yLB = _detp->coordY( iy ) ;
        double yLB = _detp->topology().localPosition( SIUTIL::MeasurementPoint( 0.0, (double)( iy ) ) ).y() ;

	gsl_sf_result result ;

	int status = gsl_sf_erf_Q_e( ( yLB - CloudCenterY )/SigmaY, &result) ;

	if (status != 0) throw(string("SiPixelDetUnitDigitizer::induce_signal :could not compute gaussian tail probability for the threshold chosen"));

	LowerBound = 1. - result.val ; //cout <<" LOWERB PIXEL "<<oLowerBound<<" " <<LowerBound<<" " <<oLowerBound-LowerBound<<endl;
#ifdef _DEBUG_
cout <<" LB, int_(-inf)^("<< ( yLB - CloudCenterY )/SigmaY<<") = "<< result.val << " yLB, CloudCenterY = "
     << yLB <<", "<< CloudCenterY<< "  SigmaY = "<<SigmaY << endl ;
#endif
      }

      if ( iy == _numPixelsY - 1 ){ 
        UpperBound = 1. ;
#ifdef _DEBUG_
cout <<" max iy = "<< iy << " CloudCenterY = " << CloudCenterY<< "  SigmaY = "<<SigmaY << endl ;
#endif
      } else { //SSh mp = MeasurementPoint( 0.0, double(iy+1) );

//        double yUB = _detp->coordY( iy + 1 ) ;
        double yUB = _detp->topology().localPosition( SIUTIL::MeasurementPoint( 0.0, (double)( iy + 1 ) ) ).y() ;

	gsl_sf_result result ;

	int status = gsl_sf_erf_Q_e( ( yUB - CloudCenterY )/SigmaY, &result) ;

	if (status != 0) throw(string("SiPixelDetUnitDigitizer::induce_signal :could not compute gaussian tail probability for the threshold chosen"));

	UpperBound = 1. - result.val ; //cout <<" LOWERB PIXEL "<<oUpperBound<<" " <<UpperBound<<" " <<oUpperBound-UpperBound<<endl;
#ifdef _DEBUG_
cout <<" UB, int_(-inf)^("<< ( yUB - CloudCenterY )/SigmaY<<") = "<< result.val << " yUB, CloudCenterY = "
     << yUB <<", "<< CloudCenterY<< "  SigmaY = "<<SigmaY << endl ;
#endif
      }

      double   TotalIntegrationRange = UpperBound - LowerBound ;
      y[iy] = TotalIntegrationRange ; // save strip integral
#ifdef _DEBUG_
cout << i_SP <<"=sp; y["<<iy<<"] = "<< y[iy] <<" UB, LB = "<< UpperBound<<","<<LowerBound <<" Npix Y All="<<_numPixelsY << endl ;
#endif

    } //  for (unsigned int iy.

#ifdef _DEBUG_
cout << endl << i_SP <<"=sp; Get the 2D charge integrals by folding x and y strips: Charge = "<< Charge << endl ;
#endif
    
    double ChargeSum = 0. ;

    for(unsigned int ix = IPixLeftDownX ; ix <= IPixRightUpX ; ix++ ){  // loop over x index

      for(unsigned int iy = IPixLeftDownY ; iy <= IPixRightUpY ; iy++ ){ //loope over y ind

        double ChargeFraction = Charge * x[ix] * y[iy];
	
	ChargeSum += ChargeFraction ;
        
        if( ChargeFraction > 0. ) {
 	  int chan ;
          chan = SIUTIL::PixelDigi::pixelToChannel( ix, iy ) ;  // Get index 
          hit_signal[chan] += ChargeFraction ;

#include "check_inc/check_is2.CC"
	}
      } // end for iy
    } //end for ix

#ifdef _DEBUG_
#include "check_inc/check_is3.CC"
#endif

    if( fabs(Charge - ChargeSum)/Charge > 0.01 )
     cout <<" i_SP = "<<i_SP<<" WARNING in SiPixelDetUnitDigitizer::induce_signal: Charge = "<<Charge<<" != "<< ChargeSum <<" = ChargeSum. Continue..."<< endl ;

  } //for ( vector<SignalPoint>::const_iterator i .

  _sum_induce_signals += hit_signal.size() ;

  _detp->add_signal_from_simHit( hit_signal, hit ) ;

#ifdef  _DEBUG_ 
cout<<" hit_signal.size() = "<< hit_signal.size()
    << "SiPixelDetUnitDigitizer::induce_signal: end." << endl ;
#endif
} // induce_signal.

}  // namespace SIDIGIR.
