#include "SiDigitizer.h"
#include "EVENT/MCParticle.h"
#include "CLHEP/Units/PhysicalConstants.h"

namespace SIDIGIR{


/** Generate primary ionization along the det.unit's track segment. 
 *  Divide the track into small sub-segments  
 */
void SiPixelDetUnitDigitizer::primary_ionization
                         ( SIDETUNIT::SiTrkSimHitExtended * hit ) throw(string) {

// Straight line approximation for trajectory inside active media

//SSh  const float SegmentLength = 0.0010 ; //cm, [ORCA: 10microns in cm]
  const float SegmentLength = 0.010 ; // mm

  // Get the 3D segment direction vector 
  Hep3Vector direction = hit->exitPoint() - hit->entryPoint() ;

  float eLoss = hit->getdEdx() ;  // Eloss in GeV
  float length = direction.mag() ;  // Track length in Silicon

  _NumberOfSegments = (unsigned int) ( length / SegmentLength) ; // Number of segments

  if( _NumberOfSegments < 1 ) _NumberOfSegments = 1 ;

#ifdef _DEBUG_
  cout << " primary_ionzation: _NumberOfSegments: " << _NumberOfSegments << endl ;
  cout << " entry" << hit->entryPoint() << endl ;
  cout << " exit" << hit->exitPoint() << endl ;
  cout << " [exit - entry]"<< (hit->exitPoint() - hit->entryPoint()) << endl ;
  cout << " pid= " << ( hit->getMCParticle() )->getPDG() <<"  mass="<< ( hit->getMCParticle() )->getMass() << endl;
#endif

  float* elossVector = new float[_NumberOfSegments];  // Eloss vector

  if( _fluctuateCharge ) {
  
    EVENT::MCParticle *mcPart = hit->getMCParticle() ;

    float mass = CLHEP::electron_mass_c2 ; // 0.511 ;

    if( mcPart !=0 ){
      mcPart->getMass() ;
    } else
      cout << "SiPixelDetUnitDigitizer::primary_ionization: mcParticle is not set for this hit. Let Mass eq. electon mass = "<< mass << endl ;
    
    float momentum = hit->pabs();
  
    // Generate fluctuated charge points
    fluctuateEloss( mass, momentum, eLoss, length, _NumberOfSegments, 
		    elossVector );
  
  }

  _ionization_points.resize( _NumberOfSegments ) ; // set size

  // loop over segments
  for ( unsigned int i = 0; i != _NumberOfSegments; i++) {

    // Divide the segment into equal length subsegments 
    Hep3Vector point = hit->entryPoint() + 
                         float( (i + 0.5)/_NumberOfSegments ) * direction ;

    float energy ;

    if( _fluctuateCharge ) 

      energy = elossVector[i]/_GeVperElectron ; // Convert charge to elec.

    else

      energy = hit->getdEdx()/_GeVperElectron/float(_NumberOfSegments) ;

      EnergyDepositUnit edu( energy, point) ; //define position,energy point

      _ionization_points[i] = edu ; // save

    //     cout << i << " " << _ionization_points[i].x() << " " 
    // 	 << _ionization_points[i].y() << " " 
    // 	 << _ionization_points[i].z() << " " 
    // 	 << _ionization_points[i].energy() <<endl;

  }

  delete[] elossVector ;

} //  primary_ionization.


} // namespace SIDIGIR.
