#ifndef _GEOMTILING_NAMESPACE_H_
#define _GEOMTILING_NAMESPACE_H_

//#define _DEBUG_ 
//#define _DEBUG_1_ 
//#define _DEBUG_DIGI_DU_ 

#include "SiDigitizer.h"
#include "SiUtil.h"


namespace SIDET{

/** Class SiTracker:
  *  - interface to Gear.xml
  *  - define det.units for current subdetestor
  *  - used in SiDigiProcessor and SiClusteringProcessor
  *  
  * @author S.Shulga
  * @version $Id: SiTracker.h,v 1 2007/06/06 shulga
  */
class SiTracker{

 private:
  
  // for setTechnology(), set from steering file
  SIUTIL::SubDet  _subDet ;

  SIUTIL::DetUnitType  _detUnitType ; // enum, definition: SiUtil.h

  unsigned int _lostHit ;

  unsigned int _numberOfRightFTDLayers ;

  SIDIGIR::SiPixelDetUnitDigitizer *_digitizer ;

 public:

  static SIDETUNIT::DetUnitVecVec _detUnitVecVec ;//VXD[layer,ladder], FTD[layer,ladder] (pixel mode)

  double _hitsInDeadZone ; // init in SiDigitization/src/createOutputCollections_digi.cc

  double _hitsInTwoDU ; // init in SiDigitization/src/createOutputCollections_digi.cc

  // constructor
  SiTracker() ;
  
  // Destructor
  ~SiTracker() ;

  vector<SIDETUNIT::DetUnitVec> getDetUnits() { return _detUnitVecVec; } ;

  SIUTIL::SubDet getSubDet() const { return _subDet ; }

  SIUTIL::DetUnitType getDetUnitType() const { return _detUnitType ; }
  
  unsigned int getLostHit () const { return _lostHit; } ;

  unsigned int getTotalAcceptedSimHits() const throw(string) ;
  unsigned int getTotalAcceptedRawHits() const throw(string) ;

  void setTechnology( StringVec  subDet_Technology ) throw(string) ;

  // called by SiTrkDigiProcessor::init()
  void gearInit( string ) throw(string) ;

  // called by gearInit
  void gearInitVXD( string ) throw(string) ;
  void gearInitFTD( string ) throw(string) ;
  
  void check_detUnits( string sCC ) throw(string) ;
  bool check_layerN( string sCC, int &iLayer ) throw(string) ;

  void fillDetUnits( SimTrackerHit *simHit ) throw(string)  ;

  void fillDetUnits( TrackerRawDataImpl *rawHit ) throw(string) ;

  void digitizeDetUnits( vector<LCCollection*>  & result ) throw(string) ;
  
  void clusterizeDetUnits(vector<LCCollection*>  & result) throw(string) ;
  
  void show( string ) ;

  void howManySimHits() throw(string) ;

  void howManyRawHits() throw(string) ;

  
  void clearDetUnits() ;

}; // class SiTracker


} // namespace SITRK.

#endif


