#include "SiTracker.h"
#include "SiClusterizer.h"

#include "IMPL/LCCollectionVec.h"
#include "IMPL/LCFlagImpl.h"
#include <marlin/Global.h>
#include <gear/GEAR.h>
#include <gearimpl/Vectors.h>
#include <gearimpl/ConstantBField.h>

#include <cassert>

namespace SIDET{

SIDETUNIT::DetUnitVecVec duvv ;
SIDETUNIT::DetUnitVecVec SiTracker::_detUnitVecVec = duvv;


SiTracker::SiTracker():
                _subDet(SIUTIL::DUMMY),
		_detUnitType(SIUTIL::dummy),
		_lostHit(0),
		_numberOfRightFTDLayers(0) {

  _digitizer = new SIDIGIR::SiPixelDetUnitDigitizer() ;
  _hitsInDeadZone = 0. ;
  _hitsInTwoDU = 0. ;
 // _clusterizer = new SiTrkDetUnitClusterizer() ;

} // c-tor SiTracker.




// Destructor
SiTracker::~SiTracker() { delete _digitizer ; }



unsigned int SiTracker::getTotalAcceptedSimHits() const throw(string) {

 unsigned int nB = 0 ;

 for( SIDETUNIT::DetUnitVecVec::const_iterator i =_detUnitVecVec.begin();
 				        i!=_detUnitVecVec.end(); ++i )
 
   for( SIDETUNIT::DetUnitVec::const_iterator j  = (*i).begin();
   			           j != (*i).end(); ++j ) 
	 nB += (*j)->getNumberOfSimHits() ;

 return nB ;

} //  getTotalAcceptedSimHits.





unsigned int SiTracker::getTotalAcceptedRawHits() const throw(string) {

 unsigned int nB = 0 ;

 for( SIDETUNIT::DetUnitVecVec::const_iterator i =_detUnitVecVec.begin();
 				        i!=_detUnitVecVec.end(); ++i)
 
   for( SIDETUNIT::DetUnitVec::const_iterator j  = (*i).begin();
   			           j != (*i).end(); ++j ) 
	 nB += (*j)->getNumberOfRawHits() ;

 return nB ;

} //  getTotalAcceptedRawHits.



void SiTracker::check_detUnits( string sCC ) throw(string) {

 string CC = "SiTracker::check_detUnits: " + sCC ;
 if( _detUnitVecVec.size() == 0 )
                      throw(string("SiTracker::check_detUnits: " + sCC +": det.Units are not created.")) ;

} // check_detUnits().



bool SiTracker::check_layerN( string sCC, int &iLayer ) throw(string) {
 string CC = "SiTracker::check_layerN: " + sCC ;

 switch( _subDet ){ //see: SiUtil/include/SiUtil.h:enum SubDet { VXD, SIT, FTD, DUMMY } ;

  case SIUTIL::VXD:
    iLayer-- ;
    if( (iLayer >= (int)_detUnitVecVec.size()) || ( iLayer < 0 ) )
  	throw(string(CC+" VXD: iLayer=" + SIUTIL::i2s(iLayer) + " >= " 
	         + SIUTIL::i2s(_detUnitVecVec.size()) 
		 + " =_detUnitVecVec.size() or iLayer < 0 ")) ;
    break ;

  case SIUTIL::FTD:
    if( abs(iLayer) > (int)_numberOfRightFTDLayers )
  	throw(string(CC+" FTD: iLayer=" + SIUTIL::i2s(iLayer) 
	         + " >= " + SIUTIL::i2s(_numberOfRightFTDLayers) 
		 + "=_numberOfRightFTDLayers.  _detUnitVecVec.size() = "
	         + SIUTIL::i2s(_detUnitVecVec.size())
		    )) ;
    if( abs(iLayer) > (int)_detUnitVecVec.size()/2 ) return false ;// hit in strip layer (4,5,6,7,-4,-5,-6,-7)

    if( iLayer > 0 ) iLayer-- ;
    else iLayer = (_detUnitVecVec.size()/2 - 1) - iLayer ;
    break ;

  default: throw(string(CC+" Wrong subDet or this subDet is not implemented yet."+"\n")) ;

 } // switch( _subDet.

 return true ;

} // check_layerN.



void SiTracker::fillDetUnits( SimTrackerHit *simHit ) throw(string) {

  string CC = "SiTracker::fillDetUnits(sim):" ;

  check_detUnits( CC ) ;

  int iLayer = simHit->getCellID() ;
  
  if( !check_layerN( CC, iLayer ) ) return ;

  SIDETUNIT::DetUnitVec detUnitVec = _detUnitVecVec[iLayer] ;

// Loop over d.u. to define (temporary) entry/exit hit points in each d.u.
// "Temporary" is because sim.hit is often outside all ladders
// (sim.hit from Mokka is in barrel cylinder but not in ladder)
  for( SIDETUNIT::DetUnitVec::iterator i  = detUnitVec.begin();
			               i != detUnitVec.end(); ++i ){
   (*i)->setHit_tmp( simHit ) ; 
  }

// Loop over all d.u. to find true d.u. for given sim.hit
// (true d.u. contains minimal hit's Z (in det.unit frame))

  double distMin_Hit_entry = 9999999. ;

  SIDETUNIT::DetUnit * du_true = 0 ;

  bool find = false ;

  int nb_hitDetUnits = 0 ; // Nb of hit det.units for given sim.hit
  
  for( SIDETUNIT::DetUnitVec::iterator i  = detUnitVec.begin();
			    i != detUnitVec.end(); ++i ){
			    
   if( (*i)->isSetTmpHit() ){
   
     nb_hitDetUnits++ ;

     Hep3Vector entry_currPos = ( (*i)->get_simHit_tmp() )->entryPoint() ;

     Hep3Vector hitLocal_currPos = ( (*i)->get_simHit_tmp() )->getHitLocalPos() ;

     Hep3Vector dr = hitLocal_currPos - entry_currPos ;

     double dist_curr = dr.mag() ;

     if( dist_curr < distMin_Hit_entry ){ 

      distMin_Hit_entry = dist_curr ;

      du_true = *i ; 

      find = true ;

     } // if( dist_curr < distMin_Hit_entry.

   } // if( (*i)->isSetTmpHit().
   
  } // for.

  if( nb_hitDetUnits > 3 ) throw(CC + string(":") + SIUTIL::i2s(nb_hitDetUnits) + string(" hit d.u. for one sim.hit.") ) ;

  if( nb_hitDetUnits == 2 ) _hitsInTwoDU++ ;

#ifdef _DEBUG_
const double *pos = simHit->getPosition() ;
const float  *mom = simHit->getMomentum() ;
#endif


  if( find ){
    du_true->setHit() ;

#ifdef _DEBUG_
cout<<"-----find-----begin-------------"<< endl ;
cout<<"=== DU found: ==="<< endl ;
du_true->show( simHit ) ;
cout<<"=== all DU's ==="<< endl ;
cout<<" iLayer "<< iLayer << endl ;
cout<<" pos ("<<pos[0] <<", "<<pos[1] <<", "<<pos[2]<<")" << endl ;
cout<<" mom ("<<mom[0] <<", "<<mom[1] <<", "<<mom[2]<<")" << endl << endl ;
for( SIDETUNIT::DetUnitVec::iterator i  = detUnitVec.begin(); 
	 		  i != detUnitVec.end(); ++i ) 
  if( (*i)->isSetTmpHit() ) (*i)->show( simHit ) ;
cout<<"-----find-----end-------------"<< endl ;
#endif

  } else {

   _hitsInDeadZone++ ;

#ifdef _DEBUG_
   cout<<"---- det.unit-not-found --- hit in dead zone ---"<< endl ;
   cout<<"-----not-find-----begin-------------"<< endl ;
   cout<<" iLayer "<< iLayer << endl ;
   SIUTIL::out3( " Gl.pos: ", pos ) ;
   SIUTIL::out3( " Gl.mom: ", mom ) ;

   for( SIDETUNIT::DetUnitVec::iterator i  = detUnitVec.begin(); 
			     i != detUnitVec.end(); ++i ) (*i)->show( simHit ) ;

    cout<<"-----not-find-----end-------------"<< endl ;
    cout<< string("SiTracker::fillDetUnits: sim.hit outside all det.units...") << endl ;
#endif
  }

  return ;
   
} // fillDetUnits .











void SiTracker::fillDetUnits( TrackerRawDataImpl *rawHit ) throw(string) {
// called by SiTrkClusteringProcessor::createOutputCollections_pixels

 string CC = "SiTracker::fillDetUnits(raw):" ;

 check_detUnits( CC ) ;

 for( SIDETUNIT::DetUnitVecVec::iterator iLayer  = _detUnitVecVec.begin();
                                   iLayer != _detUnitVecVec.end();
			         ++iLayer ) {
  
   SIDETUNIT::DetUnitVec detUnitVec = *iLayer ;
#ifdef _DEBUG_DIGI_DU_
   cout<< endl << CC+" detUnitVec.size()= "<< detUnitVec.size() << endl ;
#endif

   for( SIDETUNIT::DetUnitVec::iterator iDetUnit  = detUnitVec.begin(); 
		      	     iDetUnit != detUnitVec.end();
			     ++iDetUnit ) {
    // (*iDetUnit)->check() ;
#ifdef _DEBUG_DIGI_DU_
cout << CC + " Current det.unit: du(Layer,Ndu) = ("
     << (*iDetUnit)->getNLayer()<<","<< (*iDetUnit)->getDetUnitN() << ")" << endl;
#endif

   if( (*iDetUnit)->setRawHit( rawHit ) ){

#ifdef _DEBUG_DIGI_DU_
cout << CC + " After (*iDetUnit)->setRawHit( rawHit )  Det.unit found. Return." << endl ;
#endif
     return ; 
   } // if( (*iDetUnit)->.

#ifdef _DEBUG_DIGI_DU_
cout << CC + " After (*iDetUnit)->setRawHit( rawHit ). Det.unit not found yet." << endl ;
#endif

   } // for( DetUnitVec::iterator iDetUnit.

 } // for( vector<DetUnitVec>::iterator iLayer.

 throw(string(CC + " It is impossible: Det.unit for rawHit is not found.") );
 return ;
   
} // fillDetUnits .

















void SiTracker::digitizeDetUnits(vector<LCCollection*>  & result) throw(string) {

 string CC = "SiTracker::digitizeDetUnits:" ;

 check_detUnits( CC ) ;


/** Create collection for writing raw hits into LCIO file
 */
 LCCollectionVec * rawVec = new LCCollectionVec( LCIO::TRACKERRAWDATA );
 LCCollectionVec * relVec = new LCCollectionVec( LCIO::LCRELATION );

 LCFlagImpl rawFlag( 0 ) ;

 rawFlag.setBit ( LCIO::TRAWBIT_ID1 ) ;

 rawVec->setFlag( rawFlag.getFlag() ) ;

 for( SIDETUNIT::DetUnitVecVec::iterator iLayer  = _detUnitVecVec.begin();
                                   iLayer != _detUnitVecVec.end();
			         ++iLayer){
  
   SIDETUNIT::DetUnitVec detUnitVec = *iLayer ;
#ifdef _DEBUG_DIGI_DU_
   cout<< endl << CC+" detUnitVec.size()= "<< detUnitVec.size() << endl ;
#endif

   for( SIDETUNIT::DetUnitVec::iterator iDetUnit  = detUnitVec.begin(); 
		      	     iDetUnit != detUnitVec.end();
			   ++iDetUnit ){
    // (*iDetUnit)->check() ;
#ifdef _DEBUG_DIGI_DU_
cout << CC + " Current det.unit: du(Layer,Ndu) = ("
     << (*iDetUnit)->getNLayer()<<","<< (*iDetUnit)->getDetUnitN() << ")" << endl;
#endif

    _digitizer->digitize( *iDetUnit ) ;

#ifdef _DEBUG_DIGI_DU_
cout << CC + " After _digitizer->digitize." << endl;
#endif

    (*iDetUnit)->make_digis( rawVec, relVec ) ;

   } // for( DetUnitVec::iterator iDetUnit.

 } // for( vector<DetUnitVec>::iterator iLayer.


 assert( result.size()==0 ) ;

 result.push_back( rawVec ) ;  // [0]
 result.push_back( relVec ) ;  // [1]

cout << CC << " Final: "
     << _digitizer->get_sumSimHits() 
     <<" sim.hits were in digitizer. " << endl 
     <<" Previous event: total Accepted RawHits from all det.units = "
     << getTotalAcceptedRawHits() << endl
     << _digitizer->get_sumInduceSignals()
     << " induced signals were in digitizer from all det.units." << endl ;

 _digitizer->reset_sumSimHits() ;

 _digitizer->reset_sumInduceSignals() ;

 return ;

} // digitizeDetUnits.















void SiTracker::clusterizeDetUnits(vector<LCCollection*>  & result) throw(string) {

 string CC = "SiTracker::clusterizeDetUnits:" ;

 check_detUnits( CC ) ;

/** Create collection for writing raw hits into LCIO file
 */
 LCCollectionVec * recHits = new LCCollectionVec( LCIO::TRACKERHIT ) ;
 LCCollectionVec * relVec = new LCCollectionVec( LCIO::LCRELATION );

 cout<<"=================="  << endl ;
 cout<<" Layer  Nclusters"  << endl ;
 int sum = 0 ;

 for( SIDETUNIT::DetUnitVecVec::iterator iLayer  = _detUnitVecVec.begin();
                                   iLayer != _detUnitVecVec.end();
			         ++iLayer){

   SIDETUNIT::DetUnitVec detUnitVec = *iLayer ;
#ifdef _DEBUG_DIGI_DU_
   cout<< endl << CC+" detUnitVec.size()= "<< detUnitVec.size() << endl ;
#endif

   unsigned int nClusters = 0 ;
   int Layer = 0 ;
   for( SIDETUNIT::DetUnitVec::iterator iDetUnit  = detUnitVec.begin(); 
		      	     iDetUnit != detUnitVec.end();
			     ++iDetUnit ){
    Layer = (*iDetUnit)->getLayerN() ;

#ifdef _DEBUG_DIGI_DU_
vector<TrackerRawDataImpl *> rawhits = (*iDetUnit)->getRawHits() ;
cout << CC + " det.unit(Layer,Ndu)= ("
     << (*iDetUnit)->getNLayer()<<","<< (*iDetUnit)->getDetUnitN() << ") N raw hits="<<(*iDetUnit)->getNumberOfRawHits() << endl;

 for( vector<TrackerRawDataImpl *>::iterator 
            iRawHit  = rawhits.begin();
            iRawHit != rawhits.end(); ++iRawHit ) {
   cout << "oOIOooOIOooOIOo "<< (*iRawHit)->getCellID0() <<", "<<(*iRawHit)->getCellID1() <<" "<<(*iRawHit)->getCellID1() << endl ;
 } // for iRawHit.

cout << "   Loop ended. Next _clusterizer->clusterize( *iDetUnit ):" << endl;
#endif

   SICLU::SiTrkDetUnitClusterizer *clusterizer = new SICLU::SiTrkDetUnitClusterizer() ;

   clusterizer->clusterize( *iDetUnit ) ;

   vector<TrackerHitImpl*> rechitsIMPL = clusterizer->getRecHits() ;
   vector<TrackerHitImpl*> rechits_DU = (*iDetUnit)->getRecHits() ;

   if( rechitsIMPL.size() !=  rechits_DU.size() )
      throw(string(CC+" it is imposible: rechitsIMPL.size() !=  rechits_DU.size(): "+SIUTIL::i2s(rechitsIMPL.size())+" != "+SIUTIL::i2s(rechits_DU.size()))) ;

   nClusters += ( (*iDetUnit)->getRecHits() ).size() ;

   for(vector<TrackerHitImpl*>::iterator irh  = rechitsIMPL.begin(); 
 				      irh != rechitsIMPL.end(); ++irh ){
    recHits->addElement( *irh ) ;
   }

#ifdef _DEBUG_DIGI_DU_
cout << CC + " After make_clusters for (Layer, Ndu) = (" << (*iDetUnit)->getNLayer() << "," <<(*iDetUnit)->getDetUnitN() << ")" << endl ;
#endif
    
    delete clusterizer ;
    
   } // for( DetUnitVec::iterator iDetUnit.

   cout << Layer <<" "<< nClusters << endl ;
   sum += nClusters ;

 } // for( vector<DetUnitVec>::iterator iLayer.
 cout<<"=================="<<  endl ;
 cout<<"  sum = "<< sum <<"   result.size() = "<<result.size()<<  endl ;
 
 assert( result.size()==0 ) ;

 result.push_back( recHits ) ; // [0]
 result.push_back( relVec ) ;  // [1]

 cout << CC << " Final: "
     << getTotalAcceptedRawHits() << " hits were in clusterizeDetUnits from all det.units " 
     << endl << endl ;

 return ;

} // clusterizeDetUnits.
















void SiTracker::clearDetUnits() {

 for( SIDETUNIT::DetUnitVecVec::iterator iLayer  = _detUnitVecVec.begin();
                                   iLayer != _detUnitVecVec.end();
			         ++iLayer) {

   SIDETUNIT::DetUnitVec detUnitVec = *iLayer ;

   for( SIDETUNIT::DetUnitVec::iterator iDetUnit  = detUnitVec.begin(); 
		      	     iDetUnit != detUnitVec.end();
			     ++iDetUnit ) 
			        (*iDetUnit)->clear() ;
 }
} // clearDetUnits.











void SiTracker::howManyRawHits() throw(string) {

 string CC = "SiTracker::howManyRawHits:" ;

 check_detUnits( CC ) ;

 cout << "SiTracker::howManyRawHits: " << endl ;

 cout << "==================" << endl 
      << "Layer   NrawHits " << endl ;

 int sum = 0 ;
 for( SIDETUNIT::DetUnitVecVec::iterator iLayer  = _detUnitVecVec.begin();
                                   iLayer != _detUnitVecVec.end();
			         ++iLayer) {

   SIDETUNIT::DetUnitVec detUnitVec = *iLayer ;

   int Nhits = 0, iLay = 9999 ;
   bool first = true ;
   for( SIDETUNIT::DetUnitVec::iterator iDetUnit  = detUnitVec.begin(); 
		      	     iDetUnit != detUnitVec.end();
			     ++iDetUnit ){ 
   
     if(first){ iLay = (*iDetUnit)->getLayerN() ;  first = false; }
     if( iLay != (*iDetUnit)->getLayerN() ) 
       throw(string("SiTracker::howManyRawHits: iLay != (*iDetUnit)->getNLayer():")+SIUTIL::i2s(iLay)+","+SIUTIL::i2s((*iDetUnit)->getLayerN())  ) ;

     Nhits += (*iDetUnit)->getNumberOfRawHits() ; 

   }

    cout << iLay << "      " << Nhits << endl ;
    sum += Nhits ;

 } // for( vector<DetUnitVec>::iterator iLayer.

 cout << "==================" << endl ;
 cout << " sum = " << sum << endl ;

 return ;

} // howManyRawHits.











void SiTracker::howManySimHits() throw(string) {


 string CC = "SiTracker::howManySimHits:" ;

 check_detUnits( CC ) ;

 cout << "SiTracker::howManySimHits: " << endl ;

 cout << "==================" << endl 
      << "Layer   NsimHits " << endl ;

 int sum = 0 ;
 for( SIDETUNIT::DetUnitVecVec::iterator iLayer  = _detUnitVecVec.begin();
                                   iLayer != _detUnitVecVec.end();
			         ++iLayer) {

   SIDETUNIT::DetUnitVec detUnitVec = *iLayer ;

   int Nhits = 0, iLay = 9999 ;
   bool first = true ;
   for( SIDETUNIT::DetUnitVec::iterator iDetUnit  = detUnitVec.begin(); 
		      	     iDetUnit != detUnitVec.end();
			     ++iDetUnit ){ 
   
     if(first){ iLay = (*iDetUnit)->getLayerN() ;  first = false; }
     if( iLay != (*iDetUnit)->getLayerN() ) 
       throw(string("SiTracker::howManySimHits: iLay != (*iDetUnit)->getNLayer():")+SIUTIL::i2s(iLay)+","+SIUTIL::i2s((*iDetUnit)->getLayerN())  ) ;

     Nhits += (*iDetUnit)->getNumberOfSimHits() ; 

   }

    cout << iLay << "      " << Nhits << endl ;
    sum += Nhits ;

 } // for( vector<DetUnitVec>::iterator iLayer.

 cout << "==================" << endl ;
 cout << " sum = " << sum << endl ;

 return ;

} // howManySimHits.












void SiTracker::setTechnology( StringVec subDet_Technology ) 
					throw(string) {
 if( subDet_Technology.size() != 2 )
    throw(string(" SiTracker::SetTechnology: subDet_Technology.size != 2: ")
         + SIUTIL::i2s( subDet_Technology.size() ) 
	 + string(" Program terminated. ") ) ;
 
 string subDet = subDet_Technology[0] ;
 
 string technology = subDet_Technology[1] ;

 _subDet = SIUTIL::enumSubDet( subDet ) ;

 _detUnitType = SIUTIL::enumDetUnitType( technology ) ;

} //  SetTechnology.





void SiTracker::show( string CC ){

  cout<< CC + " show nothing " << endl;

} // Show.




} // namespace SIDET.
