#include "SiTracker.h"
#include "Pixel/DiskDetUnit.h"

// STUFF needed for GEAR
#include <marlin/Global.h>
#include <gear/GEAR.h>
#include <gearimpl/Vectors.h>
#include <gearimpl/ConstantBField.h>

#include <gearimpl/GearParametersImpl.h>
#include <gearimpl/Util.h>

namespace SIDET{

void SiTracker::gearInitFTD( string sCC ) throw(string) {
   string CC = "SiTracker::gearInitFTD: "+ sCC ;

   const gear::GearParametersImpl& gearPar 
        = dynamic_cast<const gear::GearParametersImpl&>
	  ( marlin::Global::GEAR->getGearParameters( "FTD" ) );

   const gear::ConstantBField Bf 
            = dynamic_cast<const gear::ConstantBField&>
          ( marlin::Global::GEAR->getBField() ) ;

   gear::Vector3D p = gear::Vector3D(0.,0.,0.) ;
   double ftdBField = ( Bf.at( p ) ).z() ;
   cout << "Digitizer::init(): Bf      = " << Bf.at( p ) << endl ;
   cout << "Digitizer::init(): ftdBField     = " << ftdBField << endl ;

   cout << " SiTracker::gearInitFTD: FTD, gearPar: " << gearPar << endl ;

   double tanLorentzAnglePerTesla = gearPar.getDoubleVal( "tanLorentzAnglePerTesla" ) ;

   double pixThresholdInNoiseUnits = gearPar.getDoubleVal( "pixThresholdInNoiseUnits" ) ;
   double seedThresholdInNoiseUnits = gearPar.getDoubleVal( "seedThresholdInNoiseUnits" ) ;
   double clusterThresholdInNoiseUnits = gearPar.getDoubleVal( "clusterThresholdInNoiseUnits" ) ;

   IntVec numberWedgeModulesInLayer = gearPar.getIntVals  ( "NumberWedgeModulesInLayer" ) ;

   DoubleVec inner_rFTD = gearPar.getDoubleVals( "sensitiveFTDInnerRadius"   ) ;
   DoubleVec outer_rFTD = gearPar.getDoubleVals( "sensitiveFTDOuterRadius"   ) ;
   DoubleVec mean_zFTD = gearPar.getDoubleVals( "sensitiveFTDZCoordinate"   ) ;
   double thicknessFTD  = gearPar.getDoubleVal ( "sensitiveFTDDiskThickness" ) ;

   double pixelSizeX_duFrame = gearPar.getDoubleVal ( "pixelSizeX_duFrame" ) ;
   double pixelSizeY_duFrame = gearPar.getDoubleVal ( "pixelSizeY_duFrame" ) ;
/*
   PixInReadOutRowFTD        = gearPar.getIntVal    ( "PixInReadOutRow" ) ;
   PixInReadOutColumnFTD     = gearPar.getIntVal    ( "PixInReadOutColumn" ) ;
   pixelNumberOfModulesFTD   = gearPar.getIntVal    ( "pixelNumberOfModules" ) ;
   pixelAngleLayerToLayerFTD = gearPar.getDoubleVal ( "pixelAngleLayerToLayer" ) ;

   stripAngleLayerToLayerFTD   = gearPar.getDoubleVal( "stripAngleLayerToLayer" ) ;
   stripWidthFTD               = gearPar.getDoubleVal( "stripWidth" ) ;
   Rate1stTo3rdTrapezoidSideFTD= gearPar.getDoubleVal( "Rate1stTo3rdTrapezoidSide" ) ; 
   Rate2ndTo3rdTrapezoidSideFTD= gearPar.getDoubleVal( "Rate2ndTo3rdTrapezoidSide" ) ;
   stripNumberOfModulesFTD     = gearPar.getIntVal   ( "stripNumberOfModules" ) ;
*/

//-- Init Pixel Layers --
   _numberOfRightFTDLayers = inner_rFTD.size() ;
   if(_numberOfRightFTDLayers==0)
     throw(string(CC + " Correct number of values: inner_rFTD.size()==0! (see gear.xml steering file)." ));

   int numberOfRightPixelLayers = gearPar.getIntVal ( "NumberOfPixelLayer" ) ;
   if( numberOfRightPixelLayers > (int)numberWedgeModulesInLayer.size() ) 
     throw(string(" Correct NumberOfPixelLayer or number of values in NumberWedgeModulesInLayer (gear.xml steering file): "+SIUTIL::i2s(numberOfRightPixelLayers)+", "+SIUTIL::i2s(numberWedgeModulesInLayer.size()) ));
     
   IntVec numberLaddersInPixelWedgeModule = gearPar.getIntVals( "NumberLaddersInPixelWedgeModule" ) ;
   if( numberOfRightPixelLayers != (int)numberLaddersInPixelWedgeModule.size() ) 
     throw(string(" Correct NumberOfPixelLayer or number of values in NumberLaddersInPixelWedgeModule (gear.xml steering file): "+SIUTIL::i2s(numberOfRightPixelLayers)+", "+SIUTIL::i2s(numberLaddersInPixelWedgeModule.size()) ));

   // loop over pixel layers to create det.units for each pixel layer
   
   _detUnitVecVec.clear() ;

//----

   IntVec::iterator iNwm = numberWedgeModulesInLayer.begin() ;
   IntVec::iterator iNlwm = numberLaddersInPixelWedgeModule.begin() ;

   for( int iLayer=0; iLayer < numberOfRightPixelLayers; 
            iLayer++, iNwm++, iNlwm++ ){ // loop over layers (iLayer > 0)

    cout <<"+"<< iLayer <<"-th layer of FTD contains "
         << *iNwm << " wedge modules. " << endl
	 <<" Each wedge consist of " << *iNlwm <<" ladders (det.units)."<< endl ;

    SIDETUNIT::DetUnitVec detUnitVec_tmp ;
     
    for( int iWedge = 0; iWedge < *iNwm; 
    	     iWedge++ ){// loop over wedges

      //cout <<"  Next wedge N " << iWedge << endl ;

      for( int iDU = 0; iDU < (*iNlwm); iDU++ ){// loop over ladder = det.unit

       //cout <<"      Next d.u. N " << iDU << endl ;
       SIDETUNIT::DiskDetUnit *du = new SIDETUNIT::DiskDetUnit( _detUnitType, 
      				iLayer + 1,
				(unsigned int)iWedge,
				(unsigned int)(*iNwm),
				(unsigned int)(*iNlwm),
				(unsigned int)iDU,
             inner_rFTD[ iLayer ],  // Rmin
	     outer_rFTD[ iLayer ],  // Rmax
	     mean_zFTD[ iLayer ] - thicknessFTD*0.5,  // Zmin
	     mean_zFTD[ iLayer ] + thicknessFTD*0.5, // Zmax
	               pixelSizeX_duFrame,
	               pixelSizeY_duFrame,
				    ftdBField,
				    tanLorentzAnglePerTesla,
			pixThresholdInNoiseUnits,
			seedThresholdInNoiseUnits,
			clusterThresholdInNoiseUnits ) ;

       detUnitVec_tmp.push_back( du ) ;

      } // for( int i=0.

    }  //  for(unsigned int iWedge.

cout << CC <<" >>> detUnitVec_tmp.size() ="<< detUnitVec_tmp.size() << endl ;

    _detUnitVecVec.push_back( detUnitVec_tmp ) ;

   } // for( int iLayer. (iLayer > 0)


   iNwm = numberWedgeModulesInLayer.begin() ;
   iNlwm = numberLaddersInPixelWedgeModule.begin() ;

   for( int iLayer=0; iLayer < numberOfRightPixelLayers; 
            iLayer++, iNwm++, iNlwm++ ){ // loop over layers (iLayer < 0)

    cout <<"-"<< iLayer <<"-th layer of FTD contains "
         << *iNwm << " wedge modules. " << endl
	 <<" Each wedge consist of " << *iNlwm <<" ladders (det.units)."<< endl ;

    SIDETUNIT::DetUnitVec detUnitVec_tmp ;

    for( int iWedge = 0; iWedge < *iNwm; iWedge++ ){// loop over wedges

      for( int iDU = 0; iDU < (*iNlwm); iDU++ ){// loop over ladder = det.unit

       SIDETUNIT::DiskDetUnit *du = new SIDETUNIT::DiskDetUnit( _detUnitType, 
      				-iLayer - 1,
				(unsigned int)iWedge,
				(unsigned int)(*iNwm),
				(unsigned int)(*iNlwm),
				(unsigned int)iDU,
              inner_rFTD[ iLayer ],  // Rmin
	      outer_rFTD[ iLayer ],  // Rmax
	     -mean_zFTD[ iLayer ] - 0.5*thicknessFTD, // Zmin
	     -mean_zFTD[ iLayer ] + 0.5*thicknessFTD, // Zmax
	               pixelSizeX_duFrame,
	               pixelSizeY_duFrame,
				    ftdBField,
				    tanLorentzAnglePerTesla,
			pixThresholdInNoiseUnits,
			seedThresholdInNoiseUnits,
			clusterThresholdInNoiseUnits ) ;

       detUnitVec_tmp.push_back( du ) ;

      } // for( int i=0.

    }  //  for(unsigned int iWedge.

cout << CC <<" >>> detUnitVec_tmp.size() ="<< detUnitVec_tmp.size() << endl ;

    _detUnitVecVec.push_back( detUnitVec_tmp ) ;

   } // for( int iLayer. (iLayer < 0)

} //  gearInitFTD().

}// namespace SITRK.
