#include "SiTracker.h"
#include "Pixel/BarrelDetUnit.h"

// STUFF needed for GEAR
#include <marlin/Global.h>
#include <gear/GEAR.h>
#include <gearimpl/Vectors.h>
#include <gearimpl/ConstantBField.h>
#include "gear/VXDLayerLayout.h"
#include <gearimpl/GearParametersImpl.h>
#include <gearimpl/Util.h>


namespace SIDET{

void SiTracker::gearInitVXD( string sCC ) throw(string) {
   string CC = "SiTracker::gearInitVXD: "+ sCC ;

//   const StringVec& keys = marlin::Global::GEAR->getGearParameterKeys() ;

   const gear::ConstantBField Bfield  
            = dynamic_cast<const gear::ConstantBField&>
          ( marlin::Global::GEAR->getBField() ) ;

   gear::Vector3D p = gear::Vector3D(0.,0.,0.) ;
   double vxdBField = ( Bfield.at( p ) ).z() ;
   cout << "Digitizer::init(): vxdBField     = " << vxdBField << endl ;

   const gear::VXDParameters& VXDgearPar = marlin::Global::GEAR->getVXDParameters() ;

   const gear::VXDLayerLayout& VXDLayers = VXDgearPar.getVXDLayerLayout() ;

   const gear::GearParametersImpl& VXDInfraGearPar 
         = dynamic_cast<const gear::GearParametersImpl&>
	   ( marlin::Global::GEAR->getGearParameters( "VXDInfra" ) ) ;

   double tanLorentzAnglePerTesla_VXD = VXDInfraGearPar.getDoubleVal( "tanLorentzAnglePerTesla" ) ;

   double pixThresholdInNoiseUnits = VXDInfraGearPar.getDoubleVal( "pixThresholdInNoiseUnits" ) ; 
   double seedThresholdInNoiseUnits = VXDInfraGearPar.getDoubleVal( "seedThresholdInNoiseUnits" ) ;
   double clusterThresholdInNoiseUnits = VXDInfraGearPar.getDoubleVal( "clusterThresholdInNoiseUnits" ) ;

   IntVec laddersInLayerVXD = VXDInfraGearPar.getIntVals  ( "LaddersInLayer" ) ;

   if( laddersInLayerVXD.size() != (unsigned int)(VXDLayers.getNLayers()) ) 
     throw(string(" Correct laddersInLayer  or number of layers in gear.xml steering file: "+SIUTIL::i2s(laddersInLayerVXD.size())+", "+SIUTIL::i2s(VXDLayers.getNLayers()) ));

   // loop over layers
   unsigned int iLayer = 0 ;// Layer number

   _detUnitVecVec.clear() ;

   for(IntVec::iterator  nLad  = laddersInLayerVXD.begin();
  			 nLad != laddersInLayerVXD.end(); nLad++, iLayer++ ){

    cout <<iLayer<<"-th layer of VXD contains "
         << *nLad << " ladders." << endl ;

    unsigned int iDU = 0 ;

    SIDETUNIT::DetUnitVec detUnitVec_tmp ;
    
    for( int i=0; i < (*nLad); i++, iDU++ ){// each ladder = det.unit

      SIDETUNIT::BarrelDetUnit *du = new SIDETUNIT::BarrelDetUnit( _detUnitType, 
      				iLayer, (unsigned int)(*nLad), iDU,
//                              VXDLayers.getLadderDistance(iLayer),
//				VXDLayers.getLadderDistance(iLayer) + VXDLayers.getLadderThickness(iLayer),
                                VXDLayers.getSensitiveDistance(iLayer),
				VXDLayers.getSensitiveDistance(iLayer) + VXDLayers.getSensitiveThickness(iLayer),
//VXDLayers.getLadderDistance(iLayer) + 0.5*VXDLayers.getLadderThickness(iLayer),
//VXDLayers.getLadderDistance(iLayer) + 1.5*VXDLayers.getLadderThickness(iLayer),
//			       -VXDLayers.getLadderLength(iLayer), 
//				VXDLayers.getLadderLength(iLayer),
			       -VXDLayers.getSensitiveLength(iLayer), 
				VXDLayers.getSensitiveLength(iLayer),
				VXDInfraGearPar.getDoubleVal( "pixelSizeX_duFrame"),
				VXDInfraGearPar.getDoubleVal( "pixelSizeY_duFrame"),
				vxdBField,
				tanLorentzAnglePerTesla_VXD,
				pixThresholdInNoiseUnits,
				seedThresholdInNoiseUnits,
				clusterThresholdInNoiseUnits ) ;

      detUnitVec_tmp.push_back( du ) ;

    } // for( int i=0.
    
    cout <<" Layer: "<<iLayer << " LadderDistance = " << VXDLayers.getLadderDistance(iLayer) << endl 
         << " LadderThickness = " << VXDLayers.getLadderThickness(iLayer) << endl << endl ;

    _detUnitVecVec.push_back( detUnitVec_tmp ) ;
  
  } //for(IntVec::iterator  iv .

} //  gearInitVXD().

}// namespace SITRK.
