#ifndef _TRACKER_PIXEL_DIGI_H_
#define _TRACKER_PIXEL_DIGI_H_

#include <utility>

#include "SiUtil.h"

using namespace std ;

namespace SIUTIL{

const int N_bit = 64 ;

/** ORCA, CMS software, 2006
 *  Adapted in MarlinReco, Sergey Shulga, May, 2007
 *
 *  Persistent digi for the Pixels.
 *  @version $Id: PixelDigi.h,v 1 2007/06/06 shulga
 */

class PixelDigi {

 public:

  typedef  TcellID  PackedDigiType; // TcellID == lcio::long64
  typedef  TcellID  ChannelType;

  PixelDigi( int packed_value) : theData(packed_value) {}

  PixelDigi( int row, int col, int adc) {
    init( row, col, adc);
  }

  PixelDigi( int chan, int adc) {
    pair<int,int> rc = channelToPixel(chan);
    init( rc.first, rc.second, adc);
  }

  PixelDigi() : theData(0) {}

  void init( int row, int col, int adc) throw(string) ;

  // Access to digi information
  int row() const     {return (theData >> the_Packing.row_shift) & 
			 the_Packing.row_mask;}
  int column() const  {return (theData >> the_Packing.column_shift) & 
			 the_Packing.column_mask;}
  int time() const    {return (theData >> the_Packing.time_shift) & 
			 the_Packing.time_mask;}
  int adc() const     {return (theData >> the_Packing.adc_shift) & 
			 the_Packing.adc_mask;}
  PackedDigiType packedData() const {return theData;}

  static pair<int,int> channelToPixel( int ch) {
    int row = ( ch >> the_Packing.column_width ) & 
                         the_Packing.row_mask ;
    int col = ch & the_Packing.column_mask ;
    return pair<int,int>(row,col);
  }

  static int pixelToChannel( int row, int col) {
    return (row << the_Packing.column_width) | col;
  }

  int channel() const {return pixelToChannel( row(), column());}

  //   int channel() const {return (theData >> the_Packing.channel_shift) & 
  // 		                        the_Packing.channel_mask;}

 private:
  PackedDigiType theData;
  /**
   * Pack the pixel information to use less memory
   */
  class Packing {
  public:

    // Constructor: pre-computes masks and shifts from field widths
    Packing(const int row_w, const int column_w, 
	    const int time_w, const int adc_w);

    // public data:
    int adc_shift;
    int time_shift;
    int row_shift;
    int column_shift;
    // int channel_shift;
    PackedDigiType adc_mask;
    PackedDigiType time_mask;
    PackedDigiType row_mask;
    PackedDigiType column_mask;
    // PackedDigiType channel_mask;

    int row_width;
    int column_width;
    int adc_width;

    int max_row;
    int max_column;
    int max_adc;
  };

  static Packing   the_Packing;
};  

// Comparison operators
inline bool operator<( const PixelDigi& one, const PixelDigi& other) {
  return one.channel() < other.channel();
}

#include<iostream>
// needed by COBRA
inline ostream & operator<<(ostream & o, const PixelDigi & digi) {
  return o << digi.channel();
}

} //namespace SIUTIL .

#endif
