#ifndef _SIUTIL_NAMESPACE_H_
#define _SIUTIL_NAMESPACE_H_

#include "math.h"
#include <string>
#include <sstream>

#include "CLHEP/Vector/ThreeVector.h"

#include "lcio.h"
#include "LCIOSTLTypes.h"
#include "IMPL/TrackerRawDataImpl.h"
#include "UTIL/CellIDDecoder.h"

using namespace std ;
using namespace CLHEP ;

/**  Auxiliary classes, types, variables, constats, functions
 *
 *   @author S.Shulga
 *   @version $Id: SiUtil.h, v 1 2007/06/06 shulga
 */

namespace SIUTIL{

typedef lcio::long64           TcellID ;
typedef std::vector< TcellID > TcellIDVec ;
typedef TcellIDVec::iterator   IcellIDVec ;


enum DetUnitType { barrelPixels,
		   barrelLaddersPixels, 
		   barrelZStrips, 
		   barrelPhiStrips, 
		   diskRectanglePixels,
		   diskPhiStrips,
		   diskRStrips, 
		   dummy } ;

const unsigned int intDummy = 99999;
const double dblDummy = 99999.;

string sDetUnitType( DetUnitType detUnitType ) throw(string) ;

DetUnitType enumDetUnitType( string S ) throw(string) ;


enum SubDet { VXD, SIT, FTD, DUMMY } ;

string sSubDet( SubDet subDet ) throw(string) ;

SubDet enumSubDet( string S ) throw(string) ;



void ShowRunHeaderParameters( string Comnt, 
                              EVENT::LCRunHeader* run ) ;


string i2s(int i) ;
string d2s(double i) ;

void out3( string c, double *p ) ;

void out3( string c, Hep3Vector p ) ;

void out3( string c, const double *p ) ;

void out3( double *p ) ;

void out3( const double *p ) ;

void out3( string c, const float *p ) ;

void out3( string c, float *p ) ;

void out2( string c, double *p ) ;

void out2( string c1, double *p1, double *p2, 
                      double *p3, double *p4 ) ;

int get_iDefault () ;

double get_dDefault () ;

string get_sDefault () ;

EVENT::StringVec get_sVecDefault () ;

EVENT::IntVec get_iVecDefault () ;

double get_epsil() ;
//double get_Pi();           // _Pi(3.14159265358979311599796)

double Grad_in_Rad() ; // = 5.72957795130823228646477e+01; // 180./3.141592653;

void detUnitBarrelFrame( Hep3Vector &v ) ;

void detUnitBarrelFrame_back( Hep3Vector &v ) ;

} // namespace SIUTIL.

#endif
