// Modify the pixel packing to make 100micron pixels possible. d.k. 2/02
//

#include "PixelDigi.h"

#include <iostream>
#include <algorithm>

namespace SIUTIL{

//bool firstA = true ;

void PixelDigi::init( int row, int col, int adc) throw(string) {
  // This check is for the maximal row or col number that can be packed
  // in a PixelDigi. The actual number of rows or columns in a detector
  // may be smaller!

  if ( row < 0 || row > the_Packing.max_row || 
       col < 0 || col > the_Packing.max_column) {
    throw(string("PixelDigi::init : row or column out packing range: row ")
        +i2s(row)+" in [0,"+i2s(the_Packing.max_row) 
	+"], column "
	+i2s(col) 
	+ " in [0,"
	+i2s(the_Packing.max_column) );
  }

  // Set adc to max_adc in case of overflow
  adc = (adc > the_Packing.max_adc) ? the_Packing.max_adc : max(adc,0);

  theData = (row << the_Packing.row_shift) | 
    (col << the_Packing.column_shift) | 
    (adc << the_Packing.adc_shift);
} // PixelDigi::init.




PixelDigi::Packing::Packing(const int row_w, const int column_w, 
			    const int time_w, const int adc_w) :
  row_width(row_w), column_width(column_w), adc_width(adc_w) {
  // Constructor: pre-computes masks and shifts from field widths
  // Order of fields (from right to left) is
  // row, column, time, adc count.

  if ( row_w+column_w+time_w+adc_w != N_bit ) {
    cout << endl << "Warning in PixelDigi::Packing constructor:" 
	 << "sum of field widths != 32" << endl;
  }
//   adc_shift     = 0;
//   time_shift    = adc_w;
//   column_shift  = time_shift + time_w;
//   row_shift     = column_shift + column_w;
//   channel_shift = column_shift;
  
  // Fields are counted from right to left!

  row_shift     = 0;
  column_shift  = row_shift + row_w;
  time_shift    = column_shift + column_w;
  adc_shift     = time_shift + time_w;
  
  row_mask     = ~(~0 << row_w);
  column_mask  = ~(~0 << column_w);
  time_mask    = ~(~0 << time_w);
  adc_mask     = ~(~0 << adc_w);
  //  channel_mask = ~(~0 << row_w+column_w);

  max_row = row_mask;
  max_column = column_mask;
  max_adc = adc_mask ; 
  //cout << endl << "<!-- ATTENTION: static member init PixelDigi::Packing constructor:" << endl
  //     <<" max_row,max_column,max_adc = "<< max_row<<","<< max_column<<","<<max_adc << endl 
  //     << " Initialization of static data members - DEFINES DIGI PACKING ! -->" << endl  << endl ;

} //  PixelDigi::Packing::Packing .

/*
// Extract from CMSIM manual (version Thu Jul 31 16:38:50 MET DST 1997)
// --------------------------------------------------------------------
// DIGI format for pixel
//
// For pixel digitization one word per fired pixel is used. 
// The information includes pixel row and column number, time
// and charge information with 7, 9, 4 and 12 bits for each as shown below. 
//
//  :DETD    :TRAK  :PXBD    4   #. no. of digitization elements
//   #. name     no. bits
//     :V          7             #. row number
//     :W          9             #. column number
//     :TIME       4             #. time (ns)
//     :ADC       12             #. charge
//
// MODIFY 19.02.2002 for ORCA_6
// Change to enable 100micron row pixels, we than have 160 pixels in the v 
// direction.
//   #. name     no. bits
//     :V          8             #. row number        (256)
//     :W          9             #. column number     (512)
//     :TIME       4             #. time (ns)         (16)
//     :ADC       11             #. charge            (2048)
*/

// Initialization of static data members - DEFINES DIGI PACKING !
//PixelDigi::Packing PixelDigi::the_Packing( 7, 9, 4, 12); //row, col, time, adc

//PixelDigi::Packing PixelDigi::the_Packing( 8, 9, 4, 11); // row, col, time, adc
// SSh: 32->64
PixelDigi::Packing PixelDigi::the_Packing( 16, 18, 8, 22); // row, col, time, adc


} // namespace SIUTIL.
