#include "SiUtil.h"


namespace SIUTIL{

void detUnitBarrelFrame( Hep3Vector &v ){ // x" = y, y" = z, z" = x
 double x_old = v.x() ;
 double y_old = v.y() ;
 double z_old = v.z() ;
 v.set( y_old, z_old, x_old ) ;
}

void detUnitBarrelFrame_back( Hep3Vector &v ){ // x = z", y = x", z = y"
 double x_old = v.x() ;
 double y_old = v.y() ;
 double z_old = v.z() ;
 v.set( z_old, x_old, y_old ) ;
}



double get_epsil() { return 1.e-7 ; }
double Grad_in_Rad(){ return 5.72957795130823228646477e+01; } // 180./3.141592653;

int get_iDefault () { return 9999 ; }

double get_dDefault () { return 9999. ; }

string get_sDefault () { return "dummy"; }

EVENT::StringVec get_sVecDefault () { EVENT::StringVec sV; return sV ; }

EVENT::IntVec get_iVecDefault () { EVENT::IntVec iV; return iV ; }


string i2s(int i){
 ostringstream ostr;  ostr << i ;
 return ostr.str() ;
}

string d2s(double i){
 ostringstream ostr;  ostr << i ;
 return ostr.str() ;
}

void out3( string c, Hep3Vector p ){
 cout << c << p[0]<<"," << p[1]  <<"," << p[2] << "; mag =" << p.mag() << endl;
}

void out3( string c, double *p ){
 cout << c << p[0]<<"," << p[1]  <<"," << p[2] << "; mag ="
      << sqrt( p[0]*p[0] + p[1]*p[1] + p[2]*p[2] ) << endl;
}

void out3( string c, const float *p ){
 cout << c << p[0]<<"," << p[1]  <<"," << p[2] << "; mag ="
      << sqrt( p[0]*p[0] + p[1]*p[1] + p[2]*p[2] ) << endl;
}

void out3( string c, float *p ){
 cout << c << p[0]<<"," << p[1]  <<"," << p[2] << "; mag ="
      << sqrt( p[0]*p[0] + p[1]*p[1] + p[2]*p[2] ) << endl;
}

void out3( double *p ){
 cout << p[0]<<"," << p[1]  <<"," << p[2] << endl;
}

void out3( const double *p ){
 cout << p[0]<<"," << p[1]  <<"," << p[2] << endl;
}

void out3( string c, const double *p ){
 cout << c << p[0]<<"," << p[1]  <<"," << p[2] << "; mag ="
      << sqrt( p[0]*p[0] + p[1]*p[1] + p[2]*p[2] ) << endl;
}

void out2( string c, double *p ){
 cout << c << p[0] <<"," << p[1] << "; mag ="
      << sqrt(p[0]*p[0] + p[1]*p[1]) << endl;
}


void out2( string c1, double *p1, double *p2, 
                      double *p3, double *p4 ){
 out2( c1, p1 );
 out2( c1, p2 );
 out2( c1, p3 );
 out2( c1, p4 );

}

void init0(double *&p, double *&pp){ 

 p[0] = 0.; pp[0] = 0.; 
 p[1] = 0.; pp[1] = 0.;

}




// header in SiTrkDigi/include/DetUnit.h
string sDetUnitType( DetUnitType detUnitType ) throw(string){

  switch( detUnitType ){
   case barrelPixels: return "barrelPixels" ;
   case barrelLaddersPixels: return "barrelLaddersPixels" ;
   case diskRectanglePixels: return "diskRectanglePixels" ;
   case diskPhiStrips: return "diskPhiStrips" ;
   case diskRStrips: return "diskRStrips" ;
   case dummy: return "dummy" ;
   default: 
     ostringstream s1 ; s1 << int( detUnitType ) ;
     throw(string("SIUTIL::sDetUnitType(DetUnitType): there is no such detUnitType " + s1.str() + "\n")) ;
  } // switch( detUnitType.

}




DetUnitType enumDetUnitType( string S ) throw(string){
 if(S=="barrelPixels") return barrelPixels ;
 if(S=="barrelLaddersPixels") return barrelLaddersPixels ;
 if(S=="barrelZStrips") return barrelZStrips ; 
 if(S=="barrelPhiStrips") return barrelPhiStrips ; 
 if(S=="diskRectanglePixels") return diskRectanglePixels ;
 if(S=="diskPhiStrips") return diskPhiStrips ;
 if(S=="diskRStrips") return diskRStrips ;
 throw(string("SIUTIL::enumDetUnitType(string): there is no such detUnitType " + S + "\n")) ;

} // enumDetUnitType.




string sSubDet( SubDet subDet ) throw(string){

  switch( subDet ){
   case VXD: return "VXD" ;
   case SIT: return "SIT" ;
   case FTD: return "FTD" ;
   default: 
     ostringstream s1 ; s1 << int( subDet ) ;
     throw(string("sSubDet(GeomTiling.h, out.cc): there is no such Subdetector " + s1.str() + "\n")) ;
  } // switch( subDet.

}

SubDet enumSubDet( string S ) throw(string){

 if(S=="VXD") return VXD ;
 if(S=="SIT") return SIT ;
 if(S=="FTD") return FTD ; 
 throw(string("SIUTIL::enumSubDet(string): there is no such SubDet " + S + "\n")) ;

}


} // namespace SIUTIL.
